@@ -26,6 +26,7 @@
 
 #include <kis_types.h>
 #include <kis_shared.h>
+#include <kis_dab_shape.h>
 #include <kritabrush_export.h>
 
 class KisQImagemask;
@@ -39,7 +40,6 @@ class KisPaintInformation;
 class KisBoundary;
 class KisPaintopLodLimitations;
 
-
 enum enumBrushType {
     INVALID,
     MASK,
@@ -88,10 +88,10 @@ protected:
     {
     public:
         PlainColoringInformation(const quint8* color);
-        virtual ~PlainColoringInformation();
-        virtual const quint8* color() const ;
-        virtual void nextColumn();
-        virtual void nextRow();
+        ~PlainColoringInformation() override;
+        const quint8* color() const override ;
+        void nextColumn() override;
+        void nextRow() override;
     private:
         const quint8* m_color;
     };
@@ -102,10 +102,10 @@ protected:
     public:
 
         PaintDeviceColoringInformation(const KisPaintDeviceSP source, int width);
-        virtual ~PaintDeviceColoringInformation();
-        virtual const quint8* color() const ;
-        virtual void nextColumn();
-        virtual void nextRow();
+        ~PaintDeviceColoringInformation() override;
+        const quint8* color() const override ;
+        void nextColumn() override;
+        void nextRow() override;
 
     private:
 
@@ -118,22 +118,25 @@ public:
     KisBrush();
     KisBrush(const QString& filename);
 
-    virtual ~KisBrush();
+    ~KisBrush() override;
 
-    virtual bool load() {
+    virtual qreal userEffectiveSize() const = 0;
+    virtual void setUserEffectiveSize(qreal value) = 0;
+
+    bool load() override {
         return false;
     }
 
-    virtual bool loadFromDevice(QIODevice *) {
+    bool loadFromDevice(QIODevice *) override {
         return false;
     }
 
 
-    virtual bool save() {
+    bool save() override {
         return false;
     }
 
-    virtual bool saveToDevice(QIODevice* ) const {
+    bool saveToDevice(QIODevice* ) const override {
         return false;
     }
 
@@ -175,12 +178,12 @@ public:
     /**
      * @return the width of the mask for the given scale and angle
      */
-    virtual qint32 maskWidth(double scale, double angle, qreal subPixelX, qreal subPixelY, const KisPaintInformation& info) const;
+    virtual qint32 maskWidth(KisDabShape const&, qreal subPixelX, qreal subPixelY, const KisPaintInformation& info) const;
 
     /**
      * @return the height of the mask for the given scale and angle
      */
-    virtual qint32 maskHeight(double scale, double angle, qreal subPixelX, qreal subPixelY, const KisPaintInformation& info) const;
+    virtual qint32 maskHeight(KisDabShape const&, qreal subPixelX, qreal subPixelY, const KisPaintInformation& info) const;
 
     /**
      * @return the logical size of the brush, that is the size measured
@@ -191,7 +194,7 @@ public:
      *         of this metric is calculation of brush-size derivatives like
      *         hotspots and spacing.
      */
-     QSizeF characteristicSize(double scaleX, double scaleY, double rotation) const;
+     virtual QSizeF characteristicSize(KisDabShape const&) const;
 
     /**
      * @return the angle of the mask adding the given angle
@@ -209,7 +212,7 @@ public:
      */
     virtual enumBrushType brushType() const;
 
-    QPointF hotSpot(double scaleX, double scaleY, double rotation, const KisPaintInformation& info) const;
+    QPointF hotSpot(KisDabShape const&, const KisPaintInformation& info) const;
 
     /**
      * Returns true if this brush can return something useful for the info. This is used
@@ -231,33 +234,50 @@ public:
      * Having got this notification the brush can update the counters
      * of dabs, generate some new random values if needed.
      *
+     * * NOTE: one should use **either** notifyCachedDabPainted() or prepareForSeqNo()
+     *
      * Currently, this is used by pipe'd brushes to implement
      * incremental and random parasites
      */
     virtual void notifyCachedDabPainted(const KisPaintInformation& info);
 
+    /**
+     * Is called by the multithreaded queue to prepare a specific brush
+     * tip for the particular seqNo.
+     *
+     * NOTE: one should use **either** notifyCachedDabPainted() or prepareForSeqNo()
+     *
+     * Currently, this is used by pipe'd brushes to implement
+     * incremental and random parasites
+     */
+    virtual void prepareForSeqNo(const KisPaintInformation& info, int seqNo);
+
+    /**
+     * Notify the brush if it can use QtConcurrent's threading capabilities in its
+     * internal routines. By default it is allowed, but some paintops (who do their
+     * own multithreading) may ask the brush to avoid internal threading.
+     */
+    void setThreadingAllowed(bool value);
+
+    /**
+     * \see setThreadingAllowed() for details
+     */
+    bool threadingAllowed() const;
+
     /**
      * Return a fixed paint device that contains a correctly scaled image dab.
      */
     virtual KisFixedPaintDeviceSP paintDevice(const KoColorSpace * colorSpace,
-            double scale, double angle,
+            KisDabShape const&,
             const KisPaintInformation& info,
             double subPixelX = 0, double subPixelY = 0) const;
 
-    /**
-     * Apply the brush mask to the pixels in dst. Dst should be big enough!
-     */
-    void mask(KisFixedPaintDeviceSP dst,
-              double scaleX, double scaleY, double angle,
-              const KisPaintInformation& info,
-              double subPixelX = 0, double subPixelY = 0, qreal softnessFactor = DEFAULT_SOFTNESS_FACTOR) const;
-
     /**
      * clear dst fill it with a mask colored with KoColor
      */
     void mask(KisFixedPaintDeviceSP dst,
               const KoColor& color,
-              double scaleX, double scaleY, double angle,
+              KisDabShape const& shape,
               const KisPaintInformation& info,
               double subPixelX = 0, double subPixelY = 0, qreal softnessFactor = DEFAULT_SOFTNESS_FACTOR) const;
 
@@ -266,7 +286,7 @@ public:
      */
     void mask(KisFixedPaintDeviceSP dst,
               const KisPaintDeviceSP src,
-              double scaleX, double scaleY, double angle,
+              KisDabShape const& shape,
               const KisPaintInformation& info,
               double subPixelX = 0, double subPixelY = 0, qreal softnessFactor = DEFAULT_SOFTNESS_FACTOR) const;
 
@@ -295,7 +315,7 @@ public:
      */
     virtual void generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
             ColoringInformation* coloringInfo,
-            double scaleX, double scaleY, double angle,
+            KisDabShape const&,
             const KisPaintInformation& info,
             double subPixelX = 0, double subPixelY = 0, qreal softnessFactor = DEFAULT_SOFTNESS_FACTOR) const;
 
@@ -315,14 +335,13 @@ public:
     virtual void setAngle(qreal _angle);
     qreal angle() const;
 
-    void prepareBrushPyramid() const;
     void clearBrushPyramid();
 
     virtual void lodLimitations(KisPaintopLodLimitations *l) const;
 
     virtual KisBrush* clone() const = 0;
 
-//protected:
+protected:
 
     KisBrush(const KisBrush& rhs);
 
@@ -332,21 +351,21 @@ public:
 
     void setHotSpot(QPointF);
 
-    /**
-     * The image is used to represent the brush in the gui, and may also, depending on the brush type
-     * be used to define the actual brush instance.
-     */
-    virtual void setBrushTipImage(const QImage& image);
-
     /**
      * XXX
      */
     virtual void setBrushType(enumBrushType type);
 
-    friend class KisBrushTest;
-
     virtual void setHasColor(bool hasColor);
 
+public:
+
+    /**
+     * The image is used to represent the brush in the gui, and may also, depending on the brush type
+     * be used to define the actual brush instance.
+     */
+    virtual void setBrushTipImage(const QImage& image);
+
     /**
      * Returns true if the brush has a bunch of pixels almost
      * fully transparent in the very center. If the brush is pierced,
@@ -364,7 +383,6 @@ protected:
     void predefinedBrushToXML(const QString &type, QDomElement& e) const;
 
 private:
-    friend class KisImagePipeBrushTest;
 
     // Initialize our boundary
     void generateBoundary() const;
