@@ -19,17 +19,16 @@
 #include <brushengine/kis_paint_information.h>
 
 #include <QDomElement>
-#include <QScopedPointer>
+#include <boost/optional.hpp>
 
 #include "kis_paintop.h"
 #include "kis_algebra_2d.h"
 #include "kis_lod_transform.h"
+#include "kis_spacing_information.h"
 
 #include <kis_dom_utils.h>
 
 struct KisPaintInformation::Private {
-    EIGEN_MAKE_ALIGNED_OPERATOR_NEW
-
     Private(const QPointF & pos_,
             qreal pressure_,
             qreal xTilt_, qreal yTilt_,
@@ -51,7 +50,7 @@ struct KisPaintInformation::Private {
         speed(speed_),
         isHoveringMode(isHoveringMode_),
         randomSource(0),
-        currentDistanceInfo(0),
+        perStrokeRandomSource(0),
         levelOfDetail(0)
     {
     }
@@ -59,7 +58,7 @@ struct KisPaintInformation::Private {
 
 
     ~Private() {
-        KIS_ASSERT_RECOVER_NOOP(!currentDistanceInfo);
+        KIS_ASSERT_RECOVER_NOOP(!sanityIsRegistered);
     }
     Private(const Private &rhs) {
         copy(rhs);
@@ -81,11 +80,13 @@ struct KisPaintInformation::Private {
         speed = rhs.speed;
         isHoveringMode = rhs.isHoveringMode;
         randomSource = rhs.randomSource;
-        currentDistanceInfo = rhs.currentDistanceInfo;
+        perStrokeRandomSource = rhs.perStrokeRandomSource;
+        sanityIsRegistered = false; // HINT: we do not copy registration mark!
+        directionHistoryInfo = rhs.directionHistoryInfo;
         canvasRotation = rhs.canvasRotation;
         canvasMirroredH = rhs.canvasMirroredH;
         if (rhs.drawingAngleOverride) {
-            drawingAngleOverride.reset(new qreal(*rhs.drawingAngleOverride));
+            drawingAngleOverride = *rhs.drawingAngleOverride;
         }
 
         levelOfDetail = rhs.levelOfDetail;
@@ -103,20 +104,52 @@ struct KisPaintInformation::Private {
     qreal speed;
     bool isHoveringMode;
     KisRandomSourceSP randomSource;
-    int canvasRotation;
-    bool canvasMirroredH;
+    KisPerStrokeRandomSourceSP perStrokeRandomSource;
+    int canvasRotation {0};
+    bool canvasMirroredH {false};
 
-    QScopedPointer<qreal> drawingAngleOverride;
-    KisDistanceInformation *currentDistanceInfo;
+    boost::optional<qreal> drawingAngleOverride;
+    bool sanityIsRegistered = false;
+
+    struct DirectionHistoryInfo {
+        DirectionHistoryInfo() {}
+        DirectionHistoryInfo(qreal _totalDistance,
+                             int _currentDabSeqNo,
+                             qreal _lastAngle,
+                             QPointF _lastPosition,
+                             boost::optional<qreal> _lockedDrawingAngle)
+            : totalStrokeLength(_totalDistance),
+              currentDabSeqNo(_currentDabSeqNo),
+              lastAngle(_lastAngle),
+              lastPosition(_lastPosition),
+              lockedDrawingAngle(_lockedDrawingAngle)
+        {
+        }
+
+        qreal totalStrokeLength = 0.0;
+        int currentDabSeqNo = 0;
+        qreal lastAngle = 0.0;
+        QPointF lastPosition;
+        boost::optional<qreal> lockedDrawingAngle;
+    };
+    boost::optional<DirectionHistoryInfo> directionHistoryInfo;
 
     int levelOfDetail;
 
     void registerDistanceInfo(KisDistanceInformation *di) {
-        currentDistanceInfo = di;
+        directionHistoryInfo = DirectionHistoryInfo(di->scalarDistanceApprox(),
+                                                    di->currentDabSeqNo(),
+                                                    di->lastDrawingAngle(),
+                                                    di->lastPosition(),
+                                                    di->lockedDrawingAngleOptional());
+
+
+        KIS_SAFE_ASSERT_RECOVER_NOOP(!sanityIsRegistered);
+        sanityIsRegistered = true;
     }
 
     void unregisterDistanceInfo() {
-        currentDistanceInfo = 0;
+        sanityIsRegistered = false;
     }
 };
 
@@ -127,10 +160,18 @@ DistanceInformationRegistrar(KisPaintInformation *_p, KisDistanceInformation *di
     p->d->registerDistanceInfo(distanceInfo);
 }
 
+KisPaintInformation::DistanceInformationRegistrar::DistanceInformationRegistrar(KisPaintInformation::DistanceInformationRegistrar &&rhs)
+    : p(0)
+{
+    std::swap(p, rhs.p);
+}
+
 KisPaintInformation::DistanceInformationRegistrar::
 ~DistanceInformationRegistrar()
 {
-    p->d->unregisterDistanceInfo();
+    if (p) {
+        p->d->unregisterDistanceInfo();
+    }
 }
 
 KisPaintInformation::KisPaintInformation(const QPointF & pos,
@@ -269,8 +310,8 @@ void KisPaintInformation::toXML(QDomDocument&, QDomElement& e) const
     e.setAttribute("rotation", QString::number(rotation(), 'g', 15));
     e.setAttribute("tangentialPressure", QString::number(tangentialPressure(), 'g', 15));
     e.setAttribute("perspective", QString::number(perspective(), 'g', 15));
-    e.setAttribute("time", d->time);
-    e.setAttribute("speed", d->speed);
+    e.setAttribute("time", QString::number(d->time, 'g', 15));
+    e.setAttribute("speed", QString::number(d->speed, 'g', 15));
 }
 
 KisPaintInformation KisPaintInformation::fromXML(const QDomElement& e)
@@ -322,15 +363,19 @@ qreal KisPaintInformation::yTilt() const
 
 void KisPaintInformation::overrideDrawingAngle(qreal angle)
 {
-    d->drawingAngleOverride.reset(new qreal(angle));
+    d->drawingAngleOverride = angle;
 }
 
 qreal KisPaintInformation::drawingAngleSafe(const KisDistanceInformation &distance) const
 {
-    if (d->drawingAngleOverride) return *d->drawingAngleOverride;
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!d->directionHistoryInfo, 0.0);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(distance.hasLastDabInformation(), 0.0);
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!d->drawingAngleOverride, 0.0);
+
+    return KisAlgebra2D::directionBetweenPoints(distance.lastPosition(),
+                                                pos(),
+                                                distance.lastDrawingAngle());
 
-    QVector2D diff(pos() - distance.lastPosition());
-    return atan2(diff.y(), diff.x());
 }
 
 KisPaintInformation::DistanceInformationRegistrar
@@ -339,78 +384,42 @@ KisPaintInformation::registerDistanceInformation(KisDistanceInformation *distanc
     return DistanceInformationRegistrar(this, distance);
 }
 
-qreal KisPaintInformation::drawingAngle() const
+qreal KisPaintInformation::drawingAngle(bool considerLockedAngle) const
 {
     if (d->drawingAngleOverride) return *d->drawingAngleOverride;
 
-    if (!d->currentDistanceInfo || !d->currentDistanceInfo->hasLastDabInformation()) {
-        warnKrita << "KisPaintInformation::drawingAngle()" << "Cannot access Distance Info last dab data";
+    if (!d->directionHistoryInfo) {
+        warnKrita << "KisPaintInformation::drawingAngleSafe()" << "DirectionHistoryInfo object is not available";
         return 0.0;
     }
 
-    if (d->currentDistanceInfo->hasLockedDrawingAngle()) {
-        return d->currentDistanceInfo->lockedDrawingAngle();
+    if (considerLockedAngle &&
+        d->directionHistoryInfo->lockedDrawingAngle) {
+
+        return *d->directionHistoryInfo->lockedDrawingAngle;
     }
 
-    QVector2D diff(pos() - d->currentDistanceInfo->lastPosition());
-    return atan2(diff.y(), diff.x());
-}
-
-void KisPaintInformation::lockCurrentDrawingAngle(qreal alpha_unused) const
-{
-    Q_UNUSED(alpha_unused);
-
-    if (!d->currentDistanceInfo) {
-        warnKrita << "KisPaintInformation::lockCurrentDrawingAngle()" << "Cannot access Distance Info last dab data";
-        return;
-    }
-
-    const QVector2D diff(pos() - d->currentDistanceInfo->lastPosition());
-    const qreal angle = atan2(diff.y(), diff.x());
-
-    qreal newAngle = angle;
-
-    if (d->currentDistanceInfo->hasLockedDrawingAngle()) {
-        const qreal stabilizingCoeff = 20.0;
-        const qreal dist = stabilizingCoeff * d->currentDistanceInfo->currentSpacing().scalarApprox();
-        const qreal alpha = qMax(0.0, dist - d->currentDistanceInfo->scalarDistanceApprox()) / dist;
-
-        const qreal oldAngle = d->currentDistanceInfo->lockedDrawingAngle();
-
-        if (shortestAngularDistance(oldAngle, newAngle) < M_PI / 6) {
-            newAngle = (1.0 - alpha) * oldAngle + alpha * newAngle;
-        } else {
-            newAngle = oldAngle;
-        }
-    }
-
-    d->currentDistanceInfo->setLockedDrawingAngle(newAngle);
+    // If the start and end positions are the same, we can't compute an angle. In that case, use the
+    // provided default.
+    return KisAlgebra2D::directionBetweenPoints(d->directionHistoryInfo->lastPosition,
+                                                pos(),
+                                                d->directionHistoryInfo->lastAngle);
 }
 
 QPointF KisPaintInformation::drawingDirectionVector() const
 {
-    if (d->drawingAngleOverride) {
-        qreal angle = *d->drawingAngleOverride;
-        return QPointF(cos(angle), sin(angle));
-    }
-
-    if (!d->currentDistanceInfo || !d->currentDistanceInfo->hasLastDabInformation()) {
-        warnKrita << "KisPaintInformation::drawingDirectionVector()" << "Cannot access Distance Info last dab data";
-        return QPointF(1.0, 0.0);
-    }
-
-    QPointF diff(pos() - d->currentDistanceInfo->lastPosition());
-    return KisAlgebra2D::normalize(diff);
+    const qreal angle = drawingAngle(false);
+    return QPointF(cos(angle), sin(angle));
 }
 
 qreal KisPaintInformation::drawingDistance() const
 {
-    if (!d->currentDistanceInfo || !d->currentDistanceInfo->hasLastDabInformation()) {
-        warnKrita << "KisPaintInformation::drawingDistance()" << "Cannot access Distance Info last dab data";
+    if (!d->directionHistoryInfo) {
+        warnKrita << "KisPaintInformation::drawingDistance()" << "DirectionHistoryInfo object is not available";
         return 1.0;
     }
 
-    QVector2D diff(pos() - d->currentDistanceInfo->lastPosition());
+    QVector2D diff(pos() - d->directionHistoryInfo->lastPosition);
     qreal length = diff.length();
 
     if (d->levelOfDetail) {
@@ -445,9 +454,30 @@ qreal KisPaintInformation::currentTime() const
     return d->time;
 }
 
+int KisPaintInformation::currentDabSeqNo() const
+{
+    if (!d->directionHistoryInfo) {
+        warnKrita << "KisPaintInformation::currentDabSeqNo()" << "DirectionHistoryInfo object is not available";
+        return 0;
+    }
+
+    return d->directionHistoryInfo->currentDabSeqNo;
+}
+
+qreal KisPaintInformation::totalStrokeLength() const
+{
+    if (!d->directionHistoryInfo) {
+        warnKrita << "KisPaintInformation::totalStrokeLength()" << "DirectionHistoryInfo object is not available";
+        return 0;
+    }
+
+    return d->directionHistoryInfo->totalStrokeLength;
+}
+
 KisRandomSourceSP KisPaintInformation::randomSource() const
 {
     if (!d->randomSource) {
+        qWarning() << "Accessing uninitialized random source!";
         d->randomSource = new KisRandomSource();
     }
 
@@ -459,7 +489,22 @@ void KisPaintInformation::setRandomSource(KisRandomSourceSP value)
     d->randomSource = value;
 }
 
-void KisPaintInformation::setLevelOfDetail(int levelOfDetail) const
+KisPerStrokeRandomSourceSP KisPaintInformation::perStrokeRandomSource() const
+{
+    if (!d->perStrokeRandomSource) {
+        qWarning() << "Accessing uninitialized per stroke random source!";
+        d->perStrokeRandomSource = new KisPerStrokeRandomSource();
+    }
+
+    return d->perStrokeRandomSource;
+}
+
+void KisPaintInformation::setPerStrokeRandomSource(KisPerStrokeRandomSourceSP value)
+{
+    d->perStrokeRandomSource = value;
+}
+
+void KisPaintInformation::setLevelOfDetail(int levelOfDetail)
 {
     d->levelOfDetail = levelOfDetail;
 }
@@ -487,18 +532,7 @@ QDebug operator<<(QDebug dbg, const KisPaintInformation &info)
 KisPaintInformation KisPaintInformation::mixOnlyPosition(qreal t, const KisPaintInformation& mixedPi, const KisPaintInformation& basePi)
 {
     QPointF pt = (1 - t) * mixedPi.pos() + t * basePi.pos();
-    KisPaintInformation result(pt,
-                               basePi.pressure(),
-                               basePi.xTilt(),
-                               basePi.yTilt(),
-                               basePi.rotation(),
-                               basePi.tangentialPressure(),
-                               basePi.perspective(),
-                               basePi.currentTime(),
-                               basePi.drawingSpeed());
-
-    result.setRandomSource(basePi.randomSource());
-    return result;
+    return mixImpl(pt, t, mixedPi, basePi, true, false);
 }
 
 KisPaintInformation KisPaintInformation::mix(qreal t, const KisPaintInformation& pi1, const KisPaintInformation& pi2)
@@ -509,36 +543,76 @@ KisPaintInformation KisPaintInformation::mix(qreal t, const KisPaintInformation&
 
 KisPaintInformation KisPaintInformation::mix(const QPointF& p, qreal t, const KisPaintInformation& pi1, const KisPaintInformation& pi2)
 {
-    qreal pressure = (1 - t) * pi1.pressure() + t * pi2.pressure();
-    qreal xTilt = (1 - t) * pi1.xTilt() + t * pi2.xTilt();
-    qreal yTilt = (1 - t) * pi1.yTilt() + t * pi2.yTilt();
+    return mixImpl(p, t, pi1, pi2, false, true);
+}
 
-    qreal rotation = pi1.rotation();
+KisPaintInformation KisPaintInformation::mixWithoutTime(qreal t, const KisPaintInformation& pi1, const KisPaintInformation& pi2)
+{
+    QPointF pt = (1 - t) * pi1.pos() + t * pi2.pos();
+    return mixWithoutTime(pt, t, pi1, pi2);
+}
 
-    if (pi1.rotation() != pi2.rotation()) {
-        qreal a1 = kisDegreesToRadians(pi1.rotation());
-        qreal a2 = kisDegreesToRadians(pi2.rotation());
-        qreal distance = shortestAngularDistance(a2, a1);
+KisPaintInformation KisPaintInformation::mixWithoutTime(const QPointF& p, qreal t, const KisPaintInformation& pi1, const KisPaintInformation& pi2)
+{
+    return mixImpl(p, t, pi1, pi2, false, false);
+}
 
-        rotation = kisRadiansToDegrees(incrementInDirection(a1, t * distance, a2));
-    }
+void KisPaintInformation::mixOtherOnlyPosition(qreal t, const KisPaintInformation& other)
+{
+    QPointF pt = (1 - t) * other.pos() + t * this->pos();
+    this->mixOtherImpl(pt, t, other, true, false);
+}
 
-    qreal tangentialPressure = (1 - t) * pi1.tangentialPressure() + t * pi2.tangentialPressure();
-    qreal perspective = (1 - t) * pi1.perspective() + t * pi2.perspective();
-    qreal time = (1 - t) * pi1.currentTime() + t * pi2.currentTime();
-    qreal speed = (1 - t) * pi1.drawingSpeed() + t * pi2.drawingSpeed();
-
-    KisPaintInformation result(p, pressure, xTilt, yTilt, rotation, tangentialPressure, perspective, time, speed);
-    KIS_ASSERT_RECOVER_NOOP(pi1.isHoveringMode() == pi2.isHoveringMode());
-    result.d->isHoveringMode = pi1.isHoveringMode();
-    result.d->levelOfDetail = pi1.d->levelOfDetail;
-    result.d->randomSource = pi1.d->randomSource;
-    result.d->canvasRotation = pi2.canvasRotation();
-    result.d->canvasMirroredH = pi2.canvasMirroredH();
+void KisPaintInformation::mixOtherWithoutTime(qreal t, const KisPaintInformation& other)
+{
+    QPointF pt = (1 - t) * other.pos() + t * this->pos();
+    this->mixOtherImpl(pt, t, other, false, false);
+}
 
+KisPaintInformation KisPaintInformation::mixImpl(const QPointF &p, qreal t, const KisPaintInformation &pi1, const KisPaintInformation &pi2, bool posOnly, bool mixTime)
+{
+    KisPaintInformation result(pi2);
+    result.mixOtherImpl(p, t, pi1, posOnly, mixTime);
     return result;
 }
 
+void KisPaintInformation::mixOtherImpl(const QPointF &p, qreal t, const KisPaintInformation &other, bool posOnly, bool mixTime)
+{
+    if (posOnly) {
+        this->d->pos = p;
+        this->d->isHoveringMode = false;
+        this->d->levelOfDetail = 0;
+        return;
+    }
+    else {
+        qreal pressure = (1 - t) * other.pressure() + t * this->pressure();
+        qreal xTilt = (1 - t) * other.xTilt() + t * this->xTilt();
+        qreal yTilt = (1 - t) * other.yTilt() + t * this->yTilt();
+
+        qreal rotation = other.rotation();
+
+        if (other.rotation() != this->rotation()) {
+            qreal a1 = kisDegreesToRadians(other.rotation());
+            qreal a2 = kisDegreesToRadians(this->rotation());
+            qreal distance = shortestAngularDistance(a2, a1);
+
+            rotation = kisRadiansToDegrees(incrementInDirection(a1, t * distance, a2));
+        }
+
+        qreal tangentialPressure = (1 - t) * other.tangentialPressure() + t * this->tangentialPressure();
+        qreal perspective = (1 - t) * other.perspective() + t * this->perspective();
+        qreal time = mixTime ? ((1 - t) * other.currentTime() + t * this->currentTime()) : this->currentTime();
+        qreal speed = (1 - t) * other.drawingSpeed() + t * this->drawingSpeed();
+
+        KIS_ASSERT_RECOVER_NOOP(other.isHoveringMode() == this->isHoveringMode());
+        *(this->d) = Private(p, pressure, xTilt, yTilt, rotation, tangentialPressure, perspective, time, speed, other.isHoveringMode());
+        this->d->randomSource = other.d->randomSource;
+        this->d->perStrokeRandomSource = other.d->perStrokeRandomSource;
+        // this->d->isHoveringMode = other.isHoveringMode();
+        this->d->levelOfDetail = other.d->levelOfDetail;
+    }
+}
+
 qreal KisPaintInformation::tiltDirection(const KisPaintInformation& info, bool normalize)
 {
     qreal xTilt = info.xTilt();
@@ -567,4 +641,3 @@ qreal KisPaintInformation::tiltElevation(const KisPaintInformation& info, qreal
     // mapping to 0.0..1.0 if normalize is true
     return normalize ? (tiltElevation / (M_PI * qreal(0.5))) : tiltElevation;
 }
-
