@@ -53,6 +53,7 @@ KisFilterRegistry* KisFilterRegistry::instance()
 {
     KisFilterRegistry *reg = qApp->findChild<KisFilterRegistry *>(QString());
     if (!reg) {
+        dbgRegistry << "initializing KisFilterRegistry";
         reg = new KisFilterRegistry(qApp);
         KoPluginLoader::instance()->load("Krita/Filter", "Type == 'Service' and ([X-Krita-Version] == 28)");
     }
@@ -74,7 +75,7 @@ KisFilterConfigurationSP KisFilterRegistry::cloneConfiguration(const KisFilterCo
 {
     Q_ASSERT(kfc);
     KisFilterSP filter = value(kfc->name());
-    KisFilterConfigurationSP newkfc = filter->defaultConfiguration();
+    KisFilterConfigurationSP newkfc(filter->defaultConfiguration());
     newkfc->fromXML(kfc->toXML());
     return newkfc;
 }
