@@ -38,8 +38,8 @@
 #include "KoColor.h"
 #include "KoColorProfile.h"
 #include <KoCompositeOpRegistry.h>
+#include "KisProofingConfiguration.h"
 
-#include "recorder/kis_action_recorder.h"
 #include "kis_adjustment_layer.h"
 #include "kis_annotation.h"
 #include "kis_change_profile_visitor.h"
@@ -52,6 +52,7 @@
 #include "kis_meta_data_merge_strategy_registry.h"
 #include "kis_name_server.h"
 #include "kis_paint_layer.h"
+#include "kis_projection_leaf.h"
 #include "kis_painter.h"
 #include "kis_selection.h"
 #include "kis_transaction.h"
@@ -63,6 +64,7 @@
 #include "kis_image_signal_router.h"
 #include "kis_image_animation_interface.h"
 #include "kis_stroke_strategy.h"
+#include "kis_simple_stroke_strategy.h"
 #include "kis_image_barrier_locker.h"
 
 
@@ -94,7 +96,7 @@
 #include "kis_layer_projection_plane.h"
 
 #include "kis_update_time_monitor.h"
-#include "kis_image_barrier_locker.h"
+#include "tiles3/kis_lockless_stack.h"
 
 #include <QtCore>
 
@@ -114,24 +116,78 @@
 #endif
 
 
+struct KisImageSPStaticRegistrar {
+    KisImageSPStaticRegistrar() {
+        qRegisterMetaType<KisImageSP>("KisImageSP");
+    }
+};
+static KisImageSPStaticRegistrar __registrar;
+
 class KisImage::KisImagePrivate
 {
 public:
-    KisImagePrivate(KisImage *_q, qint32 w, qint32 h, const KoColorSpace *c, KisUndoStore *u)
+    KisImagePrivate(KisImage *_q, qint32 w, qint32 h,
+                    const KoColorSpace *c,
+                    KisUndoStore *undo,
+                    KisImageAnimationInterface *_animationInterface)
         : q(_q)
         , lockedForReadOnly(false)
         , width(w)
         , height(h)
-        , colorSpace(c)
+        , colorSpace(c ? c : KoColorSpaceRegistry::instance()->rgb8())
         , nserver(1)
-        , undoStore(u)
-        , legacyUndoAdapter(u, _q)
-        , postExecutionUndoAdapter(u, _q)
-        , recorder(_q)
+        , undoStore(undo ? undo : new KisDumbUndoStore())
+        , legacyUndoAdapter(undoStore.data(), _q)
+        , postExecutionUndoAdapter(undoStore.data(), _q)
         , signalRouter(_q)
-        , animationInterface(0)
-        , scheduler(_q)
-    {}
+        , animationInterface(_animationInterface)
+        , scheduler(_q, _q)
+        , axesCenter(QPointF(0.5, 0.5))
+    {
+        {
+            KisImageConfig cfg;
+            if (cfg.enableProgressReporting()) {
+                scheduler.setProgressProxy(&compositeProgressProxy);
+            }
+
+            // Each of these lambdas defines a new factory function.
+            scheduler.setLod0ToNStrokeStrategyFactory(
+                [=](bool forgettable) {
+                    return KisLodSyncPair(
+                        new KisSyncLodCacheStrokeStrategy(KisImageWSP(q), forgettable),
+                        KisSyncLodCacheStrokeStrategy::createJobsData(KisImageWSP(q)));
+                });
+
+            scheduler.setSuspendUpdatesStrokeStrategyFactory(
+                [=]() {
+                    return KisSuspendResumePair(
+                        new KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP(q), true),
+                        KisSuspendProjectionUpdatesStrokeStrategy::createSuspendJobsData(KisImageWSP(q)));
+                });
+
+            scheduler.setResumeUpdatesStrokeStrategyFactory(
+                [=]() {
+                    return KisSuspendResumePair(
+                        new KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP(q), false),
+                        KisSuspendProjectionUpdatesStrokeStrategy::createResumeJobsData(KisImageWSP(q)));
+                });
+        }
+
+        connect(q, SIGNAL(sigImageModified()), KisMemoryStatisticsServer::instance(), SLOT(notifyImageChanged()));
+    }
+
+    ~KisImagePrivate() {
+        /**
+         * Stop animation interface. It may use the rootLayer.
+         */
+        delete animationInterface;
+
+        /**
+         * First delete the nodes, while strokes
+         * and undo are still alive
+         */
+        rootLayer.clear();
+    }
 
     KisImage *q;
 
@@ -145,25 +201,25 @@ public:
     double yres = 1.0;
 
     const KoColorSpace * colorSpace;
+    KisProofingConfigurationSP proofingConfig;
 
     KisSelectionSP deselectedGlobalSelection;
     KisGroupLayerSP rootLayer; // The layers are contained in here
-    QList<KisLayer*> dirtyLayers; // for thumbnails
-    QList<KisLayerComposition*> compositions;
+    QList<KisLayerCompositionSP> compositions;
     KisNodeSP isolatedRootNode;
     bool wrapAroundModePermitted = false;
 
     KisNameServer nserver;
 
-    KisUndoStore *undoStore;
+    QScopedPointer<KisUndoStore> undoStore;
     KisLegacyUndoAdapter legacyUndoAdapter;
     KisPostExecutionUndoAdapter postExecutionUndoAdapter;
 
-    KisActionRecorder recorder;
-
     vKisAnnotationSP annotations;
 
     QAtomicInt disableUIUpdateSignals;
+    KisLocklessStack<QRect> savedDisabledUIUpdates;
+
     KisProjectionUpdatesFilterSP projectionUpdatesFilter;
     KisImageSignalRouter signalRouter;
     KisImageAnimationInterface *animationInterface;
@@ -175,6 +231,8 @@ public:
 
     bool blockLevelOfDetail = false;
 
+    QPointF axesCenter;
+
     bool tryCancelCurrentStrokeAsync();
 
     void notifyProjectionUpdatedInPatches(const QRect &rc);
@@ -183,56 +241,16 @@ public:
 KisImage::KisImage(KisUndoStore *undoStore, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name)
         : QObject(0)
         , KisShared()
+        , m_d(new KisImagePrivate(this, width, height,
+                                  colorSpace, undoStore,
+                                  new KisImageAnimationInterface(this)))
 {
+    // make sure KisImage belongs to the GUI thread
+    moveToThread(qApp->thread());
+    connect(this, SIGNAL(sigInternalStopIsolatedModeRequested()), SLOT(stopIsolatedMode()));
+
     setObjectName(name);
-    // Handle undoStore == 0 and colorSpace == 0 cases
-    if (!undoStore) {
-        undoStore = new KisDumbUndoStore();
-    }
-
-    const KoColorSpace *c;
-    if (colorSpace != 0) {
-        c = colorSpace;
-    } else {
-        c = KoColorSpaceRegistry::instance()->rgb8();
-    }
-    m_d = new KisImagePrivate(this, width, height, c, undoStore);
-
-
-    {
-        KisImageConfig cfg;
-        if (cfg.enableProgressReporting()) {
-            m_d->scheduler.setProgressProxy(&m_d->compositeProgressProxy);
-        }
-
-        // Each of these lambdas defines a new factory function.
-        m_d->scheduler.setLod0ToNStrokeStrategyFactory(
-            [=](bool forgettable) {
-                return KisLodSyncPair(
-                    new KisSyncLodCacheStrokeStrategy(KisImageWSP(this), forgettable),
-                    KisSyncLodCacheStrokeStrategy::createJobsData(KisImageWSP(this)));
-            });
-
-        m_d->scheduler.setSuspendUpdatesStrokeStrategyFactory(
-            [=]() {
-                return KisSuspendResumePair(
-                    new KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP(this), true),
-                    KisSuspendProjectionUpdatesStrokeStrategy::createSuspendJobsData(KisImageWSP(this)));
-            });
-
-        m_d->scheduler.setResumeUpdatesStrokeStrategyFactory(
-            [=]() {
-                return KisSuspendResumePair(
-                    new KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP(this), false),
-                    KisSuspendProjectionUpdatesStrokeStrategy::createResumeJobsData(KisImageWSP(this)));
-            });
-    }
-
     setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE_U8));
-
-    m_d->animationInterface = new KisImageAnimationInterface(this);
-
-    connect(this, SIGNAL(sigImageModified()), KisMemoryStatisticsServer::instance(), SLOT(notifyImageChanged()));
 }
 
 KisImage::~KisImage()
@@ -244,22 +262,86 @@ KisImage::~KisImage()
      */
     waitForDone();
 
-    /**
-     * Stop animation interface. It may use the rootLayer.
-     */
-    delete m_d->animationInterface;
-
-    /**
-     * First delete the nodes, while strokes
-     * and undo are still alive
-     */
-    m_d->rootLayer = 0;
-
-    delete m_d->undoStore;
     delete m_d;
     disconnect(); // in case Qt gets confused
 }
 
+KisImage *KisImage::clone(bool exactCopy)
+{
+    return new KisImage(*this, 0, exactCopy);
+}
+
+KisImage::KisImage(const KisImage& rhs, KisUndoStore *undoStore, bool exactCopy)
+    : KisNodeFacade(),
+      KisNodeGraphListener(),
+      KisShared(),
+      m_d(new KisImagePrivate(this,
+                              rhs.width(), rhs.height(),
+                              rhs.colorSpace(),
+                              undoStore ? undoStore : new KisDumbUndoStore(),
+                              new KisImageAnimationInterface(*rhs.animationInterface(), this)))
+{
+    // make sure KisImage belongs to the GUI thread
+    moveToThread(qApp->thread());
+    connect(this, SIGNAL(sigInternalStopIsolatedModeRequested()), SLOT(stopIsolatedMode()));
+
+    setObjectName(rhs.objectName());
+
+    m_d->xres = rhs.m_d->xres;
+    m_d->yres = rhs.m_d->yres;
+
+
+    if (rhs.m_d->proofingConfig) {
+        m_d->proofingConfig = toQShared(new KisProofingConfiguration(*rhs.m_d->proofingConfig));
+    }
+
+    KisNodeSP newRoot = rhs.root()->clone();
+    newRoot->setGraphListener(this);
+    newRoot->setImage(this);
+    m_d->rootLayer = dynamic_cast<KisGroupLayer*>(newRoot.data());
+    setRoot(newRoot);
+
+    if (exactCopy || rhs.m_d->isolatedRootNode) {
+        QQueue<KisNodeSP> linearizedNodes;
+        KisLayerUtils::recursiveApplyNodes(rhs.root(),
+            [&linearizedNodes](KisNodeSP node) {
+                linearizedNodes.enqueue(node);
+            });
+        KisLayerUtils::recursiveApplyNodes(newRoot,
+            [&linearizedNodes, exactCopy, &rhs, this](KisNodeSP node) {
+                KisNodeSP refNode = linearizedNodes.dequeue();
+
+                if (exactCopy) {
+                    node->setUuid(refNode->uuid());
+                }
+
+                if (rhs.m_d->isolatedRootNode &&
+                    rhs.m_d->isolatedRootNode == refNode) {
+
+                    m_d->isolatedRootNode = node;
+                }
+            });
+    }
+
+    Q_FOREACH (KisLayerCompositionSP comp, rhs.m_d->compositions) {
+        m_d->compositions << toQShared(new KisLayerComposition(*comp, this));
+    }
+
+    rhs.m_d->nserver = KisNameServer(rhs.m_d->nserver);
+
+    vKisAnnotationSP newAnnotations;
+    Q_FOREACH (KisAnnotationSP annotation, rhs.m_d->annotations) {
+        newAnnotations << annotation->clone();
+    }
+    m_d->annotations = newAnnotations;
+
+    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->projectionUpdatesFilter);
+    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->disableUIUpdateSignals);
+    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->disableDirtyRequests);
+
+    m_d->blockLevelOfDetail = rhs.m_d->blockLevelOfDetail;
+}
+
 void KisImage::aboutToAddANode(KisNode *parent, int index)
 {
     KisNodeGraphListener::aboutToAddANode(parent, index);
@@ -275,7 +357,7 @@ void KisImage::nodeHasBeenAdded(KisNode *parent, int index)
 
     KisNodeSP newNode = parent->at(index);
     if (!dynamic_cast<KisSelectionMask*>(newNode.data())) {
-        stopIsolatedMode();
+        emit sigInternalStopIsolatedModeRequested();
     }
 }
 
@@ -283,7 +365,7 @@ void KisImage::aboutToRemoveANode(KisNode *parent, int index)
 {
     KisNodeSP deletedNode = parent->at(index);
     if (!dynamic_cast<KisSelectionMask*>(deletedNode.data())) {
-        stopIsolatedMode();
+        emit sigInternalStopIsolatedModeRequested();
     }
 
     KisNodeGraphListener::aboutToRemoveANode(parent, index);
@@ -425,9 +507,9 @@ bool KisImage::tryBarrierLock(bool readOnly)
     return result;
 }
 
-bool KisImage::isIdle()
+bool KisImage::isIdle(bool allowLocked)
 {
-    return !locked() && m_d->scheduler.isIdle();
+    return (allowLocked || !locked()) && m_d->scheduler.isIdle();
 }
 
 void KisImage::lock()
@@ -514,7 +596,7 @@ void KisImage::cropImage(const QRect& newRect)
 
 void KisImage::cropNode(KisNodeSP node, const QRect& newRect)
 {
-    bool isLayer = dynamic_cast<KisLayer*>(node.data());
+    bool isLayer = qobject_cast<KisLayer*>(node.data());
     KUndo2MagicString actionName = isLayer ?
         kundo2_i18n("Crop Layer") :
         kundo2_i18n("Crop Mask");
@@ -690,7 +772,11 @@ void KisImage::rotateImage(double radians)
 
 void KisImage::rotateNode(KisNodeSP node, double radians)
 {
-    rotateImpl(kundo2_i18n("Rotate Layer"), node, false, radians);
+    if (node->inherits("KisMask")) {
+        rotateImpl(kundo2_i18n("Rotate Mask"), node, false, radians);
+    } else {
+        rotateImpl(kundo2_i18n("Rotate Layer"), node, false, radians);
+    }
 }
 
 void KisImage::shearImpl(const KUndo2MagicString &actionName,
@@ -757,8 +843,13 @@ void KisImage::shearNode(KisNodeSP node, double angleX, double angleY)
 {
     QPointF shearOrigin = QRectF(bounds()).center();
 
-    shearImpl(kundo2_i18n("Shear layer"), node, false,
-              angleX, angleY, shearOrigin);
+    if (node->inherits("KisMask")) {
+        shearImpl(kundo2_i18n("Shear Mask"), node, false,
+                  angleX, angleY, shearOrigin);
+    } else {
+        shearImpl(kundo2_i18n("Shear Layer"), node, false,
+                  angleX, angleY, shearOrigin);
+    }
 }
 
 void KisImage::shear(double angleX, double angleY)
@@ -800,7 +891,9 @@ bool KisImage::assignImageProfile(const KoColorProfile *profile)
     m_d->colorSpace = dstCs;
 
     KisChangeProfileVisitor visitor(srcCs, dstCs);
-    return m_d->rootLayer->accept(visitor);
+    bool retval = m_d->rootLayer->accept(visitor);
+    m_d->signalRouter.emitNotification(ProfileChangedSignal);
+    return retval;
 
 }
 
@@ -856,10 +949,10 @@ QPointF KisImage::documentToPixel(const QPointF &documentCoord) const
     return QPointF(documentCoord.x() * xRes(), documentCoord.y() * yRes());
 }
 
-QPoint KisImage::documentToIntPixel(const QPointF &documentCoord) const
+QPoint KisImage::documentToImagePixelFloored(const QPointF &documentCoord) const
 {
     QPointF pixelCoord = documentToPixel(documentCoord);
-    return QPoint((int)pixelCoord.x(), (int)pixelCoord.y());
+    return QPoint(qFloor(pixelCoord.x()), qFloor(pixelCoord.y()));
 }
 
 QRectF KisImage::documentToPixel(const QRectF &documentRect) const
@@ -867,11 +960,6 @@ QRectF KisImage::documentToPixel(const QRectF &documentRect) const
     return QRectF(documentToPixel(documentRect.topLeft()), documentToPixel(documentRect.bottomRight()));
 }
 
-QRect KisImage::documentToIntPixel(const QRectF &documentRect) const
-{
-    return documentToPixel(documentRect).toAlignedRect();
-}
-
 QPointF KisImage::pixelToDocument(const QPointF &pixelCoord) const
 {
     return QPointF(pixelCoord.x() / xRes(), pixelCoord.y() / yRes());
@@ -938,9 +1026,9 @@ qint32 KisImage::nHiddenLayers() const
     return visitor.count();
 }
 
-void KisImage::flatten()
+void KisImage::flatten(KisNodeSP activeNode)
 {
-    KisLayerUtils::flattenImage(this);
+    KisLayerUtils::flattenImage(this, activeNode);
 }
 
 void KisImage::mergeMultipleLayers(QList<KisNodeSP> mergedNodes, KisNodeSP putAfter)
@@ -1041,7 +1129,10 @@ QRect KisImage::effectiveLodBounds() const
 
 KisPostExecutionUndoAdapter* KisImage::postExecutionUndoAdapter() const
 {
-    return &m_d->postExecutionUndoAdapter;
+    const int lod = currentLevelOfDetail();
+    return lod > 0 ?
+        m_d->scheduler.lodNPostExecutionUndoAdapter() :
+        &m_d->postExecutionUndoAdapter;
 }
 
 void KisImage::setUndoStore(KisUndoStore *undoStore)
@@ -1049,13 +1140,12 @@ void KisImage::setUndoStore(KisUndoStore *undoStore)
 
     m_d->legacyUndoAdapter.setUndoStore(undoStore);
     m_d->postExecutionUndoAdapter.setUndoStore(undoStore);
-    delete m_d->undoStore;
-    m_d->undoStore = undoStore;
+    m_d->undoStore.reset(undoStore);
 }
 
 KisUndoStore* KisImage::undoStore()
 {
-    return m_d->undoStore;
+    return m_d->undoStore.data();
 }
 
 KisUndoAdapter* KisImage::undoAdapter() const
@@ -1063,11 +1153,6 @@ KisUndoAdapter* KisImage::undoAdapter() const
     return &m_d->legacyUndoAdapter;
 }
 
-KisActionRecorder* KisImage::actionRecorder() const
-{
-    return &m_d->recorder;
-}
-
 void KisImage::setDefaultProjectionColor(const KoColor &color)
 {
     KIS_ASSERT_RECOVER_RETURN(m_d->rootLayer);
@@ -1085,7 +1170,7 @@ KoColor KisImage::defaultProjectionColor() const
 
 void KisImage::setRootLayer(KisGroupLayerSP rootLayer)
 {
-    stopIsolatedMode();
+    emit sigInternalStopIsolatedModeRequested();
 
     KoColor defaultProjectionColor(Qt::transparent, m_d->colorSpace);
 
@@ -1094,16 +1179,16 @@ void KisImage::setRootLayer(KisGroupLayerSP rootLayer)
         m_d->rootLayer->disconnect();
 
         KisPaintDeviceSP original = m_d->rootLayer->original();
-        defaultProjectionColor.setColor(original->defaultPixel(), original->colorSpace());
+        defaultProjectionColor = original->defaultPixel();
     }
 
     m_d->rootLayer = rootLayer;
     m_d->rootLayer->disconnect();
     m_d->rootLayer->setGraphListener(this);
+    m_d->rootLayer->setImage(this);
 
     KisPaintDeviceSP newOriginal = m_d->rootLayer->original();
-    defaultProjectionColor.convertTo(newOriginal->colorSpace());
-    newOriginal->setDefaultPixel(defaultProjectionColor.data());
+    newOriginal->setDefaultPixel(defaultProjectionColor);
 
     setRoot(m_d->rootLayer.data());
 }
@@ -1216,18 +1301,39 @@ void KisImage::KisImagePrivate::notifyProjectionUpdatedInPatches(const QRect &rc
 
 bool KisImage::startIsolatedMode(KisNodeSP node)
 {
-    if (!tryBarrierLock()) return false;
+    struct StartIsolatedModeStroke : public KisSimpleStrokeStrategy {
+        StartIsolatedModeStroke(KisNodeSP node, KisImageSP image)
+            : KisSimpleStrokeStrategy("start-isolated-mode", kundo2_noi18n("start-isolated-mode")),
+              m_node(node),
+              m_image(image)
+        {
+            this->enableJob(JOB_INIT, true, KisStrokeJobData::SEQUENTIAL, KisStrokeJobData::EXCLUSIVE);
+            setClearsRedoOnStart(false);
+        }
 
-    unlock();
+        void initStrokeCallback() {
+            // pass-though node don't have any projection prepared, so we should
+            // explicitly regenerate it before activating isolated mode.
+            m_node->projectionLeaf()->explicitlyRegeneratePassThroughProjection();
 
-    m_d->isolatedRootNode = node;
-    emit sigIsolatedModeChanged();
+            m_image->m_d->isolatedRootNode = m_node;
+            emit m_image->sigIsolatedModeChanged();
 
-    // the GUI uses our thread to do the color space conversion so we
-    // need to emit this signal in multiple threads
-    m_d->notifyProjectionUpdatedInPatches(bounds());
+            // the GUI uses our thread to do the color space conversion so we
+            // need to emit this signal in multiple threads
+            m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds());
+
+            m_image->invalidateAllFrames();
+        }
+
+    private:
+        KisNodeSP m_node;
+        KisImageSP m_image;
+    };
+
+    KisStrokeId id = startStroke(new StartIsolatedModeStroke(node, this));
+    endStroke(id);
 
-    invalidateAllFrames();
     return true;
 }
 
@@ -1235,21 +1341,41 @@ void KisImage::stopIsolatedMode()
 {
     if (!m_d->isolatedRootNode)  return;
 
-    KisNodeSP oldRootNode = m_d->isolatedRootNode;
-    m_d->isolatedRootNode = 0;
+    struct StopIsolatedModeStroke : public KisSimpleStrokeStrategy {
+        StopIsolatedModeStroke(KisImageSP image)
+            : KisSimpleStrokeStrategy("stop-isolated-mode", kundo2_noi18n("stop-isolated-mode")),
+              m_image(image)
+        {
+            this->enableJob(JOB_INIT);
+            setClearsRedoOnStart(false);
+        }
 
-    emit sigIsolatedModeChanged();
+        void initStrokeCallback() {
+            if (!m_image->m_d->isolatedRootNode)  return;
 
-    // the GUI uses our thread to do the color space conversion so we
-    // need to emit this signal in multiple threads
-    m_d->notifyProjectionUpdatedInPatches(bounds());
-    invalidateAllFrames();
+            //KisNodeSP oldRootNode = m_image->m_d->isolatedRootNode;
+            m_image->m_d->isolatedRootNode = 0;
 
-    // TODO: Substitute notifyProjectionUpdated() with this code
-    // when update optimization is implemented
-    //
-    // QRect updateRect = bounds() | oldRootNode->extent();
-    // oldRootNode->setDirty(updateRect);
+            emit m_image->sigIsolatedModeChanged();
+
+            // the GUI uses our thread to do the color space conversion so we
+            // need to emit this signal in multiple threads
+            m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds());
+            m_image->invalidateAllFrames();
+
+            // TODO: Substitute notifyProjectionUpdated() with this code
+            // when update optimization is implemented
+            //
+            // QRect updateRect = bounds() | oldRootNode->extent();
+            // oldRootNode->setDirty(updateRect);
+        }
+
+    private:
+        KisImageSP m_image;
+    };
+
+    KisStrokeId id = startStroke(new StopIsolatedModeStroke(this));
+    endStroke(id);
 }
 
 KisNodeSP KisImage::isolatedModeRoot() const
@@ -1290,6 +1416,11 @@ void KisImage::requestStrokeCancellation()
     }
 }
 
+UndoResult KisImage::tryUndoUnfinishedLod0Stroke()
+{
+    return m_d->scheduler.tryUndoLastStrokeAsync();
+}
+
 void KisImage::requestStrokeEnd()
 {
     emit sigStrokeEndRequested();
@@ -1358,7 +1489,7 @@ void KisImage::addSpontaneousJob(KisSpontaneousJob *spontaneousJob)
 
 void KisImage::setProjectionUpdatesFilter(KisProjectionUpdatesFilterSP filter)
 {
-    // udpate filters are *not* recursive!
+    // update filters are *not* recursive!
     KIS_ASSERT_RECOVER_NOOP(!filter || !m_d->projectionUpdatesFilter);
 
     m_d->projectionUpdatesFilter = filter;
@@ -1384,9 +1515,18 @@ void KisImage::disableUIUpdates()
     m_d->disableUIUpdateSignals.ref();
 }
 
-void KisImage::enableUIUpdates()
+QVector<QRect> KisImage::enableUIUpdates()
 {
     m_d->disableUIUpdateSignals.deref();
+
+    QRect rect;
+    QVector<QRect> postponedUpdates;
+
+    while (m_d->savedDisabledUIUpdates.pop(rect)) {
+        postponedUpdates.append(rect);
+    }
+
+    return postponedUpdates;
 }
 
 void KisImage::notifyProjectionUpdated(const QRect &rc)
@@ -1400,9 +1540,21 @@ void KisImage::notifyProjectionUpdated(const QRect &rc)
         if (dirtyRect.isEmpty()) return;
 
         emit sigImageUpdated(dirtyRect);
+    } else {
+        m_d->savedDisabledUIUpdates.push(rc);
     }
 }
 
+void KisImage::setWorkingThreadsLimit(int value)
+{
+    m_d->scheduler.setThreadsLimit(value);
+}
+
+int KisImage::workingThreadsLimit() const
+{
+    return m_d->scheduler.threadsLimit();
+}
+
 void KisImage::notifySelectionChanged()
 {
     /**
@@ -1425,24 +1577,25 @@ void KisImage::notifySelectionChanged()
 }
 
 void KisImage::requestProjectionUpdateImpl(KisNode *node,
-                                           const QRect &rect,
+                                           const QVector<QRect> &rects,
                                            const QRect &cropRect)
 {
-    if (rect.isEmpty()) return;
+    if (rects.isEmpty()) return;
 
-    KisNodeGraphListener::requestProjectionUpdate(node, rect);
-    m_d->scheduler.updateProjection(node, rect, cropRect);
+    m_d->scheduler.updateProjection(node, rects, cropRect);
 }
 
-void KisImage::requestProjectionUpdate(KisNode *node, const QRect& rect)
+void KisImage::requestProjectionUpdate(KisNode *node, const QVector<QRect> &rects, bool resetAnimationCache)
 {
     if (m_d->projectionUpdatesFilter
-        && m_d->projectionUpdatesFilter->filter(this, node, rect)) {
+        && m_d->projectionUpdatesFilter->filter(this, node, rects, resetAnimationCache)) {
 
         return;
     }
 
-    m_d->animationInterface->notifyNodeChanged(node, rect, false);
+    if (resetAnimationCache) {
+        m_d->animationInterface->notifyNodeChanged(node, rects, false);
+    }
 
     /**
      * Here we use 'permitted' instead of 'active' intentively,
@@ -1451,15 +1604,21 @@ void KisImage::requestProjectionUpdate(KisNode *node, const QRect& rect)
      * supporting the wrap-around mode will not make much harm.
      */
     if (m_d->wrapAroundModePermitted) {
-        const QRect boundRect = effectiveLodBounds();
-        KisWrappedRect splitRect(rect, boundRect);
+        QVector<QRect> allSplitRects;
 
-        Q_FOREACH (const QRect &rc, splitRect) {
-            requestProjectionUpdateImpl(node, rc, boundRect);
+        const QRect boundRect = effectiveLodBounds();
+        Q_FOREACH (const QRect &rc, rects) {
+            KisWrappedRect splitRect(rc, boundRect);
+            allSplitRects.append(splitRect);
         }
+
+        requestProjectionUpdateImpl(node, allSplitRects, boundRect);
+
     } else {
-        requestProjectionUpdateImpl(node, rect, bounds());
+        requestProjectionUpdateImpl(node, rects, bounds());
     }
+
+    KisNodeGraphListener::requestProjectionUpdate(node, rects, resetAnimationCache);
 }
 
 void KisImage::invalidateFrames(const KisTimeRange &range, const QRect &rect)
@@ -1472,20 +1631,19 @@ void KisImage::requestTimeSwitch(int time)
     m_d->animationInterface->requestTimeSwitchNonGUI(time);
 }
 
-QList<KisLayerComposition*> KisImage::compositions()
+QList<KisLayerCompositionSP> KisImage::compositions()
 {
     return m_d->compositions;
 }
 
-void KisImage::addComposition(KisLayerComposition* composition)
+void KisImage::addComposition(KisLayerCompositionSP composition)
 {
     m_d->compositions.append(composition);
 }
 
-void KisImage::removeComposition(KisLayerComposition* composition)
+void KisImage::removeComposition(KisLayerCompositionSP composition)
 {
     m_d->compositions.removeAll(composition);
-    delete composition;
 }
 
 bool checkMasksNeedConversion(KisNodeSP root, const QRect &bounds)
@@ -1513,6 +1671,10 @@ bool checkMasksNeedConversion(KisNodeSP root, const QRect &bounds)
 
 void KisImage::setWrapAroundModePermitted(bool value)
 {
+    if (m_d->wrapAroundModePermitted != value) {
+        requestStrokeEnd();
+    }
+
     m_d->wrapAroundModePermitted = value;
 
     if (m_d->wrapAroundModePermitted &&
@@ -1594,3 +1756,27 @@ KisImageAnimationInterface* KisImage::animationInterface() const
 {
     return m_d->animationInterface;
 }
+
+void KisImage::setProofingConfiguration(KisProofingConfigurationSP proofingConfig)
+{
+    m_d->proofingConfig = proofingConfig;
+    emit sigProofingConfigChanged();
+}
+
+KisProofingConfigurationSP KisImage::proofingConfiguration() const
+{
+    if (m_d->proofingConfig) {
+        return m_d->proofingConfig;
+    }
+    return KisProofingConfigurationSP();
+}
+
+QPointF KisImage::mirrorAxesCenter() const
+{
+    return m_d->axesCenter;
+}
+
+void KisImage::setMirrorAxesCenter(const QPointF &value) const
+{
+    m_d->axesCenter = value;
+}
