@@ -18,9 +18,12 @@
 
 #include "kis_image_animation_interface.h"
 
+#include <QFileInfo>
+
 #include "kis_global.h"
 #include "kis_image.h"
 #include "kis_regenerate_frame_stroke_strategy.h"
+#include "kis_switch_time_stroke_strategy.h"
 #include "kis_keyframe_channel.h"
 #include "kis_time_range.h"
 
@@ -33,17 +36,33 @@ struct KisImageAnimationInterface::Private
 {
     Private()
         : image(0),
-          currentTime(0),
-          currentUITime(0),
           externalFrameActive(false),
           frameInvalidationBlocked(false),
-          cachedLastFrameValue(-1)
+          cachedLastFrameValue(-1),
+          audioChannelMuted(false),
+          audioChannelVolume(0.5),
+          m_currentTime(0),
+          m_currentUITime(0)
+    {
+    }
+
+    Private(const Private &rhs, KisImage *newImage)
+        : image(newImage),
+          externalFrameActive(false),
+          frameInvalidationBlocked(false),
+          fullClipRange(rhs.fullClipRange),
+          playbackRange(rhs.playbackRange),
+          framerate(rhs.framerate),
+          cachedLastFrameValue(-1),
+          audioChannelFileName(rhs.audioChannelFileName),
+          audioChannelMuted(rhs.audioChannelMuted),
+          audioChannelVolume(rhs.audioChannelVolume),
+          m_currentTime(rhs.m_currentTime),
+          m_currentUITime(rhs.m_currentUITime)
     {
     }
 
     KisImage *image;
-    int currentTime;
-    int currentUITime;
     bool externalFrameActive;
     bool frameInvalidationBlocked;
 
@@ -51,18 +70,48 @@ struct KisImageAnimationInterface::Private
     KisTimeRange playbackRange;
     int framerate;
     int cachedLastFrameValue;
+    QString audioChannelFileName;
+    bool audioChannelMuted;
+    qreal audioChannelVolume;
+
+    KisSwitchTimeStrokeStrategy::SharedTokenWSP switchToken;
+
+    inline int currentTime() const {
+        return m_currentTime;
+    }
+
+    inline int currentUITime() const {
+        return m_currentUITime;
+    }
+    inline void setCurrentTime(int value) {
+        m_currentTime = value;
+    }
+
+    inline void setCurrentUITime(int value) {
+        m_currentUITime = value;
+    }
+private:
+    int m_currentTime;
+    int m_currentUITime;
 };
 
 
 KisImageAnimationInterface::KisImageAnimationInterface(KisImage *image)
-    : m_d(new Private)
+    : QObject(image),
+      m_d(new Private)
 {
     m_d->image = image;
 
     m_d->framerate = 24;
     m_d->fullClipRange = KisTimeRange::fromTime(0, 100);
 
-    connect(this, SIGNAL(sigInternalRequestTimeSwitch(int)), SLOT(switchCurrentTimeAsync(int)));
+    connect(this, SIGNAL(sigInternalRequestTimeSwitch(int, bool)), SLOT(switchCurrentTimeAsync(int, bool)));
+}
+
+KisImageAnimationInterface::KisImageAnimationInterface(const KisImageAnimationInterface &rhs, KisImage *newImage)
+    : m_d(new Private(*rhs.m_d, newImage))
+{
+    connect(this, SIGNAL(sigInternalRequestTimeSwitch(int, bool)), SLOT(switchCurrentTimeAsync(int, bool)));
 }
 
 KisImageAnimationInterface::~KisImageAnimationInterface()
@@ -84,12 +133,12 @@ bool KisImageAnimationInterface::hasAnimation() const
 
 int KisImageAnimationInterface::currentTime() const
 {
-    return m_d->currentTime;
+    return m_d->currentTime();
 }
 
 int KisImageAnimationInterface::currentUITime() const
 {
-    return m_d->currentUITime;
+    return m_d->currentUITime();
 }
 
 const KisTimeRange& KisImageAnimationInterface::fullClipRange() const
@@ -97,11 +146,24 @@ const KisTimeRange& KisImageAnimationInterface::fullClipRange() const
     return m_d->fullClipRange;
 }
 
-void KisImageAnimationInterface::setFullClipRange(const KisTimeRange range) {
+void KisImageAnimationInterface::setFullClipRange(const KisTimeRange range)
+{
     m_d->fullClipRange = range;
     emit sigFullClipRangeChanged();
 }
 
+void KisImageAnimationInterface::setFullClipRangeStartTime(int column)
+{
+    KisTimeRange newRange(column,  m_d->fullClipRange.end(), false);
+    setFullClipRange(newRange);
+}
+
+void KisImageAnimationInterface::setFullClipRangeEndTime(int column)
+{
+    KisTimeRange newRange(m_d->fullClipRange.start(), column, false);
+    setFullClipRange(newRange);
+}
+
 const KisTimeRange& KisImageAnimationInterface::playbackRange() const
 {
     return m_d->playbackRange.isValid() ? m_d->playbackRange : m_d->fullClipRange;
@@ -118,6 +180,43 @@ int KisImageAnimationInterface::framerate() const
     return m_d->framerate;
 }
 
+QString KisImageAnimationInterface::audioChannelFileName() const
+{
+    return m_d->audioChannelFileName;
+}
+
+void KisImageAnimationInterface::setAudioChannelFileName(const QString &fileName)
+{
+    QFileInfo info(fileName);
+
+    KIS_SAFE_ASSERT_RECOVER_NOOP(fileName.isEmpty() || info.isAbsolute());
+    m_d->audioChannelFileName = fileName.isEmpty() ? fileName : info.absoluteFilePath();
+
+    emit sigAudioChannelChanged();
+}
+
+bool KisImageAnimationInterface::isAudioMuted() const
+{
+    return m_d->audioChannelMuted;
+}
+
+void KisImageAnimationInterface::setAudioMuted(bool value)
+{
+    m_d->audioChannelMuted = value;
+    emit sigAudioChannelChanged();
+}
+
+qreal KisImageAnimationInterface::audioVolume() const
+{
+    return m_d->audioChannelVolume;
+}
+
+void KisImageAnimationInterface::setAudioVolume(qreal value)
+{
+    m_d->audioChannelVolume = value;
+    emit sigAudioVolumeChanged();
+}
+
 void KisImageAnimationInterface::setFramerate(int fps)
 {
     m_d->framerate = fps;
@@ -136,36 +235,67 @@ bool KisImageAnimationInterface::externalFrameActive() const
 
 void KisImageAnimationInterface::requestTimeSwitchWithUndo(int time)
 {
-    if (m_d->currentUITime == time) return;
-
-    KisSwitchCurrentTimeCommand *cmd =
-        new KisSwitchCurrentTimeCommand(m_d->image, time);
-
-    cmd->redo();
-    m_d->image->postExecutionUndoAdapter()->addCommand(toQShared(cmd));
+    if (currentUITime() == time) return;
+    requestTimeSwitchNonGUI(time, true);
 }
 
-void KisImageAnimationInterface::requestTimeSwitchNonGUI(int time)
+void KisImageAnimationInterface::setDefaultProjectionColor(const KoColor &color)
 {
-    emit sigInternalRequestTimeSwitch(time);
+    int savedTime = 0;
+    saveAndResetCurrentTime(currentTime(), &savedTime);
+
+    m_d->image->setDefaultProjectionColor(color);
+
+    restoreCurrentTime(&savedTime);
 }
 
-void KisImageAnimationInterface::switchCurrentTimeAsync(int frameId)
+void KisImageAnimationInterface::requestTimeSwitchNonGUI(int time, bool useUndo)
 {
-    if (m_d->currentUITime == frameId) return;
+    emit sigInternalRequestTimeSwitch(time, useUndo);
+}
 
-    m_d->image->barrierLock();
-    m_d->currentTime = frameId;
-    m_d->currentUITime = frameId;
-    m_d->image->unlock();
+void KisImageAnimationInterface::explicitlySetCurrentTime(int frameId)
+{
+    m_d->setCurrentTime(frameId);
+}
 
-    KisStrokeStrategy *strategy =
-        new KisRegenerateFrameStrokeStrategy(this);
+void KisImageAnimationInterface::switchCurrentTimeAsync(int frameId, bool useUndo)
+{
+    if (currentUITime() == frameId) return;
 
-    KisStrokeId stroke = m_d->image->startStroke(strategy);
-    m_d->image->endStroke(stroke);
+    const KisTimeRange range = KisTimeRange::calculateIdenticalFramesRecursive(m_d->image->root(), currentUITime());
+    const bool needsRegeneration = !range.contains(frameId);
 
-    emit sigTimeChanged(frameId);
+    KisSwitchTimeStrokeStrategy::SharedTokenSP token =
+        m_d->switchToken.toStrongRef();
+
+    if (!token || !token->tryResetDestinationTime(frameId, needsRegeneration)) {
+
+        {
+            KisPostExecutionUndoAdapter *undoAdapter = useUndo ?
+                m_d->image->postExecutionUndoAdapter() : 0;
+
+            KisSwitchTimeStrokeStrategy *strategy =
+                new KisSwitchTimeStrokeStrategy(frameId, needsRegeneration,
+                                                this, undoAdapter);
+
+            m_d->switchToken = strategy->token();
+
+            KisStrokeId stroke = m_d->image->startStroke(strategy);
+            m_d->image->endStroke(stroke);
+        }
+
+        if (needsRegeneration) {
+            KisStrokeStrategy *strategy =
+                new KisRegenerateFrameStrokeStrategy(this);
+
+            KisStrokeId strokeId = m_d->image->startStroke(strategy);
+            m_d->image->endStroke(strokeId);
+        }
+    }
+
+    m_d->setCurrentUITime(frameId);
+    emit sigUiTimeChanged(frameId);
 }
 
 void KisImageAnimationInterface::requestFrameRegeneration(int frameId, const QRegion &dirtyRegion)
@@ -187,19 +317,19 @@ void KisImageAnimationInterface::requestFrameRegeneration(int frameId, const QRe
 void KisImageAnimationInterface::saveAndResetCurrentTime(int frameId, int *savedValue)
 {
     m_d->externalFrameActive = true;
-    *savedValue = m_d->currentTime;
-    m_d->currentTime = frameId;
+    *savedValue = m_d->currentTime();
+    m_d->setCurrentTime(frameId);
 }
 
 void KisImageAnimationInterface::restoreCurrentTime(int *savedValue)
 {
-    m_d->currentTime = *savedValue;
+    m_d->setCurrentTime(*savedValue);
     m_d->externalFrameActive = false;
 }
 
 void KisImageAnimationInterface::notifyFrameReady()
 {
-    emit sigFrameReady(m_d->currentTime);
+    emit sigFrameReady(m_d->currentTime());
 }
 
 void KisImageAnimationInterface::notifyFrameCancelled()
@@ -216,23 +346,33 @@ void KisImageAnimationInterface::notifyNodeChanged(const KisNode *node,
                                                    const QRect &rect,
                                                    bool recursive)
 {
-    if (externalFrameActive() || m_d->frameInvalidationBlocked) return;
+    notifyNodeChanged(node, QVector<QRect>({rect}), recursive);
+}
 
-    KisKeyframeChannel *channel =
-        node->getKeyframeChannel(KisKeyframeChannel::Content.id());
+void KisImageAnimationInterface::notifyNodeChanged(const KisNode *node,
+                                                   const QVector<QRect> &rects,
+                                                   bool recursive)
+{
+    if (externalFrameActive() || m_d->frameInvalidationBlocked) return;
+    if (node->inherits("KisSelectionMask")) return;
+
+
+    const int currentTime = m_d->currentTime();
+    KisTimeRange invalidateRange;
 
     if (recursive) {
-        KisTimeRange affectedRange;
-        KisTimeRange::calculateTimeRangeRecursive(node, currentTime(), affectedRange, false);
-
-        invalidateFrames(affectedRange, rect);
-    } else if (channel) {
-        const int currentTime = m_d->currentTime;
-
-        invalidateFrames(channel->affectedFrames(currentTime), rect);
+        invalidateRange = KisTimeRange::calculateAffectedFramesRecursive(node, currentTime);
     } else {
-        invalidateFrames(KisTimeRange::infinite(0), rect);
+        invalidateRange = KisTimeRange::calculateNodeAffectedFrames(node, currentTime);
     }
+
+    // we compress the updated rect (atm, no one uses it anyway)
+    QRect unitedRect;
+    Q_FOREACH (const QRect &rc, rects) {
+        unitedRect |= rc;
+    }
+
+    invalidateFrames(invalidateRange, unitedRect);
 }
 
 void KisImageAnimationInterface::invalidateFrames(const KisTimeRange &range, const QRect &rect)
@@ -276,7 +416,7 @@ int KisImageAnimationInterface::totalLength()
     int lastKey = m_d->cachedLastFrameValue;
 
     lastKey  = std::max(lastKey, m_d->fullClipRange.end());
-    lastKey  = std::max(lastKey, m_d->currentUITime);
+    lastKey  = std::max(lastKey, m_d->currentUITime());
 
     return lastKey + 1;
 }
