@@ -87,12 +87,12 @@ public:
     /**
      * Delete this node
      */
-    virtual ~KisNode();
+    ~KisNode() override;
 
     virtual KisNodeSP clone() const = 0;
 
-    virtual bool accept(KisNodeVisitor &v);
-    virtual void accept(KisProcessingVisitor &visitor, KisUndoAdapter *undoAdapter);
+    bool accept(KisNodeVisitor &v) override;
+    void accept(KisProcessingVisitor &visitor, KisUndoAdapter *undoAdapter) override;
 
     /**
      * Re-implement this method to add constraints for the
@@ -114,7 +114,7 @@ public:
      * this percolates up to parent nodes all the way to the root
      * node.
      */
-    virtual void setDirty(const QRect & rect);
+    void setDirty(const QRect & rect);
 
     /**
      * Add the given rects to the set of dirty rects for this node;
@@ -128,7 +128,28 @@ public:
      * this percolates up to parent nodes all the way to the root
      * node, if propagate is true;
      */
-    virtual void setDirty(const QRegion &region);
+    void setDirty(const QRegion &region);
+
+    /**
+     * Convenience override of multirect version of setDirtyDontResetAnimationCache()
+     *
+     * @see setDirtyDontResetAnimationCache(const QVector<QRect> &rects)
+     */
+    void setDirtyDontResetAnimationCache();
+
+    /**
+     * Convenience override of multirect version of setDirtyDontResetAnimationCache()
+     *
+     * @see setDirtyDontResetAnimationCache(const QVector<QRect> &rects)
+     */
+    void setDirtyDontResetAnimationCache(const QRect &rect);
+
+    /**
+     * @brief setDirtyDontResetAnimationCache does almost the same thing as usual
+     * setDirty() call, but doesn't reset the animation cache (since onlion skins are
+     * not used when rendering animation.
+     */
+    void setDirtyDontResetAnimationCache(const QVector<QRect> &rects);
 
     /**
      * Informs that the frames in the given range are no longer valid
@@ -157,25 +178,6 @@ public:
      */
     virtual KisAbstractProjectionPlaneSP projectionPlane() const;
 
-    /**
-     * Return the keyframe channels associated with this node
-     * @return list of keyframe channels
-     */
-    QList<KisKeyframeChannel *> keyframeChannels() const;
-
-    /**
-     * Get the keyframe channel with given id.
-     * @param id internal name for channel
-     * @return keyframe channel with the id, or null if not found
-     */
-    KisKeyframeChannel *getKeyframeChannel(const QString &id) const;
-
-    bool useInTimeline() const;
-    void setUseInTimeline(bool value);
-
-    bool isAnimated() const;
-    virtual void enableAnimation();
-
     /**
      * Synchronizes LoD caches of the node with the current state of it.
      * The current level of detail is fetched from the image pointed by
@@ -194,18 +196,6 @@ public:
      */
     virtual KisProjectionLeafSP projectionLeaf() const;
 
-protected:
-
-    /**
-     * Add a keyframe channel for this node. The channel will be added
-     * to the common hash table which will be available to the UI.
-     *
-     * WARNING: the \p channel object *NOT* become owned by the node!
-     *          The caller must ensure manually that the lifetime of
-     *          the object coincide with the lifetime of the node.
-     */
-    void addKeyframeChannel(KisKeyframeChannel* channel);
-
 protected:
 
     /**
@@ -256,6 +246,13 @@ protected:
      */
     virtual QRect accessRect(const QRect &rect, PositionToFilthy pos = N_FILTHY) const;
 
+    /**
+     * Called each time direct child nodes are added or removed under this
+     * node as parent. This does not track changes inside the child nodes
+     * or the child nodes' properties.
+     */
+    virtual void childNodeChanged(KisNodeSP changedChildNode);
+
 public: // Graph methods
 
     /**
@@ -352,6 +349,12 @@ public: // Graph methods
      */
     KisNodeSP findChildByName(const QString &name);
 
+Q_SIGNALS:
+    /**
+     * Don't use this signal anywhere other than KisNodeShape. It's a hack.
+     */
+    void sigNodeChangedInternal();
+
 public:
 
     /**
@@ -373,10 +376,13 @@ private:
     void createNodeProgressProxy();
 
 protected:
-    KisBaseNodeSP parentCallback() const;
-    void notifyParentVisibilityChanged(bool value);
-    void baseNodeChangedCallback();
-    void baseNodeInvalidateAllFramesCallback();
+    KisBaseNodeSP parentCallback() const override;
+    void notifyParentVisibilityChanged(bool value) override;
+    void baseNodeChangedCallback() override;
+    void baseNodeInvalidateAllFramesCallback() override;
+
+protected:
+    void addKeyframeChannel(KisKeyframeChannel* channel) override;
 private:
 
     friend class KisNodeFacade;
