@@ -30,7 +30,8 @@ public:
         : m_paintDevice(paintDevice),
           m_exactBoundsCache(paintDevice),
           m_nonDefaultPixelAreaCache(paintDevice),
-          m_regionCache(paintDevice)
+          m_regionCache(paintDevice),
+          m_sequenceNumber(0)
     {
     }
 
@@ -38,7 +39,8 @@ public:
         : m_paintDevice(rhs.m_paintDevice),
           m_exactBoundsCache(rhs.m_paintDevice),
           m_nonDefaultPixelAreaCache(rhs.m_paintDevice),
-          m_regionCache(rhs.m_paintDevice)
+          m_regionCache(rhs.m_paintDevice),
+          m_sequenceNumber(0)
     {
     }
 
@@ -51,6 +53,7 @@ public:
         m_exactBoundsCache.invalidate();
         m_nonDefaultPixelAreaCache.invalidate();
         m_regionCache.invalidate();
+        m_sequenceNumber++;
     }
 
     QRect exactBounds() {
@@ -61,17 +64,14 @@ public:
         QRect bounds;
         bool result = m_exactBoundsCache.tryGetValue(bounds);
 
-        const int msecThreshold = 1000;
-
         if (!result) {
-            if (!m_lastCalculatedExactBounds.isValid() ||
-                m_lastCalculatedExactBounds.elapsed() > msecThreshold) {
-
-                m_lastCalculatedExactBounds.restart();
-                bounds = exactBounds();
-            } else {
-                bounds = m_paintDevice->extent();
-            }
+            /**
+             * The calculation of the exact bounds might be too slow
+             * in some special cases, e.g. for an empty canvas of 7k
+             * by 6k.  So we just always return extent, when the exact
+             * bounds is not available.
+             */
+            bounds = m_paintDevice->extent();
         }
 
         return bounds;
@@ -85,37 +85,44 @@ public:
         return m_regionCache.getValue();
     }
 
-    QImage createThumbnail(qint32 w, qint32 h, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags) {
+    QImage createThumbnail(qint32 w, qint32 h, qreal oversample, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags) {
         QImage thumbnail;
 
-        if(m_thumbnailsValid) {
-            thumbnail = findThumbnail(w, h);
+        if (h == 0 || w == 0) {
+            return thumbnail;
+        }
+
+        if (m_thumbnailsValid) {
+            thumbnail = findThumbnail(w, h, oversample);
         }
         else {
             m_thumbnails.clear();
             m_thumbnailsValid = true;
         }
 
-        if(thumbnail.isNull()) {
-            thumbnail = m_paintDevice->createThumbnail(w, h, QRect(), renderingIntent, conversionFlags);
-            cacheThumbnail(w, h, thumbnail);
+        if (thumbnail.isNull()) {
+            thumbnail = m_paintDevice->createThumbnail(w, h, QRect(), oversample, renderingIntent, conversionFlags);
+            cacheThumbnail(w, h, oversample, thumbnail);
         }
 
-        Q_ASSERT(!thumbnail.isNull() || m_paintDevice->extent().isEmpty());
         return thumbnail;
     }
 
+    int sequenceNumber() const {
+        return m_sequenceNumber;
+    }
+
 private:
-    inline QImage findThumbnail(qint32 w, qint32 h) {
+    inline QImage findThumbnail(qint32 w, qint32 h, qreal oversample) {
         QImage resultImage;
-        if (m_thumbnails.contains(w) && m_thumbnails[w].contains(h)) {
-            resultImage = m_thumbnails[w][h];
+        if (m_thumbnails.contains(w) && m_thumbnails[w].contains(h) && m_thumbnails[w][h].contains(oversample)) {
+            resultImage = m_thumbnails[w][h][oversample];
         }
         return resultImage;
     }
 
-    inline void cacheThumbnail(qint32 w, qint32 h, QImage image) {
-        m_thumbnails[w][h] = image;
+    inline void cacheThumbnail(qint32 w, qint32 h, qreal oversample, QImage image) {
+        m_thumbnails[w][h][oversample] = image;
     }
 
 private:
@@ -124,7 +131,7 @@ private:
     struct ExactBoundsCache : KisLockFreeCache<QRect> {
         ExactBoundsCache(KisPaintDevice *paintDevice) : m_paintDevice(paintDevice) {}
 
-        QRect calculateNewValue() const {
+        QRect calculateNewValue() const override {
             return m_paintDevice->calculateExactBounds(false);
         }
     private:
@@ -134,7 +141,7 @@ private:
     struct NonDefaultPixelCache : KisLockFreeCache<QRect> {
         NonDefaultPixelCache(KisPaintDevice *paintDevice) : m_paintDevice(paintDevice) {}
 
-        QRect calculateNewValue() const {
+        QRect calculateNewValue() const override {
             return m_paintDevice->calculateExactBounds(true);
         }
     private:
@@ -144,7 +151,7 @@ private:
     struct RegionCache : KisLockFreeCache<QRegion> {
         RegionCache(KisPaintDevice *paintDevice) : m_paintDevice(paintDevice) {}
 
-        QRegion calculateNewValue() const {
+        QRegion calculateNewValue() const override {
             return m_paintDevice->dataManager()->region();
         }
     private:
@@ -154,10 +161,10 @@ private:
     ExactBoundsCache m_exactBoundsCache;
     NonDefaultPixelCache m_nonDefaultPixelAreaCache;
     RegionCache m_regionCache;
-    QElapsedTimer m_lastCalculatedExactBounds;
 
     bool m_thumbnailsValid;
-    QMap<int, QMap<int, QImage> > m_thumbnails;
+    QMap<int, QMap<int, QMap<qreal,QImage> > > m_thumbnails;
+    QAtomicInt m_sequenceNumber;
 };
 
 #endif /* __KIS_PAINT_DEVICE_CACHE_H */
