@@ -21,6 +21,7 @@
 #include <QDebug>
 #include "kis_keyframe_channel.h"
 #include "kis_node.h"
+#include "kis_layer_utils.h"
 
 struct KisTimeRangeStaticRegistrar {
     KisTimeRangeStaticRegistrar() {
@@ -37,26 +38,74 @@ QDebug operator<<(QDebug dbg, const KisTimeRange &r)
     return dbg.space();
 }
 
-void KisTimeRange::calculateTimeRangeRecursive(const KisNode *node, int time, KisTimeRange &range, bool exclusive)
+KisTimeRange KisTimeRange::calculateIdenticalFramesRecursive(const KisNode *node, int time)
 {
-    KisKeyframeChannel *channel =
-            node->getKeyframeChannel(KisKeyframeChannel::Content.id());
+    KisTimeRange range = KisTimeRange::infinite(0);
 
-    if (channel) {
-        if (exclusive) {
-            // Intersection
-            range &= channel->affectedFrames(time);
-        } else {
-            // Union
-            range |= channel->affectedFrames(time);
-        }
+    KisLayerUtils::recursiveApplyNodes(node,
+        [&range, time] (const KisNode *node) {
+            if (node->visible()) {
+                range &= calculateNodeIdenticalFrames(node, time);
+            }
+    });
+
+    return range;
+}
+
+KisTimeRange KisTimeRange::calculateAffectedFramesRecursive(const KisNode *node, int time)
+{
+    KisTimeRange range;
+
+    KisLayerUtils::recursiveApplyNodes(node,
+        [&range, time] (const KisNode *node) {
+            if (node->visible()) {
+                range |= calculateNodeIdenticalFrames(node, time);
+            }
+    });
+
+    return range;
+}
+
+KisTimeRange KisTimeRange::calculateNodeIdenticalFrames(const KisNode *node, int time)
+{
+    KisTimeRange range = KisTimeRange::infinite(0);
+
+    const QMap<QString, KisKeyframeChannel*> channels =
+        node->keyframeChannels();
+
+    Q_FOREACH (const KisKeyframeChannel *channel, channels) {
+        // Intersection
+        range &= channel->identicalFrames(time);
     }
 
-    KisNodeSP child = node->firstChild();
-    while (child) {
-        calculateTimeRangeRecursive(child, time, range, exclusive);
-        child = child->nextSibling();
+    return range;
+}
+
+KisTimeRange KisTimeRange::calculateNodeAffectedFrames(const KisNode *node, int time)
+{
+    KisTimeRange range;
+
+    if (!node->visible()) return range;
+
+    const QMap<QString, KisKeyframeChannel*> channels =
+        node->keyframeChannels();
+
+    // TODO: channels should report to the image which channel exactly has changed
+    //       to avoid the dirty range to be stretched into infinity!
+
+    if (channels.isEmpty() ||
+        !channels.contains(KisKeyframeChannel::Content.id())) {
+
+        range = KisTimeRange::infinite(0);
+        return range;
     }
+
+    Q_FOREACH (const KisKeyframeChannel *channel, channels) {
+        // Union
+        range |= channel->affectedFrames(time);
+    }
+
+    return range;
 }
 
 namespace KisDomUtils {
