@@ -46,8 +46,16 @@ uint qHash(KisWeakSharedPtr<T> ptr) {
  * Define lots of shared pointer versions of Krita classes.
  * Shared pointer classes have the advantage of near automatic
  * memory management (but beware of circular references)
- * and the disadvantage that inheritiance relations are no longer
- * recognizable
+ * These types should never be passed by reference,
+ * because that will mess up their reference counter.
+ *
+ * An example of the naming pattern used:
+ *
+ * KisPaintDeviceSP is a KisSharedPtr of KisPaintDevice
+ * KisPaintDeviceWSP is a KisWeakSharedPtr of KisPaintDevice
+ * vKisPaintDeviceSP is a QVector of KisPaintDeviceSP
+ * vKisPaintDeviceSP_it is an iterator of vKisPaintDeviceSP
+ *
  */
 class KisImage;
 typedef KisSharedPtr<KisImage> KisImageSP;
@@ -97,6 +105,10 @@ class KisTransparencyMask;
 typedef KisSharedPtr<KisTransparencyMask> KisTransparencyMaskSP;
 typedef KisWeakSharedPtr<KisTransparencyMask> KisTransparencyMaskWSP;
 
+class KisColorizeMask;
+typedef KisSharedPtr<KisColorizeMask> KisColorizeMaskSP;
+typedef KisWeakSharedPtr<KisColorizeMask> KisColorizeMaskWSP;
+
 class KisLayer;
 typedef KisSharedPtr<KisLayer> KisLayerSP;
 typedef KisWeakSharedPtr<KisLayer> KisLayerWSP;
@@ -121,6 +133,10 @@ class KisGroupLayer;
 typedef KisSharedPtr<KisGroupLayer> KisGroupLayerSP;
 typedef KisWeakSharedPtr<KisGroupLayer> KisGroupLayerWSP;
 
+class KisFileLayer;
+typedef KisSharedPtr<KisFileLayer> KisFileLayerSP;
+typedef KisWeakSharedPtr<KisFileLayer> KisFileLayerWSP;
+
 class KisSelection;
 typedef KisSharedPtr<KisSelection> KisSelectionSP;
 typedef KisWeakSharedPtr<KisSelection> KisSelectionWSP;
@@ -128,9 +144,6 @@ typedef KisWeakSharedPtr<KisSelection> KisSelectionWSP;
 class KisSelectionComponent;
 typedef KisSharedPtr<KisSelectionComponent> KisSelectionComponentSP;
 
-class KisBackground;
-typedef KisSharedPtr<KisBackground> KisBackgroundSP;
-
 class KisSelectionMask;
 typedef KisSharedPtr<KisSelectionMask> KisSelectionMaskSP;
 
@@ -164,6 +177,14 @@ class KisAnimationFrameCache;
 typedef KisSharedPtr<KisAnimationFrameCache> KisAnimationFrameCacheSP;
 typedef KisWeakSharedPtr<KisAnimationFrameCache> KisAnimationFrameCacheWSP;
 
+class KisPaintingAssistant;
+typedef QSharedPointer<KisPaintingAssistant> KisPaintingAssistantSP;
+typedef QWeakPointer<KisPaintingAssistant> KisPaintingAssistantWSP;
+
+class KisReferenceImage;
+typedef QSharedPointer<KisReferenceImage> KisReferenceImageSP;
+typedef QWeakPointer<KisReferenceImage> KisReferenceImageWSP;
+
 // Repeat iterators
 class KisHLineIterator2;
 template<class T> class KisRepeatHLineIteratorPixelBase;
@@ -206,8 +227,15 @@ class KisPaintOpPreset;
 typedef KisSharedPtr<KisPaintOpPreset> KisPaintOpPresetSP;
 typedef KisWeakSharedPtr<KisPaintOpPreset> KisPaintOpPresetWSP;
 
+template <typename T>
+class KisPinnedSharedPtr;
+
 class KisPaintOpSettings;
-typedef KisSharedPtr<KisPaintOpSettings> KisPaintOpSettingsSP;
+typedef KisPinnedSharedPtr<KisPaintOpSettings> KisPaintOpSettingsSP;
+
+template <typename T>
+class KisRestrictedSharedPtr;
+typedef KisRestrictedSharedPtr<KisPaintOpSettings> KisPaintOpSettingsRestrictedSP;
 
 class KisPaintOp;
 typedef KisSharedPtr<KisPaintOp> KisPaintOpSP;
@@ -236,7 +264,13 @@ typedef QWeakPointer<KisStroke> KisStrokeWSP;
 typedef KisStrokeWSP KisStrokeId;
 
 class KisFilterConfiguration;
-typedef QSharedPointer<KisFilterConfiguration> KisSafeFilterConfigurationSP;
+typedef KisPinnedSharedPtr<KisFilterConfiguration> KisFilterConfigurationSP;
+
+class KisPropertiesConfiguration;
+typedef KisPinnedSharedPtr<KisPropertiesConfiguration> KisPropertiesConfigurationSP;
+
+class KisLockedProperties;
+typedef KisSharedPtr<KisLockedProperties> KisLockedPropertiesSP;
 
 class KisProjectionUpdatesFilter;
 typedef QSharedPointer<KisProjectionUpdatesFilter> KisProjectionUpdatesFilterSP;
@@ -253,9 +287,24 @@ class KisKeyframe;
 typedef QSharedPointer<KisKeyframe> KisKeyframeSP;
 typedef QWeakPointer<KisKeyframe> KisKeyframeWSP;
 
+class KisFilterChain;
+typedef KisSharedPtr<KisFilterChain> KisFilterChainSP;
+
+class KisProofingConfiguration;
+typedef QSharedPointer<KisProofingConfiguration> KisProofingConfigurationSP;
+typedef QWeakPointer<KisProofingConfiguration> KisProofingConfigurationWSP;
+
+class KisLayerComposition;
+typedef QSharedPointer<KisLayerComposition> KisLayerCompositionSP;
+typedef QWeakPointer<KisLayerComposition> KisLayerCompositionWSP;
+
+
+
 #include <QSharedPointer>
 #include <QWeakPointer>
 #include <kis_shared_ptr.h>
+#include <kis_restricted_shared_ptr.h>
+#include <kis_pinned_shared_ptr.h>
 
 #endif // KISTYPES_H_
 
