@@ -2,7 +2,7 @@
  * Copyright (C) 1998-1999 Torben Weis       <weis@kde.org>
  * Copyright (C) 2000-2005 David Faure       <faure@kde.org>
  * Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
- * Copyright (C) 2010-2012 Boudewijn Rempt   <boud@kogmbh.com>
+ * Copyright (C) 2010-2012 Boudewijn Rempt   <boud@valdyas.org>
  * Copyright (C) 2011 Inge Wallin            <ingwa@kogmbh.com>
  * Copyright (C) 2015 Michael Abrahams       <miabraha@gmail.com>
  *
@@ -30,11 +30,12 @@
 #include <KoColorSpaceEngine.h>
 #include <KoCanvasBase.h>
 #include <KoToolManager.h>
-#include <KoShapeBasedDocumentBase.h>
+#include <KoShapeControllerBase.h>
 #include <KoResourceServerProvider.h>
 #include <kis_icon.h>
 
 #include "KisApplication.h"
+#include "KisMainWindow.h"
 #include "KisDocument.h"
 #include "KisView.h"
 #include "KisViewManager.h"
@@ -57,25 +58,21 @@
 #include <QDomElement>
 #include <QGlobalStatic>
 #include <KisMimeDatabase.h>
+#include <dialogs/KisSessionManagerDialog.h>
 
-#include "KisView.h"
-#include "KisDocument.h"
 #include "kis_config.h"
 #include "kis_shape_controller.h"
-#include "kis_resource_server_provider.h"
+#include "KisResourceServerProvider.h"
 #include "kis_animation_cache_populator.h"
 #include "kis_idle_watcher.h"
 #include "kis_image.h"
-#include "KisImportExportManager.h"
-#include "KisDocument.h"
-#include "KoToolManager.h"
 #include "KisOpenPane.h"
 
 #include "kis_color_manager.h"
-#include "kis_debug.h"
 
 #include "kis_action.h"
 #include "kis_action_registry.h"
+#include "KisSessionResource.h"
 
 Q_GLOBAL_STATIC(KisPart, s_instance)
 
@@ -99,28 +96,26 @@ public:
     QList<QPointer<KisView> > views;
     QList<QPointer<KisMainWindow> > mainWindows;
     QList<QPointer<KisDocument> > documents;
-
     KActionCollection *actionCollection{0};
-
     KisIdleWatcher idleWatcher;
     KisAnimationCachePopulator animationCachePopulator;
-    void loadActions();
-};
 
-// Basically, we are going to insert the current UI/MainWindow ActionCollection
-// into the KisActionRegistry.
-void KisPart::loadActions()
-{
-    d->actionCollection = currentMainwindow()->viewManager()->actionCollection();
+    KisSessionResource *currentSession = nullptr;
+    bool closingSession{false};
+    QScopedPointer<KisSessionManagerDialog> sessionManager;
 
-    KisActionRegistry * actionRegistry = KisActionRegistry::instance();
+    bool queryCloseDocument(KisDocument *document) {
+        Q_FOREACH(auto view, views) {
+            if (view && view->isVisible() && view->document() == document) {
+                return view->queryClose();
+            }
+        }
 
-    Q_FOREACH (auto action, d->actionCollection->actions()) {
-        auto name = action->objectName();
-        actionRegistry->addAction(action->objectName(), action);
+        return true;
     }
 };
 
+
 KisPart* KisPart::instance()
 {
     return s_instance;
@@ -146,6 +141,7 @@ KisPart::KisPart()
     connect(&d->idleWatcher, SIGNAL(startedIdleMode()),
             &d->animationCachePopulator, SLOT(slotRequestRegeneration()));
 
+
     d->animationCachePopulator.slotRequestRegeneration();
 }
 
@@ -171,7 +167,9 @@ void KisPart::updateIdleWatcherConnections()
     QVector<KisImageSP> images;
 
     Q_FOREACH (QPointer<KisDocument> document, documents()) {
-        images << document->image();
+        if (document->image()) {
+            images << document->image();
+        }
     }
 
     d->idleWatcher.setTrackedImages(images);
@@ -184,6 +182,8 @@ void KisPart::addDocument(KisDocument *document)
     if (!d->documents.contains(document)) {
         d->documents.append(document);
         emit documentOpened('/'+objectName());
+        emit sigDocumentAdded(document);
+        connect(document, SIGNAL(sigSavingFinished()), SLOT(slotDocumentSaved()));
     }
 }
 
@@ -208,16 +208,16 @@ void KisPart::removeDocument(KisDocument *document)
 {
     d->documents.removeAll(document);
     emit documentClosed('/'+objectName());
+    emit sigDocumentRemoved(document->url().toLocalFile());
     document->deleteLater();
 }
 
-KisMainWindow *KisPart::createMainWindow()
+KisMainWindow *KisPart::createMainWindow(QUuid id)
 {
-    KisMainWindow *mw = new KisMainWindow();
-
+    KisMainWindow *mw = new KisMainWindow(id);
     dbgUI <<"mainWindow" << (void*)mw << "added to view" << this;
     d->mainWindows.append(mw);
-
+    emit sigWindowAdded(mw);
     return mw;
 }
 
@@ -260,8 +260,6 @@ void KisPart::addView(KisView *view)
         d->views.append(view);
     }
 
-    connect(view, SIGNAL(destroyed()), this, SLOT(viewDestroyed()));
-
     emit sigViewAdded(view);
 }
 
@@ -269,6 +267,14 @@ void KisPart::removeView(KisView *view)
 {
     if (!view) return;
 
+    /**
+     * HACK ALERT: we check here explicitly if the document (or main
+     *             window), is saving the stuff. If we close the
+     *             document *before* the saving is completed, a crash
+     *             will happen.
+     */
+    KIS_ASSERT_RECOVER_RETURN(!view->mainWindow()->hackIsSaving());
+
     emit sigViewRemoved(view);
 
     QPointer<KisDocument> doc = view->document();
@@ -309,6 +315,55 @@ int KisPart::viewCount(KisDocument *doc) const
     }
 }
 
+bool KisPart::closingSession() const
+{
+    return d->closingSession;
+}
+
+bool KisPart::closeSession(bool keepWindows)
+{
+    d->closingSession = true;
+
+    Q_FOREACH(auto document, d->documents) {
+        if (!d->queryCloseDocument(document.data())) {
+            d->closingSession = false;
+            return false;
+        }
+    }
+
+    if (d->currentSession) {
+        KisConfig kisCfg;
+        if (kisCfg.saveSessionOnQuit(false)) {
+
+            d->currentSession->storeCurrentWindows();
+            d->currentSession->save();
+
+            KConfigGroup cfg = KSharedConfig::openConfig()->group("session");
+            cfg.writeEntry("previousSession", d->currentSession->name());
+        }
+
+        d->currentSession = nullptr;
+    }
+
+    if (!keepWindows) {
+        Q_FOREACH (auto window, d->mainWindows) {
+            window->close();
+        }
+
+        if (d->sessionManager) {
+            d->sessionManager->close();
+        }
+    }
+
+    d->closingSession = false;
+    return true;
+}
+
+void KisPart::slotDocumentSaved()
+{
+    KisDocument *doc = qobject_cast<KisDocument*>(sender());
+    emit sigDocumentSaved(doc->url().toLocalFile());
+}
 
 void KisPart::removeMainWindow(KisMainWindow *mainWindow)
 {
@@ -345,6 +400,17 @@ KisMainWindow *KisPart::currentMainwindow() const
 
 }
 
+KisMainWindow * KisPart::windowById(QUuid id) const
+{
+    Q_FOREACH(QPointer<KisMainWindow> mainWindow, d->mainWindows) {
+        if (mainWindow->id() == id) {
+            return mainWindow;
+        }
+    }
+
+    return nullptr;
+}
+
 KisIdleWatcher* KisPart::idleWatcher() const
 {
     return &d->idleWatcher;
@@ -357,24 +423,12 @@ KisAnimationCachePopulator* KisPart::cachePopulator() const
 
 void KisPart::openExistingFile(const QUrl &url)
 {
-    Q_ASSERT(url.isLocalFile());
-    qApp->setOverrideCursor(Qt::BusyCursor);
-    KisDocument *document = createDocument();
-    if (!document->openUrl(url)) {
-        delete document;
-        return;
-    }
-    if (!document->image()) {
-        delete document;
-        return;
-    }
-    document->setModified(false);
-    addDocument(document);
+    // TODO: refactor out this method!
 
     KisMainWindow *mw = currentMainwindow();
-    mw->addViewAndNotifyLoadingCompleted(document);
+    KIS_SAFE_ASSERT_RECOVER_RETURN(mw);
 
-    qApp->restoreOverrideCursor();
+    mw->openDocument(url, KisMainWindow::None);
 }
 
 void KisPart::updateShortcuts()
@@ -422,11 +476,16 @@ void KisPart::openTemplate(const QUrl &url)
         mimeType.remove( QRegExp( "-template$" ) );
         document->setMimeTypeAfterLoading(mimeType);
         document->resetURL();
-        document->setEmpty();
     }
     else {
-        document->showLoadingErrorDialog();
-        document->initEmpty();
+        if (document->errorMessage().isEmpty()) {
+            QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("Could not create document from template\n%1", document->localFilePath()));
+        }
+        else {
+            QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("Could not create document from template\n%1\nReason: %2", document->localFilePath(), document->errorMessage()));
+        }
+        delete document;
+        return;
     }
     addDocument(document);
 
@@ -436,21 +495,13 @@ void KisPart::openTemplate(const QUrl &url)
     KisOpenPane *pane = qobject_cast<KisOpenPane*>(sender());
     if (pane) {
         pane->hide();
-        delete pane;
+        pane->deleteLater();
 
     }
 
     qApp->restoreOverrideCursor();
 }
 
-void KisPart::viewDestroyed()
-{
-    KisView *view = qobject_cast<KisView*>(sender());
-    if (view) {
-        removeView(view);
-    }
-}
-
 void KisPart::addRecentURLToAllMainWindows(QUrl url)
 {
     // Add to recent actions list in our mainWindows
@@ -467,7 +518,7 @@ void KisPart::startCustomDocument(KisDocument* doc)
     KisOpenPane *pane = qobject_cast<KisOpenPane*>(sender());
     if (pane) {
         pane->hide();
-        delete pane;
+        pane->deleteLater();
     }
     mw->addViewAndNotifyLoadingCompleted(doc);
 
@@ -475,6 +526,43 @@ void KisPart::startCustomDocument(KisDocument* doc)
 
 KisInputManager* KisPart::currentInputManager()
 {
-    return instance()->currentMainwindow()->viewManager()->inputManager();
+    KisMainWindow *mw = currentMainwindow();
+    KisViewManager *manager = mw ? mw->viewManager() : 0;
+    return manager ? manager->inputManager() : 0;
 }
 
+void KisPart::showSessionManager()
+{
+    if (d->sessionManager.isNull()) {
+        d->sessionManager.reset(new KisSessionManagerDialog());
+    }
+
+    d->sessionManager->show();
+    d->sessionManager->activateWindow();
+}
+
+void KisPart::startBlankSession()
+{
+    KisMainWindow *window = createMainWindow();
+    window->initializeGeometry();
+    window->show();
+
+}
+
+bool KisPart::restoreSession(const QString &sessionName)
+{
+    if (sessionName.isNull()) return false;
+
+    KoResourceServer<KisSessionResource> * rserver = KisResourceServerProvider::instance()->sessionServer();
+    auto *session = rserver->resourceByName(sessionName);
+    if (!session || !session->valid()) return false;
+
+    session->restore();
+
+    return true;
+}
+
+void KisPart::setCurrentSession(KisSessionResource *session)
+{
+    d->currentSession = session;
+}
