@@ -2,7 +2,7 @@
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
    Copyright (C) 2000-2005 David Faure  <faure@kde.org>
    Copyright (C) 2007 Thorsten Zachmann <zachmann@kde.org>
-   Copyright (C) 2010 Boudewijn Rempt   <boud@kogmbh.com>
+   Copyright (C) 2010 Boudewijn Rempt   <boud@valdyas.org>
    Copyright (C) 2015 Michael Abrahams  <miabraha@gmail.com>
 
    This library is free software; you can redistribute it and/or
@@ -27,20 +27,23 @@
 #include <QList>
 #include <QPointer>
 #include <QUrl>
+#include <QUuid>
 
 #include "kritaui_export.h"
+#include <KConfigCore/kconfiggroup.h>
 #include <KoConfig.h>
 #include <KisMainWindow.h>
 
 namespace KIO {
 }
 
+class KisAction;
 class KisDocument;
 class KisView;
-class QGraphicsItem;
 class KisDocument;
 class KisIdleWatcher;
 class KisAnimationCachePopulator;
+class KisSessionResource;
 
 /**
  * KisPart is the Great Deku Tree of Krita.
@@ -77,7 +80,7 @@ public:
      * The destructor does not delete any attached KisView objects and it does not
      * delete the attached widget as returned by widget().
      */
-    ~KisPart();
+    ~KisPart() override;
 
     // ----------------- Document management -----------------
 
@@ -105,10 +108,11 @@ public:
 
     // ----------------- MainWindow management -----------------
 
+
     /**
      * Create a new main window.
      */
-    KisMainWindow *createMainWindow();
+    KisMainWindow *createMainWindow(QUuid id = QUuid());
 
     /**
      * Removes a main window from the list of managed windows.
@@ -134,10 +138,7 @@ public:
      */
     KisMainWindow *currentMainwindow() const;
 
-    /**
-     * Load actions for currently active main window into KisActionRegistry.
-     */
-    void loadActions();
+    KisMainWindow *windowById(QUuid id) const;
 
     /**
      * @return the application-wide KisIdleWatcher.
@@ -149,7 +150,6 @@ public:
      */
     KisAnimationCachePopulator *cachePopulator() const;
 
-
 public Q_SLOTS:
 
     /**
@@ -173,29 +173,32 @@ public Q_SLOTS:
 
 private Q_SLOTS:
 
-    void viewDestroyed();
-
     void updateIdleWatcherConnections();
 
     void updateShortcuts();
 
 Q_SIGNALS:
     /**
-     * emitted when a new document is opened.
+     * emitted when a new document is opened. (for the idle watcher)
      */
     void documentOpened(const QString &ref);
 
     /**
-     * emitted when an old document is closed.
+     * emitted when an old document is closed. (for the idle watcher)
      */
     void documentClosed(const QString &ref);
 
+    // These signals are for libkis or sketch
     void sigViewAdded(KisView *view);
     void sigViewRemoved(KisView *view);
+    void sigDocumentAdded(KisDocument *document);
+    void sigDocumentSaved(const QString &url);
+    void sigDocumentRemoved(const QString &filename);
+    void sigWindowAdded(KisMainWindow *window);
 
 public:
 
-    static KisInputManager *currentInputManager();
+    KisInputManager *currentInputManager();
 
     //------------------ View management ------------------
 
@@ -232,24 +235,34 @@ public:
      */
     int viewCount(KisDocument *doc) const;
 
-    /**
-     * @return a QGraphicsItem canvas displaying this document. There is only one QGraphicsItem canvas that can
-     * be shown by many QGraphicsView subclasses (those should reimplement KoCanvasController
-     * as well).
-     *
-     * @param create if true, a new canvas item is created if there wasn't one.
-     */
-    QGraphicsItem *canvasItem(KisDocument *document, bool create = true);
+    //------------------ Session management ------------------
 
+    void showSessionManager();
 
-protected:
+    void startBlankSession();
 
     /**
-     * Override this to create a QGraphicsItem that does not rely
-     * on proxying a KoCanvasController.
+     * Restores a saved session by name
      */
-    QGraphicsItem *createCanvasItem(KisDocument *document);
+    bool restoreSession(const QString &sessionName);
 
+    void setCurrentSession(KisSessionResource *session);
+
+    /**
+     * Attempts to save the session and close all windows.
+     * This may involve asking the user to save open files.
+     * @return false, if closing was cancelled by the user
+     */
+    bool closeSession(bool keepWindows = false);
+
+    /**
+     * Are we in the process of closing the application through closeSession().
+     */
+    bool closingSession() const;
+
+private Q_SLOTS:
+
+    void slotDocumentSaved();
 
 private:
 
@@ -258,7 +271,6 @@ private:
     class Private;
     Private *const d;
 
-
 };
 
 #endif
