@@ -24,7 +24,6 @@
 #include <KoDockFactoryBase.h>
 #include <KoDockRegistry.h>
 #include <KoDocumentInfo.h>
-#include "KoDocumentInfo.h"
 #include "KoPageLayout.h"
 #include <KoToolManager.h>
 
@@ -35,7 +34,6 @@
 #include <kis_debug.h>
 #include <kselectaction.h>
 #include <kconfiggroup.h>
-#include <kactioncollection.h>
 
 #include <QMenu>
 #include <QMessageBox>
@@ -50,10 +48,9 @@
 #include <QList>
 #include <QPrintDialog>
 #include <QToolBar>
-#include <QUrl>
 #include <QStatusBar>
 #include <QMoveEvent>
-#include <QTemporaryFile>
+#include <QMdiSubWindow>
 
 #include <kis_image.h>
 #include <kis_node.h>
@@ -78,15 +75,15 @@
 #include "KisPrintJob.h"
 #include "kis_shape_controller.h"
 #include "kis_tool_freehand.h"
-#include "KisUndoStackAction.h"
 #include "KisViewManager.h"
 #include "kis_zoom_manager.h"
-#include "kis_composite_progress_proxy.h"
 #include "kis_statusbar.h"
 #include "kis_painting_assistants_decoration.h"
+#include "KisReferenceImagesDecoration.h"
 #include "kis_progress_widget.h"
 #include "kis_signal_compressor.h"
 #include "kis_filter_manager.h"
+#include "kis_file_layer.h"
 #include "krita_utils.h"
 #include "input/kis_input_manager.h"
 #include "KisRemoteFileFetcher.h"
@@ -104,7 +101,8 @@ bool KisView::s_firstView = true;
 class Q_DECL_HIDDEN KisView::Private
 {
 public:
-    Private(KisView *_q, KisDocument *document,
+    Private(KisView *_q,
+            KisDocument *document,
             KoCanvasResourceManager *resourceManager,
             KActionCollection *actionCollection)
         : actionCollection(actionCollection)
@@ -112,14 +110,12 @@ public:
         , canvasController(_q, actionCollection)
         , canvas(&viewConverter, resourceManager, _q, document->shapeController())
         , zoomManager(_q, &this->viewConverter, &this->canvasController)
-        , paintingAssistantsDecoration(_q)
+        , paintingAssistantsDecoration(new KisPaintingAssistantsDecoration(_q))
+        , referenceImagesDecoration(new KisReferenceImagesDecoration(_q, document))
         , floatingMessageCompressor(100, KisSignalCompressor::POSTPONE)
     {
     }
 
-    KisUndoStackAction *undo = 0;
-    KisUndoStackAction *redo = 0;
-
     bool inOperation; //in the middle of an operation (no screen refreshing)?
 
     QPointer<KisDocument> document; // our KisDocument
@@ -140,11 +136,16 @@ public:
     KisZoomManager zoomManager;
     KisViewManager *viewManager = 0;
     KisNodeSP currentNode;
-    KisPaintingAssistantsDecoration paintingAssistantsDecoration;
+    KisPaintingAssistantsDecorationSP paintingAssistantsDecoration;
+    KisReferenceImagesDecorationSP referenceImagesDecoration;
     bool isCurrent = false;
     bool showFloatingMessage = false;
     QPointer<KisFloatingMessage> savedFloatingMessage;
     KisSignalCompressor floatingMessageCompressor;
+    QMdiSubWindow *subWindow{nullptr};
+
+    bool softProofing = false;
+    bool gamutCheck = false;
 
     // Hmm sorry for polluting the private class with such a big inner class.
     // At the beginning it was a little struct :)
@@ -216,13 +217,10 @@ KisView::KisView(KisDocument *document, KoCanvasResourceManager *resourceManager
 
     setFocusPolicy(Qt::StrongFocus);
 
-    d->undo = new KisUndoStackAction(d->document->undoStack(), KisUndoStackAction::UNDO);
-    d->redo = new KisUndoStackAction(d->document->undoStack(), KisUndoStackAction::RED0);
-
     QStatusBar * sb = statusBar();
     if (sb) { // No statusbar in e.g. konqueror
-        connect(d->document, SIGNAL(statusBarMessage(const QString&)),
-                this, SLOT(slotActionStatusText(const QString&)));
+        connect(d->document, SIGNAL(statusBarMessage(const QString&, int)),
+                this, SLOT(slotSavingStatusMessage(const QString&, int)));
         connect(d->document, SIGNAL(clearStatusBarMessage()),
                 this, SLOT(slotClearStatusText()));
     }
@@ -233,8 +231,6 @@ KisView::KisView(KisDocument *document, KoCanvasResourceManager *resourceManager
 
     d->canvasController.setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
     d->canvasController.setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
-    d->canvasController.setDrawShadow(false);
-    d->canvasController.setCanvasMode(KoCanvasController::Infinite);
     d->canvasController.setVastScrolling(cfg.vastScrolling());
     d->canvasController.setCanvas(&d->canvas);
 
@@ -247,26 +243,30 @@ KisView::KisView(KisDocument *document, KoCanvasResourceManager *resourceManager
     connect(d->document, SIGNAL(sigLoadingFinished()), this, SLOT(slotLoadingFinished()));
     connect(d->document, SIGNAL(sigSavingFinished()), this, SLOT(slotSavingFinished()));
 
-    d->canvas.addDecoration(&d->paintingAssistantsDecoration);
-    d->paintingAssistantsDecoration.setVisible(true);
+    d->canvas.addDecoration(d->paintingAssistantsDecoration);
+    d->paintingAssistantsDecoration->setVisible(true);
+
+    d->canvas.addDecoration(d->referenceImagesDecoration);
+    d->referenceImagesDecoration->setVisible(true);
 
     d->showFloatingMessage = cfg.showCanvasMessages();
+    d->zoomManager.updateScreenResolution(this);
 }
 
 KisView::~KisView()
 {
     if (d->viewManager) {
-        KoProgressProxy *proxy = d->viewManager->statusBar()->progress()->progressProxy();
-        KIS_ASSERT_RECOVER_NOOP(proxy);
-        image()->compositeProgressProxy()->removeProxy(proxy);
-
         if (d->viewManager->filterManager()->isStrokeRunning()) {
             d->viewManager->filterManager()->cancel();
         }
+
+        d->viewManager->mainWindow()->notifyChildViewDestroyed(this);
     }
 
-    KisPart::instance()->removeView(this);
     KoToolManager::instance()->removeCanvasController(&d->canvasController);
+    d->canvasController.setCanvas(0);
+
+    KisPart::instance()->removeView(this);
     delete d;
 }
 
@@ -309,6 +309,11 @@ void KisView::showFloatingMessageImpl(const QString &message, const QIcon& icon,
     }
 }
 
+bool KisView::canvasIsMirrored() const
+{
+    return d->canvas.xAxisMirrored() || d->canvas.yAxisMirrored();
+}
+
 void KisView::setViewManager(KisViewManager *view)
 {
     d->viewManager = view;
@@ -348,15 +353,6 @@ void KisView::setViewManager(KisViewManager *view)
             SLOT(slotContinueRemoveNode(KisNodeSP)),
             Qt::AutoConnection);
 
-    /*
-     * WARNING: Currently we access the global progress bar in two ways:
-     * connecting to composite progress proxy (strokes) and creating
-     * progress updaters. The latter way should be deprecated in favour
-     * of displaying the status of the global strokes queue
-     */
-    image()->compositeProgressProxy()->addProxy(d->viewManager->statusBar()->progress()->progressProxy());
-    connect(d->viewManager->statusBar()->progress(), SIGNAL(sigCancellationRequested()), image(), SLOT(requestStrokeCancellation()));
-
     d->viewManager->updateGUI();
 
     KoToolManager::instance()->switchToolRequested("KritaShape/KisToolBrush");
@@ -383,7 +379,7 @@ void KisView::slotContinueAddNode(KisNodeSP newActiveNode)
      */
 
     if (!d->isCurrent &&
-        (!d->currentNode || !d->currentNode->parent())) {
+            (!d->currentNode || !d->currentNode->parent())) {
 
         d->currentNode = newActiveNode;
     }
@@ -402,16 +398,6 @@ void KisView::slotContinueRemoveNode(KisNodeSP newActiveNode)
     }
 }
 
-QAction *KisView::undoAction() const
-{
-    return d->undo;
-}
-
-QAction *KisView::redoAction() const
-{
-    return d->redo;
-}
-
 KoZoomController *KisView::zoomController() const
 {
     return d->zoomManager.zoomController();
@@ -526,20 +512,36 @@ void KisView::dropEvent(QDropEvent *event)
             QAction *insertAsNewLayer = new QAction(i18n("Insert as New Layer"), &popup);
             QAction *insertManyLayers = new QAction(i18n("Insert Many Layers"), &popup);
 
+            QAction *insertAsNewFileLayer = new QAction(i18n("Insert as New File Layer"), &popup);
+            QAction *insertManyFileLayers = new QAction(i18n("Insert Many File Layers"), &popup);
+
             QAction *openInNewDocument = new QAction(i18n("Open in New Document"), &popup);
             QAction *openManyDocuments = new QAction(i18n("Open Many Documents"), &popup);
 
+            QAction *insertAsReferenceImage = new QAction(i18n("Insert as Reference Image"), &popup);
+            QAction *insertAsReferenceImages = new QAction(i18n("Insert as Reference Images"), &popup);
+
             QAction *cancel = new QAction(i18n("Cancel"), &popup);
 
             popup.addAction(insertAsNewLayer);
+            popup.addAction(insertAsNewFileLayer);
             popup.addAction(openInNewDocument);
+            popup.addAction(insertAsReferenceImage);
+
             popup.addAction(insertManyLayers);
+            popup.addAction(insertManyFileLayers);
             popup.addAction(openManyDocuments);
+            popup.addAction(insertAsReferenceImages);
 
             insertAsNewLayer->setEnabled(image() && urls.count() == 1);
+            insertAsNewFileLayer->setEnabled(image() && urls.count() == 1);
             openInNewDocument->setEnabled(urls.count() == 1);
+            insertAsReferenceImage->setEnabled(image() && urls.count() == 1);
+
             insertManyLayers->setEnabled(image() && urls.count() > 1);
+            insertManyFileLayers->setEnabled(image() && urls.count() > 1);
             openManyDocuments->setEnabled(urls.count() > 1);
+            insertAsReferenceImages->setEnabled(image() && urls.count() > 1);
 
             popup.addSeparator();
             popup.addAction(cancel);
@@ -547,7 +549,7 @@ void KisView::dropEvent(QDropEvent *event)
             QAction *action = popup.exec(QCursor::pos());
             if (action != 0 && action != cancel) {
                 QTemporaryFile *tmp = 0;
-                Q_FOREACH (QUrl url, urls) {
+                for (QUrl url : urls) {
 
                     if (!url.isLocalFile()) {
                         // download the file and substitute the url
@@ -565,12 +567,28 @@ void KisView::dropEvent(QDropEvent *event)
                             d->viewManager->imageManager()->importImage(url);
                             activateWindow();
                         }
-                        else {
-                            Q_ASSERT(action == openInNewDocument || action == openManyDocuments);
+                        else if (action == insertAsNewFileLayer || action == insertManyFileLayers) {
+                            KisNodeCommandsAdapter adapter(viewManager());
+                            KisFileLayer *fileLayer = new KisFileLayer(image(), "", url.toLocalFile(),
+                                                                       KisFileLayer::None, image()->nextLayerName(), OPACITY_OPAQUE_U8);
+                            adapter.addNode(fileLayer, viewManager()->activeNode()->parent(), viewManager()->activeNode());
+                        }
+                        else if (action == openInNewDocument || action == openManyDocuments) {
                             if (mainWindow()) {
-                                mainWindow()->openDocument(url);
+                                mainWindow()->openDocument(url, KisMainWindow::None);
                             }
                         }
+                        else if (action == insertAsReferenceImage || action == insertAsReferenceImages) {
+                            auto *reference = KisReferenceImage::fromFile(url.toLocalFile(), d->viewConverter, this);
+
+                            if (reference) {
+                                reference->setPosition(d->viewConverter.imageToDocument(cursorPos));
+                                d->referenceImagesDecoration->addReferenceImage(reference);
+
+                                KoToolManager::instance()->switchToolRequested("ToolReferenceImages");
+                            }
+                        }
+
                     }
                     delete tmp;
                     tmp = 0;
@@ -592,8 +610,8 @@ void KisView::setDocument(KisDocument *document)
     d->document = document;
     QStatusBar *sb = statusBar();
     if (sb) { // No statusbar in e.g. konqueror
-        connect(d->document, SIGNAL(statusBarMessage(const QString&)),
-                this, SLOT(slotActionStatusText(const QString&)));
+        connect(d->document, SIGNAL(statusBarMessage(const QString&, int)),
+                this, SLOT(slotSavingStatusMessage(const QString&, int)));
         connect(d->document, SIGNAL(clearStatusBarMessage()),
                 this, SLOT(slotClearStatusText()));
     }
@@ -604,11 +622,6 @@ void KisView::setDocumentDeleted()
     d->documentDeleted = true;
 }
 
-KoPageLayout KisView::pageLayout() const
-{
-    return document()->pageLayout();
-}
-
 QPrintDialog *KisView::createPrintDialog(KisPrintJob *printJob, QWidget *parent)
 {
     Q_UNUSED(parent);
@@ -624,24 +637,40 @@ KisMainWindow * KisView::mainWindow() const
     return dynamic_cast<KisMainWindow *>(window());
 }
 
+void KisView::setSubWindow(QMdiSubWindow *subWindow)
+{
+    d->subWindow = subWindow;
+}
+
 QStatusBar * KisView::statusBar() const
 {
     KisMainWindow *mw = mainWindow();
     return mw ? mw->statusBar() : 0;
 }
 
-void KisView::slotActionStatusText(const QString &text)
+void KisView::slotSavingStatusMessage(const QString &text, int timeout, bool isAutoSaving)
 {
     QStatusBar *sb = statusBar();
-    if (sb)
-        sb->showMessage(text);
+    if (sb) {
+        sb->showMessage(text, timeout);
+    }
+
+    KisConfig cfg;
+
+    if (!sb || sb->isHidden() ||
+        (!isAutoSaving && cfg.forceShowSaveMessages()) ||
+        (cfg.forceShowAutosaveMessages() && isAutoSaving)) {
+
+        viewManager()->showFloatingMessage(text, QIcon());
+    }
 }
 
 void KisView::slotClearStatusText()
 {
     QStatusBar *sb = statusBar();
-    if (sb)
+    if (sb) {
         sb->clearMessage();
+    }
 }
 
 QList<QAction*> KisView::createChangeUnitActions(bool addPixelUnit)
@@ -652,16 +681,16 @@ QList<QAction*> KisView::createChangeUnitActions(bool addPixelUnit)
 
 void KisView::closeEvent(QCloseEvent *event)
 {
-    // Check whether we're the last view
+    // Check whether we're the last (user visible) view
     int viewCount = KisPart::instance()->viewCount(document());
-    if (viewCount > 1) {
+    if (viewCount > 1 || !isVisible()) {
         // there are others still, so don't bother the user
         event->accept();
         return;
     }
 
     if (queryClose()) {
-        d->viewManager->removeStatusBarItem(zoomManager()->zoomActionWidget());
+        d->viewManager->statusBar()->setView(0);
         event->accept();
         return;
     }
@@ -675,6 +704,8 @@ bool KisView::queryClose()
     if (!document())
         return true;
 
+    document()->waitForSavingToComplete();
+
     if (document()->isModified()) {
         QString name;
         if (document()->documentInfo()) {
@@ -693,15 +724,20 @@ bool KisView::queryClose()
 
         switch (res) {
         case QMessageBox::Yes : {
-            bool isNative = (document()->outputMimeType() == document()->nativeFormatMimeType());
-            if (!viewManager()->mainWindow()->saveDocument(document(), !isNative))
+            bool isNative = (document()->mimeType() == document()->nativeFormatMimeType());
+            if (!viewManager()->mainWindow()->saveDocument(document(), !isNative, false))
                 return false;
             break;
         }
-        case QMessageBox::No :
+        case QMessageBox::No : {
+            KisImageSP image = document()->image();
+            image->requestStrokeCancellation();
+            viewManager()->blockUntilOperationsFinishedForced(image);
+
             document()->removeAutoSaveFiles();
             document()->setModified(false);   // Now when queryClose() is called by closeEvent it won't do anything.
             break;
+        }
         default : // case QMessageBox::Cancel :
             return false;
         }
@@ -711,6 +747,11 @@ bool KisView::queryClose()
 
 }
 
+void KisView::slotScreenChanged()
+{
+    d->zoomManager.updateScreenResolution(this);
+}
+
 void KisView::resetImageSizeAndScroll(bool changeCentering,
                                       const QPointF &oldImageStillPoint,
                                       const QPointF &newImageStillPoint)
@@ -764,9 +805,48 @@ void KisView::resetImageSizeAndScroll(bool changeCentering,
     d->canvasController.setPreferredCenter(oldPreferredCenter - oldStillPoint + newStillPoint);
 }
 
+void KisView::syncLastActiveNodeToDocument()
+{
+    KisDocument *doc = document();
+    if (doc) {
+        doc->setPreActivatedNode(d->currentNode);
+    }
+}
+
+void KisView::saveViewState(KisPropertiesConfiguration &config) const
+{
+    config.setProperty("file", d->document->url());
+    config.setProperty("window", mainWindow()->windowStateConfig().name());
+
+    if (d->subWindow) {
+        config.setProperty("geometry", d->subWindow->saveGeometry().toBase64());
+    }
+
+    config.setProperty("zoomMode", (int)zoomController()->zoomMode());
+    config.setProperty("zoom", d->canvas.coordinatesConverter()->zoom());
+
+    d->canvasController.saveCanvasState(config);
+}
+
+void KisView::restoreViewState(const KisPropertiesConfiguration &config)
+{
+    if (d->subWindow) {
+        QByteArray geometry = QByteArray::fromBase64(config.getString("geometry", "").toLatin1());
+        d->subWindow->restoreGeometry(QByteArray::fromBase64(geometry));
+    }
+
+    qreal zoom = config.getFloat("zoom", 1.0f);
+    int zoomMode = config.getInt("zoomMode", (int)KoZoomMode::ZOOM_PAGE);
+    d->zoomManager.zoomController()->setZoom((KoZoomMode::Mode)zoomMode, zoom);
+    d->canvasController.restoreCanvasState(config);
+}
+
 void KisView::setCurrentNode(KisNodeSP node)
 {
     d->currentNode = node;
+    d->canvas.slotTrySwitchShapeManager();
+
+    syncLastActiveNodeToDocument();
 }
 
 KisNodeSP KisView::currentNode() const
@@ -784,7 +864,7 @@ KisLayerSP KisView::currentLayer() const
     else {
         node = d->currentNode;
     }
-    return dynamic_cast<KisLayer*>(node.data());
+    return qobject_cast<KisLayer*>(node.data());
 }
 
 KisMaskSP KisView::currentMask() const
@@ -804,6 +884,58 @@ KisSelectionSP KisView::selection()
     return 0;
 }
 
+void KisView::slotSoftProofing(bool softProofing)
+{
+    d->softProofing = softProofing;
+    QString message;
+    if (canvasBase()->image()->colorSpace()->colorDepthId().id().contains("F"))
+    {
+        message = i18n("Soft Proofing doesn't work in floating point.");
+        viewManager()->showFloatingMessage(message,QIcon());
+        return;
+    }
+    if (softProofing){
+        message = i18n("Soft Proofing turned on.");
+    } else {
+        message = i18n("Soft Proofing turned off.");
+    }
+    viewManager()->showFloatingMessage(message,QIcon());
+    canvasBase()->slotSoftProofing(softProofing);
+}
+
+void KisView::slotGamutCheck(bool gamutCheck)
+{
+    d->gamutCheck = gamutCheck;
+    QString message;
+    if (canvasBase()->image()->colorSpace()->colorDepthId().id().contains("F"))
+    {
+        message = i18n("Gamut Warnings don't work in floating point.");
+        viewManager()->showFloatingMessage(message,QIcon());
+        return;
+    }
+
+    if (gamutCheck){
+        message = i18n("Gamut Warnings turned on.");
+        if (!d->softProofing){
+            message += "\n "+i18n("But Soft Proofing is still off.");
+        }
+    } else {
+        message = i18n("Gamut Warnings turned off.");
+    }
+    viewManager()->showFloatingMessage(message,QIcon());
+    canvasBase()->slotGamutCheck(gamutCheck);
+}
+
+bool KisView::softProofing()
+{
+    return d->softProofing;
+}
+
+bool KisView::gamutCheck()
+{
+    return d->gamutCheck;
+}
+
 void KisView::slotLoadingFinished()
 {
     if (!document()) return;
@@ -835,7 +967,6 @@ void KisView::slotLoadingFinished()
     connect(image(), SIGNAL(sigSizeChanged(QPointF,QPointF)), this, SIGNAL(sigSizeChanged(QPointF,QPointF)));
 
     KisNodeSP activeNode = document()->preActivatedNode();
-    document()->setPreActivatedNode(0); // to make sure that we don't keep a reference to a layer the user can later delete.
 
     if (!activeNode) {
         activeNode = image()->rootLayer()->lastChild();
@@ -846,6 +977,7 @@ void KisView::slotLoadingFinished()
     }
 
     setCurrentNode(activeNode);
+    connect(d->viewManager->mainWindow(), SIGNAL(screenChanged()), SLOT(slotScreenChanged()));
     zoomManager()->updateImageBoundsSnapping();
 }
 
@@ -880,3 +1012,8 @@ void KisView::slotImageSizeChanged(const QPointF &oldStillPoint, const QPointF &
     zoomManager()->updateImageBoundsSnapping();
     zoomManager()->updateGUI();
 }
+
+void KisView::closeView()
+{
+    d->subWindow->close();
+}
