@@ -51,6 +51,7 @@ class KoCanvasResourceManager;
 // KDE classes
 class QAction;
 class KActionCollection;
+class KConfigGroup;
 
 // Qt classes
 class QDragEnterEvent;
@@ -58,6 +59,7 @@ class QDropEvent;
 class QPrintDialog;
 class QCloseEvent;
 class QStatusBar;
+class QMdiSubWindow;
 
 /**
  * This class is used to display a @ref KisDocument.
@@ -73,10 +75,7 @@ public:
      * Creates a new view for the document.
      */
     KisView(KisDocument *document, KoCanvasResourceManager *resourceManager, KActionCollection *actionCollection, QWidget *parent = 0);
-    ~KisView();
-
-    QAction *undoAction() const;
-    QAction *redoAction() const;
+    ~KisView() override;
 
     // Temporary while teasing apart view and mainwindow
     void setViewManager(KisViewManager *view);
@@ -106,13 +105,6 @@ public:
      */
     KisPrintJob * createPrintJob();
 
-    /**
-     * @return the page layout to be used for printing.
-     * Default is the documents layout.
-     * Reimplement if your application needs to use a different layout.
-     */
-    KoPageLayout pageLayout() const;
-
     /**
      * Create a QPrintDialog based on the @p printJob
      */
@@ -121,12 +113,17 @@ public:
     /**
      * @return the KisMainWindow in which this view is currently.
      */
-    KisMainWindow * mainWindow() const;
+    KisMainWindow *mainWindow() const;
+
+    /**
+     * Tells this view which subwindow it is part of.
+     */
+    void setSubWindow(QMdiSubWindow *subWindow);
 
     /**
      * @return the statusbar of the KisMainWindow in which this view is currently.
      */
-    QStatusBar * statusBar() const;
+    QStatusBar *statusBar() const;
 
     /**
      * This adds a widget to the statusbar for this view.
@@ -150,6 +147,8 @@ public:
     /// create a list of actions that when activated will change the unit on the document.
     QList<QAction*> createChangeUnitActions(bool addPixelUnit = false);
 
+    void closeView();
+
 public:
 
     /**
@@ -192,6 +191,16 @@ public:
     KisNodeSP currentNode() const;
     KisLayerSP currentLayer() const;
     KisMaskSP currentMask() const;
+    /**
+     * @brief softProofing
+     * @return whether or not we're softproofing in this view.
+     */
+    bool softProofing();
+    /**
+     * @brief gamutCheck
+     * @return whether or not we're using gamut warnings in this view.
+     */
+    bool gamutCheck();
 
     /// Convenience method to get at the active selection (the
     /// selection of the current layer, or, if that does not exist,
@@ -203,22 +212,42 @@ public:
     void setShowFloatingMessage(bool show);
     void showFloatingMessageImpl(const QString &message, const QIcon& icon, int timeout, KisFloatingMessage::Priority priority, int alignment);
 
+    bool canvasIsMirrored() const;
+
+    void syncLastActiveNodeToDocument();
+
+    void saveViewState(KisPropertiesConfiguration &config) const;
+    void restoreViewState(const KisPropertiesConfiguration &config);
+
 public Q_SLOTS:
 
     /**
      * Display a message in the status bar (calls QStatusBar::message())
      * @todo rename to something more generic
+     * @param value determines autosaving
      */
-    void slotActionStatusText(const QString &text);
+    void slotSavingStatusMessage(const QString &text, int timeout, bool isAutoSaving = false);
 
     /**
      * End of the message in the status bar (calls QStatusBar::clear())
      * @todo rename to something more generic
      */
     void slotClearStatusText();
+    /**
+     * @brief slotSoftProofing set whether or not we're softproofing in this view.
+     * Will be setting the same in the canvas belonging to the view.
+     */
+    void slotSoftProofing(bool softProofing);
+    /**
+     * @brief slotGamutCheck set whether or not we're gamutchecking in this view.
+     * Will be setting the same in the vans belonging to the view.
+     */
+    void slotGamutCheck(bool gamutCheck);
 
     bool queryClose();
 
+    void slotScreenChanged();
+
 private Q_SLOTS:
     void slotImageNodeAdded(KisNodeSP node);
     void slotContinueAddNode(KisNodeSP newActiveNode);
@@ -239,10 +268,9 @@ Q_SIGNALS:
 protected:
 
     // QWidget overrides
-    void dragEnterEvent(QDragEnterEvent * event);
-    void dropEvent(QDropEvent * event);
-    bool event( QEvent* event );
-    void closeEvent(QCloseEvent *event);
+    void dragEnterEvent(QDragEnterEvent * event) override;
+    void dropEvent(QDropEvent * event) override;
+    void closeEvent(QCloseEvent *event) override;
 
     /**
      * Generate a name for this view.
@@ -255,6 +283,7 @@ public Q_SLOTS:
     void slotImageResolutionChanged();
     void slotImageSizeChanged(const QPointF &oldStillPoint, const QPointF &newStillPoint);
 
+
 private:
 
     class Private;
