@@ -30,16 +30,11 @@
 #include <KoShapeController.h>
 #include <KoShapeRegistry.h>
 #include <KoCompositeOpRegistry.h>
-#include <KoOdfPaste.h>
-#include <KoOdfLoadingContext.h>
-#include <KoOdfReadStore.h>
 #include <KoShapeManager.h>
 #include <KoSelection.h>
-#include <KoDrag.h>
-#include <KoShapeOdfSaveHelper.h>
-#include <KoShapeController.h>
 #include <KoDocumentResourceManager.h>
 #include <KoShapeStroke.h>
+#include <KoDocumentInfo.h>
 
 #include "KisViewManager.h"
 #include "kis_canvas_resource_provider.h"
@@ -62,16 +57,24 @@
 #include "kis_transaction_based_command.h"
 #include "kis_selection_filters.h"
 #include "kis_shape_selection.h"
-#include "KisPart.h"
 #include "kis_shape_layer.h"
 #include <kis_shape_controller.h>
+#include "kis_image_animation_interface.h"
+#include "kis_time_range.h"
+#include "kis_keyframe_channel.h"
+
 
 #include <processing/fill_processing_visitor.h>
 #include <kis_selection_tool_helper.h>
 
+#include "kis_figure_painting_tool_helper.h"
+
 namespace ActionHelper {
 
-    void copyFromDevice(KisViewManager *view, KisPaintDeviceSP device, bool makeSharpClip = false)
+    void copyFromDevice(KisViewManager *view,
+                        KisPaintDeviceSP device,
+                        bool makeSharpClip = false,
+                        const KisTimeRange &range = KisTimeRange())
     {
         KisImageWSP image = view->image();
         if (!image) return;
@@ -126,7 +129,7 @@ namespace ActionHelper {
             }
         }
 
-        KisClipboard::instance()->setClip(clip, rc.topLeft());
+        KisClipboard::instance()->setClip(clip, rc.topLeft(), range);
     }
 
 }
@@ -147,9 +150,10 @@ void KisSelectAllActionFactory::run(KisViewManager *view)
     struct SelectAll : public KisTransactionBasedCommand {
         SelectAll(KisImageSP image) : m_image(image) {}
         KisImageSP m_image;
-        KUndo2Command* paint() {
+        KUndo2Command* paint() override {
             KisSelectionSP selection = m_image->globalSelection();
             KisSelectionTransaction transaction(selection->pixelSelection());
+            selection->pixelSelection()->clear();
             selection->pixelSelection()->select(m_image->bounds());
             return transaction.endAndTake();
         }
@@ -212,7 +216,7 @@ void KisFillActionFactory::run(const QString &fillSource, KisViewManager *view)
                                        kundo2_i18n("Flood Fill Layer"));
 
     KisResourcesSnapshotSP resources =
-        new KisResourcesSnapshot(view->image(), node, 0, view->resourceProvider()->resourceManager());
+        new KisResourcesSnapshot(view->image(), node, view->resourceProvider()->resourceManager());
     if (!fillSource.contains("opacity")) {
         resources->setOpacity(1.0);
     }
@@ -235,6 +239,8 @@ void KisFillActionFactory::run(const QString &fillSource, KisViewManager *view)
                             KisStrokeJobData::EXCLUSIVE);
 
     applicator.end();
+
+    view->resourceProvider()->slotPainting();
 }
 
 void KisClearActionFactory::run(KisViewManager *view)
@@ -302,63 +308,69 @@ void KisCutCopyActionFactory::run(bool willCut, bool makeSharpClip, KisViewManag
                 return;
             }
 
-            ActionHelper::copyFromDevice(view, dev, makeSharpClip);
-        }
+            KisTimeRange range;
 
-        if (willCut) {
-            KUndo2Command *command = 0;
-
-            if (willCut && node->hasEditablePaintDevice()) {
-                struct ClearSelection : public KisTransactionBasedCommand {
-                    ClearSelection(KisNodeSP node, KisSelectionSP sel)
-                        : m_node(node), m_sel(sel) {}
-                    KisNodeSP m_node;
-                    KisSelectionSP m_sel;
-
-                    KUndo2Command* paint() {
-                        KisSelectionSP cutSelection = m_sel;
-                        // Shrinking the cutting area was previously used
-                        // for getting seamless cut-paste. Now we use makeSharpClip
-                        // instead.
-                        // QRect originalRect = cutSelection->selectedExactRect();
-                        // static const int preciseSelectionThreshold = 16;
-                        //
-                        // if (originalRect.width() > preciseSelectionThreshold ||
-                        //     originalRect.height() > preciseSelectionThreshold) {
-                        //     cutSelection = new KisSelection(*m_sel);
-                        //     delete cutSelection->flatten();
-                        //
-                        //     KisSelectionFilter* filter = new KisShrinkSelectionFilter(1, 1, false);
-                        //
-                        //     QRect processingRect = filter->changeRect(originalRect);
-                        //     filter->process(cutSelection->pixelSelection(), processingRect);
-                        // }
-
-                        KisTransaction transaction(m_node->paintDevice());
-                        m_node->paintDevice()->clearSelection(cutSelection);
-                        m_node->setDirty(cutSelection->selectedRect());
-                        return transaction.endAndTake();
-                    }
-                };
-
-                command = new ClearSelection(node, selection);
+            KisKeyframeChannel *channel = node->getKeyframeChannel(KisKeyframeChannel::Content.id());
+            if (channel) {
+                const int currentTime = image->animationInterface()->currentTime();
+                range = channel->affectedFrames(currentTime);
             }
 
-            KUndo2MagicString actionName = willCut ?
-                                           kundo2_i18n("Cut") :
-                                           kundo2_i18n("Copy");
-            KisProcessingApplicator *ap = beginAction(view, actionName);
-
-            if (command) {
-                ap->applyCommand(command,
-                                 KisStrokeJobData::SEQUENTIAL,
-                                 KisStrokeJobData::NORMAL);
-            }
-
-            KisOperationConfiguration config(id());
-            config.setProperty("will-cut", willCut);
-            endAction(ap, config.toXML());
+            ActionHelper::copyFromDevice(view, dev, makeSharpClip, range);
         }
+
+        KUndo2Command *command = 0;
+
+        if (willCut && node->hasEditablePaintDevice()) {
+            struct ClearSelection : public KisTransactionBasedCommand {
+                ClearSelection(KisNodeSP node, KisSelectionSP sel)
+                    : m_node(node), m_sel(sel) {}
+                KisNodeSP m_node;
+                KisSelectionSP m_sel;
+
+                KUndo2Command* paint() override {
+                    KisSelectionSP cutSelection = m_sel;
+                    // Shrinking the cutting area was previously used
+                    // for getting seamless cut-paste. Now we use makeSharpClip
+                    // instead.
+                    // QRect originalRect = cutSelection->selectedExactRect();
+                    // static const int preciseSelectionThreshold = 16;
+                    //
+                    // if (originalRect.width() > preciseSelectionThreshold ||
+                    //     originalRect.height() > preciseSelectionThreshold) {
+                    //     cutSelection = new KisSelection(*m_sel);
+                    //     delete cutSelection->flatten();
+                    //
+                    //     KisSelectionFilter* filter = new KisShrinkSelectionFilter(1, 1, false);
+                    //
+                    //     QRect processingRect = filter->changeRect(originalRect);
+                    //     filter->process(cutSelection->pixelSelection(), processingRect);
+                    // }
+
+                    KisTransaction transaction(m_node->paintDevice());
+                    m_node->paintDevice()->clearSelection(cutSelection);
+                    m_node->setDirty(cutSelection->selectedRect());
+                    return transaction.endAndTake();
+                }
+            };
+
+            command = new ClearSelection(node, selection);
+        }
+
+        KUndo2MagicString actionName = willCut ?
+                    kundo2_i18n("Cut") :
+                    kundo2_i18n("Copy");
+        KisProcessingApplicator *ap = beginAction(view, actionName);
+
+        if (command) {
+            ap->applyCommand(command,
+                             KisStrokeJobData::SEQUENTIAL,
+                             KisStrokeJobData::NORMAL);
+        }
+
+        KisOperationConfiguration config(id());
+        config.setProperty("will-cut", willCut);
+        endAction(ap, config.toXML());
     }
 }
 
@@ -366,6 +378,7 @@ void KisCopyMergedActionFactory::run(KisViewManager *view)
 {
     KisImageWSP image = view->image();
     if (!image) return;
+    if (!view->blockUntilOperationsFinished(image)) return;
 
     image->barrierLock();
     KisPaintDeviceSP dev = image->root()->projection();
@@ -376,28 +389,6 @@ void KisCopyMergedActionFactory::run(KisViewManager *view)
     endAction(ap, KisOperationConfiguration(id()).toXML());
 }
 
-void KisPasteActionFactory::run(KisViewManager *view)
-{
-    KisImageWSP image = view->image();
-    if (!image) return;
-
-    KisPaintDeviceSP clip = KisClipboard::instance()->clip(image->bounds(), true);
-
-    if (clip) {
-        KisPaintLayer *newLayer = new KisPaintLayer(image.data(), image->nextLayerName() + i18n("(pasted)"), OPACITY_OPAQUE_U8, clip);
-        KisNodeSP aboveNode = view->activeLayer();
-        KisNodeSP parentNode = aboveNode ? aboveNode->parent() : image->root();
-
-        KUndo2Command *cmd = new KisImageLayerAddCommand(image, newLayer, parentNode, aboveNode);
-        KisProcessingApplicator *ap = beginAction(view, cmd->text());
-        ap->applyCommand(cmd, KisStrokeJobData::SEQUENTIAL, KisStrokeJobData::NORMAL);
-        endAction(ap, KisOperationConfiguration(id()).toXML());
-    } else {
-        // XXX: "Add saving of XML data for Paste of shapes"
-        view->canvasBase()->toolProxy()->paste();
-    }
-}
-
 void KisPasteNewActionFactory::run(KisViewManager *viewManager)
 {
     Q_UNUSED(viewManager);
@@ -409,14 +400,14 @@ void KisPasteNewActionFactory::run(KisViewManager *viewManager)
     if (rect.isEmpty()) return;
 
     KisDocument *doc = KisPart::instance()->createDocument();
-
+    doc->documentInfo()->setAboutInfo("title", i18n("Untitled"));
     KisImageSP image = new KisImage(doc->createUndoStore(),
                                     rect.width(),
                                     rect.height(),
                                     clip->colorSpace(),
                                     i18n("Pasted"));
     KisPaintLayerSP layer =
-        new KisPaintLayer(image.data(), image->nextLayerName() + i18n("(pasted)"),
+        new KisPaintLayer(image.data(), image->nextLayerName() + " " + i18n("(pasted)"),
                           OPACITY_OPAQUE_U8, clip->colorSpace());
 
     KisPainter::copyAreaOptimized(QPoint(), clip, layer->paintDevice(), rect);
@@ -429,7 +420,7 @@ void KisPasteNewActionFactory::run(KisViewManager *viewManager)
     win->addViewAndNotifyLoadingCompleted(doc);
 }
 
-void KisInvertSelectionOperaton::runFromXML(KisViewManager* view, const KisOperationConfiguration& config)
+void KisInvertSelectionOperation::runFromXML(KisViewManager* view, const KisOperationConfiguration& config)
 {
     KisSelectionFilter* filter = new KisInvertSelectionFilter();
     runFilter(filter, view, config);
@@ -460,67 +451,45 @@ void KisSelectionToVectorActionFactory::run(KisViewManager *view)
 
     KisProcessingApplicator *ap = beginAction(view, kundo2_i18n("Convert to Vector Selection"));
 
-    ap->applyCommand(view->canvasBase()->shapeController()->addShape(shape),
+    ap->applyCommand(view->canvasBase()->shapeController()->addShape(shape, 0),
                      KisStrokeJobData::SEQUENTIAL,
                      KisStrokeJobData::EXCLUSIVE);
 
     endAction(ap, KisOperationConfiguration(id()).toXML());
 }
 
-
-class KisShapeSelectionPaste : public KoOdfPaste
-{
-public:
-    KisShapeSelectionPaste(KisViewManager* view) : m_view(view)
-    {
-    }
-
-    virtual ~KisShapeSelectionPaste() {
-    }
-
-    virtual bool process(const KoXmlElement & body, KoOdfReadStore & odfStore) {
-        KoOdfLoadingContext loadingContext(odfStore.styles(), odfStore.store());
-        KoShapeLoadingContext context(loadingContext, m_view->canvasBase()->shapeController()->resourceManager());
-        KoXmlElement child;
-
-        QList<KoShape*> shapes;
-        forEachElement(child, body) {
-            KoShape * shape = KoShapeRegistry::instance()->createShapeFromOdf(child, context);
-            if (shape) {
-                shapes.append(shape);
-            }
-        }
-        if (!shapes.isEmpty()) {
-            KisSelectionToolHelper helper(m_view->canvasBase(), kundo2_i18n("Convert shapes to vector selection"));
-            helper.addSelectionShapes(shapes);
-        }
-        return true;
-    }
-private:
-    KisViewManager* m_view;
-};
-
 void KisShapesToVectorSelectionActionFactory::run(KisViewManager* view)
 {
-    QList<KoShape*> shapes = view->canvasBase()->shapeManager()->selection()->selectedShapes();
+    const QList<KoShape*> originalShapes = view->canvasBase()->shapeManager()->selection()->selectedShapes();
 
-    KoShapeOdfSaveHelper saveHelper(shapes);
-    KoDrag drag;
-    drag.setOdf(KoOdf::mimeType(KoOdf::Text), saveHelper);
-    QMimeData* mimeData = drag.mimeData();
+    bool hasSelectionShapes = false;
+    QList<KoShape*> clonedShapes;
 
-    Q_ASSERT(mimeData->hasFormat(KoOdf::mimeType(KoOdf::Text)));
+    Q_FOREACH (KoShape *shape, originalShapes) {
+        if (dynamic_cast<KisShapeSelectionMarker*>(shape->userData())) {
+            hasSelectionShapes = true;
+            continue;
+        }
+        clonedShapes << shape->cloneShape();
+    }
 
-    KisShapeSelectionPaste paste(view);
-    paste.paste(KoOdf::Text, mimeData);
+    if (clonedShapes.isEmpty()) {
+        if (hasSelectionShapes) {
+            view->showFloatingMessage(i18nc("floating message",
+                                            "The shape already belongs to a selection"),
+                                      QIcon(), 2000, KisFloatingMessage::Low);
+        }
+        return;
+    }
+
+    KisSelectionToolHelper helper(view->canvasBase(), kundo2_i18n("Convert shapes to vector selection"));
+    helper.addSelectionShapes(clonedShapes);
 }
 
 void KisSelectionToShapeActionFactory::run(KisViewManager *view)
 {
     KisSelectionSP selection = view->selection();
-
     if (!selection->outlineCacheValid()) {
-
         return;
     }
 
@@ -531,8 +500,109 @@ void KisSelectionToShapeActionFactory::run(KisViewManager *view)
     shape->setShapeId(KoPathShapeId);
 
     KoColor fgColor = view->canvasBase()->resourceManager()->resource(KoCanvasResourceManager::ForegroundColor).value<KoColor>();
-    KoShapeStroke* border = new KoShapeStroke(1.0, fgColor.toQColor());
+    KoShapeStrokeSP border(new KoShapeStroke(1.0, fgColor.toQColor()));
     shape->setStroke(border);
 
     view->document()->shapeController()->addShape(shape);
 }
+
+void KisStrokeSelectionActionFactory::run(KisViewManager *view, StrokeSelectionOptions params)
+{
+    KisImageWSP image = view->image();
+    if (!image) {
+        return;
+    }
+
+    KisSelectionSP selection = view->selection();
+    if (!selection) {
+        return;
+    }
+
+    int size = params.lineSize;
+
+    KisPixelSelectionSP pixelSelection = selection->projection();
+    if (!pixelSelection->outlineCacheValid()) {
+        pixelSelection->recalculateOutlineCache();
+    }
+
+    QPainterPath outline = pixelSelection->outlineCache();
+    QColor color = params.color.toQColor();
+
+    KisNodeSP currentNode = view->resourceProvider()->resourceManager()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeWSP>();
+    if (!currentNode->inherits("KisShapeLayer") && currentNode->paintDevice()) {
+        KoCanvasResourceManager * rManager = view->resourceProvider()->resourceManager();
+        KisPainter::StrokeStyle strokeStyle =  KisPainter::StrokeStyleBrush;
+        KisPainter::FillStyle fillStyle =  params.fillStyle();
+
+        KisFigurePaintingToolHelper helper(kundo2_i18n("Draw Polyline"),
+                                       image,
+                                       currentNode,
+                                       rManager ,
+                                       strokeStyle,
+                                       fillStyle);
+        helper.setFGColorOverride(params.color);
+        helper.setSelectionOverride(0);
+        QPen pen(Qt::red, size);
+        pen.setJoinStyle(Qt::RoundJoin);
+
+        if (fillStyle != KisPainter::FillStyleNone) {
+            helper.paintPainterPathQPenFill(outline, pen, params.fillColor);
+        }
+        else {
+            helper.paintPainterPathQPen(outline, pen, params.fillColor);
+        }
+    }
+    else if (currentNode->inherits("KisShapeLayer")) {
+
+        QTransform transform = view->canvasBase()->coordinatesConverter()->imageToDocumentTransform();
+
+        KoShape *shape = KoPathShape::createShapeFromPainterPath(transform.map(outline));
+        shape->setShapeId(KoPathShapeId);
+
+        KoShapeStrokeSP border(new KoShapeStroke(size, color));
+        shape->setStroke(border);
+
+        view->document()->shapeController()->addShape(shape);
+    }
+    image->setModified();
+
+}
+
+void KisStrokeBrushSelectionActionFactory::run(KisViewManager *view, StrokeSelectionOptions params)
+{
+    KisImageWSP image = view->image();
+    if (!image) {
+        return;
+    }
+
+    KisSelectionSP selection = view->selection();
+    if (!selection) {
+        return;
+    }
+
+    KisPixelSelectionSP pixelSelection = selection->projection();
+    if (!pixelSelection->outlineCacheValid()) {
+        pixelSelection->recalculateOutlineCache();
+    }
+
+    KisNodeSP currentNode = view->resourceProvider()->resourceManager()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeWSP>();
+    if (!currentNode->inherits("KisShapeLayer") && currentNode->paintDevice())
+    {
+        KoCanvasResourceManager * rManager = view->resourceProvider()->resourceManager();
+        QPainterPath outline = pixelSelection->outlineCache();
+        KisPainter::StrokeStyle strokeStyle =  KisPainter::StrokeStyleBrush;
+        KisPainter::FillStyle fillStyle =  KisPainter::FillStyleNone;
+        KoColor color = params.color;
+
+        KisFigurePaintingToolHelper helper(kundo2_i18n("Draw Polyline"),
+                                       image,
+                                       currentNode,
+                                       rManager ,
+                                       strokeStyle,
+                                       fillStyle);
+        helper.setFGColorOverride(color);
+        helper.setSelectionOverride(0);
+        helper.paintPainterPath(outline);
+        image->setModified();
+    }
+}
