@@ -40,28 +40,29 @@ class KRITAUI_EXPORT KisCanvasWidgetBase : public KisAbstractCanvasWidget
 public:
     KisCanvasWidgetBase(KisCanvas2 * canvas, KisCoordinatesConverter *coordinatesConverter);
 
-    virtual ~KisCanvasWidgetBase();
+    ~KisCanvasWidgetBase() override;
 
 public: // KisAbstractCanvasWidget
 
-    virtual KoToolProxy *toolProxy() const;
+    KoToolProxy *toolProxy() const override;
 
 
     /// set the specified display filter on the canvas
-    virtual void setDisplayFilter(KisDisplayFilter *displayFilter);
+    void setDisplayFilter(QSharedPointer<KisDisplayFilter> displayFilter) override = 0;
 
     /**
      * Draw the specified decorations on the view.
      */
-    virtual void drawDecorations(QPainter & gc, const QRect &updateWidgetRect) const;
+    void drawDecorations(QPainter & gc, const QRect &updateWidgetRect) const override;
 
-    virtual void addDecoration(KisCanvasDecoration* deco);
-    virtual KisCanvasDecoration* decoration(const QString& id) const;
+    void addDecoration(KisCanvasDecorationSP deco) override;
+    void removeDecoration(const QString& id) override;
+    KisCanvasDecorationSP decoration(const QString& id) const override;
 
-    virtual void setDecorations(const QList<QPointer<KisCanvasDecoration> > &);
-    virtual QList<QPointer<KisCanvasDecoration> > decorations() const;
+    void setDecorations(const QList<KisCanvasDecorationSP > &) override;
+    QList<KisCanvasDecorationSP > decorations() const override;
 
-    virtual void setWrapAroundViewingMode(bool value);
+    void setWrapAroundViewingMode(bool value) override;
 
     /**
      * Returns the color of the border, i.e. the part of the canvas
@@ -78,6 +79,9 @@ public: // KisAbstractCanvasWidget
 
     KisCoordinatesConverter* coordinatesConverter() const;
 
+    QVector<QRect> updateCanvasProjection(const QVector<KisUpdateInfoSP> &infoObjects) override;
+    using KisAbstractCanvasWidget::updateCanvasProjection;
+
 protected:
     KisCanvas2 *canvas() const;
 
