@@ -46,7 +46,7 @@ namespace _Private
         typedef QRectF Result;
         static QRectF map(const QTransform& transform, const QRectF& rc)  { return transform.mapRect(rc); }
     };
-    
+
     template<> struct Traits<QRect>:    public Traits<QRectF>    { };
     template<> struct Traits<QPoint>:   public Traits<QPointF>   { };
     template<> struct Traits<QPolygon>: public Traits<QPolygonF> { };
@@ -57,22 +57,23 @@ class KRITAUI_EXPORT KisCoordinatesConverter: public KoZoomHandler
 {
 public:
     KisCoordinatesConverter();
-    ~KisCoordinatesConverter();
+    ~KisCoordinatesConverter() override;
 
     void setCanvasWidgetSize(QSize size);
+    void setDevicePixelRatio(qreal value);
     void setImage(KisImageWSP image);
     void setDocumentOffset(const QPoint &offset);
 
     QPoint documentOffset() const;
     qreal rotationAngle() const;
-    
+
     QPoint rotate(QPointF center, qreal angle);
-    QPoint mirror(QPointF center, bool mirrorXAxis, bool mirrorYAxis, bool keepOrientation=false);
+    QPoint mirror(QPointF center, bool mirrorXAxis, bool mirrorYAxis);
     bool xAxisMirrored() const;
     bool yAxisMirrored() const;
     QPoint resetRotation(QPointF center);
-    
-    virtual void setZoom(qreal zoom);
+
+    void setZoom(qreal zoom) override;
 
     /**
      * A composition of to scale methods: zoom level + image resolution
@@ -83,22 +84,22 @@ public:
     imageToViewport(const T& obj) const { return _Private::Traits<T>::map(imageToViewportTransform(), obj); }
     template<class T> typename _Private::Traits<T>::Result
     viewportToImage(const T& obj) const { return _Private::Traits<T>::map(imageToViewportTransform().inverted(), obj); }
-    
+
     template<class T> typename _Private::Traits<T>::Result
     flakeToWidget(const T& obj) const { return _Private::Traits<T>::map(flakeToWidgetTransform(), obj); }
     template<class T> typename _Private::Traits<T>::Result
     widgetToFlake(const T& obj) const { return _Private::Traits<T>::map(flakeToWidgetTransform().inverted(), obj); }
-    
+
     template<class T> typename _Private::Traits<T>::Result
     widgetToViewport(const T& obj) const { return _Private::Traits<T>::map(viewportToWidgetTransform().inverted(), obj); }
     template<class T> typename _Private::Traits<T>::Result
     viewportToWidget(const T& obj) const { return _Private::Traits<T>::map(viewportToWidgetTransform(), obj); }
-    
+
     template<class T> typename _Private::Traits<T>::Result
     documentToWidget(const T& obj) const { return _Private::Traits<T>::map(documentToWidgetTransform(), obj); }
     template<class T> typename _Private::Traits<T>::Result
     widgetToDocument(const T& obj) const { return _Private::Traits<T>::map(documentToWidgetTransform().inverted(), obj); }
-    
+
     template<class T> typename _Private::Traits<T>::Result
     imageToDocument(const T& obj) const { return _Private::Traits<T>::map(imageToDocumentTransform(), obj); }
     template<class T> typename _Private::Traits<T>::Result
@@ -108,7 +109,7 @@ public:
     documentToFlake(const T& obj) const { return _Private::Traits<T>::map(documentToFlakeTransform(), obj); }
     template<class T> typename _Private::Traits<T>::Result
     flakeToDocument(const T& obj) const { return _Private::Traits<T>::map(documentToFlakeTransform().inverted(), obj); }
-    
+
     template<class T> typename _Private::Traits<T>::Result
     imageToWidget(const T& obj) const { return _Private::Traits<T>::map(imageToWidgetTransform(), obj); }
     template<class T> typename _Private::Traits<T>::Result
@@ -124,20 +125,22 @@ public:
 
     void getQPainterCheckersInfo(QTransform *transform,
                                  QPointF *brushOrigin,
-                                 QPolygonF *poligon) const;
+                                 QPolygonF *poligon,
+                                 const bool scrollCheckers) const;
 
     void getOpenGLCheckersInfo(const QRectF &viewportRect,
                                QTransform *textureTransform,
                                QTransform *modelTransform,
                                QRectF *textureRect,
                                QRectF *modelRect,
-                               bool scrollCheckers) const;
+                               const bool scrollCheckers) const;
 
     QPointF imageCenterInWidgetPixel() const;
     QRectF imageRectInWidgetPixels() const;
     QRectF imageRectInViewportPixels() const;
     QSizeF imageSizeInFlakePixels() const;
     QRectF widgetRectInFlakePixels() const;
+    QRectF widgetRectInImagePixels() const;
     QRect imageRectInImagePixels() const;
     QRectF imageRectInDocumentPixels() const;
 
@@ -145,6 +148,7 @@ public:
     QPointF widgetCenterPoint() const;
 
     void imageScale(qreal *scaleX, qreal *scaleY) const;
+    void imagePhysicalScale(qreal *scaleX, qreal *scaleY) const;
 
 private:
     friend class KisZoomAndPanTest;
