@@ -30,6 +30,7 @@
 
 #include <kis_icon.h>
 
+#include "kis_canvas2.h"
 #include "kis_canvas_resource_provider.h"
 #include "KisViewManager.h"
 #include "KisView.h"
@@ -46,6 +47,10 @@ public:
         , resourceProvider(0)
         , mirrorHorizontal(false)
         , mirrorVertical(false)
+        , lockHorizontal(false)
+        , lockVertical(false)
+        , hideVerticalDecoration(false)
+        , hideHorizontalDecoration(false)
         , handleSize(32.f)
         , xActive(false)
         , yActive(false)
@@ -65,9 +70,17 @@ public:
     KisMirrorAxis* q;
 
     KisCanvasResourceProvider* resourceProvider;
+    KisImageWSP image;
     bool mirrorHorizontal;
     bool mirrorVertical;
 
+    bool lockHorizontal;
+    bool lockVertical;
+    bool hideVerticalDecoration;
+    bool hideHorizontalDecoration;
+
+
+
     float handleSize;
     QPixmap horizontalHandleIcon;
     QPixmap verticalHandleIcon;
@@ -95,6 +108,9 @@ KisMirrorAxis::KisMirrorAxis(KisCanvasResourceProvider* provider, QPointer<KisVi
 {
     d->resourceProvider = provider;
     connect(d->resourceProvider, SIGNAL(mirrorModeChanged()), SLOT(mirrorModeChanged()));
+    connect(d->resourceProvider, SIGNAL(moveMirrorVerticalCenter()), SLOT(moveVerticalAxisToCenter()));
+    connect(d->resourceProvider, SIGNAL(moveMirrorHorizontalCenter()), SLOT(moveHorizontalAxisToCenter()));
+
     d->mirrorHorizontal = d->resourceProvider->mirrorHorizontal();
     d->mirrorVertical = d->resourceProvider->mirrorVertical();
     d->horizontalIcon = KisIconUtils::loadIcon("mirrorAxis-HorizontalMove").pixmap(d->handleSize, QIcon::Normal, QIcon::On);
@@ -103,10 +119,8 @@ KisMirrorAxis::KisMirrorAxis(KisCanvasResourceProvider* provider, QPointer<KisVi
     d->verticalHandleIcon = KisIconUtils::loadIcon("transform-move").pixmap(d->handleSize, QIcon::Normal, QIcon::On);
     setVisible(d->mirrorHorizontal || d->mirrorVertical);
 
-    int imageWidth = parent->canvasBase()->image()->width();
-    int imageHeight = parent->canvasBase()->image()->height();
-    QPointF point(imageWidth / 2, imageHeight / 2);
-    d->resourceProvider->resourceManager()->setResource(KisCanvasResourceProvider::MirrorAxesCenter, point);
+
+    d->image = parent->canvasBase()->image();
 }
 
 KisMirrorAxis::~KisMirrorAxis()
@@ -138,6 +152,7 @@ void KisMirrorAxis::drawDecoration(QPainter& gc, const QRectF& updateArea, const
     Q_UNUSED(converter);
     Q_UNUSED(canvas);
 
+    gc.save();
     gc.setPen(QPen(QColor(0, 0, 0, 128), 1));
     gc.setBrush(Qt::white);
     gc.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
@@ -158,7 +173,7 @@ void KisMirrorAxis::drawDecoration(QPainter& gc, const QRectF& updateArea, const
 
     d->recomputeVisibleAxes(gc.viewport());
 
-    if(d->mirrorHorizontal) {
+    if(d->mirrorHorizontal && !d->hideHorizontalDecoration) {
         if (!d->horizontalAxis.isNull()) {
            // QPointF horizontalIndicatorCenter = d->horizontalAxis.unitVector().pointAt(15);
            // QRectF horizontalIndicator = QRectF(horizontalIndicatorCenter.x() - halfHandleSize, horizontalIndicatorCenter.y() - halfHandleSize, d->handleSize, d->handleSize);
@@ -173,16 +188,19 @@ void KisMirrorAxis::drawDecoration(QPainter& gc, const QRectF& updateArea, const
            // gc.drawEllipse(horizontalIndicator);
           //  gc.drawPixmap(horizontalIndicator.adjusted(5, 5, -5, -5).toRect(), d->horizontalIcon);
 
-            gc.setPen(QPen(QColor(0, 0, 0, 128), 2));
-            gc.drawEllipse(d->horizontalHandle);
-            gc.drawPixmap(d->horizontalHandle.adjusted(5, 5, -5, -5).toRect(), d->horizontalIcon);
+            // don't draw the handles if we are locking the axis for movement
+            if (!d->lockHorizontal) {
+                gc.setPen(QPen(QColor(0, 0, 0, 128), 2));
+                gc.drawEllipse(d->horizontalHandle);
+                gc.drawPixmap(d->horizontalHandle.adjusted(5, 5, -5, -5).toRect(), d->horizontalIcon);
+            }
 
         } else {
             d->horizontalHandle = QRectF();
         }
     }
 
-    if(d->mirrorVertical) {
+    if(d->mirrorVertical && !d->hideVerticalDecoration) {
         if (!d->verticalAxis.isNull()) {
 
             gc.setPen(QPen(QColor(0, 0, 0, 64), 2, Qt::DashDotDotLine, Qt::RoundCap, Qt::RoundJoin));
@@ -196,11 +214,13 @@ void KisMirrorAxis::drawDecoration(QPainter& gc, const QRectF& updateArea, const
             QPointF verticalHandleCenter = d->verticalAxis.unitVector().pointAt(verticalHandlePosition);
             d->verticalHandle = QRectF(verticalHandleCenter.x() - halfHandleSize, verticalHandleCenter.y() - halfHandleSize, d->handleSize, d->handleSize);
 
-           // gc.drawEllipse(verticalIndicator);
-          //  gc.drawPixmap(verticalIndicator.adjusted(5, 5, -5, -5).toRect(), d->verticalIcon);
-            gc.setPen(QPen(QColor(0, 0, 0, 128), 2));
-            gc.drawEllipse(d->verticalHandle);
-            gc.drawPixmap(d->verticalHandle.adjusted(5, 5, -5, -5).toRect(), d->verticalIcon);
+            // don't draw the handles if we are locking the axis for movement
+            if (!d->lockVertical) {
+                gc.setPen(QPen(QColor(0, 0, 0, 128), 2));
+                gc.drawEllipse(d->verticalHandle);
+                gc.drawPixmap(d->verticalHandle.adjusted(5, 5, -5, -5).toRect(), d->verticalIcon);
+            }
+
         } else {
             d->verticalHandle = QRectF();
         }
@@ -212,6 +232,8 @@ void KisMirrorAxis::drawDecoration(QPainter& gc, const QRectF& updateArea, const
         gc.endNativePainting();
     }
 
+    gc.restore();
+
 }
 
 bool KisMirrorAxis::eventFilter(QObject* target, QEvent* event)
@@ -228,14 +250,17 @@ bool KisMirrorAxis::eventFilter(QObject* target, QEvent* event)
         QTabletEvent *te = dynamic_cast<QTabletEvent*>(event);
         QPoint pos = me ? me->pos() : (te ? te->pos() : QPoint(77,77));
 
-        if(d->mirrorHorizontal && d->horizontalHandle.contains(pos)) {
+
+
+
+        if(d->mirrorHorizontal && d->horizontalHandle.contains(pos) && !d->lockHorizontal && !d->hideHorizontalDecoration ) {
             d->xActive = true;
             QApplication::setOverrideCursor(Qt::ClosedHandCursor);
             event->accept();
             return true;
         }
 
-        if(d->mirrorVertical && d->verticalHandle.contains(pos)) {
+        if(d->mirrorVertical && d->verticalHandle.contains(pos) && !d->lockVertical && !d->hideVerticalDecoration) {
             d->yActive = true;
             QApplication::setOverrideCursor(Qt::ClosedHandCursor);
             event->accept();
@@ -266,8 +291,8 @@ bool KisMirrorAxis::eventFilter(QObject* target, QEvent* event)
             event->accept();
             return true;
         }
-        if(d->mirrorHorizontal) {
-            if(d->horizontalHandle.contains(pos)) {
+        if(d->mirrorHorizontal && !d->hideHorizontalDecoration) {
+            if(d->horizontalHandle.contains(pos) && !d->lockHorizontal) {
                 if(!d->horizontalContainsCursor) {
                     QApplication::setOverrideCursor(Qt::OpenHandCursor);
                     d->horizontalContainsCursor = true;
@@ -277,8 +302,8 @@ bool KisMirrorAxis::eventFilter(QObject* target, QEvent* event)
                 d->horizontalContainsCursor = false;
             }
         }
-        if(d->mirrorVertical) {
-            if(d->verticalHandle.contains(pos)) {
+        if(d->mirrorVertical && !d->hideVerticalDecoration) {
+            if(d->verticalHandle.contains(pos) && !d->lockVertical) {
                 if(!d->verticalContainsCursor) {
                     QApplication::setOverrideCursor(Qt::OpenHandCursor);
                     d->verticalContainsCursor = true;
@@ -312,7 +337,15 @@ void KisMirrorAxis::mirrorModeChanged()
 {
     d->mirrorHorizontal = d->resourceProvider->mirrorHorizontal();
     d->mirrorVertical = d->resourceProvider->mirrorVertical();
+
+    d->lockHorizontal = d->resourceProvider->mirrorHorizontalLock();
+    d->lockVertical = d->resourceProvider->mirrorVerticalLock();
+
+    d->hideHorizontalDecoration = d->resourceProvider->mirrorHorizontalHideDecorations();
+    d->hideVerticalDecoration = d->resourceProvider->mirrorVerticalHideDecorations();
+
     setVisible(d->mirrorHorizontal || d->mirrorVertical);
+
 }
 
 void KisMirrorAxis::setVisible(bool v)
@@ -330,20 +363,33 @@ void KisMirrorAxis::setVisible(bool v)
     }
 }
 
+void KisMirrorAxis::moveHorizontalAxisToCenter()
+{
+    d->setAxisPosition(d->image->width()/2, d->axisPosition.y());
+}
+
+void KisMirrorAxis::moveVerticalAxisToCenter()
+{
+    d->setAxisPosition(d->axisPosition.x(), d->image->height()/2 );
+}
+
+
 void KisMirrorAxis::Private::setAxisPosition(float x, float y)
 {
     QPointF newPosition = QPointF(x, y);
 
-    resourceProvider->resourceManager()->setResource(KisCanvasResourceProvider::MirrorAxesCenter, newPosition);
+    const QPointF relativePosition = KisAlgebra2D::absoluteToRelative(newPosition, image->bounds());
+    image->setMirrorAxesCenter(relativePosition);
 
     q->view()->canvasBase()->updateCanvas();
 }
 
+
 void KisMirrorAxis::Private::recomputeVisibleAxes(QRect viewport)
 {
     KisCoordinatesConverter *converter = q->view()->viewConverter();
 
-    axisPosition = resourceProvider->resourceManager()->resource(KisCanvasResourceProvider::MirrorAxesCenter).toPointF();
+    axisPosition = KisAlgebra2D::relativeToAbsolute(image->mirrorAxesCenter(), image->bounds());
 
     QPointF samplePt1 = converter->imageToWidget<QPointF>(axisPosition);
     QPointF samplePt2 = converter->imageToWidget<QPointF>(QPointF(axisPosition.x(), axisPosition.y() - 100));
