@@ -20,11 +20,14 @@
 #define KIS_INPUTMANAGER_H
 
 #include <QObject>
+#include <QPointer>
+
 #include <kritaui_export.h>
 
+#include <kis_tool_proxy.h>
+
 class QPointF;
 class QTouchEvent;
-class KisToolProxy;
 class KisCanvas2;
 /**
  * \brief Central object to manage canvas input.
@@ -56,9 +59,7 @@ public:
     /**
      * Destructor.
      */
-    ~KisInputManager();
-
-
+    ~KisInputManager() override;
 
     void addTrackedCanvas(KisCanvas2 *canvas);
     void removeTrackedCanvas(KisCanvas2 *canvas);
@@ -70,7 +71,7 @@ public:
      * Please note that KisInputManager is supposed to handle events
      * for a single receiver only. This is defined by the fact that it
      * resends some of the events back through the Qt's queue to the
-     * reciever. That is why the input manager will assert when it gets
+     * receiver. That is why the input manager will assert when it gets
      * an event with wrong destination.
      */
     void setupAsEventFilter(QObject *receiver);
@@ -78,10 +79,19 @@ public:
     /**
      * Event filter method. Overridden from QObject.
      */
-    bool eventFilter(QObject* object, QEvent* event );
-    bool eventFilterImpl(QEvent * event);
+    bool eventFilter(QObject* object, QEvent* event ) override;
 
+    /**
+     * @brief attachPriorityEventFilter
+     * @param filter
+     * @param priority
+     */
     void attachPriorityEventFilter(QObject *filter, int priority = 0);
+
+    /**
+     * @brief detachPriorityEventFilter
+     * @param filter
+     */
     void detachPriorityEventFilter(QObject *filter);
 
     /**
@@ -92,30 +102,22 @@ public:
     /**
      * The tool proxy of the current application.
      */
-    KisToolProxy *toolProxy() const;
-
-    /**
-     * Touch events are special, too.
-     *
-     * \return a touch event if there was one, otherwise 0
-     */
-    QTouchEvent *lastTouchEvent() const;
-
-    /**
-     * Convert a widget position to a document position.
-     */
-    QPointF widgetToDocument(const QPointF &position);
+    QPointer<KisToolProxy> toolProxy() const;
 
 public Q_SLOTS:
     void stopIgnoringEvents();
-    void slotFocusOnEnter(bool value);
 
 private Q_SLOTS:
+    void slotAboutToChangeTool();
     void slotToolChanged();
     void profileChanged();
     void slotCompressedMoveEvent();
 
 private:
+    bool startTouch(bool &retval);
+    void endTouch();
+
+    bool eventFilterImpl(QEvent * event);
     template <class Event>
         bool compressMoveEventCommon(Event *event);
 
