@@ -32,14 +32,15 @@
 #include <QGuiApplication>
 #include <QDesktopWidget>
 
-#include <qpa/qwindowsysteminterface.h>
-#include <qpa/qplatformscreen.h>
-#include <private/qguiapplication_p.h>
+// #include <qpa/qwindowsysteminterface.h>
+// #include <qpa/qplatformscreen.h>
+// #include <private/qguiapplication_p.h>
 
+#include <QPointer>
 #include <QScreen>
 #include <QWidget>
 #include <QLibrary>
-#include <math.h>
+#include <cmath>
 #define Q_PI M_PI
 
 #include <input/kis_extended_modifiers_mapper.h>
@@ -77,6 +78,7 @@ enum {
 QWindowsTabletSupport *QTAB = 0;
 static QPointer<QWidget> targetWindow = 0; //< Window receiving last tablet event
 static QPointer<QWidget> qt_tablet_target = 0; //< Widget receiving last tablet event
+static bool dialogOpen = false;  //< KisTabletSupportWin is not a Q_OBJECT and can't accept dialog signals
 
 HWND createDummyWindow(const QString &className, const wchar_t *windowName, WNDPROC wndProc)
 {
@@ -160,11 +162,11 @@ static QPoint mousePosition()
 
 QWindowsWinTab32DLL QWindowsTabletSupport::m_winTab32DLL;
 
-void KisTabletSupportWin::init()
+bool KisTabletSupportWin::init()
 {
     if (!QWindowsTabletSupport::m_winTab32DLL.init()) {
         qWarning() << "Failed to initialize Wintab";
-        return;
+        return false;
     }
 
     QTAB = QWindowsTabletSupport::create();
@@ -175,6 +177,7 @@ void KisTabletSupportWin::init()
                          delete QTAB;
                          QTAB = QWindowsTabletSupport::create();
                      });
+    return true;
 }
 
 
@@ -218,7 +221,7 @@ static void handleTabletEvent(QWidget *windowWidget, const QPointF &local, const
 
 
     if ((type == QEvent::TabletRelease || buttons == Qt::NoButton) && (qt_tablet_target != 0)) {
-        dbgTablet << "releasing tablet target" << qt_tablet_target;
+        dbgInput << "releasing tablet target" << qt_tablet_target;
         qt_tablet_target = 0;
     }
 
@@ -294,9 +297,35 @@ struct DefaultButtonsConverter
                  * Qt::NoButton here.
                  */
                 if (convertedButton == Qt::NoButton) {
-                    *button = Qt::NoButton;
-                    *buttons = Qt::NoButton;
-                    break;
+
+                    /**
+                     * Sometimes the driver-handled sortcuts are just
+                     * keyboard modifiers, so ideally we should handle
+                     * them as well. The problem is that we cannot
+                     * know if the shortcut was a pan/zoom action or a
+                     * shortcut. So here we use a "hackish" approash.
+                     * We just check if any modifier has been pressed
+                     * and, if so, pass the button to Krita. Of
+                     * course, if the driver uses some really complex
+                     * shortcuts like "Shift + stylus btn" to generate
+                     * some recorded shortcut, it will not work. But I
+                     * guess it will be ok for th emost of the
+                     * usecases.
+                     *
+                     * WARNING: this hack will *not* work if you bind
+                     *          any non-modifier key to the stylus
+                     *          button, e.g. Space.
+                     */
+
+                    const Qt::KeyboardModifiers keyboardModifiers = QApplication::queryKeyboardModifiers();
+
+                    if (KisTabletDebugger::instance()->shouldEatDriverShortcuts() ||
+                        keyboardModifiers == Qt::NoModifier) {
+
+                        *button = Qt::NoButton;
+                        *buttons = Qt::NoButton;
+                        break;
+                    }
                 }
             }
         }
@@ -387,8 +416,10 @@ bool QWindowsWinTab32DLL::init()
     if (wTInfo)
         return true;
     QLibrary library(QStringLiteral("wintab32"));
-    if (!library.load())
+    if (!library.load()) {
+        qWarning() << QString("Could not load wintab32 dll: %1").arg(library.errorString());
         return false;
+    }
     wTOpen         = (PtrWTOpen)         library.resolve("WTOpenW");
     wTClose        = (PtrWTClose)        library.resolve("WTClose");
     wTInfo         = (PtrWTInfo)         library.resolve("WTInfoW");
@@ -398,7 +429,20 @@ bool QWindowsWinTab32DLL::init()
     wTGet          = (PtrWTGet)          library.resolve("WTGetW");
     wTQueueSizeGet = (PtrWTQueueSizeGet) library.resolve("WTQueueSizeGet");
     wTQueueSizeSet = (PtrWTQueueSizeSet) library.resolve("WTQueueSizeSet");
-    return wTOpen && wTClose && wTInfo && wTEnable && wTOverlap && wTPacketsGet && wTQueueSizeGet && wTQueueSizeSet;
+
+    if (wTOpen && wTClose && wTInfo && wTEnable && wTOverlap && wTPacketsGet && wTQueueSizeGet && wTQueueSizeSet) {
+        return true;
+    }
+    qWarning() << "Could not resolve the following symbols:\n"
+               << "\t wTOpen" << wTOpen << "\n"
+               << "\t wtClose" << wTClose << "\n"
+               << "\t wtInfo" << wTInfo << "\n"
+               << "\t wTEnable" << wTEnable << "\n"
+               << "\t wTOverlap" << wTOverlap << "\n"
+               << "\t wTPacketsGet" << wTPacketsGet << "\n"
+               << "\t wTQueueSizeGet" << wTQueueSizeGet << "\n"
+               << "\t wTQueueSizeSet" << wTQueueSizeSet << "\n";
+    return false;
 }
 
 
@@ -494,7 +538,7 @@ QString QWindowsTabletSupport::description() const
     const unsigned size = m_winTab32DLL.wTInfo(WTI_INTERFACE, IFC_WINTABID, 0);
     if (!size)
         return QString();
-    QScopedPointer<TCHAR> winTabId(new TCHAR[size + 1]);
+    QVarLengthArray<TCHAR> winTabId(size + 1);
     m_winTab32DLL.wTInfo(WTI_INTERFACE, IFC_WINTABID, winTabId.data());
     WORD implementationVersion = 0;
     m_winTab32DLL.wTInfo(WTI_INTERFACE, IFC_IMPLVERSION, &implementationVersion);
@@ -603,18 +647,52 @@ QWindowsTabletDeviceData QWindowsTabletSupport::tabletInit(const quint64 uniqueI
     result.minTanPressure = int(axis.axMin);
     result.maxTanPressure = int(axis.axMax);
 
-    LOGCONTEXT defaultLc;
-    /* get default region */
-    QWindowsTabletSupport::m_winTab32DLL.wTInfo(WTI_DEFCONTEXT, 0, &defaultLc);
-    result.maxX = int(defaultLc.lcInExtX) - int(defaultLc.lcInOrgX);
-    result.maxY = int(defaultLc.lcInExtY) - int(defaultLc.lcInOrgY);
-    result.maxZ = int(defaultLc.lcInExtZ) - int(defaultLc.lcInOrgZ);
+    result.minX = int(lc.lcOutOrgX);
+    result.maxX = int(lc.lcOutExtX) + int(lc.lcOutOrgX);
+    result.minY = int(lc.lcOutOrgY);
+    result.maxY = -int(lc.lcOutExtY) + int(lc.lcOutOrgY);
+    // These are set to 0 when we opened the tablet context in QWindowsTabletSupport::create
+    KIS_SAFE_ASSERT_RECOVER_NOOP(lc.lcOutOrgX == 0);
+    KIS_SAFE_ASSERT_RECOVER_NOOP(lc.lcOutOrgY == 0);
+
+    result.maxZ = int(lc.lcOutExtZ) - int(lc.lcOutOrgZ);
     result.currentDevice = deviceType(cursorType);
 
+    // Define a rectangle representing the whole screen as seen by Wintab.
+    QRect qtDesktopRect = QApplication::desktop()->geometry();
+    QRect wintabDesktopRect(lc.lcSysOrgX, lc.lcSysOrgY,
+                            lc.lcSysExtX, lc.lcSysExtY);
+    qDebug() << ppVar(qtDesktopRect);
+    qDebug() << ppVar(wintabDesktopRect);
+
+    // Show screen choice dialog
+    if (!dialogOpen) {
+        KisScreenSizeChoiceDialog dlg(0,
+                                      wintabDesktopRect,
+                                      qtDesktopRect);
+
+        KisExtendedModifiersMapper mapper;
+        KisExtendedModifiersMapper::ExtendedModifiers modifiers =
+            mapper.queryExtendedModifiers();
+
+        if (modifiers.contains(Qt::Key_Shift) ||
+            (!dlg.canUseDefaultSettings() &&
+             qtDesktopRect != wintabDesktopRect)) {
+
+            dialogOpen = true;
+            dlg.exec();
+        }
+
+        result.virtualDesktopArea = dlg.screenRect();
+        dialogOpen = false;
+    } else {
+        // This branch should've been explicitly prevented.
+        KIS_SAFE_ASSERT_RECOVER_NOOP(!dialogOpen);
+        warnTablet << "Trying to init a WinTab device while screen resolution dialog is active, this should not happen!";
+        warnTablet << "Tablet coordinates could be wrong as a result.";
+        result.virtualDesktopArea = qtDesktopRect;
+    }
 
-    // These define a rectangle representing the whole screen as seen by Wintab.
-    result.virtualDesktopArea = QRect(defaultLc.lcSysOrgX, defaultLc.lcSysOrgY,
-                                      defaultLc.lcSysExtX, defaultLc.lcSysExtY);
     return result;
 };
 
@@ -623,7 +701,12 @@ QWindowsTabletDeviceData QWindowsTabletSupport::tabletInit(const quint64 uniqueI
 
 bool QWindowsTabletSupport::translateTabletProximityEvent(WPARAM /* wParam */, LPARAM lParam)
 {
-
+    if (dialogOpen) {
+        // tabletInit(...) may show the screen resolution dialog and is blocking.
+        // During this period, don't process any tablet events at all.
+        dbgTablet << "WinTab screen resolution dialog is active, ignoring WinTab proximity event";
+        return false;
+    }
     auto sendProximityEvent = [&](QEvent::Type type) {
         QPointF emptyPos;
         qreal zero = 0.0;
@@ -659,7 +742,7 @@ bool QWindowsTabletSupport::translateTabletPacketEvent()
 {
     static PACKET localPacketBuf[TabletPacketQSize];  // our own tablet packet queue.
     const int packetCount = QWindowsTabletSupport::m_winTab32DLL.wTPacketsGet(m_context, TabletPacketQSize, &localPacketBuf);
-    if (!packetCount || m_currentDevice < 0)
+    if (!packetCount || m_currentDevice < 0 || dialogOpen)
         return false;
 
     // In contrast to Qt, these will not be "const" during our loop.
@@ -718,10 +801,13 @@ bool QWindowsTabletSupport::translateTabletPacketEvent()
 
         const int z = currentDevice == QTabletEvent::FourDMouse ? int(packet.pkZ) : 0;
 
-        // This code is to delay the tablet data one cycle to sync with the mouse location.
-        QPointF globalPosF = m_oldGlobalPosF / dpr; // Convert from "native" to "device independent pixels."
-        m_oldGlobalPosF = tabletData.scaleCoordinates(packet.pkX, packet.pkY,
-                                                      tabletData.virtualDesktopArea);
+        // NOTE: we shouldn't postpone the tablet events like Qt does, because we
+        //       don't support mouse mode (which was the reason for introducing this
+        //       postponing). See bug 363284.
+        QPointF globalPosF =
+            tabletData.scaleCoordinates(packet.pkX, packet.pkY,
+                                        tabletData.virtualDesktopArea);
+        globalPosF /= dpr; // Convert from "native" to "device independent pixels."
 
         QPoint globalPos = globalPosF.toPoint();
 
@@ -853,9 +939,42 @@ void QWindowsTabletSupport::tabletUpdateCursor(const int pkCursor)
     // Check tablet name to enable Surface Pro 3 workaround.
 #ifdef UNICODE
     if (!isSurfacePro3) {
-        UINT nameLength = QWindowsTabletSupport::m_winTab32DLL.wTInfo(WTI_DEVICES, DVC_NAME, 0);
+        /**
+         * Some really "nice" tablet drivers don't know that trhey are
+         * supposed to return their name length when the buffer is
+         * null and they try to write into it effectively causing a
+         * suicide. So we cannot rely on it :(
+         *
+         * We workaround it by just allocating a big array and hoping
+         * for the best.
+         *
+         * Failing tablets:
+         *   - Adesso Cybertablet M14
+         *   - Peritab-302
+         *   - Trust Tablet TB7300
+         *   - VisTablet Realm Pro
+         *   - Aiptek 14000u (latest driver: v5.03, 2013-10-21)
+         *   - Genius G-Pen F350
+         *   - Genius G-Pen 560 (supported on win7 only)
+         */
+
+        // we cannot use the correct api :(
+        // UINT nameLength = QWindowsTabletSupport::m_winTab32DLL.wTInfo(WTI_DEVICES, DVC_NAME, 0);
+
+        // 1024 chars should be enough for everyone! (c)
+        UINT nameLength = 1024;
+
         TCHAR* dvcName = new TCHAR[nameLength + 1];
-        QWindowsTabletSupport::m_winTab32DLL.wTInfo(WTI_DEVICES, DVC_NAME, dvcName);
+        memset(dvcName, 0, sizeof(TCHAR) * nameLength);
+        UINT writtenBytes = QWindowsTabletSupport::m_winTab32DLL.wTInfo(WTI_DEVICES, DVC_NAME, dvcName);
+
+        if (writtenBytes > sizeof(TCHAR) * nameLength) {
+            qWarning() << "WINTAB WARNING: tablet name is too long!" << writtenBytes;
+
+            // avoid crash when trying to read it
+            dvcName[nameLength - 1] = (TCHAR)0;
+        }
+
         QString qDvcName = QString::fromWCharArray((const wchar_t*)dvcName);
         dbgInput << "DVC_NAME =" << qDvcName;
         // Name changed between older and newer Surface Pro 3 drivers
