@@ -1,3 +1,4 @@
+
 /****************************************************************************
 **
 ** Copyright (C) 2015 The Qt Company Ltd.
@@ -52,9 +53,9 @@ QXcbConnection::QXcbConnection(bool canGrabServer, const char *displayName)
     : m_connection(0)
     , m_canGrabServer(canGrabServer)
     , m_displayName(displayName ? QByteArray(displayName) : qgetenv("DISPLAY"))
-#ifdef XCB_USE_XLIB
+    #ifdef XCB_USE_XLIB
     , m_xlib_display(0)
-#endif
+    #endif
 {
     m_connection = QX11Info::connection();
     m_xlib_display = QX11Info::display();
@@ -345,20 +346,38 @@ void QXcbConnection::notifyEnterEvent(xcb_enter_notify_event_t *event)
     while ((window = m_windowMapper.key(0,0)) != 0) {
         m_windowMapper.remove(window);
     }
-    window = event->event;
 
-    if (!m_windowMapper.contains(window)) {
-        QWidget *widget = QWidget::find(window);
+    addWindowFromXi2Id(event->event);
+}
+
+void QXcbConnection::addWindowFromXi2Id(xcb_window_t id)
+{
+    if (!m_windowMapper.contains(id)) {
+        QWidget *widget = QWidget::find(id);
         if (widget) {
             QWindow *windowHandle = widget->windowHandle();
-            m_windowMapper.insert(window, windowHandle);
+            m_windowMapper.insert(id, windowHandle);
+
+            // we should also cold-init the window events mask
+            // (Qt's own initialization got broken in Qt 5.10)
+            xi2Select(id);
         }
     }
 }
 
 QWindow* QXcbConnection::windowFromId(xcb_window_t id)
 {
-    return m_windowMapper.value(id, 0);
+    QWindow *window = m_windowMapper.value(id, 0);
+
+    // Try to fetch the window Id lazily. It is needed when the cursor gets under
+    // a popup window or a popup dialog, which doesn't produce any enter event on
+    // some systems
+    if (!window) {
+        addWindowFromXi2Id(id);
+        window = m_windowMapper.value(id, 0);
+    }
+
+    return window;
 }
 
 static int xi2ValuatorOffset(unsigned char *maskPtr, int maskLen, int number)
@@ -530,8 +549,13 @@ public:
 
     class WheelEvent : public InputEvent {
     public:
+#if QT_VERSION >= 0x050700
+        WheelEvent(QWindow *w, ulong time, const QPointF & local, const QPointF & global, QPoint pixelD, QPoint angleD, int qt4D, Qt::Orientation qt4O,
+                   Qt::KeyboardModifiers mods, Qt::ScrollPhase phase = Qt::NoScrollPhase, Qt::MouseEventSource src = Qt::MouseEventNotSynthesized)
+#else
         WheelEvent(QWindow *w, ulong time, const QPointF & local, const QPointF & global, QPoint pixelD, QPoint angleD, int qt4D, Qt::Orientation qt4O,
                    Qt::KeyboardModifiers mods, Qt::ScrollPhase phase = Qt::ScrollUpdate, Qt::MouseEventSource src = Qt::MouseEventNotSynthesized)
+#endif
             : InputEvent(w, time, Wheel, mods), pixelDelta(pixelD), angleDelta(angleD), qt4Delta(qt4D), qt4Orientation(qt4O), localPos(local), globalPos(global), phase(phase), source(src) { }
         QPoint pixelDelta;
         QPoint angleDelta;
@@ -556,18 +580,18 @@ struct EventTimerStaticInitializer
 EventTimerStaticInitializer __timerStaticInitializer;
 
 Qt::MouseButtons tabletState = Qt::NoButton;
-QWidget *tabletPressWidget = 0;
+QPointer<QWidget> tabletPressWidget = 0;
 
 void QWindowSystemInterface::handleTabletEvent(QWindow *w, const QPointF &local, const QPointF &global,
                                                int device, int pointerType, Qt::MouseButtons buttons, qreal pressure, int xTilt, int yTilt,
                                                qreal tangentialPressure, qreal rotation, int z, qint64 uid,
                                                Qt::KeyboardModifiers modifiers)
 {
-    qint64 timestamp = g_eventTimer.elapsed();
+    qint64 timestamp = g_eventTimer.msecsSinceReference() + g_eventTimer.elapsed();
 
     QWindowSystemInterfacePrivate::TabletEvent *e =
-        new QWindowSystemInterfacePrivate::TabletEvent(w, timestamp, local, global, device, pointerType, buttons, pressure,
-                                                       xTilt, yTilt, tangentialPressure, rotation, z, uid, modifiers);
+            new QWindowSystemInterfacePrivate::TabletEvent(w, timestamp, local, global, device, pointerType, buttons, pressure,
+                                                           xTilt, yTilt, tangentialPressure, rotation, z, uid, modifiers);
 
     processTabletEvent(e);
 }
@@ -581,6 +605,12 @@ void processTabletEvent(QWindowSystemInterfacePrivate::TabletEvent *e)
 
     bool localValid = true;
 
+    // It can happen that we got no tablet release event. Just catch it here
+    // and clean up the state.
+    if (type == QEvent::TabletMove && e->buttons == Qt::NoButton) {
+        tabletPressWidget = 0;
+    }
+
     QWidget *targetWidget = 0;
 
     if (tabletPressWidget) {
@@ -657,7 +687,7 @@ void processTabletEvent(QWindowSystemInterfacePrivate::TabletEvent *e)
 
 void QWindowSystemInterface::handleTabletEnterProximityEvent(int device, int pointerType, qint64 uid)
 {
-    qint64 timestamp = g_eventTimer.elapsed();
+    qint64 timestamp = g_eventTimer.msecsSinceReference() + g_eventTimer.elapsed();
 
     QTabletEvent ev(QEvent::TabletEnterProximity, QPointF(), QPointF(),
                     device, pointerType, 0, 0, 0,
@@ -669,7 +699,7 @@ void QWindowSystemInterface::handleTabletEnterProximityEvent(int device, int poi
 
 void QWindowSystemInterface::handleTabletLeaveProximityEvent(int device, int pointerType, qint64 uid)
 {
-    qint64 timestamp = g_eventTimer.elapsed();
+    qint64 timestamp = g_eventTimer.msecsSinceReference() + g_eventTimer.elapsed();
 
     QTabletEvent ev(QEvent::TabletLeaveProximity, QPointF(), QPointF(),
                     device, pointerType, 0, 0, 0,
@@ -748,7 +778,8 @@ void processWheelEvent(QWindowSystemInterfacePrivate::WheelEvent *e)
 
     //if (window->d_func()->blockedByModalWindow) {
     if (QGuiApplication::modalWindow() &&
-        QGuiApplication::modalWindow() != window) {
+            QGuiApplication::modalWindow() != window &&
+            QGuiApplication::modalWindow() != window->transientParent()) {
 
         // a modal window is blocking this window, don't allow wheel events through
         return;
