@@ -50,8 +50,8 @@
 
 struct XInput2TouchDeviceData {
     XInput2TouchDeviceData()
-    : xiDeviceInfo(0)
-    , qtTouchDevice(0)
+        : xiDeviceInfo(0)
+        , qtTouchDevice(0)
     {
     }
     XIDeviceInfo *xiDeviceInfo;
@@ -163,7 +163,7 @@ void QXcbConnection::xi2SetupDevices()
                     // Some drivers have no labels on the wheel buttons, some have no label on just one and some have no label on
                     // button 4 and the wrong one on button 5. So we just check that they are not labelled with unrelated buttons.
                     if ((!label4 || qatom(label4) == QXcbAtom::ButtonWheelUp || qatom(label4) == QXcbAtom::ButtonWheelDown) &&
-                        (!label5 || qatom(label5) == QXcbAtom::ButtonWheelUp || qatom(label5) == QXcbAtom::ButtonWheelDown))
+                            (!label5 || qatom(label5) == QXcbAtom::ButtonWheelUp || qatom(label5) == QXcbAtom::ButtonWheelDown))
                         scrollingDevice.legacyOrientations |= Qt::Vertical;
                 }
                 if (bci->num_buttons >= 7) {
@@ -220,6 +220,20 @@ void QXcbConnection::xi2SetupDevices()
             isTablet = true;
             tabletData.pointerType = QTabletEvent::Pen;
             dbgType = QLatin1String("pen");
+        } else if (name.contains("waltop") && name.contains("tablet")) {
+            // other "Genius" tablets
+            // WALTOP International Corp. Slim Tablet
+            isTablet = true;
+            tabletData.pointerType = QTabletEvent::Pen;
+            dbgType = QLatin1String("pen");
+        } else if (name.contains("uc-logic")) {
+            isTablet = true;
+            tabletData.pointerType = QTabletEvent::Pen;
+            dbgType = QLatin1String("pen");
+        } else if (name.contains("ugee")) {
+                isTablet = true;
+                tabletData.pointerType = QTabletEvent::Pen;
+                dbgType = QLatin1String("pen");
         } else {
             isTablet = false;
         }
@@ -434,9 +448,9 @@ XInput2TouchDeviceData *QXcbConnection::touchDeviceForId(int id)
                     dev->size.width() > 10000 || dev->size.height() > 10000)
                 dev->size = QSizeF(130, 110);
         }
-// WARNING: Krita edit
-//        if (!isAtLeastXI22() || type == QTouchDevice::TouchPad)
-//            caps |= QTouchDevice::MouseEmulation;
+        // WARNING: Krita edit
+        //        if (!isAtLeastXI22() || type == QTouchDevice::TouchPad)
+        //            caps |= QTouchDevice::MouseEmulation;
 
         if (type >= QTouchDevice::TouchScreen && type <= QTouchDevice::TouchPad) {
             dev->qtTouchDevice = new QTouchDevice;
@@ -659,7 +673,7 @@ Qt::MouseButton QXcbConnection::xiToQtMouseButton(uint32_t b)
     case 1: return Qt::LeftButton;
     case 2: return Qt::MiddleButton;
     case 3: return Qt::RightButton;
-    // 4-7 are for scrolling
+        // 4-7 are for scrolling
     default: break;
     }
     if (b >= 8 && b <= Qt::MaxMouseButton)
@@ -667,6 +681,36 @@ Qt::MouseButton QXcbConnection::xiToQtMouseButton(uint32_t b)
     return Qt::NoButton;
 }
 
+Qt::MouseButtons QXcbConnection::xiToQtMouseButtons(xXIDeviceEvent *xiDeviceEvent)
+{
+    /**
+     * WARNING: we haven't tested this method on different tablets. For basic tablets
+     *          without any button remapping it seems to work. For more complex configurations
+     *          I just don't know. Right now it is only safe to check if buttons is
+     *          equal to Qt::NoButton!
+     */
+
+
+    int len = xiDeviceEvent->buttons_len;
+    const uint32_t *buttons = reinterpret_cast<const quint32*>(&xiDeviceEvent[1]);
+
+    Qt::MouseButtons qtbuttons = Qt::NoButton;
+
+    const int numBits = len * 32;
+
+    for (int i = 1; i < numBits; i++) {
+        const int index = (i) / 32;
+        const int offset = (i) % 32;
+
+        const bool isActive = buttons[index] & (1 << offset);
+        if (isActive) {
+            qtbuttons |= xiToQtMouseButton(i);
+        }
+    }
+
+    return qtbuttons;
+}
+
 static QTabletEvent::TabletDevice toolIdToTabletDevice(quint32 toolId) {
     // keep in sync with wacom_intuos_inout() in Linux kernel driver wacom_wac.c
     switch (toolId) {
@@ -708,6 +752,39 @@ bool QXcbConnection::xi2HandleTabletEvent(void *event, TabletData *tabletData, Q
     xXIGenericDeviceEvent *xiEvent = static_cast<xXIGenericDeviceEvent *>(event);
     xXIDeviceEvent *xiDeviceEvent = reinterpret_cast<xXIDeviceEvent *>(xiEvent);
 
+
+    /**
+     * On some systems we can lose tablet button release event if the tablet
+     * was "closing a popup window by clicking somewhere outside the app
+     * window". It means that we get a tablet press event, but get absolutely
+     * no tablet release event. That can cause quite a lot of troubles, so here we
+     * check if reported button state is consistent and if not, just reset it.
+     *
+     * WARNING: We haven't tested xiToQtMouseButtons() functions on all the
+     *          tablet models and configurations, so at the moment we rely only
+     *          on its Qt::NoButton state. If people test it on custom tablet
+     *          button configurations, we can just stop tracking
+     *          tabletData->buttons and use this mapping instead.
+     */
+
+    if (xiEvent->evtype == XI_Motion ||
+            xiEvent->evtype == XI_ButtonPress ||
+            xiEvent->evtype == XI_ButtonRelease) {
+
+        Qt::MouseButtons expectedButtons = xiToQtMouseButtons(xiDeviceEvent);
+
+        if (expectedButtons != tabletData->buttons) {
+            if (expectedButtons == Qt::NoButton) {
+                tabletData->buttons = expectedButtons;
+            } else {
+                qWarning() << "===";
+                qWarning() << "WARNING: Tracked tablet buttons are not consistent!";
+                qWarning() << "        "  << ppVar(tabletData->buttons);
+                qWarning() << "        "  << ppVar(expectedButtons);
+            }
+        }
+    }
+
     switch (xiEvent->evtype) {
     case XI_ButtonPress: {
         Qt::MouseButton b = xiToQtMouseButton(xiDeviceEvent->detail);
@@ -776,8 +853,8 @@ bool QXcbConnection::xi2HandleTabletEvent(void *event, TabletData *tabletData, Q
                         // TODO maybe have a hash of tabletData->deviceId to device data so we can
                         // look up the tablet name here, and distinguish multiple tablets
                         qCDebug(lcQpaXInput, "XI2 proximity change on tablet %d (USB %x): last tool: %x id %x current tool: %x id %x TabletDevice %d",
-                            tabletData->deviceId, ptr[_WACSER_USB_ID], ptr[_WACSER_LAST_TOOL_SERIAL], ptr[_WACSER_LAST_TOOL_ID],
-                            ptr[_WACSER_TOOL_SERIAL], ptr[_WACSER_TOOL_ID], tabletData->tool);
+                                tabletData->deviceId, ptr[_WACSER_USB_ID], ptr[_WACSER_LAST_TOOL_SERIAL], ptr[_WACSER_LAST_TOOL_ID],
+                                ptr[_WACSER_TOOL_SERIAL], ptr[_WACSER_TOOL_ID], tabletData->tool);
                     }
                     XFree(data);
                 }
@@ -824,7 +901,7 @@ void QXcbConnection::xi2ReportTabletEvent(TabletData &tabletData, void *event)
     QRect screenArea = qApp->desktop()->rect();
 
     for (QHash<int, TabletData::ValuatorClassInfo>::iterator it = tabletData.valuatorInfo.begin(),
-            ite = tabletData.valuatorInfo.end(); it != ite; ++it) {
+         ite = tabletData.valuatorInfo.end(); it != ite; ++it) {
         int valuator = it.key();
         TabletData::ValuatorClassInfo &classInfo(it.value());
         xi2GetValuatorValueIfSet(event, classInfo.number, &classInfo.curVal);
@@ -872,10 +949,10 @@ void QXcbConnection::xi2ReportTabletEvent(TabletData &tabletData, void *event)
 
     if (Q_UNLIKELY(lcQpaXInput().isDebugEnabled()))
         qCDebug(lcQpaXInput, "XI2 event on tablet %d with tool %d type %d seq %d detail %d pos %6.1f, %6.1f root pos %6.1f, %6.1f buttons 0x%x pressure %4.2lf tilt %d, %d rotation %6.2lf",
-            tabletData.deviceId, tabletData.tool, ev->evtype, ev->sequenceNumber, ev->detail,
-            fixed1616ToReal(ev->event_x), fixed1616ToReal(ev->event_y),
-            fixed1616ToReal(ev->root_x), fixed1616ToReal(ev->root_y),
-            (int)tabletData.buttons, pressure, xTilt, yTilt, rotation);
+                tabletData.deviceId, tabletData.tool, ev->evtype, ev->sequenceNumber, ev->detail,
+                fixed1616ToReal(ev->event_x), fixed1616ToReal(ev->event_y),
+                fixed1616ToReal(ev->root_x), fixed1616ToReal(ev->root_y),
+                (int)tabletData.buttons, pressure, xTilt, yTilt, rotation);
 
     Qt::KeyboardModifiers modifiers = QApplication::queryKeyboardModifiers();
 
