@@ -25,6 +25,7 @@
 #include <QObject>
 #include <QImage>
 #include <QMessageBox>
+#include <QCheckBox>
 #include <QBuffer>
 #include <QGlobalStatic>
 
@@ -41,6 +42,8 @@
 #include <kis_debug.h>
 #include <kis_annotation.h>
 #include <kis_node.h>
+#include <kis_image.h>
+#include <kis_time_range.h>
 
 // local
 #include "kis_config.h"
@@ -74,7 +77,7 @@ KisClipboard* KisClipboard::instance()
     return s_instance;
 }
 
-void KisClipboard::setClip(KisPaintDeviceSP dev, const QPoint& topLeft)
+void KisClipboard::setClip(KisPaintDeviceSP dev, const QPoint& topLeft, const KisTimeRange &range)
 {
     if (!dev)
         return;
@@ -89,7 +92,6 @@ void KisClipboard::setClip(KisPaintDeviceSP dev, const QPoint& topLeft)
     Q_ASSERT(store);
     Q_ASSERT(!store->bad());
     
-
     // Layer data
     if (store->open("layerdata")) {
         if (!dev->write(writer)) {
@@ -101,6 +103,12 @@ void KisClipboard::setClip(KisPaintDeviceSP dev, const QPoint& topLeft)
         store->close();
     }
 
+    // copied frame time limits
+    if (range.isValid() && store->open("timeRange")) {
+        store->write(QString("%1 %2").arg(range.start()).arg(range.end()).toLatin1());
+        store->close();
+    }
+
     // Coordinates
     if (store->open("topLeft")) {
         store->write(QString("%1 %2").arg(topLeft.x()).arg(topLeft.y()).toLatin1());
@@ -161,10 +169,19 @@ void KisClipboard::setClip(KisPaintDeviceSP dev, const QPoint& topLeft)
 
 }
 
-KisPaintDeviceSP KisClipboard::clip(const QRect &imageBounds, bool showPopup)
+void KisClipboard::setClip(KisPaintDeviceSP dev, const QPoint& topLeft)
+{
+    setClip(dev, topLeft, KisTimeRange());
+}
+
+KisPaintDeviceSP KisClipboard::clip(const QRect &imageBounds, bool showPopup, KisTimeRange *clipRange)
 {
     QByteArray mimeType("application/x-krita-selection");
 
+    if (clipRange) {
+        *clipRange = KisTimeRange();
+    }
+
     QClipboard *cb = QApplication::clipboard();
     const QMimeData *cbData = cb->mimeData();
 
@@ -237,6 +254,18 @@ KisPaintDeviceSP KisClipboard::clip(const QRect &imageBounds, bool showPopup)
                     clip->setX(clip->x() + diff.x());
                     clip->setY(clip->y() + diff.y());
                 }
+
+                if (store->hasFile("timeRange") && clipRange) {
+                    store->open("timeRange");
+                    QString str = store->read(store->size());
+                    store->close();
+                    QStringList list = str.split(' ');
+                    if (list.size() == 2) {
+                        KisTimeRange range(list[0].toInt(), list[1].toInt(), true);
+                        *clipRange = range;
+                        qDebug() << "Pasted time range" << range;
+                    }
+                }
             }
         }
 
@@ -250,17 +279,29 @@ KisPaintDeviceSP KisClipboard::clip(const QRect &imageBounds, bool showPopup)
             return KisPaintDeviceSP(0);
 
         KisConfig cfg;
-
         quint32 behaviour = cfg.pasteBehaviour();
+        bool saveColorSetting = false;
+
 
         if (behaviour == PASTE_ASK && showPopup) {
             // Ask user each time.
             QMessageBox mb(qApp->activeWindow());
-            mb.setWindowTitle(i18nc("@title:window", "Pasting data from simple source"));
-            mb.setText(i18n("The image data you are trying to paste has no color profile information.\n\nOn the web and in simple applications the data are supposed to be in sRGB color format.\nImporting as web will show it as it is supposed to look.\nMost monitors are not perfect though so if you made the image yourself\nyou might want to import it as it looked on you monitor.\n\nHow do you want to interpret these data?"));
+            QCheckBox dontPrompt(i18n("Remember"), &mb);
+
+            dontPrompt.blockSignals(true);
+
+
+
+            mb.setWindowTitle(i18nc("@title:window", "Missing Color Profile"));
+            mb.setText(i18n("The image data you are trying to paste has no color profile information. How do you want to interpret these data? \n\n As Web (sRGB) -  Use standard colors that are displayed from computer monitors.  This is the most common way that images are stored. \n\nAs on Monitor - If you know a bit about color management and want to use your monitor to determine the color profile.\n\n"));
+
+            // the order of how you add these buttons matters as it determines the index.
             mb.addButton(i18n("As &Web"), QMessageBox::AcceptRole);
             mb.addButton(i18n("As on &Monitor"), QMessageBox::AcceptRole);
             mb.addButton(i18n("Cancel"), QMessageBox::RejectRole);
+            mb.addButton(&dontPrompt, QMessageBox::ActionRole);
+
+
 
             behaviour = mb.exec();
 
@@ -268,6 +309,9 @@ KisPaintDeviceSP KisClipboard::clip(const QRect &imageBounds, bool showPopup)
                 return 0;
             }
 
+            saveColorSetting = dontPrompt.isChecked(); // should we save this option to the config for next time?
+
+
         }
 
         const KoColorSpace * cs;
@@ -289,6 +333,12 @@ KisPaintDeviceSP KisClipboard::clip(const QRect &imageBounds, bool showPopup)
         QPoint diff = imageBounds.center() - clipBounds.center();
         clip->setX(diff.x());
         clip->setY(diff.y());
+
+        // save the persion's selection to the configuration if the option is checked
+        if (saveColorSetting) {
+            cfg.setPasteBehaviour(behaviour);
+        }
+
     }
 
     return clip;
@@ -385,12 +435,12 @@ QSize KisClipboard::clipSize() const
     return QSize();
 }
 
-void KisClipboard::setLayers(KisNodeList nodes, KisNodeSP imageRoot, bool forceCopy)
+void KisClipboard::setLayers(KisNodeList nodes, KisImageSP image, bool forceCopy)
 {
     /**
      * See a comment in KisMimeData::deepCopyNodes()
      */
-    QMimeData *data = KisMimeData::mimeForLayersDeepCopy(nodes, imageRoot, forceCopy);
+    QMimeData *data = KisMimeData::mimeForLayersDeepCopy(nodes, image, forceCopy);
     if (!data) return;
 
     QClipboard *cb = QApplication::clipboard();
