@@ -33,6 +33,8 @@
 #include "kis_node_insertion_adapter.h"
 #include "kis_dummies_facade_base.h"
 #include "kis_node_dummies_graph.h"
+#include "KisImportExportManager.h"
+#include "KisImageBarrierLockerWithFeedback.h"
 
 #include <KoProperties.h>
 #include <KoStore.h>
@@ -45,27 +47,31 @@
 #include <QBuffer>
 #include <QDomDocument>
 #include <QDomElement>
-#include <QTemporaryFile>
 #include <QDesktopWidget>
 #include <QDir>
 
-KisMimeData::KisMimeData(QList<KisNodeSP> nodes, bool forceCopy)
+KisMimeData::KisMimeData(QList<KisNodeSP> nodes, KisImageSP image, bool forceCopy)
     : QMimeData()
     , m_nodes(nodes)
     , m_forceCopy(forceCopy)
+    , m_image(image)
 {
     Q_ASSERT(m_nodes.size() > 0);
-    m_initialListener = m_nodes.first()->graphListener();
 }
 
 void KisMimeData::deepCopyNodes()
 {
     KisNodeList newNodes;
-    Q_FOREACH (KisNodeSP node, m_nodes) {
-        newNodes << node->clone();
+
+    {
+        KisImageBarrierLockerWithFeedbackAllowNull locker(m_image);
+        Q_FOREACH (KisNodeSP node, m_nodes) {
+            newNodes << node->clone();
+        }
     }
 
     m_nodes = newNodes;
+    m_image = 0;
 }
 
 QList<KisNodeSP> KisMimeData::nodes() const
@@ -86,7 +92,7 @@ QStringList KisMimeData::formats () const
     return f;
 }
 
-KisDocument *createDocument(QList<KisNodeSP> nodes)
+KisDocument *createDocument(QList<KisNodeSP> nodes, KisImageSP srcImage)
 {
     KisDocument *doc = KisPart::instance()->createDocument();
     QRect rc;
@@ -96,8 +102,11 @@ KisDocument *createDocument(QList<KisNodeSP> nodes)
 
     KisImageSP image = new KisImage(0, rc.width(), rc.height(), nodes.first()->colorSpace(), nodes.first()->name());
 
-    Q_FOREACH (KisNodeSP node, nodes) {
-        image->addNode(node->clone());
+    {
+        KisImageBarrierLockerWithFeedbackAllowNull locker(srcImage);
+        Q_FOREACH (KisNodeSP node, nodes) {
+            image->addNode(node->clone());
+        }
     }
 
     doc->setCurrentImage(image);
@@ -105,31 +114,36 @@ KisDocument *createDocument(QList<KisNodeSP> nodes)
     return doc;
 }
 
-QByteArray serializeToByteArray(QList<KisNodeSP> nodes)
+QByteArray serializeToByteArray(QList<KisNodeSP> nodes, KisImageSP srcImage)
 {
-    QByteArray byteArray;
-    QBuffer buffer(&byteArray);
+    QScopedPointer<KisDocument> doc(createDocument(nodes, srcImage));
+    QByteArray result = doc->serializeToNativeByteArray();
 
-    KoStore *store = KoStore::createStore(&buffer, KoStore::Write);
-    Q_ASSERT(!store->bad());
-    
-    KisDocument *doc = createDocument(nodes);
-    doc->saveNativeFormatCalligra(store);
-    delete doc;
+    // avoid a sanity check failure caused by the fact that the image outlives
+    // the document (and it does)
+    doc->setCurrentImage(0);
 
-    return byteArray;
+    return result;
 }
 
 QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type preferredType) const
 {
+    /**
+     * HACK ALERT:
+     *
+     * Sometimes Qt requests the data *after* destruction of Krita,
+     * we cannot load the nodes in that case, because we need signals
+     * and timers. So we just skip serializing.
+     */
+    if (!QApplication::instance()) return QVariant();
+
+
     Q_ASSERT(m_nodes.size() > 0);
 
     if (mimetype == "application/x-qt-image") {
         KisConfig cfg;
 
-        KisDocument *doc = createDocument(m_nodes);
-        doc->image()->refreshGraph();
-        doc->image()->waitForDone();
+        KisDocument *doc = createDocument(m_nodes, m_image);
 
         return doc->image()->projection()->convertToQImage(cfg.displayProfile(QApplication::desktop()->screenNumber(qApp->activeWindow())),
                                                            KoColorConversionTransformation::internalRenderingIntent(),
@@ -138,13 +152,13 @@ QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type prefe
     else if (mimetype == "application/x-krita-node" ||
              mimetype == "application/zip") {
 
-        QByteArray ba = serializeToByteArray(m_nodes);
+        QByteArray ba = serializeToByteArray(m_nodes, m_image);
         return ba;
 
     }
     else if (mimetype == "application/x-krita-node-url") {
 
-        QByteArray ba = serializeToByteArray(m_nodes);
+        QByteArray ba = serializeToByteArray(m_nodes, m_image);
 
         QString temporaryPath =
                 QDir::tempPath() + QDir::separator() +
@@ -167,7 +181,7 @@ QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type prefe
         QDomElement root = doc.createElement("pointer");
         root.setAttribute("application_pid", (qint64)QApplication::applicationPid());
         root.setAttribute("force_copy", m_forceCopy);
-        root.setAttribute("listener_pointer_value", (qint64)m_initialListener);
+        root.setAttribute("image_pointer_value", (qint64)m_image.data());
         doc.appendChild(root);
 
         Q_FOREACH (KisNodeSP node, m_nodes) {
@@ -184,42 +198,36 @@ QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type prefe
     }
 }
 
-void KisMimeData::initializeExternalNode(KisNodeSP &node,
+void KisMimeData::initializeExternalNode(KisNodeSP *node,
                                          KisImageWSP image,
                                          KisShapeController *shapeController)
 {
-    // layers store a link to the image, so update it
-    KisLayer *layer = dynamic_cast<KisLayer*>(node.data());
-    if (layer) {
-        layer->setImage(image);
-    }
-    KisShapeLayer *shapeLayer = dynamic_cast<KisShapeLayer*>(node.data());
+    // adjust the link to a correct image object
+    (*node)->setImage(image);
+
+    KisShapeLayer *shapeLayer = dynamic_cast<KisShapeLayer*>(node->data());
     if (shapeLayer) {
-        KisShapeLayer *shapeLayer2 = new KisShapeLayer(shapeController, image, node->name(), node->opacity());
-        QList<KoShape *> shapes = shapeLayer->shapes();
-        shapeLayer->removeAllShapes();
-        Q_FOREACH (KoShape *shape, shapes) {
-            shapeLayer2->addShape(shape);
-        }
-        node = shapeLayer2;
+        // attach the layer to a new shape controller
+        KisShapeLayer *shapeLayer2 = new KisShapeLayer(*shapeLayer, shapeController);
+        *node = shapeLayer2;
     }
 }
 
 QList<KisNodeSP> KisMimeData::tryLoadInternalNodes(const QMimeData *data,
-                                                   KisImageWSP image,
+                                                   KisImageSP image,
                                                    KisShapeController *shapeController,
                                                    bool /* IN-OUT */ &copyNode)
 {
     QList<KisNodeSP> nodes;
     bool forceCopy = false;
-    KisNodeGraphListener *initialListener = 0;
+    KisImageSP sourceImage;
 
     // Qt 4.7 and Qt 5.5 way
     const KisMimeData *mimedata = qobject_cast<const KisMimeData*>(data);
     if (mimedata) {
         nodes = mimedata->nodes();
         forceCopy = mimedata->m_forceCopy;
-        initialListener = mimedata->m_initialListener;
+        sourceImage = mimedata->m_image;
     }
 
     // Qt 4.8 way
@@ -232,8 +240,8 @@ QList<KisNodeSP> KisMimeData::tryLoadInternalNodes(const QMimeData *data,
         QDomElement element = doc.documentElement();
         qint64 pid = element.attribute("application_pid").toLongLong();
         forceCopy = element.attribute("force_copy").toInt();
-        qint64 listenerPointerValue = element.attribute("listener_pointer_value").toLongLong();
-        initialListener = reinterpret_cast<KisNodeGraphListener*>(listenerPointerValue);
+        qint64 imagePointerValue = element.attribute("image_pointer_value").toLongLong();
+        sourceImage = reinterpret_cast<KisImage*>(imagePointerValue);
 
         if (pid == QApplication::applicationPid()) {
 
@@ -251,14 +259,16 @@ QList<KisNodeSP> KisMimeData::tryLoadInternalNodes(const QMimeData *data,
         }
     }
 
-    if (!nodes.isEmpty() && (forceCopy || copyNode || nodes.first()->graphListener() != image.data())) {
+    if (!nodes.isEmpty() && (forceCopy || copyNode || sourceImage != image)) {
+        KisImageBarrierLockerWithFeedbackAllowNull locker(sourceImage);
+
         QList<KisNodeSP> clones;
         Q_FOREACH (KisNodeSP node, nodes) {
             node = node->clone();
-            if ((forceCopy || copyNode) && initialListener == image.data()) {
+            if ((forceCopy || copyNode) && sourceImage == image) {
                 KisLayerUtils::addCopyOfNameTag(node);
             }
-            initializeExternalNode(node, image, shapeController);
+            initializeExternalNode(&node, image, shapeController);
             clones << node;
         }
         nodes = clones;
@@ -278,40 +288,41 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
     QList<KisNodeSP> nodes;
 
     if (data->hasFormat("application/x-krita-node")) {
-        QByteArray ba = data->data("application/x-krita-node");
-
         KisDocument *tempDoc = KisPart::instance()->createDocument();
-        bool result = tempDoc->loadNativeFormatFromByteArray(ba);
+        QByteArray ba = data->data("application/x-krita-node");
+        QBuffer buf(&ba);
+        KisImportExportFilter *filter = tempDoc->importExportManager()->filterForMimeType(tempDoc->nativeFormatMimeType(), KisImportExportManager::Import);
+        filter->setBatchMode(true);
+        bool result = (filter->convert(tempDoc, &buf) == KisImportExportFilter::OK);
 
         if (result) {
             KisImageWSP tempImage = tempDoc->image();
             Q_FOREACH (KisNodeSP node, tempImage->root()->childNodes(QStringList(), KoProperties())) {
-                nodes << node;
                 tempImage->removeNode(node);
-                initializeExternalNode(node, image, shapeController);
+                initializeExternalNode(&node, image, shapeController);
+                nodes << node;
             }
         }
+        delete filter;
         delete tempDoc;
     }
 
     if (nodes.isEmpty() && data->hasFormat("application/x-krita-node-url")) {
         QByteArray ba = data->data("application/x-krita-node-url");
-        QString localFile = QUrl::fromEncoded(ba).toLocalFile();
-
         KisDocument *tempDoc = KisPart::instance()->createDocument();
-        bool result = tempDoc->loadNativeFormat(localFile);
+        Q_ASSERT(QUrl::fromEncoded(ba).isLocalFile());
+        bool result = tempDoc->openUrl(QUrl::fromEncoded(ba));
 
         if (result) {
             KisImageWSP tempImage = tempDoc->image();
             Q_FOREACH (KisNodeSP node, tempImage->root()->childNodes(QStringList(), KoProperties())) {
-                nodes << node;
                 tempImage->removeNode(node);
-                initializeExternalNode(node, image, shapeController);
+                initializeExternalNode(&node, image, shapeController);
+                nodes << node;
             }
         }
         delete tempDoc;
-
-        QFile::remove(localFile);
+        QFile::remove(QUrl::fromEncoded(ba).toLocalFile());
     }
 
     if (nodes.isEmpty() && data->hasImage()) {
@@ -319,7 +330,10 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
 
         KisPaintDeviceSP device = new KisPaintDevice(KoColorSpaceRegistry::instance()->rgb8());
         device->convertFromQImage(qimage, 0);
-        nodes << new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, device);
+
+		if (image) {
+			nodes << new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, device);
+		}
 
         alwaysRecenter = true;
     }
@@ -341,25 +355,25 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
     return nodes;
 }
 
-QMimeData* KisMimeData::mimeForLayers(const KisNodeList &nodes, KisNodeSP imageRoot, bool forceCopy)
+QMimeData* KisMimeData::mimeForLayers(const KisNodeList &nodes, KisImageSP image, bool forceCopy)
 {
     KisNodeList inputNodes = nodes;
     KisNodeList sortedNodes;
-    KisLayerUtils::sortMergableNodes(imageRoot, inputNodes, sortedNodes);
+    KisLayerUtils::sortMergableNodes(image->root(), inputNodes, sortedNodes);
     if (sortedNodes.isEmpty()) return 0;
 
-    KisMimeData* data = new KisMimeData(sortedNodes, forceCopy);
+    KisMimeData* data = new KisMimeData(sortedNodes, image, forceCopy);
     return data;
 }
 
-QMimeData* KisMimeData::mimeForLayersDeepCopy(const KisNodeList &nodes, KisNodeSP imageRoot, bool forceCopy)
+QMimeData* KisMimeData::mimeForLayersDeepCopy(const KisNodeList &nodes, KisImageSP image, bool forceCopy)
 {
     KisNodeList inputNodes = nodes;
     KisNodeList sortedNodes;
-    KisLayerUtils::sortMergableNodes(imageRoot, inputNodes, sortedNodes);
+    KisLayerUtils::sortMergableNodes(image->root(), inputNodes, sortedNodes);
     if (sortedNodes.isEmpty()) return 0;
 
-    KisMimeData* data = new KisMimeData(sortedNodes, forceCopy);
+    KisMimeData* data = new KisMimeData(sortedNodes, image, forceCopy);
     data->deepCopyNodes();
     return data;
 }
@@ -394,13 +408,11 @@ bool correctNewNodeLocation(KisNodeList nodes,
     return result;
 }
 
-bool KisMimeData::insertMimeLayers(const QMimeData *data,
-                                   KisImageSP image,
-                                   KisShapeController *shapeController,
-                                   KisNodeDummy *parentDummy,
-                                   KisNodeDummy *aboveThisDummy,
-                                   bool copyNode,
-                                   KisNodeInsertionAdapter *nodeInsertionAdapter)
+KisNodeList KisMimeData::loadNodesFast(
+    const QMimeData *data,
+    KisImageSP image,
+    KisShapeController *shapeController,
+    bool &copyNode)
 {
     QList<KisNodeSP> nodes =
         KisMimeData::tryLoadInternalNodes(data,
@@ -421,6 +433,19 @@ bool KisMimeData::insertMimeLayers(const QMimeData *data,
         copyNode = true;
     }
 
+    return nodes;
+}
+
+bool KisMimeData::insertMimeLayers(const QMimeData *data,
+                                   KisImageSP image,
+                                   KisShapeController *shapeController,
+                                   KisNodeDummy *parentDummy,
+                                   KisNodeDummy *aboveThisDummy,
+                                   bool copyNode,
+                                   KisNodeInsertionAdapter *nodeInsertionAdapter)
+{
+    QList<KisNodeSP> nodes = loadNodesFast(data, image, shapeController, copyNode /* IN-OUT */);
+
     if (nodes.isEmpty()) return false;
 
     bool result = true;
