@@ -44,7 +44,7 @@
  *        we store it separately.
  *
  *     2) This objects allows merging (compressing) multiple moves of
- *        a layer into a signle action. This behavior is implemented
+ *        a layer into a single action. This behavior is implemented
  *        in tryMerge() method.
  */
 struct MoveNodeStruct {
@@ -54,7 +54,9 @@ struct MoveNodeStruct {
           newParent(_parent),
           newAbove(_above),
           oldParent(_node->parent()),
-          oldAbove(_node->prevSibling())
+          oldAbove(_node->prevSibling()),
+          suppressNewParentRefresh(false),
+          suppressOldParentRefresh(false)
     {
     }
 
@@ -87,7 +89,7 @@ struct MoveNodeStruct {
     }
 
     void doRedoUpdates() {
-        if (oldParent) {
+        if (oldParent && !suppressOldParentRefresh) {
             image->refreshGraphAsync(oldParent);
         }
 
@@ -97,7 +99,7 @@ struct MoveNodeStruct {
     }
 
     void doUndoUpdates() {
-        if (newParent) {
+        if (newParent && !suppressNewParentRefresh) {
             image->refreshGraphAsync(newParent);
         }
 
@@ -106,6 +108,16 @@ struct MoveNodeStruct {
         }
     }
 
+    void resolveParentCollisions(MoveNodeStruct *rhs) const {
+        if (rhs->newParent == newParent) {
+            rhs->suppressNewParentRefresh = true;
+        }
+
+        if (rhs->oldParent == oldParent) {
+            rhs->suppressOldParentRefresh = true;
+        }
+    }
+
     KisImageSP image;
     KisNodeSP node;
     KisNodeSP newParent;
@@ -113,6 +125,8 @@ struct MoveNodeStruct {
 
     KisNodeSP oldParent;
     KisNodeSP oldAbove;
+    bool suppressNewParentRefresh;
+    bool suppressOldParentRefresh;
 };
 
 typedef QSharedPointer<MoveNodeStruct> MoveNodeStructSP;
@@ -136,7 +150,7 @@ class BatchMoveUpdateData {
 
     QMutex m_mutex;
 
-    KisNodeJugglerCompressed *m_parentJuggler;
+    QPointer<KisNodeJugglerCompressed> m_parentJuggler;
 
 public:
     BatchMoveUpdateData(KisNodeJugglerCompressed *parentJuggler)
@@ -149,6 +163,13 @@ private:
             bool result = hash->value(moveStruct->node)->tryMerge(*moveStruct);
             KIS_ASSERT_RECOVER_NOOP(result);
         } else {
+            MovedNodesHash::const_iterator it = hash->constBegin();
+            MovedNodesHash::const_iterator end = hash->constEnd();
+
+            for (; it != end; ++it) {
+                it.value()->resolveParentCollisions(moveStruct.data());
+            }
+
             hash->insert(moveStruct->node, moveStruct);
         }
     }
@@ -172,9 +193,16 @@ public:
     }
 
     void addInitialUpdate(MoveNodeStructSP moveStruct) {
-        QMutexLocker l(&m_mutex);
-        addToHashLazy(&m_movedNodesInitial, moveStruct);
-        emit m_parentJuggler->requestUpdateAsyncFromCommand();
+        {
+            QMutexLocker l(&m_mutex);
+            addToHashLazy(&m_movedNodesInitial, moveStruct);
+
+            // the juggler might directly forward the signal to processUnhandledUpdates,
+            // which would also like to get a lock, so we should release it beforehand
+        }
+        if (m_parentJuggler) {
+            emit m_parentJuggler->requestUpdateAsyncFromCommand();
+        }
     }
 
     void emitFinalUpdates(bool undo) {
@@ -210,7 +238,7 @@ public:
     {
     }
 
-    void end() {
+    void end() override {
         if (isFinalizing() && isFirstRedo()) {
             /**
              * When doing the first redo() some of the updates might
@@ -221,7 +249,7 @@ public:
         } else {
             /**
              * When being executed by real undo/redo operations, we
-             * should emit all the update signals. Noone else will do
+             * should emit all the update signals. No one else will do
              * that for us (juggler, which did it in the previous
              * case, might have already died).
              */
@@ -247,7 +275,21 @@ public:
     {
     }
 
-    void end() {
+    void init() override {
+        QList<KisSelectionMaskSP> *newActiveMasks;
+
+        if (isFinalizing()) {
+            newActiveMasks = &m_activeAfter;
+        } else {
+            newActiveMasks = &m_activeBefore;
+        }
+
+        Q_FOREACH (KisSelectionMaskSP mask, *newActiveMasks) {
+            mask->setActive(false);
+        }
+    }
+
+    void end() override {
         QList<KisSelectionMaskSP> *newActiveMasks;
 
         if (isFinalizing()) {
@@ -324,7 +366,7 @@ struct LowerRaiseLayer : public KisCommandUtils::AggregateCommand {
             AllMasks;
     }
 
-    void populateChildCommands() {
+    void populateChildCommands() override {
         KisNodeList sortedNodes = sortAndFilterNodes(m_nodes, m_image);
         KisNodeSP headNode = m_lower ? sortedNodes.first() : sortedNodes.last();
         const NodesType nodesType = getNodesType(sortedNodes);
@@ -449,7 +491,7 @@ struct DuplicateLayers : public KisCommandUtils::AggregateCommand {
           m_activeNode(activeNode),
           m_mode(mode) {}
 
-    void populateChildCommands() {
+    void populateChildCommands() override {
         KisNodeList filteredNodes = sortAndFilterNodes(m_nodes, m_image);
 
         if (filteredNodes.isEmpty()) return;
@@ -465,6 +507,14 @@ struct DuplicateLayers : public KisCommandUtils::AggregateCommand {
 
         const int indexOfActiveNode = filteredNodes.indexOf(m_activeNode);
         QList<KisSelectionMaskSP> activeMasks = findActiveSelectionMasks(filteredNodes);
+
+        // we will deactivate the masks before processing, so we should
+        // save their list in a convenient form
+        QSet<KisNodeSP> activeMaskNodes;
+        Q_FOREACH (KisSelectionMaskSP mask, activeMasks) {
+            activeMaskNodes.insert(mask);
+        }
+
         const bool haveActiveMasks = !activeMasks.isEmpty();
 
         if (!newParent) return;
@@ -474,6 +524,15 @@ struct DuplicateLayers : public KisCommandUtils::AggregateCommand {
                                                                m_image, false));
 
         if (haveActiveMasks) {
+            /**
+             * We should first disable the currently active masks, after the operation
+             * completed their cloned counterparts will be activated instead.
+             *
+             * HINT: we should deactivate the masks before cloning, because otherwise
+             *       KisGroupLayer::allowAsChild() will not let the second mask to be
+             *       added to the list of child nodes. See bug 382315.
+             */
+
             addCommand(new ActivateSelectionMasksCommand(activeMasks,
                                                          QList<KisSelectionMaskSP>(),
                                                          false));
@@ -494,7 +553,7 @@ struct DuplicateLayers : public KisCommandUtils::AggregateCommand {
                 }
 
                 newNodes << newNode;
-                if (haveActiveMasks && toActiveSelectionMask(node)) {
+                if (haveActiveMasks && activeMaskNodes.contains(node)) {
                     KisSelectionMaskSP mask = dynamic_cast<KisSelectionMask*>(newNode.data());
                     newActiveMasks << mask;
                 }
@@ -505,13 +564,13 @@ struct DuplicateLayers : public KisCommandUtils::AggregateCommand {
                 addCommand(new KisImageLayerAddCommand(m_image, newNode,
                                                        newParent,
                                                        currentAbove,
-                                                       false, false));
+                                                       true, true));
                 currentAbove = newNode;
             } else if (m_mode == MOVE) {
                 KisNodeSP newNode = node;
 
                 newNodes << newNode;
-                if (haveActiveMasks && toActiveSelectionMask(node)) {
+                if (haveActiveMasks && activeMaskNodes.contains(node)) {
                     KisSelectionMaskSP mask = dynamic_cast<KisSelectionMask*>(newNode.data());
                     newActiveMasks << mask;
                 }
@@ -529,6 +588,10 @@ struct DuplicateLayers : public KisCommandUtils::AggregateCommand {
 
 
         if (haveActiveMasks) {
+            /**
+             * Activate the cloned counterparts of the masks after the operation
+             * is complete.
+             */
             addCommand(new ActivateSelectionMasksCommand(QList<KisSelectionMaskSP>(),
                                                          newActiveMasks,
                                                          true));
@@ -577,7 +640,7 @@ struct RemoveLayers : private KisLayerUtils::RemoveNodeHelper, public KisCommand
           m_nodes(nodes),
           m_activeNode(activeNode){}
 
-    void populateChildCommands() {
+    void populateChildCommands() override {
         KisNodeList filteredNodes = m_nodes;
         KisLayerUtils::filterMergableNodes(filteredNodes, true);
 
@@ -599,7 +662,7 @@ struct RemoveLayers : private KisLayerUtils::RemoveNodeHelper, public KisCommand
                                                                m_image, true));
     }
 protected:
-    virtual void addCommandImpl(KUndo2Command *cmd) {
+    void addCommandImpl(KUndo2Command *cmd) override {
         addCommand(cmd);
     }
 
