@@ -18,9 +18,10 @@
 
 #include "kis_node_manager.h"
 
-#include <QDesktopServices>
+#include <QStandardPaths>
 #include <QMessageBox>
 #include <QSignalMapper>
+#include <QApplication>
 
 #include <kactioncollection.h>
 
@@ -36,7 +37,6 @@
 #include <KoToolManager.h>
 #include <KoProperties.h>
 
-
 #include <KoColorSpace.h>
 #include <KoColorSpaceRegistry.h>
 #include <KoColorModelStandardIds.h>
@@ -51,6 +51,7 @@
 #include <kis_painter.h>
 #include <kis_paint_layer.h>
 #include <KisMimeDatabase.h>
+#include <KisReferenceImagesLayer.h>
 
 #include "KisPart.h"
 #include "canvas/kis_canvas2.h"
@@ -76,10 +77,13 @@
 #include "kis_mimedata.h"
 #include "kis_layer_utils.h"
 #include "krita_utils.h"
+#include "kis_shape_layer.h"
 
 #include "processing/kis_mirror_processing_visitor.h"
 #include "KisView.h"
 
+#include <kis_signals_blocker.h>
+
 struct KisNodeManager::Private {
 
     Private(KisNodeManager *_q, KisViewManager *v)
@@ -103,9 +107,13 @@ struct KisNodeManager::Private {
     QScopedPointer<KisNodeSelectionAdapter> nodeSelectionAdapter;
     QScopedPointer<KisNodeInsertionAdapter> nodeInsertionAdapter;
 
+    KisAction *showInTimeline;
+
     KisNodeList selectedNodes;
     QPointer<KisNodeJugglerCompressed> nodeJuggler;
 
+    KisNodeWSP previouslyActiveNode;
+
     bool activateNodeImpl(KisNodeSP node);
 
     QSignalMapper nodeCreationSignalMapper;
@@ -144,21 +152,28 @@ bool KisNodeManager::Private::activateNodeImpl(KisNodeSP node)
         imageView->setCurrentNode(0);
         maskManager.activateMask(0);
         layerManager.activateLayer(0);
+        previouslyActiveNode = q->activeNode();
     } else {
 
+        previouslyActiveNode = q->activeNode();
+
         KoShape * shape = view->document()->shapeForNode(node);
-        Q_ASSERT(shape);
+
+        //if (!shape) return false;
+        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(shape, false);
 
         selection->select(shape);
         KoShapeLayer * shapeLayer = dynamic_cast<KoShapeLayer*>(shape);
 
-        Q_ASSERT(shapeLayer);
+        //if (!shapeLayer) return false;
+        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(shapeLayer, false);
+
 //         shapeLayer->setGeometryProtected(node->userLocked());
 //         shapeLayer->setVisible(node->visible());
         selection->setActiveLayer(shapeLayer);
 
         imageView->setCurrentNode(node);
-        if (KisLayerSP layer = dynamic_cast<KisLayer*>(node.data())) {
+        if (KisLayerSP layer = qobject_cast<KisLayer*>(node.data())) {
             maskManager.activateMask(0);
             layerManager.activateLayer(layer);
         } else if (KisMaskSP mask = dynamic_cast<KisMask*>(node.data())) {
@@ -216,16 +231,19 @@ void KisNodeManager::setView(QPointer<KisView>imageView)
                 &m_d->nodeCreationSignalMapper, SLOT(map()));           \
     }
 
-#define CONVERT_NODE_ACTION(id, layerType)                              \
+#define CONVERT_NODE_ACTION_2(id, layerType, exclude)                   \
     {                                                                   \
         action = actionManager->createAction(id);                       \
-        action->setExcludedNodeTypes(QStringList(layerType));           \
+        action->setExcludedNodeTypes(QStringList(exclude));             \
         actionManager->addAction(id, action);                           \
         m_d->nodeConversionSignalMapper.setMapping(action, layerType);  \
         connect(action, SIGNAL(triggered()),                            \
                 &m_d->nodeConversionSignalMapper, SLOT(map()));         \
     }
 
+#define CONVERT_NODE_ACTION(id, layerType)              \
+    CONVERT_NODE_ACTION_2(id, layerType, layerType)
+
 void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManager* actionManager)
 {
     m_d->layerManager.setup(actionManager);
@@ -243,9 +261,16 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
     action = actionManager->createAction("activatePreviousLayer");
     connect(action, SIGNAL(triggered()), this, SLOT(activatePreviousNode()));
 
+    action = actionManager->createAction("switchToPreviouslyActiveNode");
+    connect(action, SIGNAL(triggered()), this, SLOT(switchToPreviouslyActiveNode()));
+
     action  = actionManager->createAction("save_node_as_image");
     connect(action, SIGNAL(triggered()), this, SLOT(saveNodeAsImage()));
 
+    action  = actionManager->createAction("save_vector_node_to_svg");
+    connect(action, SIGNAL(triggered()), this, SLOT(saveVectorLayerAsImage()));
+    action->setActivationFlags(KisAction::ACTIVE_SHAPE_LAYER);
+
     action = actionManager->createAction("duplicatelayer");
     connect(action, SIGNAL(triggered()), this, SLOT(duplicateActiveNode()));
 
@@ -282,6 +307,14 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
     action = actionManager->createAction("select_unlocked_layers");
     connect(action, SIGNAL(triggered()), this, SLOT(selectUnlockedNodes()));
 
+    action = actionManager->createAction("new_from_visible");
+    connect(action, SIGNAL(triggered()), this, SLOT(createFromVisible()));
+
+    action = actionManager->createAction("show_in_timeline");
+    action->setCheckable(true);
+    connect(action, SIGNAL(toggled(bool)), this, SLOT(slotShowHideTimeline(bool)));
+    m_d->showInTimeline = action;
+
     NEW_LAYER_ACTION("add_new_paint_layer", "KisPaintLayer");
 
     NEW_LAYER_ACTION("add_new_group_layer", "KisGroupLayer");
@@ -300,6 +333,8 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
 
     NEW_LAYER_ACTION("add_new_filter_mask", "KisFilterMask");
 
+    NEW_LAYER_ACTION("add_new_colorize_mask", "KisColorizeMask");
+
     NEW_LAYER_ACTION("add_new_transform_mask", "KisTransformMask");
 
     NEW_LAYER_ACTION("add_new_selection_mask", "KisSelectionMask");
@@ -309,11 +344,15 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
 
     CONVERT_NODE_ACTION("convert_to_paint_layer", "KisPaintLayer");
 
-    CONVERT_NODE_ACTION("convert_to_selection_mask", "KisSelectionMask");
+    CONVERT_NODE_ACTION_2("convert_to_selection_mask", "KisSelectionMask", QStringList() << "KisSelectionMask" << "KisColorizeMask");
 
-    CONVERT_NODE_ACTION("convert_to_filter_mask", "KisFilterMask");
+    CONVERT_NODE_ACTION_2("convert_to_filter_mask", "KisFilterMask", QStringList() << "KisFilterMask" << "KisColorizeMask");
 
-    CONVERT_NODE_ACTION("convert_to_transparency_mask", "KisTransparencyMask");
+    CONVERT_NODE_ACTION_2("convert_to_transparency_mask", "KisTransparencyMask", QStringList() << "KisTransparencyMask" << "KisColorizeMask");
+
+    CONVERT_NODE_ACTION("convert_to_animated", "animated");
+
+    CONVERT_NODE_ACTION_2("convert_to_file_layer", "KisFileLayer", QStringList() << "KisGroupLayer" << "KisFileLayer" << "KisCloneLayer");
 
     connect(&m_d->nodeConversionSignalMapper, SIGNAL(mapped(const QString &)),
             this, SLOT(convertNode(const QString &)));
@@ -321,6 +360,18 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
     action = actionManager->createAction("isolate_layer");
     connect(action, SIGNAL(triggered(bool)), this, SLOT(toggleIsolateMode(bool)));
 
+    action = actionManager->createAction("toggle_layer_visibility");
+    connect(action, SIGNAL(triggered()), this, SLOT(toggleVisibility()));
+
+    action = actionManager->createAction("toggle_layer_lock");
+    connect(action, SIGNAL(triggered()), this, SLOT(toggleLock()));
+
+    action = actionManager->createAction("toggle_layer_inherit_alpha");
+    connect(action, SIGNAL(triggered()), this, SLOT(toggleInheritAlpha()));
+
+    action = actionManager->createAction("toggle_layer_alpha_lock");
+    connect(action, SIGNAL(triggered()), this, SLOT(toggleAlphaLock()));
+
     action  = actionManager->createAction("split_alpha_into_mask");
     connect(action, SIGNAL(triggered()), this, SLOT(slotSplitAlphaIntoMask()));
 
@@ -332,7 +383,7 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
     connect(action, SIGNAL(triggered()), this, SLOT(slotSplitAlphaSaveMerged()));
 
     connect(this, SIGNAL(sigNodeActivated(KisNodeSP)), SLOT(slotUpdateIsolateModeAction()));
-    connect(this, SIGNAL(sigNodeActivated(KisNodeSP)), SLOT(slotTryFinishIsolatedMode()));
+    connect(this, SIGNAL(sigNodeActivated(KisNodeSP)), SLOT(slotTryRestartIsolatedMode()));
 }
 
 void KisNodeManager::updateGUI()
@@ -373,10 +424,10 @@ void KisNodeManager::moveNodeAt(KisNodeSP node, KisNodeSP parent, int index)
     if (parent->allowAsChild(node)) {
         if (node->inherits("KisSelectionMask") && parent->inherits("KisLayer")) {
             KisSelectionMask *m = dynamic_cast<KisSelectionMask*>(node.data());
-            KisLayer *l = dynamic_cast<KisLayer*>(parent.data());
-            KisSelectionMaskSP selMask = l->selectionMask();
-            if (m && m->active() && l && l->selectionMask())
-                selMask->setActive(false);
+            KisLayer *l = qobject_cast<KisLayer*>(parent.data());
+            if (m && m->active() && l && l->selectionMask()) {
+                l->selectionMask()->setActive(false);
+            }
         }
         m_d->commandsAdapter.moveNode(node, parent, index);
     }
@@ -420,11 +471,13 @@ void KisNodeManager::toggleIsolateMode(bool checked)
 {
     KisImageWSP image = m_d->view->image();
 
-    if (checked) {
-        KisNodeSP activeNode = this->activeNode();
-        // Transform masks don't have pixel data...
-        if (activeNode->inherits("KisTransformMask")) return;
-        KIS_ASSERT_RECOVER_RETURN(activeNode);
+    KisNodeSP activeNode = this->activeNode();
+    if (checked && activeNode) {
+
+        // Transform and colorize masks don't have pixel data...
+        if (activeNode->inherits("KisTransformMask") ||
+            activeNode->inherits("KisColorizeMask")) return;
+
         if (!image->startIsolatedMode(activeNode)) {
             KisAction *action = m_d->view->actionManager()->actionByName("isolate_layer");
             action->setChecked(false);
@@ -445,66 +498,68 @@ void KisNodeManager::slotUpdateIsolateModeAction()
     action->setChecked(isolatedRootNode && isolatedRootNode == activeNode);
 }
 
-void KisNodeManager::slotTryFinishIsolatedMode()
+void KisNodeManager::slotTryRestartIsolatedMode()
 {
     KisNodeSP isolatedRootNode = m_d->view->image()->isolatedModeRoot();
     if (!isolatedRootNode) return;
 
-    bool belongsToIsolatedGroup = false;
-
-    KisNodeSP node = this->activeNode();
-    while(node) {
-        if (node == isolatedRootNode) {
-            belongsToIsolatedGroup = true;
-            break;
-        }
-        node = node->parent();
-    }
-
-    if (!belongsToIsolatedGroup) {
-        m_d->view->image()->stopIsolatedMode();
-    }
+    this->toggleIsolateMode(true);
 }
 
-void KisNodeManager::createNode(const QString & nodeType, bool quiet, KisPaintDeviceSP copyFrom)
+KisNodeSP  KisNodeManager::createNode(const QString & nodeType, bool quiet, KisPaintDeviceSP copyFrom)
 {
+    if (!m_d->view->blockUntilOperationsFinished(m_d->view->image())) {
+        return 0;
+    }
+
     KisNodeSP activeNode = this->activeNode();
     if (!activeNode) {
         activeNode = m_d->view->image()->root();
     }
 
-    KIS_ASSERT_RECOVER_RETURN(activeNode);
-    if (activeNode->systemLocked()) {
-        return;
-    }
+    KIS_ASSERT_RECOVER_RETURN_VALUE(activeNode, 0);
 
     // XXX: make factories for this kind of stuff,
     //      with a registry
 
     if (nodeType == "KisPaintLayer") {
-        m_d->layerManager.addLayer(activeNode);
+        return m_d->layerManager.addPaintLayer(activeNode);
     } else if (nodeType == "KisGroupLayer") {
-        m_d->layerManager.addGroupLayer(activeNode);
+        return m_d->layerManager.addGroupLayer(activeNode);
     } else if (nodeType == "KisAdjustmentLayer") {
-        m_d->layerManager.addAdjustmentLayer(activeNode);
+        return m_d->layerManager.addAdjustmentLayer(activeNode);
     } else if (nodeType == "KisGeneratorLayer") {
-        m_d->layerManager.addGeneratorLayer(activeNode);
+        return m_d->layerManager.addGeneratorLayer(activeNode);
     } else if (nodeType == "KisShapeLayer") {
-        m_d->layerManager.addShapeLayer(activeNode);
+        return m_d->layerManager.addShapeLayer(activeNode);
     } else if (nodeType == "KisCloneLayer") {
-        m_d->layerManager.addCloneLayer(activeNode);
+        return m_d->layerManager.addCloneLayer(activeNode);
     } else if (nodeType == "KisTransparencyMask") {
-        m_d->maskManager.createTransparencyMask(activeNode, copyFrom, false);
+        return m_d->maskManager.createTransparencyMask(activeNode, copyFrom, false);
     } else if (nodeType == "KisFilterMask") {
-        m_d->maskManager.createFilterMask(activeNode, copyFrom, quiet, false);
+        return m_d->maskManager.createFilterMask(activeNode, copyFrom, quiet, false);
+    } else if (nodeType == "KisColorizeMask") {
+        return m_d->maskManager.createColorizeMask(activeNode);
     } else if (nodeType == "KisTransformMask") {
-        m_d->maskManager.createTransformMask(activeNode);
+        return m_d->maskManager.createTransformMask(activeNode);
     } else if (nodeType == "KisSelectionMask") {
-        m_d->maskManager.createSelectionMask(activeNode, copyFrom, false);
+        return m_d->maskManager.createSelectionMask(activeNode, copyFrom, false);
     } else if (nodeType == "KisFileLayer") {
-        m_d->layerManager.addFileLayer(activeNode);
+        return m_d->layerManager.addFileLayer(activeNode);
     }
+    return 0;
+}
 
+void KisNodeManager::createFromVisible()
+{
+    KisLayerUtils::newLayerFromVisible(m_d->view->image(), m_d->view->image()->root()->lastChild());
+}
+
+void KisNodeManager::slotShowHideTimeline(bool value)
+{
+    Q_FOREACH (KisNodeSP node, selectedNodes()) {
+        node->setUseInTimeline(value);
+    }
 }
 
 KisLayerSP KisNodeManager::createPaintLayer()
@@ -514,7 +569,7 @@ KisLayerSP KisNodeManager::createPaintLayer()
         activeNode = m_d->view->image()->root();
     }
 
-    return m_d->layerManager.addLayer(activeNode);
+    return m_d->layerManager.addPaintLayer(activeNode);
 }
 
 void KisNodeManager::convertNode(const QString &nodeType)
@@ -533,17 +588,25 @@ void KisNodeManager::convertNode(const QString &nodeType)
 
         m_d->commandsAdapter.beginMacro(kundo2_i18n("Convert to a Selection Mask"));
 
+        bool result = false;
+
         if (nodeType == "KisSelectionMask") {
-            m_d->maskManager.createSelectionMask(activeNode, copyFrom, true);
+            result = !m_d->maskManager.createSelectionMask(activeNode, copyFrom, true).isNull();
         } else if (nodeType == "KisFilterMask") {
-            m_d->maskManager.createFilterMask(activeNode, copyFrom, false, true);
+            result = !m_d->maskManager.createFilterMask(activeNode, copyFrom, false, true).isNull();
         } else if (nodeType == "KisTransparencyMask") {
-            m_d->maskManager.createTransparencyMask(activeNode, copyFrom, true);
+            result = !m_d->maskManager.createTransparencyMask(activeNode, copyFrom, true).isNull();
         }
 
-        m_d->commandsAdapter.removeNode(activeNode);
         m_d->commandsAdapter.endMacro();
 
+        if (!result) {
+            m_d->view->blockUntilOperationsFinishedForced(m_d->imageView->image());
+            m_d->commandsAdapter.undoLastCommand();
+        }
+
+    } else if (nodeType == "KisFileLayer") {
+            m_d->layerManager.convertLayerToFileLayer(activeNode);
     } else {
         warnKrita << "Unsupported node conversion type:" << nodeType;
     }
@@ -567,7 +630,14 @@ void KisNodeManager::slotSomethingActivatedNodeImpl(KisNodeSP node)
 
 void KisNodeManager::slotNonUiActivatedNode(KisNodeSP node)
 {
+    // the node must still be in the graph, some asynchronous
+    // signals may easily break this requirement
+    if (node && !node->graphListener()) {
+        node = 0;
+    }
+
     if (node == activeNode()) return;
+
     slotSomethingActivatedNodeImpl(node);
 
     if (node) {
@@ -580,9 +650,11 @@ void KisNodeManager::slotNonUiActivatedNode(KisNodeSP node)
 
 void KisNodeManager::slotUiActivatedNode(KisNodeSP node)
 {
-    if (node == activeNode()) return;
-
-    slotSomethingActivatedNodeImpl(node);
+    // the node must still be in the graph, some asynchronous
+    // signals may easily break this requirement
+    if (node && !node->graphListener()) {
+        node = 0;
+    }
 
     if (node) {
         QStringList vectorTools = QStringList()
@@ -611,6 +683,10 @@ void KisNodeManager::slotUiActivatedNode(KisNodeSP node)
             }
         }
     }
+
+    if (node == activeNode()) return;
+
+    slotSomethingActivatedNodeImpl(node);
 }
 
 void KisNodeManager::nodesUpdated()
@@ -624,6 +700,10 @@ void KisNodeManager::nodesUpdated()
     m_d->view->updateGUI();
     m_d->view->selectionManager()->selectionChanged();
 
+    {
+        KisSignalsBlocker b(m_d->showInTimeline);
+        m_d->showInTimeline->setChecked(node->useInTimeline());
+    }
 }
 
 KisPaintDeviceSP KisNodeManager::activePaintDevice()
@@ -635,9 +715,10 @@ KisPaintDeviceSP KisNodeManager::activePaintDevice()
 
 void KisNodeManager::nodeProperties(KisNodeSP node)
 {
-    if (selectedNodes().size() > 1 || node->inherits("KisLayer")) {
+    if ((selectedNodes().size() > 1 && node->inherits("KisLayer")) || node->inherits("KisLayer")) {
         m_d->layerManager.layerProperties();
-    } else if (node->inherits("KisMask")) {
+    }
+    else if (node->inherits("KisMask")) {
         m_d->maskManager.maskProperties();
     }
 }
@@ -706,6 +787,20 @@ KisNodeInsertionAdapter* KisNodeManager::nodeInsertionAdapter() const
     return m_d->nodeInsertionAdapter.data();
 }
 
+bool KisNodeManager::isNodeHidden(KisNodeSP node, bool isGlobalSelectionHidden)
+{
+    if (dynamic_cast<KisReferenceImagesLayer *>(node.data())) {
+        return true;
+    }
+
+    if (isGlobalSelectionHidden && dynamic_cast<KisSelectionMask *>(node.data()) &&
+        (!node->parent() || !node->parent()->parent())) {
+        return true;
+    }
+
+    return false;
+}
+
 void KisNodeManager::nodeOpacityChanged(qreal opacity, bool finalChange)
 {
     KisNodeSP node = activeNode();
@@ -733,7 +828,8 @@ KisNodeJugglerCompressed* KisNodeManager::Private::lazyGetJuggler(const KUndo2Ma
 
     if (!nodeJuggler ||
         (nodeJuggler &&
-         !nodeJuggler->canMergeAction(actionName))) {
+         (nodeJuggler->isEnded() ||
+          !nodeJuggler->canMergeAction(actionName)))) {
 
         nodeJuggler = new KisNodeJugglerCompressed(actionName, image, q, 750);
         nodeJuggler->setAutoDelete(true);
@@ -805,10 +901,6 @@ void KisNodeManager::mirrorNodeY()
     mirrorNode(node, commandName, Qt::Vertical);
 }
 
-inline bool checkForGlobalSelection(KisNodeSP node) {
-    return dynamic_cast<KisSelectionMask*>(node.data()) && node->parent() && !node->parent()->parent();
-}
-
 void KisNodeManager::activateNextNode()
 {
     KisNodeSP activeNode = this->activeNode();
@@ -816,11 +908,15 @@ void KisNodeManager::activateNextNode()
 
     KisNodeSP node = activeNode->nextSibling();
 
+    while (node && node->childCount() > 0) {
+           node = node->firstChild();
+    }
+
     if (!node && activeNode->parent() && activeNode->parent()->parent()) {
         node = activeNode->parent();
     }
 
-    while(node && checkForGlobalSelection(node)) {
+    while(node && isNodeHidden(node, true)) {
         node = node->nextSibling();
     }
 
@@ -834,13 +930,21 @@ void KisNodeManager::activatePreviousNode()
     KisNodeSP activeNode = this->activeNode();
     if (!activeNode) return;
 
-    KisNodeSP node = activeNode->prevSibling();
+    KisNodeSP node;
 
-    if (!node && activeNode->parent()) {
-        node = activeNode->parent()->prevSibling();
+    if (activeNode->childCount() > 0) {
+        node = activeNode->lastChild();
+    }
+    else {
+        node = activeNode->prevSibling();
     }
 
-    while(node && checkForGlobalSelection(node)) {
+    while (!node && activeNode->parent()) {
+        node = activeNode->parent()->prevSibling();
+        activeNode = activeNode->parent();
+    }
+
+    while(node && isNodeHidden(node, true)) {
         node = node->prevSibling();
     }
 
@@ -849,16 +953,23 @@ void KisNodeManager::activatePreviousNode()
     }
 }
 
-void KisNodeManager::mergeLayer()
+void KisNodeManager::switchToPreviouslyActiveNode()
 {
-    m_d->layerManager.mergeLayer();
+    if (m_d->previouslyActiveNode && m_d->previouslyActiveNode->parent()) {
+        slotNonUiActivatedNode(m_d->previouslyActiveNode);
+    }
 }
 
 void KisNodeManager::rotate(double radians)
 {
-    // XXX: implement rotation for masks as well
-    m_d->layerManager.rotateLayer(radians);
+    if(!m_d->view->image()) return;
 
+    KisNodeSP node = activeNode();
+    if (!node) return;
+
+    if (!m_d->view->blockUntilOperationsFinished(m_d->view->image())) return;
+
+    m_d->view->image()->rotateNode(node, radians);
 }
 
 void KisNodeManager::rotate180()
@@ -878,8 +989,14 @@ void KisNodeManager::rotateRight90()
 
 void KisNodeManager::shear(double angleX, double angleY)
 {
-    // XXX: implement shear for masks as well
-    m_d->layerManager.shearLayer(angleX, angleY);
+    if (!m_d->view->image()) return;
+
+    KisNodeSP node = activeNode();
+    if (!node) return;
+
+    if(!m_d->view->blockUntilOperationsFinished(m_d->view->image())) return;
+
+    m_d->view->image()->shearNode(node, angleX, angleY);
 }
 
 void KisNodeManager::scale(double sx, double sy, KisFilterStrategy *filterStrategy)
@@ -919,8 +1036,8 @@ void KisNodeManager::Private::saveDeviceAsImage(KisPaintDeviceSP device,
 {
     KoFileDialog dialog(view->mainWindow(), KoFileDialog::SaveFile, "savenodeasimage");
     dialog.setCaption(i18n("Export \"%1\"", defaultName));
-    dialog.setDefaultDir(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
-    dialog.setMimeTypeFilters(KisImportExportManager::mimeFilter("application/x-krita", KisImportExportManager::Export));
+    dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
+    dialog.setMimeTypeFilters(KisImportExportManager::supportedMimeTypes(KisImportExportManager::Export));
     QString filename = dialog.filename();
 
     if (filename.isEmpty()) return;
@@ -929,26 +1046,24 @@ void KisNodeManager::Private::saveDeviceAsImage(KisPaintDeviceSP device,
 
     if (url.isEmpty()) return;
 
-    QString mimefilter = KisMimeDatabase::mimeTypeForFile(filename);;
+    QString mimefilter = KisMimeDatabase::mimeTypeForFile(filename, false);
 
-    QScopedPointer<KisDocument> d(KisPart::instance()->createDocument());
-    d->prepareForImport();
+    QScopedPointer<KisDocument> doc(KisPart::instance()->createDocument());
 
-    KisImageSP dst = new KisImage(d->createUndoStore(),
+    KisImageSP dst = new KisImage(doc->createUndoStore(),
                                   bounds.width(),
                                   bounds.height(),
                                   device->compositionSourceColorSpace(),
                                   defaultName);
     dst->setResolution(xRes, yRes);
-    d->setCurrentImage(dst);
+    doc->setCurrentImage(dst);
     KisPaintLayer* paintLayer = new KisPaintLayer(dst, "paint device", opacity);
     paintLayer->paintDevice()->makeCloneFrom(device, bounds);
     dst->addNode(paintLayer, dst->rootLayer(), KisLayerSP(0));
 
     dst->initialRefreshGraph();
 
-    d->setOutputMimeType(mimefilter.toLatin1());
-    d->exportDocument(url);
+    doc->exportDocumentSync(url, mimefilter.toLatin1());
 }
 
 void KisNodeManager::saveNodeAsImage()
@@ -974,6 +1089,40 @@ void KisNodeManager::saveNodeAsImage()
                            node->opacity());
 }
 
+#include "SvgWriter.h"
+
+void KisNodeManager::saveVectorLayerAsImage()
+{
+    KisShapeLayerSP shapeLayer = qobject_cast<KisShapeLayer*>(activeNode().data());
+    if (!shapeLayer) {
+        return;
+    }
+
+    KoFileDialog dialog(m_d->view->mainWindow(), KoFileDialog::SaveFile, "savenodeasimage");
+    dialog.setCaption(i18nc("@title:window", "Export to SVG"));
+    dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
+    dialog.setMimeTypeFilters(QStringList() << "image/svg+xml", "image/svg+xml");
+    QString filename = dialog.filename();
+
+    if (filename.isEmpty()) return;
+
+    QUrl url = QUrl::fromLocalFile(filename);
+
+    if (url.isEmpty()) return;
+
+    const QSizeF sizeInPx = m_d->view->image()->bounds().size();
+    const QSizeF sizeInPt(sizeInPx.width() / m_d->view->image()->xRes(),
+                          sizeInPx.height() / m_d->view->image()->yRes());
+
+    QList<KoShape*> shapes = shapeLayer->shapes();
+    std::sort(shapes.begin(), shapes.end(), KoShape::compareShapeZIndex);
+
+    SvgWriter writer(shapes);
+    if (!writer.save(filename, sizeInPt, true)) {
+        QMessageBox::warning(qApp->activeWindow(), i18nc("@title:window", "Krita"), i18n("Could not save to svg: %1", filename));
+    }
+}
+
 void KisNodeManager::slotSplitAlphaIntoMask()
 {
     KisNodeSP node = activeNode();
@@ -996,13 +1145,13 @@ void KisNodeManager::slotSplitAlphaIntoMask()
     KisSequentialIterator srcIt(srcDevice, processRect);
     KisSequentialIterator dstIt(selectionDevice, processRect);
 
-    do {
+    while (srcIt.nextPixel() && dstIt.nextPixel()) {
         quint8 *srcPtr = srcIt.rawData();
         quint8 *alpha8Ptr = dstIt.rawData();
 
         *alpha8Ptr = srcCS->opacityU8(srcPtr);
         srcCS->setOpacity(srcPtr, OPACITY_OPAQUE_U8, 1);
-    } while (srcIt.nextPixel() && dstIt.nextPixel());
+    }
 
     m_d->commandsAdapter.addExtraCommand(transaction.endAndTake());
 
@@ -1020,10 +1169,10 @@ void KisNodeManager::Private::mergeTransparencyMaskAsAlpha(bool writeToLayers)
 
     if (writeToLayers && !parentNode->hasEditablePaintDevice()) {
         QMessageBox::information(view->mainWindow(),
-                                 i18nc("@title:window", "Layer %1 is not editable").arg(parentNode->name()),
+                                 i18nc("@title:window", "Layer %1 is not editable", parentNode->name()),
                                  i18n("Cannot write alpha channel of "
                                       "the parent layer \"%1\".\n"
-                                      "The operation will be cancelled.").arg(parentNode->name()));
+                                      "The operation will be cancelled.", parentNode->name()));
         return;
     }
 
@@ -1059,12 +1208,12 @@ void KisNodeManager::Private::mergeTransparencyMaskAsAlpha(bool writeToLayers)
     KisSequentialIterator srcIt(selectionDevice, processRect);
     KisSequentialIterator dstIt(dstDevice, processRect);
 
-    do {
+    while (srcIt.nextPixel() && dstIt.nextPixel()) {
         quint8 *alpha8Ptr = srcIt.rawData();
         quint8 *dstPtr = dstIt.rawData();
 
         dstCS->setOpacity(dstPtr, *alpha8Ptr, 1);
-    } while (srcIt.nextPixel() && dstIt.nextPixel());
+    }
 
     if (writeToLayers) {
         commandsAdapter.addExtraCommand(transaction->endAndTake());
@@ -1092,13 +1241,80 @@ void KisNodeManager::slotSplitAlphaSaveMerged()
     m_d->mergeTransparencyMaskAsAlpha(false);
 }
 
+void KisNodeManager::toggleLock()
+{
+    KisNodeList nodes = this->selectedNodes();
+    KisNodeSP active = activeNode();
+    if (nodes.isEmpty() || !active) return;
+
+    bool isLocked = active->userLocked();
+
+    for (auto &node : nodes) {
+        node->setUserLocked(!isLocked);
+    }
+}
+
+void KisNodeManager::toggleVisibility()
+{
+    KisNodeList nodes = this->selectedNodes();
+    KisNodeSP active = activeNode();
+    if (nodes.isEmpty() || !active) return;
+
+    bool isVisible = active->visible();
+
+    for (auto &node : nodes) {
+        node->setVisible(!isVisible);
+        node->setDirty();
+    }
+}
+
+void KisNodeManager::toggleAlphaLock()
+{
+    KisNodeList nodes = this->selectedNodes();
+    KisNodeSP active = activeNode();
+    if (nodes.isEmpty() || !active) return;
+
+    auto layer = qobject_cast<KisPaintLayer*>(active.data());
+    if (!layer) {
+        return;
+    }
+
+    bool isAlphaLocked = layer->alphaLocked();
+    for (auto &node : nodes) {
+        auto layer = qobject_cast<KisPaintLayer*>(node.data());
+        if (layer) {
+            layer->setAlphaLocked(!isAlphaLocked);
+        }
+    }
+}
+
+void KisNodeManager::toggleInheritAlpha()
+{
+    KisNodeList nodes = this->selectedNodes();
+    KisNodeSP active = activeNode();
+    if (nodes.isEmpty() || !active) return;
+
+    auto layer = qobject_cast<KisLayer*>(active.data());
+    if (!layer) {
+        return;
+    }
+
+    bool isAlphaDisabled = layer->alphaChannelDisabled();
+    for (auto &node : nodes) {
+        auto layer = qobject_cast<KisLayer*>(node.data());
+        if (layer) {
+            layer->disableAlphaChannel(!isAlphaDisabled);
+            node->setDirty();
+        }
+    }
+}
+
 void KisNodeManager::cutLayersToClipboard()
 {
     KisNodeList nodes = this->selectedNodes();
-    KisNodeSP root = m_d->view->image()->root();
     if (nodes.isEmpty()) return;
 
-    KisClipboard::instance()->setLayers(nodes, root, false);
+    KisClipboard::instance()->setLayers(nodes, m_d->view->image(), false);
 
     KUndo2MagicString actionName = kundo2_i18n("Cut Nodes");
     KisNodeJugglerCompressed *juggler = m_d->lazyGetJuggler(actionName);
@@ -1108,9 +1324,7 @@ void KisNodeManager::cutLayersToClipboard()
 void KisNodeManager::copyLayersToClipboard()
 {
     KisNodeList nodes = this->selectedNodes();
-    KisNodeSP root = m_d->view->image()->root();
-
-    KisClipboard::instance()->setLayers(nodes, root, true);
+    KisClipboard::instance()->setLayers(nodes, m_d->view->image(), true);
 }
 
 void KisNodeManager::pasteLayersFromClipboard()
@@ -1159,6 +1373,8 @@ void KisNodeManager::createQuickGroupImpl(KisNodeJugglerCompressed *juggler,
     nodes2 = KisLayerUtils::sortMergableNodes(image->root(), selectedNodes());
     KisLayerUtils::filterMergableNodes(nodes2);
 
+    if (nodes2.size() == 0) return;
+
     if (KisLayerUtils::checkIsChildOf(active, nodes2)) {
         active = nodes2.first();
     }
