@@ -21,6 +21,8 @@
 #include <QTreeView>
 #include <QStyle>
 
+#include "krita_utils.h"
+#include <QApplication>
 
 #include <QGlobalStatic>
 Q_GLOBAL_STATIC(KisNodeViewColorScheme, s_instance)
@@ -30,15 +32,19 @@ struct KisNodeViewColorScheme::Private
     Private() {
         if (colorLabels.isEmpty()) {
             colorLabels << Qt::transparent;
-            colorLabels << QColor(252, 235, 87);
-            colorLabels << QColor(251, 183, 76);
-            colorLabels << QColor(160, 127, 110);
-            colorLabels << QColor(151, 202, 63);
-            colorLabels << QColor(123, 166, 209);
-            colorLabels << QColor(178, 138, 173);
-            colorLabels << QColor(238, 50, 51);
-            colorLabels << QColor(216, 218, 213);
-            colorLabels << QColor(145, 147, 142);
+            colorLabels << QColor(91,173,220);
+            colorLabels << QColor(151,202,63);
+            colorLabels << QColor(247,229,61);
+            colorLabels << QColor(255,170,63);
+            colorLabels << QColor(177,102,63);
+            colorLabels << QColor(238,50,51);
+            colorLabels << QColor(191,106,209);
+            colorLabels << QColor(118,119,114);
+
+            const QColor noLabelSetColor = qApp->palette().color(QPalette::Highlight);
+            for (auto it = colorLabels.begin(); it != colorLabels.end(); ++it) {
+                KritaUtils::dragColor(&(*it), noLabelSetColor, 0.35);
+            }
         }
     }
 
@@ -167,7 +173,17 @@ QRect KisNodeViewColorScheme::relExpandButtonRect() const
 
 QColor KisNodeViewColorScheme::colorLabel(int index) const
 {
-    return m_d->colorLabels[index % m_d->colorLabels.size()];
+    /**
+     * We should ensure that the index of the overflowing range
+     * will never be zero again.
+     */
+    if (index >= m_d->colorLabels.size()) {
+        index = 1 + index % (m_d->colorLabels.size() - 1);
+    } else {
+        index = index % m_d->colorLabels.size();
+    }
+
+    return m_d->colorLabels[index];
 }
 
 QVector<QColor> KisNodeViewColorScheme::allColorLabels() const
