@@ -46,7 +46,6 @@
 #include <KoID.h>
 #include <KoColorSpaceRegistry.h>
 #include <KoColorProfile.h>
-#include <KoColorSpace.h>
 #include <KoColor.h>
 #include <KoUnit.h>
 
@@ -65,6 +64,8 @@
 #include <KoColorModelStandardIds.h>
 #include "dialogs/kis_dlg_png_import.h"
 #include "kis_clipboard.h"
+#include <kis_cursor_override_hijacker.h>
+#include "kis_undo_stores.h"
 
 namespace
 {
@@ -338,10 +339,10 @@ public:
         png_uint_32 rowbytes = png_get_rowbytes(png_ptr, info_ptr);
         row_pointer = new png_byte[rowbytes];
     }
-    virtual ~KisPNGReaderLineByLine() {
+    ~KisPNGReaderLineByLine() override {
         delete[] row_pointer;
     }
-    virtual png_bytep readLine() {
+    png_bytep readLine() override {
         png_read_row(png_ptr, row_pointer, 0);
         return row_pointer;
     }
@@ -360,13 +361,13 @@ public:
         }
         png_read_image(png_ptr, row_pointers);
     }
-    virtual ~KisPNGReaderFullImage() {
+    ~KisPNGReaderFullImage() override {
         for (int i = 0; i < height; i++) {
             delete[] row_pointers[i];
         }
         delete[] row_pointers;
     }
-    virtual png_bytep readLine() {
+    png_bytep readLine() override {
         return row_pointers[y++];
     }
 private:
@@ -412,10 +413,6 @@ void _flush_fn(png_structp png_ptr)
 KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
 {
     dbgFile << "Start decoding PNG File";
-    if (!iod->open(QIODevice::ReadOnly)) {
-        dbgFile << "Failed to open PNG File";
-        return (KisImageBuilder_RESULT_FAILURE);
-    }
 
     png_byte signature[8];
     iod->peek((char*)signature, 8);
@@ -538,21 +535,21 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     qDebug() << "sRGB" << sRGBIntent;
 #endif
 
-bool fromBlender = false;
+    bool fromBlender = false;
 
-png_text* text_ptr;
+    png_text* text_ptr;
     int num_comments;
-png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
+    png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
 
-        for (int i = 0; i < num_comments; i++) {
-            QString key = QString(text_ptr[i].key).toLower();
-            if (key == "file") {
-                QString relatedFile = text_ptr[i].text;
-                if (relatedFile.contains(".blend", Qt::CaseInsensitive)){
-                    fromBlender=true;
-                }
+    for (int i = 0; i < num_comments; i++) {
+        QString key = QString(text_ptr[i].key).toLower();
+        if (key == "file") {
+            QString relatedFile = text_ptr[i].text;
+            if (relatedFile.contains(".blend", Qt::CaseInsensitive)){
+                fromBlender=true;
             }
         }
+    }
 
     const KoColorProfile* profile = 0;
     if (png_get_iCCP(png_ptr, info_ptr, &profile_name, &compression_type, &profile_data, &proflen)) {
@@ -576,34 +573,23 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
             quint32 behaviour = cfg.pasteBehaviour();
             if (behaviour == PASTE_ASK) {
                 KisDlgPngImport dlg(m_path, csName.first, csName.second);
-                QApplication::restoreOverrideCursor();
+                KisCursorOverrideHijacker hijacker;
+                Q_UNUSED(hijacker);
                 dlg.exec();
                 if (!dlg.profile().isEmpty()) {
-
-                    QString s = KoColorSpaceRegistry::instance()->colorSpaceId(csName.first, csName.second);
-
-                    const KoColorSpaceFactory * csf = KoColorSpaceRegistry::instance()->colorSpaceFactory(s);
-                    if (csf) {
-                        QList<const KoColorProfile *>  profileList = KoColorSpaceRegistry::instance()->profilesFor(csf);
-                        Q_FOREACH (const KoColorProfile *p, profileList) {
-                            if (p->name() == dlg.profile()) {
-                                profile = p;
-                                break;
-                            }
-                        }
-                    }
+                    profile = KoColorSpaceRegistry::instance()->profileByName(dlg.profile());
                 }
-                QApplication::setOverrideCursor(Qt::WaitCursor);
 
             }
         }
         dbgFile << "no embedded profile, will use the default profile";
     }
 
+    const QString colorSpaceId =
+        KoColorSpaceRegistry::instance()->colorSpaceId(csName.first, csName.second);
+
     // Check that the profile is used by the color space
-    if (profile && !KoColorSpaceRegistry::instance()->colorSpaceFactory(
-                KoColorSpaceRegistry::instance()->colorSpaceId(
-                    csName.first, csName.second))->profileIsCompatible(profile)) {
+    if (profile && !KoColorSpaceRegistry::instance()->profileIsCompatible(profile, colorSpaceId)) {
         warnFile << "The profile " << profile->name() << " is not compatible with the color space model " << csName.first << " " << csName.second;
         profile = 0;
     }
@@ -634,10 +620,10 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
         transform = KoColorSpaceRegistry::instance()->colorSpace(csName.first, csName.second, profile)->createColorConverter(cs, KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
     }
 
-    // Creating the KisImageWSP
+    // Creating the KisImageSP
     if (m_image == 0) {
-        m_image = new KisImage(m_doc->createUndoStore(), width, height, cs, "built image");
-        Q_CHECK_PTR(m_image);
+        KisUndoStore *store = m_doc ? m_doc->createUndoStore() : new KisSurrogateUndoStore();
+        m_image = new KisImage(store, width, height, cs, "built image");
     }
 
     // Read resolution
@@ -645,7 +631,7 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
     png_uint_32 x_resolution, y_resolution;
 
     png_get_pHYs(png_ptr, info_ptr, &x_resolution, &y_resolution, &unit_type);
-    if (unit_type == PNG_RESOLUTION_METER) {
+    if (x_resolution > 0 && y_resolution > 0 && unit_type == PNG_RESOLUTION_METER) {
         m_image->setResolution((double) POINT_TO_CM(x_resolution) / 100.0, (double) POINT_TO_CM(y_resolution) / 100.0); // It is the "invert" macro because we convert from pointer-per-inchs to points
     }
 
@@ -658,19 +644,21 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
         KoDocumentInfo * info = m_doc->documentInfo();
         dbgFile << "There are " << num_comments << " comments in the text";
         for (int i = 0; i < num_comments; i++) {
-            QString key = text_ptr[i].key;
-            dbgFile << "key is |" << text_ptr[i].key << "| containing " << text_ptr[i].text << " " << (key ==  "Raw profile type exif ");
+            QString key = QString(text_ptr[i].key).toLower();
+            dbgFile << "key: " << text_ptr[i].key
+                    << ", containing: " << text_ptr[i].text
+                    << ": " << (key == "raw profile type exif " ? "isExif" : "something else");
             if (key == "title") {
                 info->setAboutInfo("title", text_ptr[i].text);
-            } else if (key == "abstract") {
-                info->setAboutInfo("description", text_ptr[i].text);
+            } else if (key == "description") {
+                info->setAboutInfo("comment", text_ptr[i].text);
             } else if (key == "author") {
                 info->setAuthorInfo("creator", text_ptr[i].text);
-            } else if (key.contains("Raw profile type exif")) {
+            } else if (key.contains("raw profile type exif")) {
                 decode_meta_data(text_ptr + i, layer->metaData(), "exif", 6);
-            } else if (key.contains("Raw profile type iptc")) {
+            } else if (key.contains("raw profile type iptc")) {
                 decode_meta_data(text_ptr + i, layer->metaData(), "iptc", 14);
-            } else if (key.contains("Raw profile type xmp")) {
+            } else if (key.contains("raw profile type xmp")) {
                 decode_meta_data(text_ptr + i, layer->metaData(), "xmp", 0);
             } else if (key == "version") {
                 m_image->addAnnotation(new KisAnnotation("kpp_version", "version", QByteArray(text_ptr[i].text)));
@@ -679,16 +667,15 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
             }
         }
     }
-
     // Read image data
-    KisPNGReaderAbstract* reader = 0;
+    QScopedPointer<KisPNGReaderAbstract> reader;
     try {
         if (interlace_type == PNG_INTERLACE_ADAM7) {
-            reader = new KisPNGReaderFullImage(png_ptr, info_ptr, width, height);
+            reader.reset(new KisPNGReaderFullImage(png_ptr, info_ptr, width, height));
         } else {
-            reader = new KisPNGReaderLineByLine(png_ptr, info_ptr, width, height);
+            reader.reset(new KisPNGReaderLineByLine(png_ptr, info_ptr, width, height));
         }
-    } catch (std::bad_alloc& e) {
+    } catch (const std::bad_alloc& e) {
         // new png_byte[] may raise such an exception if the image
         // is invalid / to large.
         dbgFile << "bad alloc: " << e.what();
@@ -808,8 +795,6 @@ png_get_text(png_ptr, info_ptr, &text_ptr, &num_comments);
 
     // Freeing memory
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
-
-    delete reader;
     return KisImageBuilder_RESULT_OK;
 
 }
@@ -820,6 +805,11 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const QString &filename)
 
     QFile fp(filename);
     if (fp.exists()) {
+        if (!fp.open(QIODevice::ReadOnly)) {
+            dbgFile << "Failed to open PNG File";
+            return (KisImageBuilder_RESULT_FAILURE);
+        }
+
         return buildImage(&fp);
     }
     return (KisImageBuilder_RESULT_NOT_EXIST);
@@ -827,7 +817,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const QString &filename)
 }
 
 
-KisImageWSP KisPNGConverter::image()
+KisImageSP KisPNGConverter::image()
 {
     return m_image;
 }
@@ -851,6 +841,14 @@ bool KisPNGConverter::saveDeviceToStore(const QString &filename, const QRect &im
         options.interlace = false;
         options.tryToSaveAsIndexed = false;
         options.alpha = true;
+        options.saveSRGBProfile = false;
+
+        if (dev->colorSpace()->id() != "RGBA") {
+            dev = new KisPaintDevice(*dev.data());
+            KUndo2Command *cmd = dev->convertTo(KoColorSpaceRegistry::instance()->rgb8());
+            delete cmd;
+        }
+
         bool success = pngconv.buildFile(&io, imageRect, xRes, yRes, dev, annotIt, annotIt, options, metaDataStore);
         if (success != KisImageBuilder_RESULT_OK) {
             dbgFile << "Saving PNG failed:" << filename;
@@ -876,6 +874,11 @@ KisImageBuilder_Result KisPNGConverter::buildFile(const QString &filename, const
     dbgFile << "Start writing PNG File " << filename;
     // Open a QIODevice for writing
     QFile fp (filename);
+    if (!fp.open(QIODevice::WriteOnly)) {
+        dbgFile << "Failed to open PNG File for writing";
+        return (KisImageBuilder_RESULT_FAILURE);
+    }
+
     KisImageBuilder_Result result = buildFile(&fp, imageRect, xRes, yRes, device, annotationsStart, annotationsEnd, options, metaData);
 
     return result;
@@ -883,11 +886,6 @@ KisImageBuilder_Result KisPNGConverter::buildFile(const QString &filename, const
 
 KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRect &imageRect, const qreal xRes, const qreal yRes, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
 {
-    if (!iodevice->open(QIODevice::WriteOnly)) {
-        dbgFile << "Failed to open PNG File for writing";
-        return (KisImageBuilder_RESULT_FAILURE);
-    }
-
     if (!device)
         return KisImageBuilder_RESULT_INVALID_ARG;
 
@@ -901,10 +899,22 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
         device = tmp;
     }
 
-    if (options.forceSRGB) {
+    if (device->colorSpace()->colorDepthId() == Float16BitsColorDepthID
+            || device->colorSpace()->colorDepthId() == Float32BitsColorDepthID
+            || device->colorSpace()->colorDepthId() == Float64BitsColorDepthID) {
+        const KoColorSpace *dstcs = KoColorSpaceRegistry::instance()->colorSpace(device->colorSpace()->colorModelId().id(), Integer16BitsColorDepthID.id(), device->colorSpace()->profile());
+        KisPaintDeviceSP tmp = new KisPaintDevice(dstcs);
+        KisPainter gc(tmp);
+        gc.bitBlt(imageRect.topLeft(), device, imageRect);
+        gc.end();
+        device = tmp;
+    }
+    QStringList colormodels = QStringList() << RGBAColorModelID.id() << GrayAColorModelID.id();
+    if (options.forceSRGB || !colormodels.contains(device->colorSpace()->colorModelId().id())) {
         const KoColorSpace* cs = KoColorSpaceRegistry::instance()->colorSpace(RGBAColorModelID.id(), device->colorSpace()->colorDepthId().id(), "sRGB built-in - (lcms internal)");
         device = new KisPaintDevice(*device);
-        device->convertTo(cs);
+        KUndo2Command *cmd = device->convertTo(cs);
+        delete cmd;
     }
 
     // Initialize structures
@@ -957,15 +967,15 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
     Q_ASSERT(color_type > -1);
 
     // Try to compute a table of color if the colorspace is RGB8f
-    png_colorp palette = 0;
+    QScopedArrayPointer<png_color> palette;
     int num_palette = 0;
     if (!options.alpha && options.tryToSaveAsIndexed && KoID(device->colorSpace()->id()) == KoID("RGBA")) { // png doesn't handle indexed images and alpha, and only have indexed for RGB8
-        palette = new png_color[255];
+        palette.reset(new png_color[255]);
 
         KisSequentialIterator it(device, imageRect);
 
         bool toomuchcolor = false;
-        do {
+        while (it.nextPixel()) {
             const quint8* c = it.oldRawData();
             bool findit = false;
             for (int i = 0; i < num_palette; i++) {
@@ -986,7 +996,7 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
                 palette[num_palette].blue = c[0];
                 num_palette++;
             }
-        }  while (it.nextPixel());
+        }
 
         if (!toomuchcolor) {
             dbgFile << "Found a palette of " << num_palette << " colors";
@@ -1001,7 +1011,7 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
                 color_nb_bits = 8;
             }
         } else {
-            delete [] palette;
+            palette.reset();
         }
     }
 
@@ -1021,13 +1031,18 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
      * This automatically writes the correct gamma and chroma chunks along with the sRGB chunk, but firefox's
      * color management is bugged, so once you give it any incentive to start color managing an sRGB image it
      * will turn, for example, a nice desaturated rusty red into bright poppy red. So this is disabled for now.
-     */    
+     */
     /*if (!options.saveSRGBProfile && sRGB) {
-	    png_set_sRGB_gAMA_and_cHRM(png_ptr, info_ptr, PNG_sRGB_INTENT_PERCEPTUAL);
+        png_set_sRGB_gAMA_and_cHRM(png_ptr, info_ptr, PNG_sRGB_INTENT_PERCEPTUAL);
     }*/
+
+
+    // we should ensure we don't access non-existing palette object
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(palette || color_type != PNG_COLOR_TYPE_PALETTE, KisImageBuilder_RESULT_FAILURE);
+
     // set the palette
     if (color_type == PNG_COLOR_TYPE_PALETTE) {
-        png_set_PLTE(png_ptr, info_ptr, palette, num_palette);
+        png_set_PLTE(png_ptr, info_ptr, palette.data(), num_palette);
     }
     // Save annotation
     vKisAnnotationSP_it it = annotationsStart;
@@ -1064,31 +1079,55 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
     const KoColorProfile* colorProfile = device->colorSpace()->profile();
     QByteArray colorProfileData = colorProfile->rawData();
     if (!sRGB || options.saveSRGBProfile) {
+
 #if PNG_LIBPNG_VER_MAJOR >= 1 && PNG_LIBPNG_VER_MINOR >= 5
-        png_set_iCCP(png_ptr, info_ptr, (char*)"icc", PNG_COMPRESSION_TYPE_BASE, (const png_bytep)colorProfileData.constData(), colorProfileData . size());
+        png_set_iCCP(png_ptr, info_ptr, (png_const_charp)"icc", PNG_COMPRESSION_TYPE_BASE, (png_const_bytep)colorProfileData.constData(), colorProfileData . size());
 #else
-        png_set_iCCP(png_ptr, info_ptr, (char*)"icc", PNG_COMPRESSION_TYPE_BASE, (char*)colorProfileData.constData(), colorProfileData . size());
+        // older version of libpng has a problem with constness on the parameters
+        char typeString[] = "icc";
+        png_set_iCCP(png_ptr, info_ptr, typeString, PNG_COMPRESSION_TYPE_BASE, colorProfileData.data(), colorProfileData . size());
 #endif
     }
 
-    // read comments from the document information
+    // save comments from the document information
+    // warning: according to the official png spec, the keys need to be capitalized!
     if (m_doc) {
-        png_text texts[3];
+        png_text texts[4];
         int nbtexts = 0;
         KoDocumentInfo * info = m_doc->documentInfo();
-        QString title = info->aboutInfo("creator");
-        if (!title.isEmpty()) {
-            fillText(texts + nbtexts, "title", title);
+        QString title = info->aboutInfo("title");
+        if (!title.isEmpty() && options.storeMetaData) {
+            fillText(texts + nbtexts, "Title", title);
             nbtexts++;
         }
-        QString abstract = info->aboutInfo("description");
-        if (!abstract.isEmpty()) {
-            fillText(texts + nbtexts, "abstract", abstract);
+        QString abstract = info->aboutInfo("subject");
+        if (abstract.isEmpty()) {
+            abstract = info->aboutInfo("abstract");
+        }
+        if (!abstract.isEmpty() && options.storeMetaData) {
+            QString keywords = info->aboutInfo("keyword");
+            if (!keywords.isEmpty()) {
+                abstract = abstract + " keywords: " + keywords;
+            }
+            fillText(texts + nbtexts, "Description", abstract);
             nbtexts++;
         }
+
+        QString license = info->aboutInfo("license");
+        if (!license.isEmpty() && options.storeMetaData) {
+            fillText(texts + nbtexts, "Copyright", license);
+            nbtexts++;
+        }
+
         QString author = info->authorInfo("creator");
-        if (!author.isEmpty()) {
-            fillText(texts + nbtexts, "author", author);
+        if (!author.isEmpty() && options.storeAuthor) {
+            if (!info->authorContactInfo().isEmpty()) {
+                QString contact = info->authorContactInfo().at(0);
+                if (!contact.isEmpty()) {
+                    author = author+"("+contact+")";
+                }
+            }
+            fillText(texts + nbtexts, "Author", author);
             nbtexts++;
         }
 
@@ -1154,26 +1193,48 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
     // Write the PNG
     //     png_write_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, 0);
 
+    struct RowPointersStruct {
+        RowPointersStruct(const QSize &size, int pixelSize)
+            : numRows(size.height())
+        {
+            rows = new png_byte*[numRows];
+
+            for (int i = 0; i < numRows; i++) {
+                rows[i] = new png_byte[size.width() * pixelSize];
+            }
+        }
+
+        ~RowPointersStruct() {
+            for (int i = 0; i < numRows; i++) {
+                delete[] rows[i];
+            }
+            delete[] rows;
+        }
+
+        const int numRows = 0;
+        png_byte** rows = 0;
+    };
+
+
     // Fill the data structure
-    png_byte** row_pointers = new png_byte*[imageRect.height()];
+    RowPointersStruct rowPointers(imageRect.size(), device->pixelSize());
+
     int row = 0;
     for (int y = imageRect.y(); y < imageRect.y() + imageRect.height(); y++, row++) {
         KisHLineConstIteratorSP it = device->createHLineConstIteratorNG(imageRect.x(), y, imageRect.width());
 
-        row_pointers[row] = new png_byte[imageRect.width() * device->pixelSize()];
-
         switch (color_type) {
         case PNG_COLOR_TYPE_GRAY:
         case PNG_COLOR_TYPE_GRAY_ALPHA:
             if (color_nb_bits == 16) {
-                quint16 *dst = reinterpret_cast<quint16 *>(row_pointers[row]);
+                quint16 *dst = reinterpret_cast<quint16 *>(rowPointers.rows[row]);
                 do {
                     const quint16 *d = reinterpret_cast<const quint16 *>(it->oldRawData());
                     *(dst++) = d[0];
                     if (options.alpha) *(dst++) = d[1];
                 } while (it->nextPixel());
             } else {
-                quint8 *dst = row_pointers[row];
+                quint8 *dst = rowPointers.rows[row];
                 do {
                     const quint8 *d = it->oldRawData();
                     *(dst++) = d[0];
@@ -1184,7 +1245,7 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
         case PNG_COLOR_TYPE_RGB:
         case PNG_COLOR_TYPE_RGB_ALPHA:
             if (color_nb_bits == 16) {
-                quint16 *dst = reinterpret_cast<quint16 *>(row_pointers[row]);
+                quint16 *dst = reinterpret_cast<quint16 *>(rowPointers.rows[row]);
                 do {
                     const quint16 *d = reinterpret_cast<const quint16 *>(it->oldRawData());
                     *(dst++) = d[2];
@@ -1193,7 +1254,7 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
                     if (options.alpha) *(dst++) = d[3];
                 } while (it->nextPixel());
             } else {
-                quint8 *dst = row_pointers[row];
+                quint8 *dst = rowPointers.rows[row];
                 do {
                     const quint8 *d = it->oldRawData();
                     *(dst++) = d[2];
@@ -1204,7 +1265,7 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
             }
             break;
         case PNG_COLOR_TYPE_PALETTE: {
-            quint8 *dst = row_pointers[row];
+            quint8 *dst = rowPointers.rows[row];
             KisPNGWriteStream writestream(dst, color_nb_bits);
             do {
                 const quint8 *d = it->oldRawData();
@@ -1221,27 +1282,17 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, const QRe
         }
             break;
         default:
-            delete[] row_pointers;
             return KisImageBuilder_RESULT_UNSUPPORTED;
         }
     }
 
-    png_write_image(png_ptr, row_pointers);
+    png_write_image(png_ptr, rowPointers.rows);
 
     // Writing is over
     png_write_end(png_ptr, info_ptr);
 
     // Free memory
     png_destroy_write_struct(&png_ptr, &info_ptr);
-    for (int y = 0; y < imageRect.height(); y++) {
-        delete[] row_pointers[y];
-    }
-    delete[] row_pointers;
-
-    if (color_type == PNG_COLOR_TYPE_PALETTE) {
-        delete [] palette;
-    }
-    iodevice->close();
     return KisImageBuilder_RESULT_OK;
 }
 
