@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
    Copyright 2009 Vera Lukman <shicmap@gmail.com>
+   Copyright 2016 Scott Petrovic <scottpetrovic@gmail.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,17 +20,26 @@
 #ifndef KIS_POPUP_PALETTE_H
 #define KIS_POPUP_PALETTE_H
 
-#include <kis_types.h>
-#include <QWidget>
-#include <QQueue>
-#include <KoColorDisplayRendererInterface.h>
-
+#include <QElapsedTimer>
+#include <QPushButton>
+#include <QSlider>
+#include <QGraphicsOpacityEffect>
+#include "KisViewManager.h"
+#include "kactioncollection.h"
+#include "kis_tool_button.h"
+#include "kis_highlighted_button.h"
+#include <KisColorSelectorInterface.h>
 
 class KisFavoriteResourceManager;
 class QWidget;
 class KoColor;
 class KoTriangleColorSelector;
 class KisSignalCompressor;
+class KisBrushHud;
+class KisRoundHudButton;
+class KisCanvasResourceProvider;
+class KisVisualColorSelector;
+class KisAcyclicSignalConnector;
 
 class KisPopupPalette : public QWidget
 {
@@ -40,9 +50,10 @@ class KisPopupPalette : public QWidget
     Q_PROPERTY(int selectedColor READ selectedColor WRITE setSelectedColor)
 
 public:
-    KisPopupPalette(KisFavoriteResourceManager*, const KoColorDisplayRendererInterface *displayRenderer = KoDumbColorDisplayRenderer::instance(), QWidget *parent = 0);
-    ~KisPopupPalette();
-    QSize sizeHint() const;
+    KisPopupPalette(KisViewManager*, KisCoordinatesConverter* ,KisFavoriteResourceManager*, const KoColorDisplayRendererInterface *displayRenderer,
+                    KisCanvasResourceProvider *provider, QWidget *parent = 0);
+    ~KisPopupPalette() override;
+    QSize sizeHint() const override;
 
     void showPopupPalette(const QPoint&);
     void showPopupPalette(bool b);
@@ -53,16 +64,17 @@ public:
     //functions to set up selectedColor
     void setSelectedColor(int x);
     int selectedColor() const;
+    void setParent(QWidget *parent);
 
-    virtual void tabletEvent(QTabletEvent * event);
+    void tabletEvent(QTabletEvent *event) override;
 
 protected:
 
-    void paintEvent(QPaintEvent*);
-    void resizeEvent(QResizeEvent*);
-    void mouseReleaseEvent(QMouseEvent*);
-    void mouseMoveEvent(QMouseEvent*);
-    void mousePressEvent(QMouseEvent*);
+    void paintEvent(QPaintEvent*) override;
+    void resizeEvent(QResizeEvent*) override;
+    void mouseReleaseEvent(QMouseEvent*) override;
+    void mouseMoveEvent(QMouseEvent*) override;
+    void mousePressEvent(QMouseEvent*) override;
 
     //functions to calculate index of favorite brush or recent color in array
     //n is the total number of favorite brushes or recent colors
@@ -77,35 +89,72 @@ protected:
     void setHoveredColor(int x);
     int hoveredColor() const;
 
-
 private:
-    void setVisible(bool b);
+    void setVisible(bool b) override;
 
     QPainterPath drawDonutPathFull(int, int, int, int);
     QPainterPath drawDonutPathAngle(int, int, int);
+    QPainterPath drawRotationIndicator(qreal rotationAngle, bool canDrag);
     bool isPointInPixmap(QPointF&, int pos);
 
-    QPainterPath pathFromPresetIndex(int index);
+    QPainterPath createPathFromPresetIndex(int index);
 
     int numSlots();
+    void adjustLayout(const QPoint &p);
+
 private:
+    int m_hoveredPreset {0};
+    int m_hoveredColor {0};
+    int m_selectedColor {0};
 
-    int m_hoveredPreset;
-    int m_hoveredColor;
-    int m_selectedColor;
-    KisFavoriteResourceManager* m_resourceManager;
-    KoTriangleColorSelector* m_triangleColorSelector;
+    QElapsedTimer m_timeSinceOpening;
+    bool m_hadMousePressSinceOpening {false};
 
-    QTimer* m_timer;
 
+    KisCoordinatesConverter *m_coordinatesConverter;
+
+    KisViewManager *m_viewManager;
+    KisActionManager *m_actionManager;
+    KisFavoriteResourceManager *m_resourceManager;
+    KisColorSelectorInterface *m_triangleColorSelector {0};
     const KoColorDisplayRendererInterface *m_displayRenderer;
-
     QScopedPointer<KisSignalCompressor> m_colorChangeCompressor;
+    KActionCollection *m_actionCollection;
+
+    QTimer m_timer;
+
+    KisBrushHud *m_brushHud {0};
+    float m_popupPaletteSize {385.0};
+    float m_colorHistoryInnerRadius {72.0};
+    float m_colorHistoryOuterRadius {92.0};
+
+    KisRoundHudButton *m_settingsButton {0};
+    KisRoundHudButton *m_brushHudButton {0};
+    QPoint m_lastCenterPoint;
+    QRect m_canvasRotationIndicatorRect;
+    QRect m_resetCanvasRotationIndicatorRect;
+    bool m_isOverCanvasRotationIndicator {false};
+    bool m_isRotatingCanvasIndicator {false};
+    bool m_isZoomingCanvas {false};
+
+
+    KisHighlightedToolButton *mirrorMode {0};
+    KisHighlightedToolButton *canvasOnlyButton {0};
+    QPushButton *zoomToOneHundredPercentButton {0};
+    QSlider *zoomCanvasSlider {0};
+    int zoomSliderMinValue {10};
+    int zoomSliderMaxValue {200};
+    KisAcyclicSignalConnector *m_acyclicConnector = 0;
+
+    // updates the transparency and effects of the whole widget
+    QGraphicsOpacityEffect *opacityChange {0};
 
 Q_SIGNALS:
     void sigChangeActivePaintop(int);
     void sigUpdateRecentColor(int);
     void sigChangefGColor(const KoColor&);
+    void sigUpdateCanvas();
+    void zoomLevelChanged(int);
 
     // These are used to handle a bug:
     // If pop up palette is visible and a new colour is selected, the new colour
@@ -115,6 +164,9 @@ Q_SIGNALS:
     void sigEnableChangeFGColor(bool);
     void sigTriggerTimer();
 
+public Q_SLOTS:
+    void slotUpdateIcons();
+
 private Q_SLOTS:
     void slotExternalFgColorChanged(const KoColor &color);
     void slotEmitColorChanged();
@@ -123,6 +175,16 @@ private Q_SLOTS:
     void slotEnableChangeFGColor();
     void slotUpdate() { update(); }
     void slotHide() { showPopupPalette(false); }
+    void slotShowTagsPopup();
+    void showHudWidget(bool visible);
+    void slotmirroModeClicked();
+    void slotCanvasonlyModeClicked();
+    void slotZoomToOneHundredPercentClicked();
+    void slotZoomSliderChanged(int zoom);
+
+    void slotZoomSliderPressed();
+    void slotZoomSliderReleased();
+
 };
 
 #endif // KIS_POPUP_PALETTE_H
