@@ -81,8 +81,6 @@ public:
     void renderDecorations(QPainter *painter);
     void paintToolOutline(const QPainterPath &path);
 
-    bool needsFpsDebugging() const;
-
 public: // Implement kis_abstract_canvas_widget interface
     void setDisplayFilter(QSharedPointer<KisDisplayFilter> displayFilter) override;
     void setWrapAroundViewingMode(bool value) override;
@@ -91,19 +89,22 @@ public: // Implement kis_abstract_canvas_widget interface
     void finishResizingImage(qint32 w, qint32 h) override;
     KisUpdateInfoSP startUpdateCanvasProjection(const QRect & rc, const QBitArray &channelFlags) override;
     QRect updateCanvasProjection(KisUpdateInfoSP info) override;
+    QVector<QRect> updateCanvasProjection(const QVector<KisUpdateInfoSP> &infoObjects) override;
 
     QWidget *widget() override {
         return this;
     }
 
     bool isBusy() const override;
+    void setLodResetInProgress(bool value) override;
 
     void setDisplayFilterImpl(QSharedPointer<KisDisplayFilter> displayFilter, bool initializing);
 
     KisOpenGLImageTexturesSP openGLImageTextures() const;
 
-private Q_SLOTS:
+public Q_SLOTS:
     void slotConfigChanged();
+    void slotPixelGridModeChanged();
 
 protected: // KisCanvasWidgetBase
     bool callFocusNextPrevChild(bool next) override;
@@ -115,6 +116,7 @@ private:
     void reportFailedShaderCompilation(const QString &context);
     void drawImage();
     void drawCheckers();
+    void drawGrid();
 
 private:
 
