@@ -45,6 +45,14 @@
 #define GL_BGRA 0x80E1
 #endif
 
+// GL_EXT_texture_format_BGRA8888
+#ifndef GL_BGRA_EXT
+#define GL_BGRA_EXT 0x80E1
+#endif
+#ifndef GL_BGRA8_EXT
+#define GL_BGRA8_EXT 0x93A1
+#endif
+
 
 KisOpenGLImageTextures::ImageTexturesMap KisOpenGLImageTextures::imageTexturesMap;
 
@@ -55,7 +63,6 @@ KisOpenGLImageTextures::KisOpenGLImageTextures()
     , m_internalColorManagementActive(true)
     , m_checkerTexture(0)
     , m_glFuncs(0)
-    , m_allChannelsSelected(true)
     , m_useOcio(false)
     , m_initialized(false)
 {
@@ -65,7 +72,6 @@ KisOpenGLImageTextures::KisOpenGLImageTextures()
     m_conversionFlags = KoColorConversionTransformation::HighQuality;
     if (cfg.useBlackPointCompensation()) m_conversionFlags |= KoColorConversionTransformation::BlackpointCompensation;
     if (!cfg.allowLCMSOptimization()) m_conversionFlags |= KoColorConversionTransformation::NoOptimization;
-
     m_useOcio = cfg.useOcio();
 }
 
@@ -81,14 +87,14 @@ KisOpenGLImageTextures::KisOpenGLImageTextures(KisImageWSP image,
     , m_internalColorManagementActive(true)
     , m_checkerTexture(0)
     , m_glFuncs(0)
-    , m_allChannelsSelected(true)
     , m_useOcio(false)
     , m_initialized(false)
 {
     Q_ASSERT(renderingIntent < 4);
 }
 
-void KisOpenGLImageTextures::initGL(QOpenGLFunctions *f) {
+void KisOpenGLImageTextures::initGL(QOpenGLFunctions *f)
+{
     if (f) {
         m_glFuncs = f;
     } else {
@@ -96,11 +102,17 @@ void KisOpenGLImageTextures::initGL(QOpenGLFunctions *f) {
     }
 
     getTextureSize(&m_texturesInfo);
+
+    // we use local static object for creating pools shared among
+    // different images
+    static KisTextureTileInfoPoolRegistry s_poolRegistry;
+    m_updateInfoBuilder.setTextureInfoPool(s_poolRegistry.getPool(m_texturesInfo.width, m_texturesInfo.height));
+
     m_glFuncs->glGenTextures(1, &m_checkerTexture);
     createImageTextureTiles();
 
-    KisOpenGLUpdateInfoSP info = updateCache(m_image->bounds());
-    recalculateCache(info);
+    KisOpenGLUpdateInfoSP info = updateCache(m_image->bounds(), m_image);
+    recalculateCache(info, false);
 }
 
 KisOpenGLImageTextures::~KisOpenGLImageTextures()
@@ -163,22 +175,18 @@ KisOpenGLImageTexturesSP KisOpenGLImageTextures::getImageTextures(KisImageWSP im
     }
 }
 
-QRect KisOpenGLImageTextures::calculateTileRect(int col, int row) const
-{
-    return m_image->bounds() &
-            QRect(col * m_texturesInfo.effectiveWidth,
-                  row * m_texturesInfo.effectiveHeight,
-                  m_texturesInfo.effectiveWidth,
-                  m_texturesInfo.effectiveHeight);
-}
-
 void KisOpenGLImageTextures::createImageTextureTiles()
 {
-    KisConfig cfg;
 
     destroyImageTextureTiles();
     updateTextureFormat();
 
+    if (!m_tilesDestinationColorSpace) {
+        qDebug() << "No destination colorspace!!!!";
+        return;
+    }
+
+
     m_storedImageBounds = m_image->bounds();
     const int lastCol = xToCol(m_image->width());
     const int lastRow = yToRow(m_image->height());
@@ -202,15 +210,15 @@ void KisOpenGLImageTextures::createImageTextureTiles()
         m_textureTiles.reserve((lastRow+1)*m_numCols);
         for (int row = 0; row <= lastRow; row++) {
             for (int col = 0; col <= lastCol; col++) {
-                QRect tileRect = calculateTileRect(col, row);
+                QRect tileRect = m_updateInfoBuilder.calculateEffectiveTileRect(col, row, m_image->bounds());
 
                 KisTextureTile *tile = new KisTextureTile(tileRect,
-                                                        &m_texturesInfo,
-                                                        emptyTileData,
-                                                        mode,
-                                                        cfg.useOpenGLTextureBuffer(),
-                                                        cfg.numMipmapLevels(),
-                                                        f);
+                                                          &m_texturesInfo,
+                                                          emptyTileData,
+                                                          mode,
+                                                          config.useOpenGLTextureBuffer(),
+                                                          config.numMipmapLevels(),
+                                                          f);
                 m_textureTiles.append(tile);
             }
         }
@@ -231,111 +239,24 @@ void KisOpenGLImageTextures::destroyImageTextureTiles()
     m_storedImageBounds = QRect();
 }
 
-KisOpenGLUpdateInfoSP KisOpenGLImageTextures::updateCache(const QRect& rect)
+KisOpenGLUpdateInfoSP KisOpenGLImageTextures::updateCache(const QRect& rect, KisImageSP srcImage)
 {
-    return updateCacheImpl(rect, true);
+    return updateCacheImpl(rect, srcImage, true);
 }
 
 KisOpenGLUpdateInfoSP KisOpenGLImageTextures::updateCacheNoConversion(const QRect& rect)
 {
-    return updateCacheImpl(rect, false);
+    return updateCacheImpl(rect, m_image, false);
 }
 
-KisOpenGLUpdateInfoSP KisOpenGLImageTextures::updateCacheImpl(const QRect& rect, bool convertColorSpace)
+// TODO: add sanity checks about the conformance of the passed srcImage!
+KisOpenGLUpdateInfoSP KisOpenGLImageTextures::updateCacheImpl(const QRect& rect, KisImageSP srcImage, bool convertColorSpace)
 {
-    const KoColorSpace *dstCS = m_tilesDestinationColorSpace;
-
-    ConversionOptions options;
-
-    if (convertColorSpace) {
-        options = ConversionOptions(dstCS, m_renderingIntent, m_conversionFlags);
-    }
-
-    KisOpenGLUpdateInfoSP info = new KisOpenGLUpdateInfo(options);
-
-    QRect updateRect = rect & m_image->bounds();
-    if (updateRect.isEmpty() || !(m_initialized)) return info;
-
-    /**
-     * Why the rect is artificial? That's easy!
-     * It does not represent any real piece of the image. It is
-     * intentionally stretched to get through the overlappping
-     * stripes of neutrality and poke neighbouring tiles.
-     * Thanks to the rect we get the coordinates of all the tiles
-     * involved into update process
-     */
-
-    QRect artificialRect = stretchRect(updateRect, m_texturesInfo.border);
-    artificialRect &= m_image->bounds();
-
-    int firstColumn = xToCol(artificialRect.left());
-    int lastColumn = xToCol(artificialRect.right());
-    int firstRow = yToRow(artificialRect.top());
-    int lastRow = yToRow(artificialRect.bottom());
-
-    QBitArray channelFlags; // empty by default
-
-    if (m_channelFlags.size() != m_image->projection()->colorSpace()->channels().size()) {
-        setChannelFlags(QBitArray());
-    }
-    if (!m_useOcio) { // Ocio does its own channel flipping
-        if (!m_allChannelsSelected) { // and we do it only if necessary
-            channelFlags = m_channelFlags;
-        }
-    }
-
-    qint32 numItems = (lastColumn - firstColumn + 1) * (lastRow - firstRow + 1);
-    info->tileList.reserve(numItems);
-
-    const QRect bounds = m_image->bounds();
-    const int levelOfDetail = m_image->currentLevelOfDetail();
-
-    QRect alignedUpdateRect = updateRect;
-    QRect alignedBounds = bounds;
-
-    if (levelOfDetail) {
-        alignedUpdateRect = KisLodTransform::alignedRect(alignedUpdateRect, levelOfDetail);
-        alignedBounds = KisLodTransform::alignedRect(alignedBounds, levelOfDetail);
-    }
-
-    for (int col = firstColumn; col <= lastColumn; col++) {
-        for (int row = firstRow; row <= lastRow; row++) {
-
-            const QRect tileRect = calculateTileRect(col, row);
-            const QRect tileTextureRect = stretchRect(tileRect, m_texturesInfo.border);
-
-            QRect alignedTileTextureRect = levelOfDetail ?
-                KisLodTransform::alignedRect(tileTextureRect, levelOfDetail) :
-                tileTextureRect;
-
-            KisTextureTileUpdateInfoSP tileInfo(
-                new KisTextureTileUpdateInfo(col, row,
-                                             alignedTileTextureRect,
-                                             alignedUpdateRect,
-                                             alignedBounds,
-                                             levelOfDetail));
-            // Don't update empty tiles
-            if (tileInfo->valid()) {
-                tileInfo->retrieveData(m_image, channelFlags, m_onlyOneChannelSelected, m_selectedChannelIndex);
-
-                if (convertColorSpace) {
-                    tileInfo->convertTo(dstCS, m_renderingIntent, m_conversionFlags);
-                }
-
-                info->tileList.append(tileInfo);
-            }
-            else {
-                dbgUI << "Trying to create an empty tileinfo record" << col << row << tileTextureRect << updateRect << m_image->bounds();
-            }
-        }
-    }
-
-    info->assignDirtyImageRect(rect);
-    info->assignLevelOfDetail(levelOfDetail);
-    return info;
+    if (!m_initialized) return new KisOpenGLUpdateInfo();
+    return m_updateInfoBuilder.buildUpdateInfo(rect, srcImage, convertColorSpace);
 }
 
-void KisOpenGLImageTextures::recalculateCache(KisUpdateInfoSP info)
+void KisOpenGLImageTextures::recalculateCache(KisUpdateInfoSP info, bool blockMipmapRegeneration)
 {
     if (!m_initialized) {
         dbgUI << "OpenGL: Tried to edit image texture cache before it was initialized.";
@@ -350,7 +271,7 @@ void KisOpenGLImageTextures::recalculateCache(KisUpdateInfoSP info)
         KisTextureTile *tile = getTextureTileCR(tileInfo->tileCol(), tileInfo->tileRow());
         KIS_ASSERT_RECOVER_RETURN(tile);
 
-        tile->update(*tileInfo);
+        tile->update(*tileInfo, blockMipmapRegeneration);
     }
 }
 
@@ -374,8 +295,17 @@ void KisOpenGLImageTextures::generateCheckerTexture(const QImage &checkImage)
         if (checkImage.width() != BACKGROUND_TEXTURE_SIZE || checkImage.height() != BACKGROUND_TEXTURE_SIZE) {
             img = checkImage.scaled(BACKGROUND_TEXTURE_SIZE, BACKGROUND_TEXTURE_SIZE);
         }
-        f->glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, BACKGROUND_TEXTURE_SIZE, BACKGROUND_TEXTURE_SIZE,
-                    0, GL_BGRA, GL_UNSIGNED_BYTE, img.constBits());
+        GLint format = GL_BGRA, internalFormat = GL_RGBA8;
+        if (KisOpenGL::hasOpenGLES()) {
+            if (ctx->hasExtension(QByteArrayLiteral("GL_EXT_texture_format_BGRA8888"))) {
+                format = GL_BGRA_EXT;
+                internalFormat = GL_BGRA8_EXT;
+            } else {
+                format = GL_RGBA;
+            }
+        }
+        f->glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, BACKGROUND_TEXTURE_SIZE, BACKGROUND_TEXTURE_SIZE,
+                        0, format, GL_UNSIGNED_BYTE, img.constBits());
     }
     else {
         dbgUI << "OpenGL: Tried to generate checker texture before OpenGL was initialized.";
@@ -412,6 +342,11 @@ void KisOpenGLImageTextures::slotImageSizeChanged(qint32 /*w*/, qint32 /*h*/)
     createImageTextureTiles();
 }
 
+KisOpenGLUpdateInfoBuilder &KisOpenGLImageTextures::updateInfoBuilder()
+{
+    return m_updateInfoBuilder;
+}
+
 void KisOpenGLImageTextures::setMonitorProfile(const KoColorProfile *monitorProfile, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags)
 {
     //dbgUI << "Setting monitor profile to" << monitorProfile->name() << renderingIntent << conversionFlags;
@@ -419,28 +354,47 @@ void KisOpenGLImageTextures::setMonitorProfile(const KoColorProfile *monitorProf
     m_renderingIntent = renderingIntent;
     m_conversionFlags = conversionFlags;
 
+    m_updateInfoBuilder.setConversionOptions(
+        ConversionOptions(m_tilesDestinationColorSpace,
+                          m_renderingIntent,
+                          m_conversionFlags));
+
     createImageTextureTiles();
 }
 
 void KisOpenGLImageTextures::setChannelFlags(const QBitArray &channelFlags)
 {
-    m_channelFlags = channelFlags;
+    QBitArray targetChannelFlags = channelFlags;
     int selectedChannels = 0;
     const KoColorSpace *projectionCs = m_image->projection()->colorSpace();
     QList<KoChannelInfo*> channelInfo = projectionCs->channels();
 
-    if (m_channelFlags.size() != channelInfo.size()) {
-        m_channelFlags = QBitArray();
+    if (targetChannelFlags.size() != channelInfo.size()) {
+        targetChannelFlags = QBitArray();
     }
 
-    for (int i = 0; i < m_channelFlags.size(); ++i) {
-        if (m_channelFlags.testBit(i) && channelInfo[i]->channelType() == KoChannelInfo::COLOR) {
+    int selectedChannelIndex = -1;
+
+    for (int i = 0; i < targetChannelFlags.size(); ++i) {
+        if (targetChannelFlags.testBit(i) && channelInfo[i]->channelType() == KoChannelInfo::COLOR) {
             selectedChannels++;
-            m_selectedChannelIndex = i;
+            selectedChannelIndex = i;
         }
     }
-    m_allChannelsSelected = (selectedChannels == m_channelFlags.size());
-    m_onlyOneChannelSelected = (selectedChannels == 1);
+    const bool allChannelsSelected = (selectedChannels == targetChannelFlags.size());
+    const bool onlyOneChannelSelected = (selectedChannels == 1);
+
+    // OCIO has its own channel swizzling
+    if (allChannelsSelected || m_useOcio) {
+        m_updateInfoBuilder.setChannelFlags(QBitArray(), false, -1);
+    } else {
+        m_updateInfoBuilder.setChannelFlags(targetChannelFlags, onlyOneChannelSelected, selectedChannelIndex);
+    }
+}
+
+void KisOpenGLImageTextures::setProofingConfig(KisProofingConfigurationSP proofingConfig)
+{
+    m_updateInfoBuilder.setProofingConfig(proofingConfig);
 }
 
 void KisOpenGLImageTextures::getTextureSize(KisGLTexturesInfo *texturesInfo)
@@ -465,6 +419,10 @@ void KisOpenGLImageTextures::getTextureSize(KisGLTexturesInfo *texturesInfo)
 
     texturesInfo->effectiveWidth = texturesInfo->width - 2 * texturesInfo->border;
     texturesInfo->effectiveHeight = texturesInfo->height - 2 * texturesInfo->border;
+
+    m_updateInfoBuilder.setTextureBorder(texturesInfo->border);
+    m_updateInfoBuilder.setEffectiveTextureSize(
+        QSize(texturesInfo->effectiveWidth, texturesInfo->effectiveHeight));
 }
 
 bool KisOpenGLImageTextures::internalColorManagementActive() const
@@ -476,7 +434,6 @@ bool KisOpenGLImageTextures::setInternalColorManagementActive(bool value)
 {
     bool needsFinalRegeneration = m_internalColorManagementActive != value;
 
-
     if (needsFinalRegeneration) {
         m_internalColorManagementActive = value;
         createImageTextureTiles();
@@ -493,9 +450,22 @@ void KisOpenGLImageTextures::updateTextureFormat()
     QOpenGLContext *ctx = QOpenGLContext::currentContext();
     if (!(m_image && ctx)) return;
 
-    m_texturesInfo.internalFormat = GL_RGBA8;
-    m_texturesInfo.type = GL_UNSIGNED_BYTE;
-    m_texturesInfo.format = GL_BGRA;
+    if (!KisOpenGL::hasOpenGLES()) {
+        m_texturesInfo.internalFormat = GL_RGBA8;
+        m_texturesInfo.type = GL_UNSIGNED_BYTE;
+        m_texturesInfo.format = GL_BGRA;
+    } else {
+        m_texturesInfo.internalFormat = GL_BGRA8_EXT;
+        m_texturesInfo.type = GL_UNSIGNED_BYTE;
+        m_texturesInfo.format = GL_BGRA_EXT;
+        if(!ctx->hasExtension(QByteArrayLiteral("GL_EXT_texture_format_BGRA8888"))) {
+            // The red and blue channels are swapped, but it will be re-swapped
+            // by texture swizzle mask set in KisTextureTile::setTextureParameters
+            m_texturesInfo.internalFormat = GL_RGBA8;
+            m_texturesInfo.type = GL_UNSIGNED_BYTE;
+            m_texturesInfo.format = GL_RGBA;
+        }
+    }
 
     KoID colorModelId = m_image->colorSpace()->colorModelId();
     KoID colorDepthId = m_image->colorSpace()->colorDepthId();
@@ -508,7 +478,10 @@ void KisOpenGLImageTextures::updateTextureFormat()
     if (colorModelId == RGBAColorModelID) {
         if (colorDepthId == Float16BitsColorDepthID) {
 
-            if (ctx->hasExtension("GL_ARB_texture_float")) {
+            if (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3()) {
+                m_texturesInfo.internalFormat = GL_RGBA16F;
+                dbgUI << "Using half (GLES or GL3)";
+            } else if (ctx->hasExtension("GL_ARB_texture_float")) {
                 m_texturesInfo.internalFormat = GL_RGBA16F_ARB;
                 dbgUI << "Using ARB half";
             }
@@ -522,7 +495,11 @@ void KisOpenGLImageTextures::updateTextureFormat()
             haveBuiltInOpenExr = true;
 #endif
 
-            if (haveBuiltInOpenExr && ctx->hasExtension("GL_ARB_half_float_pixel")) {
+            if (haveBuiltInOpenExr && (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3())) {
+                m_texturesInfo.type = GL_HALF_FLOAT;
+                destinationColorDepthId = Float16BitsColorDepthID;
+                dbgUI << "Pixel type half (GLES or GL3)";
+            } else if (haveBuiltInOpenExr && ctx->hasExtension("GL_ARB_half_float_pixel")) {
                 m_texturesInfo.type = GL_HALF_FLOAT_ARB;
                 destinationColorDepthId = Float16BitsColorDepthID;
                 dbgUI << "Pixel type half";
@@ -534,7 +511,10 @@ void KisOpenGLImageTextures::updateTextureFormat()
             m_texturesInfo.format = GL_RGBA;
         }
         else if (colorDepthId == Float32BitsColorDepthID) {
-            if (ctx->hasExtension("GL_ARB_texture_float")) {
+            if (KisOpenGL::hasOpenGLES() || KisOpenGL::hasOpenGL3()) {
+                m_texturesInfo.internalFormat = GL_RGBA32F;
+                dbgUI << "Using float (GLES or GL3)";
+            } else if (ctx->hasExtension("GL_ARB_texture_float")) {
                 m_texturesInfo.internalFormat = GL_RGBA32F_ARB;
                 dbgUI << "Using ARB float";
             } else if (ctx->hasExtension("GL_ATI_texture_float")) {
@@ -547,26 +527,30 @@ void KisOpenGLImageTextures::updateTextureFormat()
             destinationColorDepthId = Float32BitsColorDepthID;
         }
         else if (colorDepthId == Integer16BitsColorDepthID) {
-            m_texturesInfo.internalFormat = GL_RGBA16;
-            m_texturesInfo.type = GL_UNSIGNED_SHORT;
-            m_texturesInfo.format = GL_BGRA;
-            destinationColorDepthId = Integer16BitsColorDepthID;
-            dbgUI << "Using 16 bits rgba";
+            if (!KisOpenGL::hasOpenGLES()) {
+                m_texturesInfo.internalFormat = GL_RGBA16;
+                m_texturesInfo.type = GL_UNSIGNED_SHORT;
+                m_texturesInfo.format = GL_BGRA;
+                destinationColorDepthId = Integer16BitsColorDepthID;
+                dbgUI << "Using 16 bits rgba";
+            }
+            // TODO: for ANGLE, see if we can convert to 16f to support 10-bit display
         }
     }
     else {
         // We will convert the colorspace to 16 bits rgba, instead of 8 bits
-        if (colorDepthId == Integer16BitsColorDepthID) {
+        if (colorDepthId == Integer16BitsColorDepthID && !KisOpenGL::hasOpenGLES()) {
             m_texturesInfo.internalFormat = GL_RGBA16;
             m_texturesInfo.type = GL_UNSIGNED_SHORT;
             m_texturesInfo.format = GL_BGRA;
             destinationColorDepthId = Integer16BitsColorDepthID;
             dbgUI << "Using conversion to 16 bits rgba";
         }
+        // TODO: for ANGLE, see if we can convert to 16f to support 10-bit display
     }
 
     if (!m_internalColorManagementActive &&
-        colorModelId != destinationColorModelId) {
+            colorModelId != destinationColorModelId) {
 
         KisConfig cfg;
         KisConfig::OcioColorManagementMode cm = cfg.ocioColorManagementMode();
@@ -580,7 +564,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
                                        "OpenColorIO will now be deactivated."));
         }
 
-        warnUI << "WARNING: Internal color management was forcely enabled";
+        warnUI << "WARNING: Internal color management was forcibly enabled";
         warnUI << "Color Management Mode: " << cm;
         warnUI << ppVar(m_image->colorSpace());
         warnUI << ppVar(destinationColorModelId);
@@ -591,9 +575,9 @@ void KisOpenGLImageTextures::updateTextureFormat()
     }
 
     const KoColorProfile *profile =
-        m_internalColorManagementActive ||
-        colorModelId != destinationColorModelId ?
-        m_monitorProfile : m_image->colorSpace()->profile();
+            m_internalColorManagementActive ||
+            colorModelId != destinationColorModelId ?
+                m_monitorProfile : m_image->colorSpace()->profile();
 
     /**
      * TODO: add an optimization so that the tile->convertTo() method
@@ -601,8 +585,13 @@ void KisOpenGLImageTextures::updateTextureFormat()
      */
 
     m_tilesDestinationColorSpace =
-        KoColorSpaceRegistry::instance()->colorSpace(destinationColorModelId.id(),
-                                                     destinationColorDepthId.id(),
-                                                     profile);
+            KoColorSpaceRegistry::instance()->colorSpace(destinationColorModelId.id(),
+                                                         destinationColorDepthId.id(),
+                                                         profile);
+
+    m_updateInfoBuilder.setConversionOptions(
+        ConversionOptions(m_tilesDestinationColorSpace,
+                          m_renderingIntent,
+                          m_conversionFlags));
 }
 
