@@ -20,6 +20,7 @@
 
 #include <QVector>
 #include <QMap>
+#include <QOpenGLFunctions>
 
 #include "kritaui_export.h"
 
@@ -27,12 +28,17 @@
 
 #include "canvas/kis_update_info.h"
 #include "opengl/kis_texture_tile.h"
+#include "KisOpenGLUpdateInfoBuilder.h"
 
 class KisOpenGLImageTextures;
-class QOpenGLFunctions;
 typedef KisSharedPtr<KisOpenGLImageTextures> KisOpenGLImageTexturesSP;
 
 class KoColorProfile;
+class KisTextureTileUpdateInfoPoolCollection;
+typedef QSharedPointer<KisTextureTileInfoPool> KisTextureTileInfoPoolSP;
+
+class KisProofingConfiguration;
+typedef QSharedPointer<KisProofingConfiguration> KisProofingConfigurationSP;
 
 /**
  * A set of OpenGL textures that contains the projection of a KisImage.
@@ -79,6 +85,7 @@ public:
     void initGL(QOpenGLFunctions *f);
 
     void setChannelFlags(const QBitArray &channelFlags);
+    void setProofingConfig(KisProofingConfigurationSP);
 
     bool internalColorManagementActive() const;
     bool setInternalColorManagementActive(bool value);
@@ -125,13 +132,15 @@ public:
         return 1.0 / m_texturesInfo.width;
     }
 
-    KisOpenGLUpdateInfoSP updateCache(const QRect& rect);
+    KisOpenGLUpdateInfoSP updateCache(const QRect& rect, KisImageSP srcImage);
     KisOpenGLUpdateInfoSP updateCacheNoConversion(const QRect& rect);
 
-    void recalculateCache(KisUpdateInfoSP info);
+    void recalculateCache(KisUpdateInfoSP info, bool blockMipmapRegeneration);
 
     void slotImageSizeChanged(qint32 w, qint32 h);
 
+    KisOpenGLUpdateInfoBuilder& updateInfoBuilder();
+
 protected:
 
     KisOpenGLImageTextures(KisImageWSP image, const KoColorProfile *monitorProfile,
@@ -146,12 +155,10 @@ protected:
 
 private:
 
-    QRect calculateTileRect(int col, int row) const;
-
     void getTextureSize(KisGLTexturesInfo *texturesInfo);
 
     void updateTextureFormat();
-    KisOpenGLUpdateInfoSP updateCacheImpl(const QRect& rect, bool convertColorSpace);
+    KisOpenGLUpdateInfoSP updateCacheImpl(const QRect& rect, KisImageSP srcImage, bool convertColorSpace);
 
 private:
     KisImageWSP m_image;
@@ -165,7 +172,7 @@ private:
      * then effectively, there is no conversion happens. That is used
      * for working with OCIO.
      */
-    const KoColorSpace* m_tilesDestinationColorSpace;
+    const KoColorSpace *m_tilesDestinationColorSpace;
 
     /**
      * Shows whether the internal color management should be enabled or not.
@@ -182,14 +189,12 @@ private:
     QVector<KisTextureTile*> m_textureTiles;
 
     QOpenGLFunctions *m_glFuncs;
-    QBitArray m_channelFlags;
-    bool m_allChannelsSelected;
-    bool m_onlyOneChannelSelected;
-    int m_selectedChannelIndex;
 
     bool m_useOcio;
     bool m_initialized;
 
+    KisOpenGLUpdateInfoBuilder m_updateInfoBuilder;
+
 private:
     typedef QMap<KisImageWSP, KisOpenGLImageTextures*> ImageTexturesMap;
     static ImageTexturesMap imageTexturesMap;
