@@ -26,25 +26,45 @@
 #include <KoPointerEvent.h>
 #include <KoCanvasResourceManager.h>
 
-#include <kis_distance_information.h>
+#include "kis_algebra_2d.h"
+#include "kis_distance_information.h"
 #include "kis_painting_information_builder.h"
-#include "kis_recording_adapter.h"
 #include "kis_image.h"
 #include "kis_painter.h"
 #include <brushengine/kis_paintop_preset.h>
 #include <brushengine/kis_paintop_utils.h>
 
 #include "kis_update_time_monitor.h"
+#include "kis_stabilized_events_sampler.h"
+#include "KisStabilizerDelayedPaintHelper.h"
+#include "kis_config.h"
 
+#include "kis_random_source.h"
+#include "KisPerStrokeRandomSource.h"
+
+#include "strokes/freehand_stroke.h"
+#include "strokes/KisFreehandStrokeInfo.h"
 
 #include <math.h>
 
 //#define DEBUG_BEZIER_CURVES
 
+// Factor by which to scale the airbrush timer's interval, relative to the actual airbrushing rate.
+// Setting this less than 1 makes the timer-generated pseudo-events happen faster than the desired
+// airbrush rate, which can improve responsiveness.
+const qreal AIRBRUSH_INTERVAL_FACTOR = 0.5;
+
+// The amount of time, in milliseconds, to allow between updates of the spacing information. Only
+// used when spacing updates between dabs are enabled.
+const qreal SPACING_UPDATE_INTERVAL = 50.0;
+
+// The amount of time, in milliseconds, to allow between updates of the timing information. Only
+// used when airbrushing.
+const qreal TIMING_UPDATE_INTERVAL = 50.0;
+
 struct KisToolFreehandHelper::Private
 {
     KisPaintingInformationBuilder *infoBuilder;
-    KisRecordingAdapter *recordingAdapter;
     KisStrokesFacade *strokesFacade;
 
     KUndo2MagicString transactionText;
@@ -57,7 +77,7 @@ struct KisToolFreehandHelper::Private
     QTime strokeTime;
     QTimer strokeTimeoutTimer;
 
-    QVector<PainterInfo*> painterInfos;
+    QVector<KisFreehandStrokeInfo*> strokeInfos;
     KisResourcesSnapshotSP resources;
     KisStrokeId strokeId;
 
@@ -66,44 +86,67 @@ struct KisToolFreehandHelper::Private
 
     KisSmoothingOptionsSP smoothingOptions;
 
+    // fake random sources for hovering outline *only*
+    KisRandomSourceSP fakeDabRandomSource;
+    KisPerStrokeRandomSourceSP fakeStrokeRandomSource;
+
+    // Timer used to generate paint updates periodically even without input events. This is only
+    // used for paintops that depend on timely updates even when the cursor is not moving, e.g. for
+    // airbrushing effects.
     QTimer airbrushingTimer;
 
     QList<KisPaintInformation> history;
     QList<qreal> distanceHistory;
 
-    KisPaintOpUtils::PositionHistory lastOutlinePos;
+    // Keeps track of past cursor positions. This is used to determine the drawing angle when
+    // drawing the brush outline or starting a stroke.
+    KisPaintOpUtils::PositionHistory lastCursorPos;
 
     // Stabilizer data
+    bool usingStabilizer;
     QQueue<KisPaintInformation> stabilizerDeque;
-    KisPaintInformation stabilizerLastPaintInfo;
     QTimer stabilizerPollTimer;
+    KisStabilizedEventsSampler stabilizedSampler;
+    KisStabilizerDelayedPaintHelper stabilizerDelayedPaintHelper;
+
+    QTimer asynchronousUpdatesThresholdTimer;
 
     int canvasRotation;
     bool canvasMirroredH;
 
-    KisPaintInformation
-    getStabilizedPaintInfo(const QQueue<KisPaintInformation> &queue,
-                           const KisPaintInformation &lastPaintInfo);
-
     qreal effectiveSmoothnessDistance() const;
 };
 
 
 KisToolFreehandHelper::KisToolFreehandHelper(KisPaintingInformationBuilder *infoBuilder,
                                              const KUndo2MagicString &transactionText,
-                                             KisRecordingAdapter *recordingAdapter)
+                                             KisSmoothingOptions *smoothingOptions)
     : m_d(new Private())
 {
     m_d->infoBuilder = infoBuilder;
-    m_d->recordingAdapter = recordingAdapter;
     m_d->transactionText = transactionText;
-    m_d->smoothingOptions = KisSmoothingOptionsSP(new KisSmoothingOptions());
+    m_d->smoothingOptions = KisSmoothingOptionsSP(
+                smoothingOptions ? smoothingOptions : new KisSmoothingOptions());
     m_d->canvasRotation = 0;
 
+    m_d->fakeDabRandomSource = new KisRandomSource();
+    m_d->fakeStrokeRandomSource = new KisPerStrokeRandomSource();
+
     m_d->strokeTimeoutTimer.setSingleShot(true);
     connect(&m_d->strokeTimeoutTimer, SIGNAL(timeout()), SLOT(finishStroke()));
     connect(&m_d->airbrushingTimer, SIGNAL(timeout()), SLOT(doAirbrushing()));
+    connect(&m_d->asynchronousUpdatesThresholdTimer, SIGNAL(timeout()), SLOT(doAsynchronousUpdate()));
     connect(&m_d->stabilizerPollTimer, SIGNAL(timeout()), SLOT(stabilizerPollAndPaint()));
+    connect(m_d->smoothingOptions.data(), SIGNAL(sigSmoothingTypeChanged()), SLOT(slotSmoothingTypeChanged()));
+
+    m_d->stabilizerDelayedPaintHelper.setPaintLineCallback(
+                [this](const KisPaintInformation &pi1, const KisPaintInformation &pi2) {
+                    paintLine(pi1, pi2);
+                });
+    m_d->stabilizerDelayedPaintHelper.setUpdateOutlineCallback(
+                [this]() {
+                    emit requestExplicitUpdateOutline();
+                });
 }
 
 KisToolFreehandHelper::~KisToolFreehandHelper()
@@ -123,18 +166,25 @@ KisSmoothingOptionsSP KisToolFreehandHelper::smoothingOptions() const
 
 QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos,
                                                    const KoPointerEvent *event,
-                                                   const KisPaintOpSettings *globalSettings,
+                                                   const KisPaintOpSettingsSP globalSettings,
                                                    KisPaintOpSettings::OutlineMode mode) const
 {
-    const KisPaintOpSettings *settings = globalSettings;
+    KisPaintOpSettingsSP settings = globalSettings;
     KisPaintInformation info = m_d->infoBuilder->hover(savedCursorPos, event);
+    QPointF prevPoint = m_d->lastCursorPos.pushThroughHistory(savedCursorPos);
+    qreal startAngle = KisAlgebra2D::directionBetweenPoints(prevPoint, savedCursorPos, 0);
     info.setCanvasRotation(m_d->canvasRotation);
     info.setCanvasHorizontalMirrorState( m_d->canvasMirroredH );
-    KisDistanceInformation distanceInfo(m_d->lastOutlinePos.pushThroughHistory(savedCursorPos), 0);
+    KisDistanceInformation distanceInfo(prevPoint, startAngle);
 
-    if (!m_d->painterInfos.isEmpty()) {
+    if (!m_d->strokeInfos.isEmpty()) {
         settings = m_d->resources->currentPaintOpPreset()->settings();
-        info = m_d->previousPaintInformation;
+        if (m_d->stabilizerDelayedPaintHelper.running() &&
+                m_d->stabilizerDelayedPaintHelper.hasLastPaintInformation()) {
+            info = m_d->stabilizerDelayedPaintHelper.lastPaintInformation();
+        } else {
+            info = m_d->previousPaintInformation;
+        }
 
         /**
          * When LoD mode is active it may happen that the helper has
@@ -146,7 +196,7 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
          * object.
          */
         KisDistanceInformation *buddyDistance =
-            m_d->painterInfos.first()->buddyDragDistance();
+            m_d->strokeInfos.first()->buddyDragDistance();
 
         if (buddyDistance) {
             /**
@@ -156,20 +206,21 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
              * local copy of the coordinates.
              */
             distanceInfo = *buddyDistance;
-            distanceInfo.overrideLastValues(m_d->lastOutlinePos.pushThroughHistory(savedCursorPos), 0);
+            distanceInfo.overrideLastValues(prevPoint, startAngle);
 
-        } else if (m_d->painterInfos.first()->dragDistance->isStarted()) {
-            distanceInfo = *m_d->painterInfos.first()->dragDistance;
+        } else if (m_d->strokeInfos.first()->dragDistance->isStarted()) {
+            distanceInfo = *m_d->strokeInfos.first()->dragDistance;
         }
     }
 
     KisPaintInformation::DistanceInformationRegistrar registrar =
         info.registerDistanceInformation(&distanceInfo);
 
+    info.setRandomSource(m_d->fakeDabRandomSource);
+    info.setPerStrokeRandomSource(m_d->fakeStrokeRandomSource);
+
     QPainterPath outline = settings->brushOutline(info, mode);
 
-
-
     if (m_d->resources &&
         m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER &&
         m_d->smoothingOptions->useDelayDistance()) {
@@ -183,23 +234,31 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
     return outline;
 }
 
+void KisToolFreehandHelper::cursorMoved(const QPointF &cursorPos)
+{
+    m_d->lastCursorPos.pushThroughHistory(cursorPos);
+}
+
 void KisToolFreehandHelper::initPaint(KoPointerEvent *event,
+                                      const QPointF &pixelCoords,
                                       KoCanvasResourceManager *resourceManager,
                                       KisImageWSP image, KisNodeSP currentNode,
                                       KisStrokesFacade *strokesFacade,
-                                      KisPostExecutionUndoAdapter *undoAdapter,
                                       KisNodeSP overrideNode,
                                       KisDefaultBoundsBaseSP bounds)
 {
+    QPointF prevPoint = m_d->lastCursorPos.pushThroughHistory(pixelCoords);
+    m_d->strokeTime.start();
     KisPaintInformation pi =
-        m_d->infoBuilder->startStroke(event, elapsedStrokeTime());
+        m_d->infoBuilder->startStroke(event, elapsedStrokeTime(), resourceManager);
+    qreal startAngle = KisAlgebra2D::directionBetweenPoints(prevPoint, pixelCoords, 0.0);
 
-    initPaintImpl(pi,
+    initPaintImpl(startAngle,
+                  pi,
                   resourceManager,
                   image,
                   currentNode,
                   strokesFacade,
-                  undoAdapter,
                   overrideNode,
                   bounds);
 }
@@ -209,17 +268,15 @@ bool KisToolFreehandHelper::isRunning() const
     return m_d->strokeId;
 }
 
-void KisToolFreehandHelper::initPaintImpl(const KisPaintInformation &previousPaintInformation,
+void KisToolFreehandHelper::initPaintImpl(qreal startAngle,
+                                          const KisPaintInformation &pi,
                                           KoCanvasResourceManager *resourceManager,
                                           KisImageWSP image,
                                           KisNodeSP currentNode,
                                           KisStrokesFacade *strokesFacade,
-                                          KisPostExecutionUndoAdapter *undoAdapter,
                                           KisNodeSP overrideNode,
                                           KisDefaultBoundsBaseSP bounds)
 {
-    Q_UNUSED(overrideNode);
-
     m_d->strokesFacade = strokesFacade;
 
     m_d->haveTangent = false;
@@ -227,46 +284,55 @@ void KisToolFreehandHelper::initPaintImpl(const KisPaintInformation &previousPai
 
     m_d->hasPaintAtLeastOnce = false;
 
-    m_d->strokeTime.start();
-
-    m_d->previousPaintInformation = previousPaintInformation;
-
-    createPainters(m_d->painterInfos,
-                   m_d->previousPaintInformation.pos(),
-                   m_d->previousPaintInformation.currentTime());
+    m_d->previousPaintInformation = pi;
 
     m_d->resources = new KisResourcesSnapshot(image,
                                               currentNode,
-                                              undoAdapter,
                                               resourceManager,
                                               bounds);
-
     if(overrideNode) {
         m_d->resources->setCurrentNode(overrideNode);
     }
 
-    if(m_d->recordingAdapter) {
-        m_d->recordingAdapter->startStroke(image, m_d->resources);
-    }
+    const bool airbrushing = m_d->resources->needsAirbrushing();
+    const bool useSpacingUpdates = m_d->resources->needsSpacingUpdates();
+
+    KisDistanceInitInfo startDistInfo(m_d->previousPaintInformation.pos(),
+                                      startAngle,
+                                      useSpacingUpdates ? SPACING_UPDATE_INTERVAL : LONG_TIME,
+                                      airbrushing ? TIMING_UPDATE_INTERVAL : LONG_TIME,
+                                      0);
+    KisDistanceInformation startDist = startDistInfo.makeDistInfo();
+
+    createPainters(m_d->strokeInfos,
+                   startDist);
 
     KisStrokeStrategy *stroke =
-        new FreehandStrokeStrategy(m_d->resources->needsIndirectPainting(),
-                                   m_d->resources->indirectPaintingCompositeOp(),
-                                   m_d->resources, m_d->painterInfos, m_d->transactionText);
+        new FreehandStrokeStrategy(m_d->resources, m_d->strokeInfos, m_d->transactionText);
 
     m_d->strokeId = m_d->strokesFacade->startStroke(stroke);
 
     m_d->history.clear();
     m_d->distanceHistory.clear();
 
-    if(m_d->resources->needsAirbrushing()) {
-        m_d->airbrushingTimer.setInterval(m_d->resources->airbrushingRate());
+    if (airbrushing) {
+        m_d->airbrushingTimer.setInterval(computeAirbrushTimerInterval());
         m_d->airbrushingTimer.start();
+    } else if (m_d->resources->presetNeedsAsynchronousUpdates()) {
+        m_d->asynchronousUpdatesThresholdTimer.setInterval(80 /* msec */);
+        m_d->asynchronousUpdatesThresholdTimer.start();
     }
 
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) {
         stabilizerStart(m_d->previousPaintInformation);
     }
+
+    // If airbrushing, paint an initial dab immediately. This is a workaround for an issue where
+    // some paintops (Dyna, Particle, Sketch) might never initialize their spacing/timing
+    // information until paintAt is called.
+    if (airbrushing) {
+        paintAt(pi);
+    }
 }
 
 void KisToolFreehandHelper::paintBezierSegment(KisPaintInformation pi1, KisPaintInformation pi2,
@@ -369,7 +435,7 @@ qreal KisToolFreehandHelper::Private::effectiveSmoothnessDistance() const
     return effectiveSmoothnessDistance;
 }
 
-void KisToolFreehandHelper::paint(KoPointerEvent *event)
+void KisToolFreehandHelper::paintEvent(KoPointerEvent *event)
 {
     KisPaintInformation info =
             m_d->infoBuilder->continueStroke(event,
@@ -379,6 +445,11 @@ void KisToolFreehandHelper::paint(KoPointerEvent *event)
 
     KisUpdateTimeMonitor::instance()->reportMouseMove(info.pos());
 
+    paint(info);
+}
+
+void KisToolFreehandHelper::paint(KisPaintInformation &info)
+{
     /**
      * Smooth the coordinates out using the history and the
      * distance. This is a heavily modified version of an algo used in
@@ -392,7 +463,7 @@ void KisToolFreehandHelper::paint(KoPointerEvent *event)
      * 2) There is no 'Quality' parameter, since the number of samples
      *    is calculated automatically
      *
-     * 3) 'Tail Aggressiveness' is used for controling the end of the
+     * 3) 'Tail Aggressiveness' is used for controlling the end of the
      *    stroke
      *
      * 4) The formila is a little bit different: 'Distance' parameter
@@ -503,20 +574,34 @@ void KisToolFreehandHelper::paint(KoPointerEvent *event)
             QPointF newTangent = (info.pos() - m_d->olderPaintInformation.pos()) /
                 qMax(qreal(1.0), info.currentTime() - m_d->olderPaintInformation.currentTime());
 
-            paintBezierSegment(m_d->olderPaintInformation, m_d->previousPaintInformation,
-                               m_d->previousTangent, newTangent);
+            if (newTangent.isNull() || m_d->previousTangent.isNull())
+            {
+                paintLine(m_d->previousPaintInformation, info);
+            } else {
+                paintBezierSegment(m_d->olderPaintInformation, m_d->previousPaintInformation,
+                                   m_d->previousTangent, newTangent);
+            }
 
             m_d->previousTangent = newTangent;
         }
         m_d->olderPaintInformation = m_d->previousPaintInformation;
-        m_d->strokeTimeoutTimer.start(100);
+
+        // Enable stroke timeout only when not airbrushing.
+        if (!m_d->airbrushingTimer.isActive()) {
+            m_d->strokeTimeoutTimer.start(100);
+        }
     }
     else if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::NO_SMOOTHING){
         paintLine(m_d->previousPaintInformation, info);
     }
 
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) {
-        m_d->stabilizerLastPaintInfo = info;
+        m_d->stabilizedSampler.addEvent(info);
+        if (m_d->stabilizerDelayedPaintHelper.running()) {
+            // Paint here so we don't have to rely on the timer
+            // This is just a tricky source for a relatively stable 7ms "timer"
+            m_d->stabilizerDelayedPaintHelper.paintSome();
+        }
     } else {
         m_d->previousPaintInformation = info;
     }
@@ -539,6 +624,10 @@ void KisToolFreehandHelper::endPaint()
         m_d->airbrushingTimer.stop();
     }
 
+    if (m_d->asynchronousUpdatesThresholdTimer.isActive()) {
+        m_d->asynchronousUpdatesThresholdTimer.stop();
+    }
+
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) {
         stabilizerEnd();
     }
@@ -549,14 +638,13 @@ void KisToolFreehandHelper::endPaint()
      * Please note that we are not in MT here, so no mutex
      * is needed
      */
-    m_d->painterInfos.clear();
+    m_d->strokeInfos.clear();
+
+    // last update to complete rendering if there is still something pending
+    doAsynchronousUpdate(true);
 
     m_d->strokesFacade->endStroke(m_d->strokeId);
     m_d->strokeId.clear();
-
-    if(m_d->recordingAdapter) {
-        m_d->recordingAdapter->endStroke();
-    }
 }
 
 void KisToolFreehandHelper::cancelPaint()
@@ -569,20 +657,24 @@ void KisToolFreehandHelper::cancelPaint()
         m_d->airbrushingTimer.stop();
     }
 
+    if (m_d->asynchronousUpdatesThresholdTimer.isActive()) {
+        m_d->asynchronousUpdatesThresholdTimer.stop();
+    }
+
     if (m_d->stabilizerPollTimer.isActive()) {
         m_d->stabilizerPollTimer.stop();
     }
 
+    if (m_d->stabilizerDelayedPaintHelper.running()) {
+        m_d->stabilizerDelayedPaintHelper.cancel();
+    }
+
     // see a comment in endPaint()
-    m_d->painterInfos.clear();
+    m_d->strokeInfos.clear();
 
     m_d->strokesFacade->cancelStroke(m_d->strokeId);
     m_d->strokeId.clear();
 
-    if(m_d->recordingAdapter) {
-        //FIXME: not implemented
-        //m_d->recordingAdapter->cancelStroke();
-    }
 }
 
 int KisToolFreehandHelper::elapsedStrokeTime() const
@@ -592,6 +684,7 @@ int KisToolFreehandHelper::elapsedStrokeTime() const
 
 void KisToolFreehandHelper::stabilizerStart(KisPaintInformation firstPaintInfo)
 {
+    m_d->usingStabilizer = true;
     // FIXME: Ugly hack, this is no a "distance" in any way
     int sampleSize = qRound(m_d->effectiveSmoothnessDistance());
     sampleSize = qMax(3, sampleSize);
@@ -601,18 +694,35 @@ void KisToolFreehandHelper::stabilizerStart(KisPaintInformation firstPaintInfo)
     for (int i = sampleSize; i > 0; i--) {
         m_d->stabilizerDeque.enqueue(firstPaintInfo);
     }
-    m_d->stabilizerLastPaintInfo = firstPaintInfo;
 
-    // Poll and draw each millisecond
-    m_d->stabilizerPollTimer.setInterval(1);
+    // Poll and draw regularly
+    KisConfig cfg;
+    int stabilizerSampleSize = cfg.stabilizerSampleSize();
+    m_d->stabilizerPollTimer.setInterval(stabilizerSampleSize);
     m_d->stabilizerPollTimer.start();
+
+    bool delayedPaintEnabled = cfg.stabilizerDelayedPaint();
+    if (delayedPaintEnabled) {
+        m_d->stabilizerDelayedPaintHelper.start(firstPaintInfo);
+    }
+
+    m_d->stabilizedSampler.clear();
+    m_d->stabilizedSampler.addEvent(firstPaintInfo);
 }
 
 KisPaintInformation
-KisToolFreehandHelper::Private::getStabilizedPaintInfo(const QQueue<KisPaintInformation> &queue,
-                                                       const KisPaintInformation &lastPaintInfo)
+KisToolFreehandHelper::getStabilizedPaintInfo(const QQueue<KisPaintInformation> &queue,
+                                              const KisPaintInformation &lastPaintInfo)
 {
-    KisPaintInformation result(lastPaintInfo);
+    KisPaintInformation result(lastPaintInfo.pos(),
+                               lastPaintInfo.pressure(),
+                               lastPaintInfo.xTilt(),
+                               lastPaintInfo.yTilt(),
+                               lastPaintInfo.rotation(),
+                               lastPaintInfo.tangentialPressure(),
+                               lastPaintInfo.perspective(),
+                               elapsedStrokeTime(),
+                               lastPaintInfo.drawingSpeed());
 
     if (queue.size() > 1) {
         QQueue<KisPaintInformation>::const_iterator it = queue.constBegin();
@@ -624,17 +734,17 @@ KisToolFreehandHelper::Private::getStabilizedPaintInfo(const QQueue<KisPaintInfo
         it++;
         int i = 2;
 
-        if (smoothingOptions->stabilizeSensors()) {
+        if (m_d->smoothingOptions->stabilizeSensors()) {
             while (it != end) {
                 qreal k = qreal(i - 1) / i; // coeff for uniform averaging
-                result = KisPaintInformation::mix(k, *it, result);
+                result.KisPaintInformation::mixOtherWithoutTime(k, *it);
                 it++;
                 i++;
             }
         } else{
             while (it != end) {
                 qreal k = qreal(i - 1) / i; // coeff for uniform averaging
-                result = KisPaintInformation::mixOnlyPosition(k, *it, result);
+                result.KisPaintInformation::mixOtherOnlyPosition(k, *it);
                 it++;
                 i++;
             }
@@ -646,66 +756,102 @@ KisToolFreehandHelper::Private::getStabilizedPaintInfo(const QQueue<KisPaintInfo
 
 void KisToolFreehandHelper::stabilizerPollAndPaint()
 {
-    KisPaintInformation newInfo =
-        m_d->getStabilizedPaintInfo(m_d->stabilizerDeque, m_d->stabilizerLastPaintInfo);
+    KisStabilizedEventsSampler::iterator it;
+    KisStabilizedEventsSampler::iterator end;
+    std::tie(it, end) = m_d->stabilizedSampler.range();
+    QVector<KisPaintInformation> delayedPaintTodoItems;
 
-    bool canPaint = true;
+    for (; it != end; ++it) {
+        KisPaintInformation sampledInfo = *it;
 
-    if (m_d->smoothingOptions->useDelayDistance()) {
-        const qreal R = m_d->smoothingOptions->delayDistance() /
-            m_d->resources->effectiveZoom();
+        bool canPaint = true;
 
-        QPointF diff = m_d->stabilizerLastPaintInfo.pos() - m_d->previousPaintInformation.pos();
-        qreal dx = sqrt(pow2(diff.x()) + pow2(diff.y()));
+        if (m_d->smoothingOptions->useDelayDistance()) {
+            const qreal R = m_d->smoothingOptions->delayDistance() /
+                    m_d->resources->effectiveZoom();
 
-        canPaint = dx > R;
+            QPointF diff = sampledInfo.pos() - m_d->previousPaintInformation.pos();
+            qreal dx = sqrt(pow2(diff.x()) + pow2(diff.y()));
+
+            if (!(dx > R)) {
+                if (m_d->resources->needsAirbrushing()) {
+                    sampledInfo.setPos(m_d->previousPaintInformation.pos());
+                }
+                else {
+                    canPaint = false;
+                }
+            }
+        }
+
+        if (canPaint) {
+            KisPaintInformation newInfo = getStabilizedPaintInfo(m_d->stabilizerDeque, sampledInfo);
+
+            if (m_d->stabilizerDelayedPaintHelper.running()) {
+                delayedPaintTodoItems.append(newInfo);
+            } else {
+                paintLine(m_d->previousPaintInformation, newInfo);
+            }
+            m_d->previousPaintInformation = newInfo;
+
+            // Push the new entry through the queue
+            m_d->stabilizerDeque.dequeue();
+            m_d->stabilizerDeque.enqueue(sampledInfo);
+        } else if (m_d->stabilizerDeque.head().pos() != m_d->previousPaintInformation.pos()) {
+
+            QQueue<KisPaintInformation>::iterator it = m_d->stabilizerDeque.begin();
+            QQueue<KisPaintInformation>::iterator end = m_d->stabilizerDeque.end();
+
+            while (it != end) {
+                *it = m_d->previousPaintInformation;
+                ++it;
+            }
+        }
     }
 
-    if (canPaint) {
-        paintLine(m_d->previousPaintInformation, newInfo);
-        m_d->previousPaintInformation = newInfo;
-
-        // Push the new entry through the queue
-        m_d->stabilizerDeque.dequeue();
-        m_d->stabilizerDeque.enqueue(m_d->stabilizerLastPaintInfo);
+    m_d->stabilizedSampler.clear();
 
+    if (m_d->stabilizerDelayedPaintHelper.running()) {
+        m_d->stabilizerDelayedPaintHelper.update(delayedPaintTodoItems);
+    } else {
         emit requestExplicitUpdateOutline();
-
-    } else if (m_d->stabilizerDeque.head().pos() != m_d->previousPaintInformation.pos()) {
-
-        QQueue<KisPaintInformation>::iterator it = m_d->stabilizerDeque.begin();
-        QQueue<KisPaintInformation>::iterator end = m_d->stabilizerDeque.end();
-
-        while (it != end) {
-            *it = m_d->previousPaintInformation;
-            ++it;
-        }
     }
 }
 
 void KisToolFreehandHelper::stabilizerEnd()
 {
-    // FIXME: Ugly hack, this is no a "distance" in any way
-    int sampleSize = m_d->smoothingOptions->smoothnessDistance();
-    assert(sampleSize > 0);
-
     // Stop the timer
     m_d->stabilizerPollTimer.stop();
 
     // Finish the line
-    for (int i = sampleSize; i > 0; i--) {
-        // In each iteration we add the latest paint info and delete the oldest
-        // After `sampleSize` iterations the deque will be filled with the latest
-        // value and we will have reached the end point.
-        if (m_d->smoothingOptions->finishStabilizedCurve()) {
-            stabilizerPollAndPaint();
-        }
+    if (m_d->smoothingOptions->finishStabilizedCurve()) {
+        // Process all the existing events first
+        stabilizerPollAndPaint();
+
+        // Draw the finish line with pending events and a time override
+        m_d->stabilizedSampler.addFinishingEvent(m_d->stabilizerDeque.size());
+        stabilizerPollAndPaint();
     }
+
+    if (m_d->stabilizerDelayedPaintHelper.running()) {
+        m_d->stabilizerDelayedPaintHelper.end();
+    }
+    m_d->usingStabilizer = false;
 }
 
-const KisPaintOp* KisToolFreehandHelper::currentPaintOp() const
+void KisToolFreehandHelper::slotSmoothingTypeChanged()
 {
-    return !m_d->painterInfos.isEmpty() ? m_d->painterInfos.first()->painter->paintOp() : 0;
+    if (!isRunning()) {
+        return;
+    }
+    KisSmoothingOptions::SmoothingType currentSmoothingType =
+            m_d->smoothingOptions->smoothingType();
+    if (m_d->usingStabilizer
+            && (currentSmoothingType != KisSmoothingOptions::STABILIZER)) {
+        stabilizerEnd();
+    } else if (!m_d->usingStabilizer
+            && (currentSmoothingType == KisSmoothingOptions::STABILIZER)) {
+        stabilizerStart(m_d->previousPaintInformation);
+    }
 }
 
 void KisToolFreehandHelper::finishStroke()
@@ -725,39 +871,57 @@ void KisToolFreehandHelper::finishStroke()
 
 void KisToolFreehandHelper::doAirbrushing()
 {
-    if(!m_d->painterInfos.isEmpty()) {
-        paintAt(m_d->previousPaintInformation);
+    // Check that the stroke hasn't ended.
+    if (!m_d->strokeInfos.isEmpty()) {
+
+        // Add a new painting update at a point identical to the previous one, except for the time
+        // and speed information.
+        const KisPaintInformation &prevPaint = m_d->previousPaintInformation;
+        KisPaintInformation nextPaint(prevPaint.pos(),
+                                      prevPaint.pressure(),
+                                      prevPaint.xTilt(),
+                                      prevPaint.yTilt(),
+                                      prevPaint.rotation(),
+                                      prevPaint.tangentialPressure(),
+                                      prevPaint.perspective(),
+                                      elapsedStrokeTime(),
+                                      0.0);
+        paint(nextPaint);
     }
 }
 
-void KisToolFreehandHelper::paintAt(int painterInfoId,
+void KisToolFreehandHelper::doAsynchronousUpdate(bool forceUpdate)
+{
+    m_d->strokesFacade->addJob(m_d->strokeId,
+                               new FreehandStrokeStrategy::UpdateData(forceUpdate));
+}
+
+int KisToolFreehandHelper::computeAirbrushTimerInterval() const
+{
+    qreal realInterval = m_d->resources->airbrushingInterval() * AIRBRUSH_INTERVAL_FACTOR;
+    return qMax(1, qFloor(realInterval));
+}
+
+void KisToolFreehandHelper::paintAt(int strokeInfoId,
                                     const KisPaintInformation &pi)
 {
     m_d->hasPaintAtLeastOnce = true;
     m_d->strokesFacade->addJob(m_d->strokeId,
-                               new FreehandStrokeStrategy::Data(m_d->resources->currentNode(),
-                                                                painterInfoId, pi));
+                               new FreehandStrokeStrategy::Data(strokeInfoId, pi));
 
-    if(m_d->recordingAdapter) {
-        m_d->recordingAdapter->addPoint(pi);
-    }
 }
 
-void KisToolFreehandHelper::paintLine(int painterInfoId,
+void KisToolFreehandHelper::paintLine(int strokeInfoId,
                                       const KisPaintInformation &pi1,
                                       const KisPaintInformation &pi2)
 {
     m_d->hasPaintAtLeastOnce = true;
     m_d->strokesFacade->addJob(m_d->strokeId,
-                               new FreehandStrokeStrategy::Data(m_d->resources->currentNode(),
-                                                                painterInfoId, pi1, pi2));
+                               new FreehandStrokeStrategy::Data(strokeInfoId, pi1, pi2));
 
-    if(m_d->recordingAdapter) {
-        m_d->recordingAdapter->addLine(pi1, pi2);
-    }
 }
 
-void KisToolFreehandHelper::paintBezierCurve(int painterInfoId,
+void KisToolFreehandHelper::paintBezierCurve(int strokeInfoId,
                                              const KisPaintInformation &pi1,
                                              const QPointF &control1,
                                              const QPointF &control2,
@@ -789,20 +953,15 @@ void KisToolFreehandHelper::paintBezierCurve(int painterInfoId,
 
     m_d->hasPaintAtLeastOnce = true;
     m_d->strokesFacade->addJob(m_d->strokeId,
-                               new FreehandStrokeStrategy::Data(m_d->resources->currentNode(),
-                                                                painterInfoId,
+                               new FreehandStrokeStrategy::Data(strokeInfoId,
                                                                 pi1, control1, control2, pi2));
 
-    if(m_d->recordingAdapter) {
-        m_d->recordingAdapter->addCurve(pi1, control1, control2, pi2);
-    }
 }
 
-void KisToolFreehandHelper::createPainters(QVector<PainterInfo*> &painterInfos,
-                                           const QPointF &lastPosition,
-                                           int lastTime)
+void KisToolFreehandHelper::createPainters(QVector<KisFreehandStrokeInfo*> &strokeInfos,
+                                           const KisDistanceInformation &startDist)
 {
-    painterInfos << new PainterInfo(lastPosition, lastTime);
+    strokeInfos << new KisFreehandStrokeInfo(startDist);
 }
 
 void KisToolFreehandHelper::paintAt(const KisPaintInformation &pi)
@@ -831,7 +990,7 @@ int KisToolFreehandHelper::canvasRotation()
 
 void KisToolFreehandHelper::setCanvasRotation(int rotation)
 {
-   m_d->canvasRotation = rotation; 
+   m_d->canvasRotation = rotation;
 }
 bool KisToolFreehandHelper::canvasMirroredH()
 {
@@ -840,6 +999,5 @@ bool KisToolFreehandHelper::canvasMirroredH()
 
 void KisToolFreehandHelper::setCanvasHorizontalMirrorState(bool mirrored)
 {
-   m_d->canvasMirroredH = mirrored; 
+   m_d->canvasMirroredH = mirrored;
 }
-
