@@ -25,6 +25,7 @@
 #include <KoCanvasBase.h>
 #include <KoCanvasController.h>
 #include <KoViewConverter.h>
+#include "kis_canvas2.h"
 
 #include "kis_rectangle_constraint_widget.h"
 
@@ -40,15 +41,17 @@ KisToolRectangleBase::KisToolRectangleBase(KoCanvasBase * canvas, KisToolRectang
     , m_forcedRatio(1.0)
     , m_forcedWidth(0)
     , m_forcedHeight(0)
+    , m_roundCornersX(0)
+    , m_roundCornersY(0)
 {
 }
 
 
 QList<QPointer<QWidget> > KisToolRectangleBase::createOptionWidgets()
 {
-  QList<QPointer<QWidget> > widgetsList = KoToolBase::createOptionWidgets();
+  QList<QPointer<QWidget> > widgetsList = KisToolShape::createOptionWidgets();
 
-  widgetsList.append(new KisRectangleConstraintWidget(0, this));
+  widgetsList.append(new KisRectangleConstraintWidget(0, this, showRoundCornersGUI()));
 
   return widgetsList;
 }
@@ -67,6 +70,12 @@ void KisToolRectangleBase::constraintsChanged(bool forceRatio, bool forceWidth,
   if (ratio < 0.0001f) m_isRatioForced = false;
 }
 
+void KisToolRectangleBase::roundCornersChanged(int rx, int ry)
+{
+    m_roundCornersX = rx;
+    m_roundCornersY = ry;
+}
+
 void KisToolRectangleBase::paint(QPainter& gc, const KoViewConverter &converter)
 {
     if(mode() == KisTool::PAINT_MODE) {
@@ -76,6 +85,13 @@ void KisToolRectangleBase::paint(QPainter& gc, const KoViewConverter &converter)
     KisToolPaint::paint(gc, converter);
 }
 
+void KisToolRectangleBase::activate(KoToolBase::ToolActivation toolActivation, const QSet<KoShape *> &shapes)
+{
+    KisToolShape::activate(toolActivation, shapes);
+
+    emit sigRequestReloadConfig();
+}
+
 void KisToolRectangleBase::deactivate()
 {
     updateArea();
@@ -197,7 +213,7 @@ void KisToolRectangleBase::endPrimaryAction(KoPointerEvent *event)
 
     updateArea();
 
-    finishRect(createRect(m_dragStart, m_dragEnd));
+    finishRect(createRect(m_dragStart, m_dragEnd), m_roundCornersX, m_roundCornersY);
     event->accept();
 }
 
@@ -226,23 +242,37 @@ QRectF KisToolRectangleBase::createRect(const QPointF &start, const QPointF &end
     return result.normalized();
 }
 
+bool KisToolRectangleBase::showRoundCornersGUI() const
+{
+    return true;
+}
+
 void KisToolRectangleBase::paintRectangle(QPainter &gc, const QRectF &imageRect)
 {
     KIS_ASSERT_RECOVER_RETURN(canvas());
 
-    QRect viewRect = pixelToView(imageRect).toAlignedRect();
+    const QRect viewRect = pixelToView(imageRect).toAlignedRect();
+
+    KisCanvas2 *kritaCanvas = dynamic_cast<KisCanvas2*>(canvas());
+    KIS_SAFE_ASSERT_RECOVER_RETURN(kritaCanvas);
+
+    const KisCoordinatesConverter *converter = kritaCanvas->coordinatesConverter();
+    const qreal roundCornersX = converter->effectiveZoom() * m_roundCornersX;
+    const qreal roundCornersY = converter->effectiveZoom() * m_roundCornersY;
 
     QPainterPath path;
-    path.addRect(viewRect);
+
+    if (m_roundCornersX > 0 || m_roundCornersY > 0) {
+        path.addRoundedRect(viewRect,
+                            roundCornersX, roundCornersY);
+    } else {
+        path.addRect(viewRect);
+    }
     paintToolOutline(&gc, path);
 }
 
 void KisToolRectangleBase::updateArea() {
-    QRectF bound;
-    bound.setTopLeft(m_dragStart);
-    bound.setBottomRight(m_dragEnd);
-
-    bound = bound.normalized();
+    const QRectF bound = createRect(m_dragStart, m_dragEnd);
 
     canvas()->updateCanvas(convertToPt(bound).adjusted(-100, -100, +200, +200));
 
