@@ -34,7 +34,7 @@
 #include <KoConfig.h>
 #include <kis_icon.h>
 
-#include <QDesktopServices>
+#include <QStandardPaths>
 
 #include <KoResourcePaths.h>
 
@@ -69,7 +69,7 @@ KisColorSpaceSelector::KisColorSpaceSelector(QWidget* parent) : QWidget(parent),
             this, SLOT(fillCmbProfiles()));
     connect(d->colorSpaceSelector->cmbProfile, SIGNAL(activated(const QString &)),
             this, SLOT(colorSpaceChanged()));
-    connect(d->colorSpaceSelector->bnInstallProfile, SIGNAL(clicked()), 
+    connect(d->colorSpaceSelector->bnInstallProfile, SIGNAL(clicked()),
             this, SLOT(installProfile()));
 
     d->defaultsuffix = " "+i18nc("This is appended to the color profile which is the default for the given colorspace and bit-depth","(Default)");
@@ -88,26 +88,25 @@ KisColorSpaceSelector::~KisColorSpaceSelector()
 
 void KisColorSpaceSelector::fillCmbProfiles()
 {
-    QString s = KoColorSpaceRegistry::instance()->colorSpaceId(d->colorSpaceSelector->cmbColorModels->currentItem(), d->colorSpaceSelector->cmbColorDepth->currentItem());
+    const QString colorSpaceId = KoColorSpaceRegistry::instance()->colorSpaceId(d->colorSpaceSelector->cmbColorModels->currentItem(), d->colorSpaceSelector->cmbColorDepth->currentItem());
+    const QString defaultProfileName = KoColorSpaceRegistry::instance()->defaultProfileForColorSpace(colorSpaceId);
+
     d->colorSpaceSelector->cmbProfile->clear();
 
-    const KoColorSpaceFactory * csf = KoColorSpaceRegistry::instance()->colorSpaceFactory(s);
-    if (csf == 0) return;
-
-    QList<const KoColorProfile *>  profileList = KoColorSpaceRegistry::instance()->profilesFor(csf);
+    QList<const KoColorProfile *>  profileList = KoColorSpaceRegistry::instance()->profilesFor(colorSpaceId);
     QStringList profileNames;
     Q_FOREACH (const KoColorProfile *profile, profileList) {
         profileNames.append(profile->name());
     }
-    qSort(profileNames);
+    std::sort(profileNames.begin(), profileNames.end());
     Q_FOREACH (QString stringName, profileNames) {
-        if (stringName==csf->defaultProfile()) {
-            d->colorSpaceSelector->cmbProfile->addSqueezedItem(stringName+d->defaultsuffix);
+        if (stringName == defaultProfileName) {
+            d->colorSpaceSelector->cmbProfile->addSqueezedItem(stringName + d->defaultsuffix);
         } else {
             d->colorSpaceSelector->cmbProfile->addSqueezedItem(stringName);
         }
     }
-    d->colorSpaceSelector->cmbProfile->setCurrent(csf->defaultProfile()+d->defaultsuffix);
+    d->colorSpaceSelector->cmbProfile->setCurrent(defaultProfileName + d->defaultsuffix);
     colorSpaceChanged();
 }
 
@@ -116,26 +115,28 @@ void KisColorSpaceSelector::fillCmbDepths(const KoID& id)
     KoID activeDepth = d->colorSpaceSelector->cmbColorDepth->currentItem();
     d->colorSpaceSelector->cmbColorDepth->clear();
     QList<KoID> depths = KoColorSpaceRegistry::instance()->colorDepthList(id, KoColorSpaceRegistry::OnlyUserVisible);
-    d->colorSpaceSelector->cmbColorDepth->setIDList(depths);
+    d->colorSpaceSelector->cmbColorDepth->setIDList(depths, false);
     if (depths.contains(activeDepth)) {
         d->colorSpaceSelector->cmbColorDepth->setCurrent(activeDepth);
     }
 }
 
+
+
 const KoColorSpace* KisColorSpaceSelector::currentColorSpace()
 {
     QString profilenamestring = d->colorSpaceSelector->cmbProfile->itemHighlighted();
     if (profilenamestring.contains(d->defaultsuffix)) {
         profilenamestring.remove(d->defaultsuffix);
         return KoColorSpaceRegistry::instance()->colorSpace(
-               d->colorSpaceSelector->cmbColorModels->currentItem().id(),
-               d->colorSpaceSelector->cmbColorDepth->currentItem().id(),
-               profilenamestring);
+                    d->colorSpaceSelector->cmbColorModels->currentItem().id(),
+                    d->colorSpaceSelector->cmbColorDepth->currentItem().id(),
+                    profilenamestring);
     } else {
         return KoColorSpaceRegistry::instance()->colorSpace(
-               d->colorSpaceSelector->cmbColorModels->currentItem().id(),
-               d->colorSpaceSelector->cmbColorDepth->currentItem().id(),
-               profilenamestring);
+                    d->colorSpaceSelector->cmbColorModels->currentItem().id(),
+                    d->colorSpaceSelector->cmbColorDepth->currentItem().id(),
+                    profilenamestring);
     }
 }
 
@@ -158,9 +159,16 @@ void KisColorSpaceSelector::setCurrentProfile(const QString& name)
 
 void KisColorSpaceSelector::setCurrentColorSpace(const KoColorSpace* colorSpace)
 {
-  setCurrentColorModel(colorSpace->colorModelId());
-  setCurrentColorDepth(colorSpace->colorDepthId());
-  setCurrentProfile(colorSpace->profile()->name());
+    if (!colorSpace) {
+        return;
+    }
+    setCurrentColorModel(colorSpace->colorModelId());
+    setCurrentColorDepth(colorSpace->colorDepthId());
+    setCurrentProfile(colorSpace->profile()->name());
+}
+
+void KisColorSpaceSelector::showColorBrowserButton(bool showButton) {
+    d->colorSpaceSelector->bnAdvanced->setVisible(showButton);
 }
 
 void KisColorSpaceSelector::colorSpaceChanged()
@@ -179,9 +187,9 @@ void KisColorSpaceSelector::installProfile()
     QStringList mime;
     KoFileDialog dialog(this, KoFileDialog::OpenFiles, "OpenDocumentICC");
     dialog.setCaption(i18n("Install Color Profiles"));
-    dialog.setDefaultDir(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
+    dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
     dialog.setMimeTypeFilters(QStringList() << "application/vnd.iccprofile", "application/vnd.iccprofile");
-    
+
     QStringList profileNames = dialog.filenames();
 
     KoColorSpaceEngine *iccEngine = KoColorSpaceEngineRegistry::instance()->get("icc");
@@ -207,7 +215,9 @@ void KisColorSpaceSelector::slotOpenAdvancedSelector()
     if (!m_advancedSelector) {
         m_advancedSelector = new KisAdvancedColorSpaceSelector(this, "Select a Colorspace");
         m_advancedSelector->setModal(true);
-        m_advancedSelector->setCurrentColorSpace(currentColorSpace());
+        if (currentColorSpace()) {
+            m_advancedSelector->setCurrentColorSpace(currentColorSpace());
+        }
         connect(m_advancedSelector, SIGNAL(selectionChanged(bool)), this, SLOT(slotProfileValid(bool)) );
     }
 
