@@ -19,6 +19,8 @@
 #ifndef _KIS_CURVE_WIDGET_P_H_
 #define _KIS_CURVE_WIDGET_P_H_
 #include <kis_cubic_curve.h>
+#include <QApplication>
+#include <QPalette>
 
 enum enumState {
     ST_NORMAL,
@@ -65,8 +67,10 @@ public:
     QSpinBox *m_intOut;
 
     /* Working range of them */
-    int m_inOutMin;
-    int m_inOutMax;
+    int m_inMin;
+    int m_inMax;
+    int m_outMin;
+    int m_outMax;
 
     /**
      * State functions.
@@ -79,10 +83,10 @@ public:
 
 
 
-    /*** Internal routins ***/
+    /*** Internal routines ***/
 
     /**
-     * Common update routins
+     * Common update routines
      */
     void setCurveModified();
     void setCurveRepaint();
@@ -93,8 +97,8 @@ public:
      * In/Out controls to normalized
      * range of spline (and reverse)
      */
-    double io2sp(int x);
-    int sp2io(double x);
+    double io2sp(int x, int min, int max);
+    int sp2io(double x, int min, int max);
 
 
     /**
@@ -102,7 +106,7 @@ public:
      * with any of existing ones from @m_points and adjusts its coordinates.
      * @skipIndex is the index of the point, that shouldn't be taken
      * into account during the search
-     * (e.g. beacuse it's @pt itself)
+     * (e.g. because it's @pt itself)
      *
      * Returns false in case the point can't be placed anywhere
      * without overlapping
@@ -133,16 +137,16 @@ KisCurveWidget::Private::Private(KisCurveWidget *parent)
     m_curveWidget = parent;
 }
 
-double KisCurveWidget::Private::io2sp(int x)
+double KisCurveWidget::Private::io2sp(int x, int min, int max)
 {
-    int rangeLen = m_inOutMax - m_inOutMin;
-    return double(x - m_inOutMin) / rangeLen;
+    int rangeLen = max - min;
+    return double(x - min) / rangeLen;
 }
 
-int KisCurveWidget::Private::sp2io(double x)
+int KisCurveWidget::Private::sp2io(double x, int min, int max)
 {
-    int rangeLen = m_inOutMax - m_inOutMin;
-    return int(x*rangeLen + 0.5) + m_inOutMin;
+    int rangeLen = max - min;
+    return int(x*rangeLen + 0.5) + min;
 }
 
 
@@ -202,7 +206,9 @@ void KisCurveWidget::Private::drawGrid(QPainter &p, int wWidth, int wHeight)
      * That is not mandatory but desirable
      */
 
-    p.setPen(QPen(Qt::gray, 1, Qt::SolidLine));
+    QPalette appPalette = QApplication::palette();
+
+    p.setPen(QPen(appPalette.color(QPalette::Background), 1, Qt::SolidLine));
     p.drawLine(div4_round(wWidth), 0, div4_round(wWidth), wHeight);
     p.drawLine(div2_round(wWidth), 0, div2_round(wWidth), wHeight);
     p.drawLine(div4_round(3*wWidth), 0, div4_round(3*wWidth), wHeight);
@@ -227,8 +233,8 @@ void KisCurveWidget::Private::syncIOControls()
         m_intIn->blockSignals(true);
         m_intOut->blockSignals(true);
 
-        m_intIn->setValue(sp2io(m_curve.points()[m_grab_point_index].x()));
-        m_intOut->setValue(sp2io(m_curve.points()[m_grab_point_index].y()));
+        m_intIn->setValue(sp2io(m_curve.points()[m_grab_point_index].x(), m_inMin, m_inMax));
+        m_intOut->setValue(sp2io(m_curve.points()[m_grab_point_index].y(), m_outMin, m_outMax));
 
         m_intIn->blockSignals(false);
         m_intOut->blockSignals(false);
