@@ -38,16 +38,20 @@
 #include "KisViewManager.h"
 #include <KisView.h>
 #include "kis_image.h"
+#include "kis_global.h"
 #include "kis_display_color_converter.h"
 
 
 class KisColorPreviewPopup : public QWidget {
 public:
-    KisColorPreviewPopup(KisColorSelectorBase* parent) : QWidget(), m_parent(parent)
+    KisColorPreviewPopup(KisColorSelectorBase* parent)
+        : QWidget(parent), m_parent(parent)
     {
-        setWindowFlags(Qt::ToolTip);
+        setWindowFlags(Qt::ToolTip | Qt::NoDropShadowWindowHint);
         setQColor(QColor(0,0,0));
-        setMouseTracking(true);
+        m_baseColor = QColor(0,0,0,0);
+        m_previousColor = QColor(0,0,0,0);
+        m_lastUsedColor = QColor(0,0,0,0);
     }
 
     void show()
@@ -70,7 +74,8 @@ public:
         } else {
             targetPos =  QPoint(parentPos.x(), parentPos.y() + m_parent->height());
         }
-        setGeometry(targetPos.x(), targetPos.y(), 100, 100);
+        setGeometry(targetPos.x(), targetPos.y(), 100, 150);
+        setAttribute(Qt::WA_TranslucentBackground);
     }
 
     void setQColor(const QColor& color)
@@ -79,16 +84,48 @@ public:
         update();
     }
 
+    void setPreviousColor()
+    {
+        m_previousColor = m_baseColor;
+    }
+
+    void setBaseColor(const QColor& color)
+    {
+        m_baseColor = color;
+        update();
+    }
+
+    void setLastUsedColor(const QColor& color)
+    {
+        m_lastUsedColor = color;
+        update();
+    }
+
 protected:
-    void paintEvent(QPaintEvent *e) {
+    void paintEvent(QPaintEvent *e) override {
         Q_UNUSED(e);
         QPainter p(this);
-        p.fillRect(0,0, width(), width(), m_color);
+        p.fillRect(0, 0, width(), width(), m_color);
+        p.fillRect(50, width(), width(), height(), m_previousColor);
+        p.fillRect(0, width(), 50, height(), m_lastUsedColor);
+    }
+
+    void enterEvent(QEvent *e) override {
+        QWidget::enterEvent(e);
+        m_parent->tryHideAllPopups();
+    }
+
+    void leaveEvent(QEvent *e) override {
+        QWidget::leaveEvent(e);
+        m_parent->tryHideAllPopups();
     }
 
 private:
     KisColorSelectorBase* m_parent;
     QColor m_color;
+    QColor m_baseColor;
+    QColor m_previousColor;
+    QColor m_lastUsedColor;
 };
 
 KisColorSelectorBase::KisColorSelectorBase(QWidget *parent) :
@@ -117,6 +154,7 @@ KisColorSelectorBase::KisColorSelectorBase(QWidget *parent) :
 
 KisColorSelectorBase::~KisColorSelectorBase()
 {
+    delete m_popup;
     delete m_colorPreviewPopup;
 }
 
@@ -151,6 +189,9 @@ void KisColorSelectorBase::setCanvas(KisCanvas2 *canvas)
         connect(m_canvas->displayColorConverter(), SIGNAL(displayConfigurationChanged()),
                 SLOT(reset()));
 
+        connect(canvas->imageView()->resourceProvider(), SIGNAL(sigFGColorUsed(KoColor)),
+                this,                               SLOT(updateLastUsedColorPreview(KoColor)), Qt::UniqueConnection);
+
         if (m_canvas->viewManager() && m_canvas->viewManager()->resourceProvider()) {
             setColor(Acs::currentColor(m_canvas->viewManager()->resourceProvider(), Acs::Foreground));
         }
@@ -202,12 +243,14 @@ void KisColorSelectorBase::mousePressEvent(QMouseEvent* event)
         if(y+m_popup->height()>availRect.y()+availRect.height())
             y = availRect.y()+availRect.height()-m_popup->height();
 
-
         m_popup->move(x, y);
         m_popup->setHidingTime(200);
         showPopup(DontMove);
 
     } else if (m_isPopup && event->button() == Qt::MidButton) {
+        if (m_colorPreviewPopup) {
+            m_colorPreviewPopup->hide();
+        }
         hide();
     } else {
         showColorPreview();
@@ -221,16 +264,18 @@ void KisColorSelectorBase::mouseReleaseEvent(QMouseEvent *e) {
 
     if (e->button() == Qt::MidButton) {
         e->accept();
-    } else if (m_isPopup && m_hideOnMouseClick==true && !m_hideTimer->isActive()) {
-        showColorPreview();
+    } else if (m_isPopup &&
+               (m_hideOnMouseClick && !m_popupOnMouseOver) &&
+               !m_hideTimer->isActive()) {
+        if (m_colorPreviewPopup) {
+            m_colorPreviewPopup->hide();
+        }
         hide();
     }
 }
 
 void KisColorSelectorBase::enterEvent(QEvent *e)
 {
-    Q_UNUSED(e);
-
     if (m_popup && m_popup->isVisible()) {
         m_popup->m_hideTimer->stop();
     }
@@ -248,51 +293,26 @@ void KisColorSelectorBase::enterEvent(QEvent *e)
 
         lazyCreatePopup();
 
-        QRect availRect = QApplication::desktop()->availableGeometry(this);
-        QRect forbiddenRect = QRect(parentWidget()->mapToGlobal(QPoint(0,0)),
-                                    QSize(parentWidget()->width(), parentWidget()->height()));
+        const QRect availRect = QApplication::desktop()->availableGeometry(this);
 
-        int x,y;
-        if(forbiddenRect.y()+forbiddenRect.height()/2 > availRect.height()/2) {
-            //popup above forbiddenRect
-            y = forbiddenRect.y()-m_popup->height();
-        }
-        else {
-            //popup below forbiddenRect
-            y = forbiddenRect.y()+forbiddenRect.height();
-        }
+        QPoint proposedTopLeft = rect().center() - m_popup->rect().center();
+        proposedTopLeft = mapToGlobal(proposedTopLeft);
 
-        if(forbiddenRect.x()+forbiddenRect.width()/2 < availRect.width()/2) {
-            //left edge of popup justified with left edge of popup
-            x = forbiddenRect.x();
-        }
-        else {
-            //the other way round
-            x = forbiddenRect.x()+forbiddenRect.width()-m_popup->width();
-        }
+        QRect popupRect = QRect(proposedTopLeft, m_popup->size());
+        popupRect = kisEnsureInRect(popupRect, availRect);
 
-        m_popup->move(x, y);
+        m_popup->setGeometry(popupRect);
         m_popup->setHidingTime(200);
         showPopup(DontMove);
     }
+
+    QWidget::enterEvent(e);
 }
 
 void KisColorSelectorBase::leaveEvent(QEvent *e)
 {
-    Q_UNUSED(e);
-
-    if (m_colorPreviewPopup->isVisible()) {
-        m_colorUpdateSelf=false; //this is for allowing advanced selector to listen to outside colour-change events.
-        m_colorPreviewPopup->hide();
-    }
-
-    if (m_popup && m_popup->isVisible()) {
-        m_popup->m_hideTimer->start();
-    }
-
-    if (m_isPopup && !m_hideTimer->isActive()) {
-        m_hideTimer->start();
-    }
+    tryHideAllPopups();
+    QWidget::leaveEvent(e);
 }
 
 void KisColorSelectorBase::keyPressEvent(QKeyEvent *)
@@ -363,7 +383,20 @@ void KisColorSelectorBase::lazyCreatePopup()
     if (!m_popup) {
         m_popup = createPopup();
         Q_ASSERT(m_popup);
-        m_popup->setWindowFlags(Qt::FramelessWindowHint|Qt::SubWindow|Qt::X11BypassWindowManagerHint);
+        m_popup->setParent(this);
+
+        /**
+         * On Linux passing Qt::X11BypassWindowManagerHint makes
+         * the window never hide if one switches it with Alt+Tab
+         * or something like that. The window also don't get any
+         * mouse-enter/leave events. So we have to make it normal
+         * window (which is visible to the window manager and
+         * appears in the tasks bar). Therefore we don't use it
+         * anyomore.
+         */
+        m_popup->setWindowFlags(Qt::FramelessWindowHint |
+                                Qt::Window |
+                                Qt::NoDropShadowWindowHint);
         m_popup->m_parent = this;
         m_popup->m_isPopup=true;
     }
@@ -380,13 +413,17 @@ void KisColorSelectorBase::showPopup(Move move)
     QPoint cursorPos = QCursor::pos();
 
     if (move == MoveToMousePosition) {
-        m_popup->move(cursorPos.x()-m_popup->width()/2, cursorPos.y()-m_popup->height()/2);
+        m_popup->move(QPoint(cursorPos.x()-m_popup->width()/2, cursorPos.y()-m_popup->height()/2));
         QRect rc = m_popup->geometry();
         if (rc.x() < 0) rc.setX(0);
         if (rc.y() < 0) rc.setY(0);
         m_popup->setGeometry(rc);
-
     }
+
+    if (m_colorPreviewPopup) {
+        m_colorPreviewPopup->hide();
+    }
+
     m_popup->show();
     m_popup->m_colorPreviewPopup->show();
 }
@@ -465,7 +502,8 @@ void KisColorSelectorBase::updateSettings()
 
     if(m_isPopup) {
         m_hideOnMouseClick = cfg.readEntry("hidePopupOnClickCheck", false);
-        resize(cfg.readEntry("zoomSize", 280), cfg.readEntry("zoomSize", 280));
+        const int zoomSize = cfg.readEntry("zoomSize", 280);
+        resize(zoomSize, zoomSize);
     }
 
     reset();
@@ -476,11 +514,42 @@ void KisColorSelectorBase::reset()
     update();
 }
 
+void KisColorSelectorBase::updateBaseColorPreview(const KoColor &color)
+{
+    m_colorPreviewPopup->setBaseColor(converter()->toQColor(color));
+}
+
+void KisColorSelectorBase::updatePreviousColorPreview()
+{
+    m_colorPreviewPopup->setPreviousColor();
+}
+
+void KisColorSelectorBase::updateLastUsedColorPreview(const KoColor &color)
+{
+    m_colorPreviewPopup->setLastUsedColor(converter()->toQColor(color));
+}
+
 KisDisplayColorConverter* KisColorSelectorBase::converter() const
 {
     return m_canvas ?
         m_canvas->displayColorConverter() :
-        KisDisplayColorConverter::dumbConverterInstance();
+                KisDisplayColorConverter::dumbConverterInstance();
+}
+
+void KisColorSelectorBase::tryHideAllPopups()
+{
+    if (m_colorPreviewPopup->isVisible()) {
+        m_colorUpdateSelf=false; //this is for allowing advanced selector to listen to outside colour-change events.
+        m_colorPreviewPopup->hide();
+    }
+
+    if (m_popup && m_popup->isVisible()) {
+        m_popup->m_hideTimer->start();
+    }
+
+    if (m_isPopup && !m_hideTimer->isActive()) {
+        m_hideTimer->start();
+    }
 }
 
 
