@@ -34,7 +34,6 @@
 #include "flake/kis_shape_controller.h"
 #include "kis_undo_adapter.h"
 
-
 #include "timeline_frames_view.h"
 #include "timeline_frames_model.h"
 
@@ -43,6 +42,11 @@
 #include "commands/kis_image_layer_add_command.h"
 #include "commands/kis_image_layer_remove_command.h"
 
+#include "kis_double_parse_spin_box.h"
+#include "kis_int_parse_spin_box.h"
+
+#include  <sdk/tests/kistest.h>
+#include <sdk/tests/testutil.h>
 
 void TimelineModelTest::init()
 {
@@ -118,7 +122,7 @@ struct TestingInterface : TimelineFramesModel::NodeManipulationInterface
 {
     TestingInterface(KisImageSP image) : m_image(image) {}
 
-    KisLayerSP addPaintLayer() const {
+    KisLayerSP addPaintLayer() const override {
         KisNodeSP parent = m_image->root();
         KisNodeSP after = parent->lastChild();
 
@@ -138,7 +142,7 @@ struct TestingInterface : TimelineFramesModel::NodeManipulationInterface
         return layer;
     }
 
-    void removeNode(KisNodeSP node) const {
+    void removeNode(KisNodeSP node) const override {
         m_image->undoAdapter()->addCommand(
             new KisImageLayerRemoveCommand(m_image, node));
     }
@@ -149,20 +153,24 @@ private:
 
 void TimelineModelTest::testView()
 {
+#ifndef ENABLE_GUI_TESTS
+    return;
+#endif
+
     QDialog dlg;
 
     QFont font;
     font.setPointSizeF(9);
     dlg.setFont(font);
 
-    QSpinBox *intFps = new QSpinBox(&dlg);
+    QSpinBox *intFps = new KisIntParseSpinBox(&dlg);
     intFps->setValue(12);
 
-    QSpinBox *intTime = new QSpinBox(&dlg);
+    QSpinBox *intTime = new KisIntParseSpinBox(&dlg);
     intTime->setValue(0);
     intTime->setMaximum(10000);
 
-    QSpinBox *intLayer = new QSpinBox(&dlg);
+    QSpinBox *intLayer = new KisIntParseSpinBox(&dlg);
     intLayer->setValue(0);
     intLayer->setMaximum(100);
 
@@ -190,7 +198,7 @@ void TimelineModelTest::testView()
     connect(intTime, SIGNAL(valueChanged(int)),
             SLOT(setCurrentTime(int)));
 
-    connect(m_image->animationInterface(), SIGNAL(sigTimeChanged(int)),
+    connect(m_image->animationInterface(), SIGNAL(sigUiTimeChanged(int)),
             intTime, SLOT(setValue(int)));
 
     connect(intLayer, SIGNAL(valueChanged(int)),
@@ -268,6 +276,9 @@ void TimelineModelTest::slotGuiChangedNode(KisNodeSP node)
 
 void TimelineModelTest::testOnionSkins()
 {
+#ifndef ENABLE_GUI_TESTS
+    return;
+#endif
     QDialog dlg;
 
     QFont font;
@@ -293,4 +304,4 @@ void TimelineModelTest::slotBang()
     ENTER_FUNCTION() << "!!!!";
 }
 
-QTEST_MAIN(TimelineModelTest)
+KISTEST_MAIN(TimelineModelTest)
