@@ -23,12 +23,12 @@
 
 #include <QStringList>
 #include <QDir>
-#include <QDesktopServices>
+#include <QStandardPaths>
+#include <QMessageBox>
 
 #include <klocalizedstring.h>
 #include <kpluginfactory.h>
 
-
 #include <KisImportExportManager.h>
 #include <KoFileDialog.h>
 #include <KisDocument.h>
@@ -39,7 +39,6 @@
 #include <KisViewManager.h>
 #include <kis_image.h>
 #include <kis_action.h>
-#include <KisDocument.h>
 #include <kis_paint_layer.h>
 #include <kis_painter.h>
 #include <kis_paint_device.h>
@@ -50,7 +49,7 @@
 K_PLUGIN_FACTORY_WITH_JSON(ImagesplitFactory, "kritaimagesplit.json", registerPlugin<Imagesplit>();)
 
 Imagesplit::Imagesplit(QObject *parent, const QVariantList &)
-    : KisViewPlugin(parent)
+    : KisActionPlugin(parent)
 {
     KisAction *action  = createAction("imagesplit");
     connect(action, SIGNAL(triggered()), this, SLOT(slotImagesplit()));
@@ -60,14 +59,13 @@ Imagesplit::~Imagesplit()
 {
 }
 
-void Imagesplit::saveAsImage(const QRect &imgSize, const QString &mimeType, const QString &url)
+bool Imagesplit::saveAsImage(const QRect &imgSize, const QString &mimeType, const QString &url)
 {
-    KisImageWSP image = m_view->image();
+    KisImageSP image = viewManager()->image();
 
     KisDocument *document = KisPart::instance()->createDocument();
-    document->prepareForImport();
 
-    KisImageWSP dst = new KisImage(document->createUndoStore(), imgSize.width(), imgSize.height(), image->colorSpace(), image->objectName());
+    KisImageSP dst = new KisImage(document->createUndoStore(), imgSize.width(), imgSize.height(), image->colorSpace(), image->objectName());
     dst->setResolution(image->xRes(), image->yRes());
     document->setCurrentImage(dst);
 
@@ -78,37 +76,53 @@ void Imagesplit::saveAsImage(const QRect &imgSize, const QString &mimeType, cons
     dst->addNode(paintLayer, KisNodeSP(0));
     dst->refreshGraph();
     document->setFileBatchMode(true);
-    document->setOutputMimeType(mimeType.toLatin1());
-    document->exportDocument(QUrl::fromLocalFile(url));
+    if (!document->exportDocumentSync(QUrl::fromLocalFile(url), mimeType.toLatin1())) {
+        if (document->errorMessage().isEmpty()) {
+            QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("Could not save\n%1", document->localFilePath()));
+        } else {
+            QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("Could not save %1\nReason: %2", document->localFilePath(), document->errorMessage()));
+        }
+        return false;
+    }
 
     delete document;
+
+    return true;
 }
 
 void Imagesplit::slotImagesplit()
 {
     // Taking the title - url from caption function and removing file extension
-    QStringList strList = ((m_view->document())->caption()).split('.');
+    QStringList strList = ((viewManager()->document())->caption()).split('.');
     QString suffix = strList.at(0);
 
     // Getting all mime types and converting them into names which are displayed at combo box
-    QStringList listMimeFilter = KisImportExportManager::mimeFilter("application/x-krita", KisImportExportManager::Export);
+    QStringList listMimeFilter = KisImportExportManager::supportedMimeTypes(KisImportExportManager::Export);
+    QString defaultMime = QString::fromLatin1(viewManager()->document()->mimeType());
+    int defaultMimeIndex = 0;
+
     listMimeFilter.sort();
     QStringList filteredMimeTypes;
     QStringList listFileType;
+    int i = 0;
     Q_FOREACH (const QString & mimeType, listMimeFilter) {
         listFileType.append(KisMimeDatabase::descriptionForMimeType(mimeType));
         filteredMimeTypes.append(mimeType);
+        if (mimeType == defaultMime) {
+            defaultMimeIndex = i;
+        }
+        i++;
     }
 
     listMimeFilter = filteredMimeTypes;
 
     Q_ASSERT(listMimeFilter.size() == listFileType.size());
 
-    DlgImagesplit *dlgImagesplit = new DlgImagesplit(m_view, suffix, listFileType);
+    DlgImagesplit *dlgImagesplit = new DlgImagesplit(viewManager(), suffix, listFileType, defaultMimeIndex);
     dlgImagesplit->setObjectName("Imagesplit");
     Q_CHECK_PTR(dlgImagesplit);
 
-    KisImageWSP image = m_view->image();
+    KisImageWSP image = viewManager()->image();
 
     if (dlgImagesplit->exec() == QDialog::Accepted) {
 
@@ -119,11 +133,23 @@ void Imagesplit::slotImagesplit()
         int img_height = image->height() / (numHorizontalLines + 1);
 
 
+        bool stop = false;
         if (dlgImagesplit->autoSave()) {
+            KoFileDialog dialog(viewManager()->mainWindow(), KoFileDialog::OpenDirectory, "OpenDocument");
+            dialog.setCaption(i18n("Save Image on Split"));
+            dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
+            QStringList mimeFilter = viewManager()->document()->importExportManager()->supportedMimeTypes(KisImportExportManager::Export);
+            QString defaultMime = QString::fromLatin1(viewManager()->document()->mimeType());
+            dialog.setMimeTypeFilters(mimeFilter, defaultMime);
+
+            QUrl directory = QUrl::fromUserInput(dialog.filename());
+
+            if (directory.isEmpty())
+                return;
             for (int i = 0, k = 1; i < (numVerticalLines + 1); i++) {
                 for (int j = 0; j < (numHorizontalLines + 1); j++, k++) {
                     QString mimeTypeSelected = listMimeFilter.at(dlgImagesplit->cmbIndex);
-                    QString homepath = QDir::homePath();
+                    QString homepath = directory.toLocalFile();
                     QString suffix = KisMimeDatabase::suffixesForMimeType(mimeTypeSelected).first();
                     qDebug() << "suffix" << suffix;
                     if (suffix.startsWith("*.")) {
@@ -136,7 +162,13 @@ void Imagesplit::slotImagesplit()
                     qDebug() << "\tsuffix" << suffix;
                     QString fileName = dlgImagesplit->suffix() + '_' + QString::number(k) + suffix;
                     QString url = homepath  + '/' + fileName;
-                    saveAsImage(QRect((i * img_width), (j * img_height), img_width, img_height), listMimeFilter.at(dlgImagesplit->cmbIndex), url);
+                    if (!saveAsImage(QRect((i * img_width), (j * img_height), img_width, img_height), listMimeFilter.at(dlgImagesplit->cmbIndex), url)) {
+                        stop = true;
+                        break;
+                    }
+                }
+                if (stop) {
+                    break;
                 }
             }
         }
@@ -144,17 +176,24 @@ void Imagesplit::slotImagesplit()
 
             for (int i = 0; i < (numVerticalLines + 1); i++) {
                 for (int j = 0; j < (numHorizontalLines + 1); j++) {
-                    KoFileDialog dialog(m_view->mainWindow(), KoFileDialog::SaveFile, "OpenDocument");
+                    KoFileDialog dialog(viewManager()->mainWindow(), KoFileDialog::SaveFile, "OpenDocument");
                     dialog.setCaption(i18n("Save Image on Split"));
-                    dialog.setDefaultDir(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
-                    dialog.setMimeTypeFilters(listMimeFilter);
+                    dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
+                    dialog.setMimeTypeFilters(listMimeFilter, defaultMime);
+
                     QUrl url = QUrl::fromUserInput(dialog.filename());
 
-                    QString mimefilter = KisMimeDatabase::mimeTypeForFile(url.toLocalFile());
+                    QString mimefilter = KisMimeDatabase::mimeTypeForFile(url.toLocalFile(), false);
 
                     if (url.isEmpty())
                         return;
-                    saveAsImage(QRect((i * img_width), (j * img_height), img_width, img_height), mimefilter, url.toLocalFile());
+                    if (!saveAsImage(QRect((i * img_width), (j * img_height), img_width, img_height), mimefilter, url.toLocalFile())) {
+                        stop = true;
+                        break;
+                    }
+                }
+                if (stop) {
+                    break;
                 }
             }
 
