@@ -27,6 +27,7 @@
 
 #include <QMessageBox>
 #include <QGlobalStatic>
+#include <QStandardPaths>
 
 #include <klocalizedstring.h>
 #include <KoResourcePaths.h>
@@ -40,13 +41,15 @@
 #include <kis_debug.h>
 #include <kis_action.h>
 #include <KisViewManager.h>
-#include <kis_resource_server_provider.h>
+#include <KisResourceServerProvider.h>
 #include <kis_workspace_resource.h>
 #include <brushengine/kis_paintop_preset.h>
 #include <kis_brush_server.h>
 #include <kis_paintop_settings.h>
 #include "dlg_bundle_manager.h"
 #include "dlg_create_bundle.h"
+#include <KisPaintopSettingsIds.h>
+#include "krita_container_utils.h"
 
 class ResourceManager::Private {
 
@@ -54,12 +57,12 @@ public:
 
     Private()
     {
-        brushServer = KisBrushServer::instance()->brushServer(false);
-        paintopServer = KisResourceServerProvider::instance()->paintOpPresetServer(false);
-        gradientServer = KoResourceServerProvider::instance()->gradientServer(false);
-        patternServer = KoResourceServerProvider::instance()->patternServer(false);
-        paletteServer = KoResourceServerProvider::instance()->paletteServer(false);
-        workspaceServer = KisResourceServerProvider::instance()->workspaceServer(false);
+        brushServer = KisBrushServer::instance()->brushServer();
+        paintopServer = KisResourceServerProvider::instance()->paintOpPresetServer();
+        gradientServer = KoResourceServerProvider::instance()->gradientServer();
+        patternServer = KoResourceServerProvider::instance()->patternServer();
+        paletteServer = KoResourceServerProvider::instance()->paletteServer();
+        workspaceServer = KisResourceServerProvider::instance()->workspaceServer();
     }
 
     KisBrushResourceServer* brushServer;
@@ -74,7 +77,7 @@ public:
 K_PLUGIN_FACTORY_WITH_JSON(ResourceManagerFactory, "kritaresourcemanager.json", registerPlugin<ResourceManager>();)
 
 ResourceManager::ResourceManager(QObject *parent, const QVariantList &)
-    : KisViewPlugin(parent)
+    : KisActionPlugin(parent)
     , d(new Private())
 {
     KisAction *action = new KisAction(i18n("Import Bundles..."), this);
@@ -128,9 +131,10 @@ void ResourceManager::slotCreateBundle()
     saveBundle(dlgCreateBundle);
 }
 
-void ResourceManager::saveBundle(const DlgCreateBundle &dlgCreateBundle)
+KisResourceBundle *ResourceManager::saveBundle(const DlgCreateBundle &dlgCreateBundle)
 {
     QString bundlePath =  dlgCreateBundle.saveLocation() + "/" + dlgCreateBundle.bundleName() + ".bundle";
+
     KisResourceBundle *newBundle = new KisResourceBundle(bundlePath);
 
     newBundle->addMeta("name", dlgCreateBundle.bundleName());
@@ -139,6 +143,7 @@ void ResourceManager::saveBundle(const DlgCreateBundle &dlgCreateBundle)
     newBundle->addMeta("license", dlgCreateBundle.license());
     newBundle->addMeta("website", dlgCreateBundle.website());
     newBundle->addMeta("description", dlgCreateBundle.description());
+    newBundle->setThumbnail(dlgCreateBundle.previewImage());
 
     QStringList res = dlgCreateBundle.selectedBrushes();
     Q_FOREACH (const QString &r, res) {
@@ -170,12 +175,19 @@ void ResourceManager::saveBundle(const DlgCreateBundle &dlgCreateBundle)
         KoResource *res = preset.data();
         newBundle->addResource("kis_paintoppresets", res->filename(), d->paintopServer->assignedTagsList(res), res->md5());
         KisPaintOpSettingsSP settings = preset->settings();
-        if (settings->hasProperty("requiredBrushFile")) {
-            QString brushFile = settings->getString("requiredBrushFile");
-            KisBrush *brush = d->brushServer->resourceByFilename(brushFile).data();
-            newBundle->addResource("kis_brushes", brushFile, d->brushServer->assignedTagsList(brush), brush->md5());
-        }
 
+        QStringList requiredFiles = settings->getStringList(KisPaintOpUtils::RequiredBrushFilesListTag);
+        requiredFiles << settings->getString(KisPaintOpUtils::RequiredBrushFileTag);
+        KritaUtils::makeContainerUnique(requiredFiles);
+
+        Q_FOREACH (const QString &brushFile, requiredFiles) {
+            KisBrush *brush = d->brushServer->resourceByFilename(brushFile).data();
+            if (brush) {
+                newBundle->addResource("kis_brushes", brushFile, d->brushServer->assignedTagsList(brush), brush->md5());
+            } else {
+                qWarning() << "There is no brush with name" << brushFile;
+            }
+        }
     }
 
     res = dlgCreateBundle.selectedWorkspaces();
@@ -187,18 +199,28 @@ void ResourceManager::saveBundle(const DlgCreateBundle &dlgCreateBundle)
     newBundle->addMeta("fileName", bundlePath);
     newBundle->addMeta("created", QDate::currentDate().toString("dd/MM/yyyy"));
 
-    newBundle->setThumbnail(dlgCreateBundle.previewImage());
-
     if (!newBundle->save()) {
-        QMessageBox::critical(m_view->mainWindow(), i18nc("@title:window", "Krita"), i18n("Could not create the new bundle."));
+        QMessageBox::critical(viewManager()->mainWindow(), i18nc("@title:window", "Krita"), i18n("Could not create the new bundle."));
+    }
+    else {
+        newBundle->setValid(true);
+        if (QDir(KisResourceBundleServerProvider::instance()->resourceBundleServer()->saveLocation()) != QDir(QFileInfo(bundlePath).path())) {
+            newBundle->setFilename(KisResourceBundleServerProvider::instance()->resourceBundleServer()->saveLocation() + "/" + dlgCreateBundle.bundleName() + ".bundle");
+        }
+        if (KisResourceBundleServerProvider::instance()->resourceBundleServer()->resourceByName(newBundle->name())) {
+            KisResourceBundleServerProvider::instance()->resourceBundleServer()->removeResourceFromServer(
+                        KisResourceBundleServerProvider::instance()->resourceBundleServer()->resourceByName(newBundle->name()));
+        }
+        KisResourceBundleServerProvider::instance()->resourceBundleServer()->addResource(newBundle, true);
+        newBundle->load();
     }
 
-
+    return newBundle;
 }
 
 void ResourceManager::slotManageBundles()
 {
-    DlgBundleManager* dlg = new DlgBundleManager(this, m_view->actionManager());
+    DlgBundleManager* dlg = new DlgBundleManager(this, viewManager()->actionManager());
     if (dlg->exec() != QDialog::Accepted) {
         return;
     }
@@ -206,7 +228,8 @@ void ResourceManager::slotManageBundles()
 
 QStringList ResourceManager::importResources(const QString &title, const QStringList &mimes) const
 {
-    KoFileDialog dialog(m_view->mainWindow(), KoFileDialog::OpenFiles, "krita_resources");
+    KoFileDialog dialog(viewManager()->mainWindow(), KoFileDialog::OpenFiles, "krita_resources");
+    dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
     dialog.setCaption(title);
     dialog.setMimeTypeFilters(mimes);
     return dialog.filenames();
@@ -246,29 +269,29 @@ void ResourceManager::slotImportBundles()
 {
     QStringList resources = importResources(i18n("Import Bundles"), QStringList() << "application/x-krita-bundle");
     Q_FOREACH (const QString &res, resources) {
-        KisResourceBundle *bundle = KisResourceServerProvider::instance()->resourceBundleServer()->createResource(res);
+        KisResourceBundle *bundle = KisResourceBundleServerProvider::instance()->resourceBundleServer()->createResource(res);
         bundle->load();
         if (bundle->valid()) {
             if (!bundle->install()) {
-                QMessageBox::warning(0, i18nc("@title:window", "Krita"), i18n("Could not install the resources for bundle %1.").arg(res));
+                QMessageBox::warning(0, i18nc("@title:window", "Krita"), i18n("Could not install the resources for bundle %1.", res));
             }
         }
         else {
-            QMessageBox::warning(0, i18nc("@title:window", "Krita"), i18n("Could not load bundle %1.").arg(res));
+            QMessageBox::warning(0, i18nc("@title:window", "Krita"), i18n("Could not load bundle %1.", res));
         }
 
         QFileInfo fi(res);
-        QString newFilename = KisResourceServerProvider::instance()->resourceBundleServer()->saveLocation() + fi.baseName() + bundle->defaultFileExtension();
+        QString newFilename = KisResourceBundleServerProvider::instance()->resourceBundleServer()->saveLocation() + fi.baseName() + bundle->defaultFileExtension();
         QFileInfo fileInfo(newFilename);
 
         int i = 1;
         while (fileInfo.exists()) {
-            fileInfo.setFile(KisResourceServerProvider::instance()->resourceBundleServer()->saveLocation() + fi.baseName() + QString("%1").arg(i) + bundle->defaultFileExtension());
+            fileInfo.setFile(KisResourceBundleServerProvider::instance()->resourceBundleServer()->saveLocation() + fi.baseName() + QString("%1").arg(i) + bundle->defaultFileExtension());
             i++;
         }
         bundle->setFilename(fileInfo.filePath());
         QFile::copy(res, newFilename);
-        KisResourceServerProvider::instance()->resourceBundleServer()->addResource(bundle, false);
+        KisResourceBundleServerProvider::instance()->resourceBundleServer()->addResource(bundle, false);
     }
 }
 
