@@ -165,6 +165,56 @@ QPair<QString, QString> getColorSpaceForColorType(uint16 sampletype, uint16 colo
 }
 }
 
+KisPropertiesConfigurationSP KisTIFFOptions::toProperties() const
+{
+    QHash<int, int> compToIndex;
+    compToIndex[COMPRESSION_NONE] = 0;
+    compToIndex[COMPRESSION_JPEG] = 1;
+    compToIndex[COMPRESSION_DEFLATE] = 2;
+    compToIndex[COMPRESSION_LZW] = 3;
+    compToIndex[COMPRESSION_PIXARLOG] = 8;
+
+    KisPropertiesConfigurationSP cfg = new KisPropertiesConfiguration();
+
+    cfg->setProperty("compressiontype", compToIndex.value(compressionType, 0));
+    cfg->setProperty("predictor", predictor - 1);
+    cfg->setProperty("alpha", alpha);
+    cfg->setProperty("flatten", flatten);
+    cfg->setProperty("quality", jpegQuality);
+    cfg->setProperty("deflate", deflateCompress);
+    cfg->setProperty("pixarlog", pixarLogCompress);
+    cfg->setProperty("saveProfile", saveProfile);
+
+    return cfg;
+}
+
+void KisTIFFOptions::fromProperties(KisPropertiesConfigurationSP cfg)
+{
+    QHash<int, int> indexToComp;
+    indexToComp[0] = COMPRESSION_NONE;
+    indexToComp[1] = COMPRESSION_JPEG;
+    indexToComp[2] = COMPRESSION_DEFLATE;
+    indexToComp[3] = COMPRESSION_LZW;
+    indexToComp[4] = COMPRESSION_PIXARLOG;
+
+    // old value that might be still stored in a config (remove after Krita 5.0 :) )
+    indexToComp[8] = COMPRESSION_PIXARLOG;
+
+    compressionType =
+        indexToComp.value(
+            cfg->getInt("compressiontype", 0),
+            COMPRESSION_NONE);
+
+    predictor = cfg->getInt("predictor", 0) + 1;
+    alpha = cfg->getBool("alpha", true);
+    flatten = cfg->getBool("flatten", true);
+    jpegQuality = cfg->getInt("quality", 80);
+    deflateCompress = cfg->getInt("deflate", 6);
+    pixarLogCompress = cfg->getInt("pixarlog", 6);
+    saveProfile = cfg->getBool("saveProfile", true);
+}
+
+
 KisTIFFConverter::KisTIFFConverter(KisDocument *doc)
 {
     m_doc = doc;
@@ -262,13 +312,13 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory(TIFF* image)
     }
 
     uint8 dstDepth = 0;
-    QPair<QString, QString> colorSpaceId = getColorSpaceForColorType(sampletype, color_type, depth, image, nbchannels, extrasamplescount, dstDepth);
-    if (colorSpaceId.first.isEmpty()) {
+    QPair<QString, QString> colorSpaceIdTag = getColorSpaceForColorType(sampletype, color_type, depth, image, nbchannels, extrasamplescount, dstDepth);
+    if (colorSpaceIdTag.first.isEmpty()) {
         dbgFile << "Image has an unsupported colorspace :" << color_type << " for this depth :" << depth;
         TIFFClose(image);
         return KisImageBuilder_RESULT_UNSUPPORTED_COLORSPACE;
     }
-    dbgFile << "Colorspace is :" << colorSpaceId.first << colorSpaceId.second << " with a depth of" << depth << " and with a nb of channels of" << nbchannels;
+    dbgFile << "Colorspace is :" << colorSpaceIdTag.first << colorSpaceIdTag.second << " with a depth of" << depth << " and with a nb of channels of" << nbchannels;
 
     // Read image profile
     dbgFile << "Reading profile";
@@ -281,30 +331,37 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory(TIFF* image)
         QByteArray rawdata;
         rawdata.resize(EmbedLen);
         memcpy(rawdata.data(), EmbedBuffer, EmbedLen);
-        profile = KoColorSpaceRegistry::instance()->createColorProfile(colorSpaceId.first, colorSpaceId.second, rawdata);
-    }
-    else {
-        dbgFile << "No Profile found";
+        profile = KoColorSpaceRegistry::instance()->createColorProfile(colorSpaceIdTag.first, colorSpaceIdTag.second, rawdata);
     }
 
+    const QString colorSpaceId =
+        KoColorSpaceRegistry::instance()->colorSpaceId(colorSpaceIdTag.first, colorSpaceIdTag.second);
+
     // Check that the profile is used by the color space
-
-    if (profile && !KoColorSpaceRegistry::instance()->colorSpaceFactory(KoColorSpaceRegistry::instance()->colorSpaceId(colorSpaceId.first, colorSpaceId.second))->profileIsCompatible(profile)) {
-        warnFile << "The profile " << profile->name() << " is not compatible with the color space model " << colorSpaceId.first << " " << colorSpaceId.second;
+    if (profile && !KoColorSpaceRegistry::instance()->profileIsCompatible(profile, colorSpaceId)) {
+        dbgFile << "The profile " << profile->name() << " is not compatible with the color space model " << colorSpaceIdTag.first << " " << colorSpaceIdTag.second;
         profile = 0;
     }
 
+    // Do not use the linear gamma profile for 16 bits/channel by default, tiff files are usually created with
+    // gamma correction. XXX: Should we ask the user?
+    if (!profile && colorSpaceIdTag.first == RGBAColorModelID.id() && colorSpaceIdTag.second == Integer16BitsColorDepthID.id()) {
+        profile = KoColorSpaceRegistry::instance()->profileByName("sRGB-elle-V2-srgbtrc.icc");
+        dbgFile << "Getting srgb profile" << profile;
+     }
+
     // Retrieve a pointer to the colorspace
     const KoColorSpace* cs = 0;
     if (profile && profile->isSuitableForOutput()) {
         dbgFile << "image has embedded profile:" << profile -> name() << "";
-        cs = KoColorSpaceRegistry::instance()->colorSpace(colorSpaceId.first, colorSpaceId.second, profile);
+        cs = KoColorSpaceRegistry::instance()->colorSpace(colorSpaceIdTag.first, colorSpaceIdTag.second, profile);
     }
     else {
-        cs = KoColorSpaceRegistry::instance()->colorSpace(colorSpaceId.first, colorSpaceId.second, 0);
+        cs = KoColorSpaceRegistry::instance()->colorSpace(colorSpaceIdTag.first, colorSpaceIdTag.second, 0);
     }
+
     if (cs == 0) {
-        dbgFile << "Colorspace" << colorSpaceId.first << colorSpaceId.second << " is not available, please check your installation.";
+        dbgFile << "Colorspace" << colorSpaceIdTag.first << colorSpaceIdTag.second << " is not available, please check your installation.";
         TIFFClose(image);
         return KisImageBuilder_RESULT_UNSUPPORTED_COLORSPACE;
     }
@@ -313,19 +370,24 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory(TIFF* image)
     KoColorTransformation* transform = 0;
     if (profile && !profile->isSuitableForOutput()) {
         dbgFile << "The profile can't be used in krita, need conversion";
-        transform = KoColorSpaceRegistry::instance()->colorSpace(colorSpaceId.first, colorSpaceId.second, profile)->createColorConverter(cs, KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
+        transform = KoColorSpaceRegistry::instance()->colorSpace(colorSpaceIdTag.first, colorSpaceIdTag.second, profile)->createColorConverter(cs, KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
     }
 
     // Check if there is an alpha channel
     int8 alphapos = -1; // <- no alpha
     // Check which extra is alpha if any
     dbgFile << "There are" << nbchannels << " channels and" << extrasamplescount << " extra channels";
-    if (sampleinfo) { // index images don't have any sampleinfo, and therefor sampleinfo == 0
+    if (sampleinfo) { // index images don't have any sampleinfo, and therefore sampleinfo == 0
         for (int i = 0; i < extrasamplescount; i ++) {
-            dbgFile << i << "" << extrasamplescount << ""  << (cs->colorChannelCount()) <<  nbchannels << "" << sampleinfo[i];
+            dbgFile << "sample" << i << "extra sample count" << extrasamplescount << "color channel count" << (cs->colorChannelCount()) << "Number of channels" <<  nbchannels << "sample info" << sampleinfo[i];
+            if (sampleinfo[i] == EXTRASAMPLE_UNSPECIFIED) {
+                qWarning() << "Extra sample type not defined for this file, assuming unassociated alpha.";
+                alphapos = i;
+            }
             if (sampleinfo[i] == EXTRASAMPLE_ASSOCALPHA) {
                 // XXX: dangelo: the color values are already multiplied with
                 // the alpha value.  This needs to be reversed later (postprocessor?)
+                qWarning() << "Associated alpha in this file: krita does not handle plremultiplied alpha.";
                 alphapos = i;
             }
 
@@ -359,7 +421,7 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory(TIFF* image)
         TIFFClose(image);
         return KisImageBuilder_RESULT_INVALID_ARG;
     }
-    // Creating the KisImageWSP
+    // Creating the KisImageSP
     if (! m_image) {
         m_image = new KisImage(m_doc->createUndoStore(), width, height, cs, "built image");
         m_image->setResolution( POINT_TO_INCH(xres), POINT_TO_INCH(yres )); // It is the "invert" macro because we convert from pointer-per-inchs to points
@@ -620,13 +682,13 @@ KisImageBuilder_Result KisTIFFConverter::buildImage(const QString &filename)
 }
 
 
-KisImageWSP KisTIFFConverter::image()
+KisImageSP KisTIFFConverter::image()
 {
     return m_image;
 }
 
 
-KisImageBuilder_Result KisTIFFConverter::buildFile(const QString &filename, KisImageWSP kisimage, KisTIFFOptions options)
+KisImageBuilder_Result KisTIFFConverter::buildFile(const QString &filename, KisImageSP kisimage, KisTIFFOptions options)
 {
     dbgFile << "Start writing TIFF File";
     if (!kisimage)
@@ -664,6 +726,7 @@ KisImageBuilder_Result KisTIFFConverter::buildFile(const QString &filename, KisI
         TIFFClose(image);
         return KisImageBuilder_RESULT_FAILURE;
     }
+
     KisTIFFWriterVisitor* visitor = new KisTIFFWriterVisitor(image, &options);
     if (!visitor->visit(root)) {
         TIFFClose(image);
@@ -679,4 +742,3 @@ void KisTIFFConverter::cancel()
 {
     m_stop = true;
 }
-
