@@ -19,6 +19,7 @@
 #ifndef KIS_CURVE_PAINTOP_SETTINGS_H_
 #define KIS_CURVE_PAINTOP_SETTINGS_H_
 
+#include <QScopedPointer>
 #include <brushengine/kis_paintop_settings.h>
 
 class KisCurvePaintOpSettings : public KisPaintOpSettings
@@ -26,8 +27,17 @@ class KisCurvePaintOpSettings : public KisPaintOpSettings
 
 public:
     KisCurvePaintOpSettings();
-    virtual ~KisCurvePaintOpSettings() {}
+    ~KisCurvePaintOpSettings() override;
 
-    virtual bool paintIncremental();
+    void setPaintOpSize(qreal value) override;
+    qreal paintOpSize() const override;
+
+    bool paintIncremental() override;
+
+    QList<KisUniformPaintOpPropertySP> uniformProperties(KisPaintOpSettingsSP settings) override;
+
+private:
+    struct Private;
+    const QScopedPointer<Private> m_d;
 };
 #endif
