@@ -35,6 +35,7 @@
 #include <QComboBox>
 #include <QApplication>
 #include <QMatrix4x4>
+#include <QMenu>
 
 #include <kis_debug.h>
 #include <klocalizedstring.h>
@@ -64,7 +65,6 @@
 #include <kis_pixel_selection.h>
 #include <kis_shape_selection.h>
 #include <kis_selection_manager.h>
-#include <kis_system_locker.h>
 #include <krita_utils.h>
 #include <kis_resources_snapshot.h>
 
@@ -82,12 +82,15 @@
 #include "kis_transform_mask.h"
 #include "kis_transform_mask_adapter.h"
 
+#include "krita_container_utils.h"
+#include "kis_layer_utils.h"
+#include <KisDelayedUpdateNodeInterface.h>
+
 #include "strokes/transform_stroke_strategy.h"
 
 KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
     : KisTool(canvas, KisCursor::rotateCursor())
     , m_workRecursively(true)
-    , m_changesTracker(&m_transaction)
     , m_warpStrategy(
         new KisWarpTransformStrategy(
             dynamic_cast<KisCanvas2*>(canvas)->coordinatesConverter(),
@@ -99,7 +102,7 @@ KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
     , m_liquifyStrategy(
         new KisLiquifyTransformStrategy(
             dynamic_cast<KisCanvas2*>(canvas)->coordinatesConverter(),
-            m_currentArgs, m_transaction))
+            m_currentArgs, m_transaction, canvas->resourceManager()))
     , m_freeStrategy(
         new KisFreeTransformStrategy(
             dynamic_cast<KisCanvas2*>(canvas)->coordinatesConverter(),
@@ -118,6 +121,23 @@ KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
     useCursor(KisCursor::selectCursor());
     m_optionsWidget = 0;
 
+    warpAction = new KisAction(i18n("Warp"));
+    liquifyAction = new KisAction(i18n("Liquify"));
+    cageAction = new KisAction(i18n("Cage"));
+    freeTransformAction = new KisAction(i18n("Free"));
+    perspectiveAction = new KisAction(i18n("Perspective"));
+
+    // extra actions for free transform that are in the tool options
+    mirrorHorizontalAction = new KisAction(i18n("Mirror Horizontal"));
+    mirrorVericalAction = new KisAction(i18n("Mirror Vertical"));
+    rotateNinteyCWAction = new KisAction(i18n("Rotate 90 degrees Clockwise"));
+    rotateNinteyCCWAction = new KisAction(i18n("Rotate 90 degrees CounterClockwise"));
+
+    applyTransformation = new KisAction(i18n("Apply"));
+    resetTransformation = new KisAction(i18n("Reset"));
+
+    m_contextMenu.reset(new QMenu());
+
     connect(m_warpStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
     connect(m_cageStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
     connect(m_liquifyStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
@@ -129,8 +149,8 @@ KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
     connect(m_perspectiveStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
     connect(m_perspectiveStrategy.data(), SIGNAL(requestShowImageTooBig(bool)), SLOT(imageTooBigRequested(bool)));
 
-    connect(&m_changesTracker, SIGNAL(sigConfigChanged()),
-            this, SLOT(slotTrackerChangedConfig()));
+    connect(&m_changesTracker, SIGNAL(sigConfigChanged(KisToolChangesTrackerDataSP)),
+            this, SLOT(slotTrackerChangedConfig(KisToolChangesTrackerDataSP)));
 }
 
 KisToolTransform::~KisToolTransform()
@@ -151,9 +171,7 @@ void KisToolTransform::canvasUpdateRequested()
 
 void KisToolTransform::resetCursorStyle()
 {
-    KisTool::resetCursorStyle();
-
-    overrideCursorIfNotEditable();
+    setFunctionalCursor();
 }
 
 void KisToolTransform::resetRotationCenterButtonsRequested()
@@ -323,6 +341,35 @@ void KisToolTransform::endActionImpl(KoPointerEvent *event, bool usePrimaryActio
     updateApplyResetAvailability();
 }
 
+QMenu*  KisToolTransform::popupActionsMenu()
+{
+    if (m_contextMenu) {
+        m_contextMenu->clear();
+
+        // add a quick switch to different transform types
+        m_contextMenu->addAction(freeTransformAction);
+        m_contextMenu->addAction(perspectiveAction);
+        m_contextMenu->addAction(warpAction);
+        m_contextMenu->addAction(cageAction);
+        m_contextMenu->addAction(liquifyAction);
+
+        // extra options if free transform is selected
+        if (transformMode() == FreeTransformMode) {
+            m_contextMenu->addSeparator();
+            m_contextMenu->addAction(mirrorHorizontalAction);
+            m_contextMenu->addAction(mirrorVericalAction);
+            m_contextMenu->addAction(rotateNinteyCWAction);
+            m_contextMenu->addAction(rotateNinteyCCWAction);
+        }
+
+        m_contextMenu->addSeparator();
+        m_contextMenu->addAction(applyTransformation);
+        m_contextMenu->addAction(resetTransformation);
+    }
+
+    return m_contextMenu.data();
+}
+
 void KisToolTransform::beginPrimaryAction(KoPointerEvent *event)
 {
     beginActionImpl(event, true, KisTool::NONE);
@@ -338,9 +385,21 @@ void KisToolTransform::endPrimaryAction(KoPointerEvent *event)
     endActionImpl(event, true, KisTool::NONE);
 }
 
+void KisToolTransform::activatePrimaryAction()
+{
+    currentStrategy()->activatePrimaryAction();
+    setFunctionalCursor();
+}
+
+void KisToolTransform::deactivatePrimaryAction()
+{
+    currentStrategy()->deactivatePrimaryAction();
+}
+
 void KisToolTransform::activateAlternateAction(AlternateAction action)
 {
     currentStrategy()->activateAlternateAction(action);
+    setFunctionalCursor();
 }
 
 void KisToolTransform::deactivateAlternateAction(AlternateAction action)
@@ -374,7 +433,7 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *event)
 
     cursorOutlineUpdateRequested(mousePos);
 
-    if (!MOVE_CONDITION(event, KisTool::PAINT_MODE)) {
+    if (this->mode() != KisTool::PAINT_MODE) {
         currentStrategy()->hoverActionCommon(event);
         setFunctionalCursor();
         KisTool::mouseMoveEvent(event);
@@ -387,72 +446,6 @@ void KisToolTransform::mouseReleaseEvent(KoPointerEvent *event)
     KisTool::mouseReleaseEvent(event);
 }
 
-void KisToolTransform::touchEvent( QTouchEvent* event )
-{
-    //Count all moving touch points
-    int touchCount = 0;
-    Q_FOREACH ( QTouchEvent::TouchPoint tp, event->touchPoints() ) {
-        if( tp.state() == Qt::TouchPointMoved ) {
-            touchCount++;
-        }
-    }
-
-    //Use the touch point count to determine the gesture
-    switch( touchCount ) {
-    case 1: { //Panning
-        QTouchEvent::TouchPoint tp = event->touchPoints().at( 0 );
-        QPointF diff = tp.screenPos() - tp.lastScreenPos();
-
-        m_currentArgs.setTransformedCenter( m_currentArgs.transformedCenter() + diff );
-        outlineChanged();
-        break;
-    }
-    case 2: { //Scaling
-        QTouchEvent::TouchPoint tp1 = event->touchPoints().at( 0 );
-        QTouchEvent::TouchPoint tp2 = event->touchPoints().at( 1 );
-
-        float lastZoom = (tp1.lastScreenPos() - tp2.lastScreenPos()).manhattanLength();
-        float newZoom = (tp1.screenPos() - tp2.screenPos()).manhattanLength();
-
-        float diff = (newZoom - lastZoom) / 100;
-
-        m_currentArgs.setScaleX( m_currentArgs.scaleX() + diff );
-        m_currentArgs.setScaleY( m_currentArgs.scaleY() + diff );
-
-        outlineChanged();
-        break;
-    }
-    case 3: { //Rotation
-
-/* TODO: implement touch-based rotation.
-
-            Vector2f center;
-            Q_FOREACH ( const QTouchEvent::TouchPoint &tp, event->touchPoints() ) {
-                if( tp.state() == Qt::TouchPointMoved ) {
-                    center += Vector2f( tp.screenPos().x(), tp.screenPos().y() );
-                }
-            }
-            center /= touchCount;
-
-            QTouchEvent::TouchPoint tp = event->touchPoints().at(0);
-
-            Vector2f oldPosition = (Vector2f( tp.lastScreenPos().x(), tp.lastScreenPos().y() ) - center).normalized();
-            Vector2f newPosition = (Vector2f( tp.screenPos().x(), tp.screenPos().y() ) - center).normalized();
-
-            float oldAngle = qAcos( oldPosition.dot( Vector2f( 0.0f, 0.0f ) ) );
-            float newAngle = qAcos( newPosition.dot( Vector2f( 0.0f, 0.0f ) ) );
-
-            float diff = newAngle - oldAngle;
-
-            m_currentArgs.setAZ( m_currentArgs.aZ() + diff );
-
-            outlineChanged();
-*/
-        break;
-    }
-    }
-}
-
 void KisToolTransform::applyTransform()
 {
     slotApplyTransform();
@@ -651,7 +644,7 @@ bool KisToolTransform::tryInitTransformModeFromNode(KisNodeSP node)
             dynamic_cast<KisTransformMaskAdapter*>(savedParams.data());
 
         if (adapter) {
-            m_currentArgs = adapter->savedArgs();
+            m_currentArgs = adapter->transformArgs();
             initGuiAfterTransformMode();
             result = true;
         }
@@ -666,19 +659,25 @@ bool KisToolTransform::tryFetchArgsFromCommandAndUndo(ToolTransformArgs *args, T
 
     const KUndo2Command *lastCommand = image()->undoAdapter()->presentCommand();
     KisNodeSP oldRootNode;
+    KisNodeList oldTransformedNodes;
 
     if (lastCommand &&
-        TransformStrokeStrategy::fetchArgsFromCommand(lastCommand, args, &oldRootNode) &&
+        TransformStrokeStrategy::fetchArgsFromCommand(lastCommand, args, &oldRootNode, &oldTransformedNodes) &&
         args->mode() == mode &&
         oldRootNode == currentNode) {
 
-        args->saveContinuedState();
+        KisNodeList perspectiveTransformedNodes = fetchNodesList(mode, currentNode, m_workRecursively);
 
-        image()->undoAdapter()->undoLastCommand();
-        // FIXME: can we make it async?
-        image()->waitForDone();
+        if (KritaUtils::compareListsUnordered(oldTransformedNodes, perspectiveTransformedNodes)) {
+            args->saveContinuedState();
+            image()->undoAdapter()->undoLastCommand();
 
-        result = true;
+            // FIXME: can we make it async?
+            image()->waitForDone();
+            forceRepaintDelayedLayers(oldRootNode);
+
+            result = true;
+        }
     }
 
     return result;
@@ -730,7 +729,7 @@ void KisToolTransform::updateSelectionPath()
     m_selectionPath = QPainterPath();
 
     KisResourcesSnapshotSP resources =
-        new KisResourcesSnapshot(image(), currentNode(), 0, this->canvas()->resourceManager());
+        new KisResourcesSnapshot(image(), currentNode(), this->canvas()->resourceManager());
 
     QPainterPath selectionOutline;
     KisSelectionSP selection = resources->activeSelection();
@@ -769,7 +768,7 @@ void KisToolTransform::initThumbnailImage(KisPaintDeviceSP previewDevice)
         origImg = m_selectedPortionCache->
             createThumbnail(thumbRect.width(),
                             thumbRect.height(),
-                            srcRect,
+                            srcRect, 1,
                             KoColorConversionTransformation::internalRenderingIntent(),
                             KoColorConversionTransformation::internalConversionFlags());
         thumbToImageTransform = scaleTransform.inverted();
@@ -795,7 +794,7 @@ void KisToolTransform::activate(ToolActivation toolActivation, const QSet<KoShap
     KisTool::activate(toolActivation, shapes);
 
     if (currentNode()) {
-        m_transaction = TransformTransactionProperties(QRectF(), &m_currentArgs, currentNode());
+        m_transaction = TransformTransactionProperties(QRectF(), &m_currentArgs, KisNodeSP(), {});
     }
 
     startStroke(ToolTransformArgs::FREE_TRANSFORM, false);
@@ -812,7 +811,11 @@ void KisToolTransform::requestUndoDuringStroke()
 {
     if (!m_strokeData.strokeId()) return;
 
-    m_changesTracker.requestUndo();
+    if (m_changesTracker.isEmpty()) {
+        cancelStroke();
+    } else {
+        m_changesTracker.requestUndo();
+    }
 }
 
 void KisToolTransform::requestStrokeEnd()
@@ -822,17 +825,19 @@ void KisToolTransform::requestStrokeEnd()
 
 void KisToolTransform::requestStrokeCancellation()
 {
-    cancelStroke();
+    if (m_currentArgs.isIdentity()) {
+        cancelStroke();
+    } else {
+        slotResetTransform();
+    }
 }
 
 void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool forceReset)
 {
     Q_ASSERT(!m_strokeData.strokeId());
 
-    KisPaintDeviceSP dev;
-
     KisResourcesSnapshotSP resources =
-            new KisResourcesSnapshot(image(), currentNode(), 0, this->canvas()->resourceManager());
+            new KisResourcesSnapshot(image(), currentNode(), this->canvas()->resourceManager());
 
     KisNodeSP currentNode = resources->currentNode();
 
@@ -855,6 +860,12 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
         return;
     }
 
+    /**
+     * We must ensure that the currently selected subtree
+     * has finished all its updates.
+     */
+    forceRepaintDelayedLayers(currentNode);
+
     ToolTransformArgs fetchedArgs;
     bool fetchedFromCommand = false;
 
@@ -867,7 +878,24 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
             !currentNode->paintDevice();
     }
 
-    TransformStrokeStrategy *strategy = new TransformStrokeStrategy(currentNode, resources->activeSelection(), image()->postExecutionUndoAdapter());
+    QList<KisNodeSP> nodesList = fetchNodesList(mode, currentNode, m_workRecursively);
+
+    if (nodesList.isEmpty()) {
+        KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
+        kisCanvas->viewManager()->
+            showFloatingMessage(
+                i18nc("floating message in transformation tool",
+                      "Selected layer cannot be transformed with active transformation mode "),
+                 koIcon("object-locked"), 4000, KisFloatingMessage::High);
+
+        // force-reset transform mode to default
+        initTransformMode(mode);
+
+        return;
+    }
+
+
+    TransformStrokeStrategy *strategy = new TransformStrokeStrategy(currentNode, nodesList, resources->activeSelection(), image().data());
     KisPaintDeviceSP previewDevice = strategy->previewDevice();
 
     KisSelectionSP selection = strategy->realSelection();
@@ -892,10 +920,13 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
                       "Cannot transform empty layer "),
                 QIcon(), 1000, KisFloatingMessage::Medium);
 
+        // force-reset transform mode to default
+        initTransformMode(mode);
+
         return;
     }
 
-    m_transaction = TransformTransactionProperties(srcRect, &m_currentArgs, currentNode);
+    m_transaction = TransformTransactionProperties(srcRect, &m_currentArgs, currentNode, nodesList);
 
     initThumbnailImage(previewDevice);
     updateSelectionPath();
@@ -909,7 +940,7 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
 
     m_strokeData = StrokeData(image()->startStroke(strategy));
 
-    bool haveInvisibleNodes = clearDevices(m_transaction.rootNode(), m_workRecursively);
+    bool haveInvisibleNodes = clearDevices(nodesList);
     if (haveInvisibleNodes) {
         KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
         kisCanvas->viewManager()->
@@ -929,13 +960,13 @@ void KisToolTransform::endStroke()
     if (!m_strokeData.strokeId()) return;
 
     if (!m_currentArgs.isIdentity()) {
-        transformDevices(m_transaction.rootNode(), m_workRecursively);
+        transformClearedDevices();
 
         image()->addJob(m_strokeData.strokeId(),
                         new TransformStrokeStrategy::TransformData(
                             TransformStrokeStrategy::TransformData::SELECTION,
                             m_currentArgs,
-                            m_transaction.rootNode()));
+                            m_transaction.rootNode())); // root node is used for progress only
 
         image()->endStroke(m_strokeData.strokeId());
     } else {
@@ -944,6 +975,8 @@ void KisToolTransform::endStroke()
 
     m_strokeData.clear();
     m_changesTracker.reset();
+    m_transaction = TransformTransactionProperties(QRectF(), &m_currentArgs, KisNodeSP(), {});
+    outlineChanged();
 }
 
 void KisToolTransform::cancelStroke()
@@ -957,6 +990,8 @@ void KisToolTransform::cancelStroke()
         image()->cancelStroke(m_strokeData.strokeId());
         m_strokeData.clear();
         m_changesTracker.reset();
+        m_transaction = TransformTransactionProperties(QRectF(), &m_currentArgs, KisNodeSP(), {});
+        outlineChanged();
     }
 }
 
@@ -964,60 +999,75 @@ void KisToolTransform::commitChanges()
 {
     if (!m_strokeData.strokeId()) return;
 
-    m_changesTracker.commitConfig(m_currentArgs);
+    m_changesTracker.commitConfig(toQShared(m_currentArgs.clone()));
 }
 
-void KisToolTransform::slotTrackerChangedConfig()
+void KisToolTransform::slotTrackerChangedConfig(KisToolChangesTrackerDataSP status)
 {
+    const ToolTransformArgs *newArgs = dynamic_cast<const ToolTransformArgs*>(status.data());
+    KIS_SAFE_ASSERT_RECOVER_RETURN(newArgs);
+
+    *m_transaction.currentConfig() = *newArgs;
+
     slotUiChangedConfig();
     updateOptionWidget();
 }
 
-bool KisToolTransform::clearDevices(KisNodeSP node, bool recursive)
+QList<KisNodeSP> KisToolTransform::fetchNodesList(ToolTransformArgs::TransformMode mode, KisNodeSP root, bool recursive)
 {
-    bool haveInvisibleNodes = false;
-    if (!node->isEditable(false)) return haveInvisibleNodes;
+    QList<KisNodeSP> result;
 
-    haveInvisibleNodes = !node->visible(false);
+    auto fetchFunc =
+        [&result, mode, root] (KisNodeSP node) {
+            if (node->isEditable(node == root) &&
+                (!node->inherits("KisShapeLayer") || mode == ToolTransformArgs::FREE_TRANSFORM) &&
+                !node->inherits("KisFileLayer") &&
+                (!node->inherits("KisTransformMask") || node == root)) {
+
+                result << node;
+            }
+    };
 
     if (recursive) {
-        // simple tail-recursive iteration
-        KisNodeSP prevNode = node->lastChild();
-        while(prevNode) {
-            haveInvisibleNodes |= clearDevices(prevNode, recursive);
-            prevNode = prevNode->prevSibling();
-        }
+        KisLayerUtils::recursiveApplyNodes(root, fetchFunc);
+    } else {
+        fetchFunc(root);
     }
 
-    image()->addJob(m_strokeData.strokeId(),
-                    new TransformStrokeStrategy::ClearSelectionData(node));
+    return result;
+}
 
-    /**
-     * It might happen that the editablity state of the node would
-     * change during the stroke, so we need to save the set of
-     * applicable nodes right in the beginning of the processing
-     */
-    m_strokeData.addClearedNode(node);
+bool KisToolTransform::clearDevices(const QList<KisNodeSP> &nodes)
+{
+    bool haveInvisibleNodes = false;
+
+    Q_FOREACH (KisNodeSP node, nodes) {
+        haveInvisibleNodes |= !node->visible(false);
+
+        image()->addJob(m_strokeData.strokeId(),
+                        new TransformStrokeStrategy::ClearSelectionData(node));
+
+        /**
+         * It might happen that the editablity state of the node would
+         * change during the stroke, so we need to save the set of
+         * applicable nodes right in the beginning of the processing
+         */
+        m_strokeData.addClearedNode(node);
+    }
 
     return haveInvisibleNodes;
 }
 
-void KisToolTransform::transformDevices(KisNodeSP node, bool recursive)
+void KisToolTransform::transformClearedDevices()
 {
-    if (!node->isEditable()) return;
-
-    KIS_ASSERT_RECOVER_RETURN(recursive ||
-                              (m_strokeData.clearedNodes().size() == 1 &&
-                               KisNodeSP(m_strokeData.clearedNodes().first()) == node));
-
-    Q_FOREACH (KisNodeSP currentNode, m_strokeData.clearedNodes()) {
-        KIS_ASSERT_RECOVER_RETURN(currentNode);
+    Q_FOREACH (KisNodeSP node, m_strokeData.clearedNodes()) {
+        KIS_ASSERT_RECOVER_RETURN(node);
 
         image()->addJob(m_strokeData.strokeId(),
                         new TransformStrokeStrategy::TransformData(
                             TransformStrokeStrategy::TransformData::PAINT_DEVICE,
                             m_currentArgs,
-                            currentNode));
+                            node));
     }
 }
 
@@ -1048,6 +1098,23 @@ QWidget* KisToolTransform::createOptionWidget() {
     connect(m_optionsWidget, SIGNAL(sigEditingFinished()),
             this, SLOT(slotEditingFinished()));
 
+
+    connect(mirrorHorizontalAction, SIGNAL(triggered(bool)), m_optionsWidget, SLOT(slotFlipX()));
+    connect(mirrorVericalAction, SIGNAL(triggered(bool)), m_optionsWidget, SLOT(slotFlipY()));
+    connect(rotateNinteyCWAction, SIGNAL(triggered(bool)), m_optionsWidget, SLOT(slotRotateCW()));
+    connect(rotateNinteyCCWAction, SIGNAL(triggered(bool)), m_optionsWidget, SLOT(slotRotateCCW()));
+
+
+    connect(warpAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToWarpType()));
+    connect(perspectiveAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToPerspectiveType()));
+    connect(freeTransformAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToFreeTransformType()));
+    connect(liquifyAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToLiquifyType()));
+    connect(cageAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToCageType()));
+
+    connect(applyTransformation, SIGNAL(triggered(bool)), this, SLOT(slotApplyTransform()));
+    connect(resetTransformation, SIGNAL(triggered(bool)), this, SLOT(slotResetTransform()));
+
+
     updateOptionWidget();
 
     return m_optionsWidget;
@@ -1100,14 +1167,30 @@ void KisToolTransform::slotResetTransform()
     if (m_currentArgs.continuedTransform()) {
         ToolTransformArgs::TransformMode savedMode = m_currentArgs.mode();
 
-        if (m_currentArgs.continuedTransform()->mode() == savedMode) {
+        /**
+         * Our reset transform button can be used for two purposes:
+         *
+         * 1) Reset current transform to the initial one, which was
+         *    loaded from the previous user action.
+         *
+         * 2) Reset transform frame to infinity when the frame is unchanged
+         */
+
+        const bool transformDiffers = !m_currentArgs.continuedTransform()->isSameMode(m_currentArgs);
+
+        if (transformDiffers &&
+            m_currentArgs.continuedTransform()->mode() == savedMode) {
+
             m_currentArgs.restoreContinuedState();
             initGuiAfterTransformMode();
             slotEditingFinished();
 
         } else {
+            KisNodeSP root = m_transaction.rootNode() ? m_transaction.rootNode() : image()->root();
+
             cancelStroke();
             image()->waitForDone();
+            forceRepaintDelayedLayers(root);
             startStroke(savedMode, true);
 
             KIS_ASSERT_RECOVER_NOOP(!m_currentArgs.continuedTransform());
@@ -1122,17 +1205,55 @@ void KisToolTransform::slotRestartTransform()
 {
     if (!m_strokeData.strokeId()) return;
 
+    KisNodeSP root = m_transaction.rootNode();
+    KIS_ASSERT_RECOVER_RETURN(root); // the stroke is guaranteed to be started by an 'if' above
+
     ToolTransformArgs savedArgs(m_currentArgs);
     cancelStroke();
     image()->waitForDone();
+    forceRepaintDelayedLayers(root);
     startStroke(savedArgs.mode(), true);
 }
 
+
+void KisToolTransform::forceRepaintDelayedLayers(KisNodeSP root)
+{
+    KIS_SAFE_ASSERT_RECOVER_RETURN(root);
+
+    KisLayerUtils::forceAllDelayedNodesUpdate(root);
+    image()->waitForDone();
+}
+
 void KisToolTransform::slotEditingFinished()
 {
     commitChanges();
 }
 
+void KisToolTransform::slotUpdateToWarpType()
+{
+    setTransformMode(KisToolTransform::TransformToolMode::WarpTransformMode);
+}
+
+void KisToolTransform::slotUpdateToPerspectiveType()
+{
+    setTransformMode(KisToolTransform::TransformToolMode::PerspectiveTransformMode);
+}
+
+void KisToolTransform::slotUpdateToFreeTransformType()
+{
+    setTransformMode(KisToolTransform::TransformToolMode::FreeTransformMode);
+}
+
+void KisToolTransform::slotUpdateToLiquifyType()
+{
+    setTransformMode(KisToolTransform::TransformToolMode::LiquifyTransformMode);
+}
+
+void KisToolTransform::slotUpdateToCageType()
+{
+    setTransformMode(KisToolTransform::TransformToolMode::CageTransformMode);
+}
+
 void KisToolTransform::setShearY(double shear)
 {
     m_optionsWidget->slotSetShearY(shear);
