@@ -26,6 +26,7 @@
 
 #include "KisMainWindow.h"
 #include "KisViewManager.h"
+#include "kis_transform_utils.h"
 
 
 template<typename T> inline T sign(T x) {
@@ -45,6 +46,10 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     setupUi(this);
     showDecorationsBox->setIcon(KisIconUtils::loadIcon("krita_tool_transform"));
     chkWorkRecursively->setIcon(KisIconUtils::loadIcon("krita_tool_transform_recursive"));
+    flipXButton->setIcon(KisIconUtils::loadIcon("transform_icons_mirror_x"));
+    flipYButton->setIcon(KisIconUtils::loadIcon("transform_icons_mirror_y"));
+    rotateCWButton->setIcon(KisIconUtils::loadIcon("transform_icons_rotate_cw"));
+    rotateCCWButton->setIcon(KisIconUtils::loadIcon("transform_icons_rotate_ccw"));
 
     chkWorkRecursively->setChecked(workRecursively);
     connect(chkWorkRecursively, SIGNAL(toggled(bool)), this, SIGNAL(sigRestartTransform()));
@@ -53,12 +58,12 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     cmbFilter->setIDList(KisFilterStrategyRegistry::instance()->listKeys());
     cmbFilter->setCurrent("Bicubic");
     cmbFilter->setToolTip(i18nc("@info:tooltip",
-                                "<para>Select filtering mode:\n"
-                                "<list>"
-                                "<item><interface>Bilinear</interface> for areas with uniform color to avoid artifacts</item>"
-                                "<item><interface>Bicubic</interface> for smoother results</item>"
-                                "<item><interface>Lanczos3</interface> for sharp results. May produce aerials.</item>"
-                                "</list></para>"));
+                                "<p>Select filtering mode:\n"
+                                "<ul>"
+                                "<li><b>Bilinear</b> for areas with uniform color to avoid artifacts</li>"
+                                "<li><b>Bicubic</b> for smoother results</li>"
+                                "<li><b>Lanczos3</b> for sharp results. May produce aerials.</li>"
+                                "</ul></p>"));
     connect(cmbFilter, SIGNAL(activated(const KoID &)),
             this, SLOT(slotFilterChanged(const KoID &)));
 
@@ -128,17 +133,21 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     aXBox->setSuffix(QChar(Qt::Key_degree));
     aYBox->setSuffix(QChar(Qt::Key_degree));
     aZBox->setSuffix(QChar(Qt::Key_degree));
-    aXBox->setRange(0.0, 360.0, 0);
-    aYBox->setRange(0.0, 360.0, 0);
-    aZBox->setRange(0.0, 360.0, 0);
+    aXBox->setRange(0.0, 360.0, 2);
+    aYBox->setRange(0.0, 360.0, 2);
+    aZBox->setRange(0.0, 360.0, 2);
     aXBox->setValue(0.0);
     aYBox->setValue(0.0);
     aZBox->setValue(0.0);
+    aXBox->setSingleStep(1.0);
+    aYBox->setSingleStep(1.0);
+    aZBox->setSingleStep(1.0);
 
 
 
 
     connect(m_rotationCenterButtons, SIGNAL(buttonPressed(int)), this, SLOT(slotRotationCenterChanged(int)));
+    connect(btnTransformAroundPivotPoint, SIGNAL(clicked(bool)), this, SLOT(slotTransformAroundRotationCenter(bool)));
 
     // Init Free Transform Values
     connect(scaleXBox, SIGNAL(valueChanged(int)), this, SLOT(slotSetScaleX(int)));
@@ -151,6 +160,10 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     connect(aYBox, SIGNAL(valueChanged(qreal)), this, SLOT(slotSetAY(qreal)));
     connect(aZBox, SIGNAL(valueChanged(qreal)), this, SLOT(slotSetAZ(qreal)));
     connect(aspectButton, SIGNAL(keepAspectRatioChanged(bool)), this, SLOT(slotSetKeepAspectRatio(bool)));
+    connect(flipXButton, SIGNAL(clicked(bool)), this, SLOT(slotFlipX()));
+    connect(flipYButton, SIGNAL(clicked(bool)), this, SLOT(slotFlipY()));
+    connect(rotateCWButton, SIGNAL(clicked(bool)), this, SLOT(slotRotateCW()));
+    connect(rotateCCWButton, SIGNAL(clicked(bool)), this, SLOT(slotRotateCCW()));
 
     // toggle visibility of different free buttons
     connect(freeMoveRadioButton, SIGNAL(clicked(bool)), SLOT(slotTransformAreaVisible(bool)));
@@ -166,7 +179,7 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
 
 
 
-    // Init Warp Transform Values  
+    // Init Warp Transform Values
     alphaBox->setSingleStep(0.1);
     alphaBox->setRange(0, 10, 1);
 
@@ -187,12 +200,12 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     liquifySizeSlider->setRange(KisLiquifyProperties::minSize(),
                                 KisLiquifyProperties::maxSize(), 2);
     liquifySizeSlider->setExponentRatio(4);
-    liquifySizeSlider->setValue(50.0);
+    liquifySizeSlider->setValue(60.0);
     connect(liquifySizeSlider, SIGNAL(valueChanged(qreal)), this, SLOT(liquifySizeChanged(qreal)));
     liquifySizeSlider->setToolTip(i18nc("@info:tooltip", "Size of the deformation brush"));
 
     liquifyAmountSlider->setRange(0.0, 1.0, 2);
-    liquifyAmountSlider->setValue(1.0);
+    liquifyAmountSlider->setValue(0.05);
     connect(liquifyAmountSlider, SIGNAL(valueChanged(qreal)), this, SLOT(liquifyAmountChanged(qreal)));
     liquifyAmountSlider->setToolTip(i18nc("@info:tooltip", "Amount of the deformation you get"));
 
@@ -201,12 +214,10 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     connect(liquifyFlowSlider, SIGNAL(valueChanged(qreal)), this, SLOT(liquifyFlowChanged(qreal)));
     liquifyFlowSlider->setToolTip(i18nc("@info:tooltip", "When in non-buildup mode, shows how fast the deformation limit is reached."));
 
-
     buidupModeComboBox->setCurrentIndex(0); // set to build-up mode by default
     connect(buidupModeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(liquifyBuildUpChanged(int)));
     buidupModeComboBox->setToolTip(i18nc("@info:tooltip", "Switch between Build Up and Wash mode of painting. Build Up mode adds deformations one on top of the other without any limits. Wash mode gradually deforms the piece to the selected deformation level."));
 
-
     liquifySpacingSlider->setRange(0.0, 3.0, 2);
     liquifySizeSlider->setExponentRatio(3);
     liquifySpacingSlider->setSingleStep(0.01);
@@ -216,11 +227,11 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
 
     liquifySizePressureBox->setChecked(true);
     connect(liquifySizePressureBox, SIGNAL(toggled(bool)), this, SLOT(liquifySizePressureChanged(bool)));
-    liquifySizePressureBox->setToolTip(i18nc("@info:tooltip", "Scale <interface>Size</interface> value according to current stylus pressure"));
+    liquifySizePressureBox->setToolTip(i18nc("@info:tooltip", "Scale <b>Size</b> value according to current stylus pressure"));
 
     liquifyAmountPressureBox->setChecked(true);
     connect(liquifyAmountPressureBox, SIGNAL(toggled(bool)), this, SLOT(liquifyAmountPressureChanged(bool)));
-    liquifyAmountPressureBox->setToolTip(i18nc("@info:tooltip", "Scale <interface>Amount</interface> value according to current stylus pressure"));
+    liquifyAmountPressureBox->setToolTip(i18nc("@info:tooltip", "Scale <b>Amount</b> value according to current stylus pressure"));
 
     liquifyReverseDirectionChk->setChecked(false);
     connect(liquifyReverseDirectionChk, SIGNAL(toggled(bool)), this, SLOT(liquifyReverseDirectionChanged(bool)));
@@ -257,11 +268,13 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     connect(cmbWarpType, SIGNAL(currentIndexChanged(int)), this, SLOT(notifyEditingFinished()));
     connect(m_rotationCenterButtons, SIGNAL(buttonPressed(int)), this, SLOT(notifyEditingFinished()));
     connect(aspectButton, SIGNAL(keepAspectRatioChanged(bool)), this, SLOT(notifyEditingFinished()));
-    connect(defaultRadioButton, SIGNAL(clicked(bool)), this, SLOT(notifyEditingFinished()));
-    connect(customRadioButton, SIGNAL(clicked(bool)), this, SLOT(notifyEditingFinished()));
+
     connect(lockUnlockPointsButton, SIGNAL(clicked()), this, SLOT(notifyEditingFinished()));
     connect(resetPointsButton, SIGNAL(clicked()), this, SLOT(notifyEditingFinished()));
 
+    connect(defaultRadioButton, SIGNAL(clicked(bool)), this, SLOT(notifyEditingFinished()));
+    connect(customRadioButton, SIGNAL(clicked(bool)), this, SLOT(notifyEditingFinished()));
+
     // Liquify
     //
     // liquify brush options do not affect the image directly and are not
@@ -311,6 +324,8 @@ void KisToolTransformConfigWidget::slotUpdateIcons()
     middleBottomButton->setIcon(KisIconUtils::loadIcon("arrow-down"));
     bottomRightButton->setIcon(KisIconUtils::loadIcon("arrow-downright"));
 
+    btnTransformAroundPivotPoint->setIcon(KisIconUtils::loadIcon("pivot-point"));
+
 
     // pressure icons
     liquifySizePressureBox->setIcon(KisIconUtils::loadIcon("transform_icons_penPressure"));
@@ -510,13 +525,15 @@ void KisToolTransformConfigWidget::updateConfig(const ToolTransformArgs &config)
     if (config.mode() == ToolTransformArgs::FREE_TRANSFORM ||
         config.mode() == ToolTransformArgs::PERSPECTIVE_4POINT) {
 
+        quickTransformGroup->setEnabled(config.mode() == ToolTransformArgs::FREE_TRANSFORM);
+
         stackedWidget->setCurrentIndex(0);
 
         bool freeTransformIsActive = config.mode() == ToolTransformArgs::FREE_TRANSFORM;
 
         if (freeTransformIsActive)
         {
-            freeTransformButton->setChecked(true);           
+            freeTransformButton->setChecked(true);
         }
         else
         {
@@ -526,19 +543,19 @@ void KisToolTransformConfigWidget::updateConfig(const ToolTransformArgs &config)
         aXBox->setEnabled(freeTransformIsActive);
         aYBox->setEnabled(freeTransformIsActive);
         aZBox->setEnabled(freeTransformIsActive);
-        Q_FOREACH (QAbstractButton *button, m_rotationCenterButtons->buttons()) {
-            button->setEnabled(freeTransformIsActive);
-            freeRotationRadioButton->setEnabled(freeTransformIsActive);
-        }
+        freeRotationRadioButton->setEnabled(freeTransformIsActive);
 
         scaleXBox->setValue(config.scaleX() * 100.);
         scaleYBox->setValue(config.scaleY() * 100.);
         shearXBox->setValue(config.shearX());
         shearYBox->setValue(config.shearY());
 
-        translateXBox->setValue(config.transformedCenter().x());
-        translateYBox->setValue(config.transformedCenter().y());
+        const QPointF anchorPoint = config.originalCenter() + config.rotationCenterOffset();
+        const KisTransformUtils::MatricesPack m(config);
+        const QPointF anchorPointView = m.finalTransform().map(anchorPoint);
 
+        translateXBox->setValue(anchorPointView.x());
+        translateYBox->setValue(anchorPointView.y());
 
         aXBox->setValue(radianToDegree(config.aX()));
         aYBox->setValue(radianToDegree(config.aY()));
@@ -558,6 +575,9 @@ void KisToolTransformConfigWidget::updateConfig(const ToolTransformArgs &config)
                 break;
             }
         }
+
+        btnTransformAroundPivotPoint->setChecked(config.transformAroundRotationCenter());
+
     } else if (config.mode() == ToolTransformArgs::WARP) {
 
         stackedWidget->setCurrentIndex(1);
@@ -768,6 +788,7 @@ void KisToolTransformConfigWidget::slotSetWarpModeButtonClicked(bool value)
 
     ToolTransformArgs *config = m_transaction->currentConfig();
     config->setMode(ToolTransformArgs::WARP);
+    config->setWarpCalculation(KisWarpTransformWorker::WarpCalculation::GRID);
     emit sigResetTransform();
 }
 
@@ -825,15 +846,30 @@ void KisToolTransformConfigWidget::slotRotationCenterChanged(int index)
                                                 j * m_transaction->originalHalfHeight()));
 
         notifyConfigChanged();
+        updateConfig(*config);
     }
 }
 
+void KisToolTransformConfigWidget::slotTransformAroundRotationCenter(bool value)
+{
+    if (m_uiSlotsBlocked) return;
+
+    ToolTransformArgs *config = m_transaction->currentConfig();
+    config->setTransformAroundRotationCenter(value);
+    notifyConfigChanged();
+    notifyEditingFinished();
+}
+
 void KisToolTransformConfigWidget::slotSetScaleX(int value)
 {
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setScaleX(value / 100.);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setScaleX(value / 100.);
+    }
 
     if (config->keepAspectRatio()) {
 
@@ -842,7 +878,10 @@ void KisToolTransformConfigWidget::slotSetScaleX(int value)
 
         scaleYBox->blockSignals(true);
         scaleYBox->setValue(calculatedValue);
-        config->setScaleY(calculatedValue / 100.);
+        {
+            KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+            config->setScaleY(calculatedValue / 100.);
+        }
         scaleYBox->blockSignals(false);
 
         unblockNotifications();
@@ -857,14 +896,21 @@ void KisToolTransformConfigWidget::slotSetScaleY(int value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setScaleY(value / 100.);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setScaleY(value / 100.);
+    }
 
     if (config->keepAspectRatio()) {
         blockNotifications();
         int calculatedValue = int(m_scaleRatio * value);
         scaleXBox->blockSignals(true);
         scaleXBox->setValue(calculatedValue);
-        config->setScaleX(calculatedValue / 100.);
+        {
+            KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+            config->setScaleX(calculatedValue / 100.);
+        }
         scaleXBox->blockSignals(false);
         unblockNotifications();
     }
@@ -878,7 +924,12 @@ void KisToolTransformConfigWidget::slotSetShearX(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setShearX((double)value);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setShearX((double)value);
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -888,7 +939,13 @@ void KisToolTransformConfigWidget::slotSetShearY(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setShearY((double)value);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setShearY((double)value);
+    }
+
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -898,7 +955,14 @@ void KisToolTransformConfigWidget::slotSetTranslateX(int value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setTransformedCenter(QPointF(value, config->transformedCenter().y()));
+
+    const QPointF anchorPoint = config->originalCenter() + config->rotationCenterOffset();
+    const KisTransformUtils::MatricesPack m(*config);
+
+    const QPointF anchorPointView = m.finalTransform().map(anchorPoint);
+    const QPointF newAnchorPointView(value, anchorPointView.y());
+    config->setTransformedCenter(config->transformedCenter() + newAnchorPointView - anchorPointView);
+
     notifyConfigChanged();
 }
 
@@ -907,7 +971,14 @@ void KisToolTransformConfigWidget::slotSetTranslateY(int value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setTransformedCenter(QPointF(config->transformedCenter().x(), value));
+
+    const QPointF anchorPoint = config->originalCenter() + config->rotationCenterOffset();
+    const KisTransformUtils::MatricesPack m(*config);
+
+    const QPointF anchorPointView = m.finalTransform().map(anchorPoint);
+    const QPointF newAnchorPointView(anchorPointView.x(), value);
+    config->setTransformedCenter(config->transformedCenter() + newAnchorPointView - anchorPointView);
+
     notifyConfigChanged();
 }
 
@@ -916,7 +987,10 @@ void KisToolTransformConfigWidget::slotSetAX(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setAX(degreeToRadian((double)value));
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAX(degreeToRadian((double)value));
+    }
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -926,7 +1000,12 @@ void KisToolTransformConfigWidget::slotSetAY(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setAY(degreeToRadian((double)value));
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAY(degreeToRadian((double)value));
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -936,7 +1015,64 @@ void KisToolTransformConfigWidget::slotSetAZ(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setAZ(degreeToRadian((double)value));
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAZ(degreeToRadian((double)value));
+    }
+
+    notifyConfigChanged();
+    notifyEditingFinished();
+}
+
+void KisToolTransformConfigWidget::slotFlipX()
+{
+    ToolTransformArgs *config = m_transaction->currentConfig();
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setScaleX(config->scaleX() * -1);
+    }
+
+    notifyConfigChanged();
+    notifyEditingFinished();
+}
+
+void KisToolTransformConfigWidget::slotFlipY()
+{
+    ToolTransformArgs *config = m_transaction->currentConfig();
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setScaleY(config->scaleY() * -1);
+    }
+
+    notifyConfigChanged();
+    notifyEditingFinished();
+}
+
+void KisToolTransformConfigWidget::slotRotateCW()
+{
+    ToolTransformArgs *config = m_transaction->currentConfig();
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAZ(normalizeAngle(config->aZ() + M_PI_2));
+    }
+
+    notifyConfigChanged();
+    notifyEditingFinished();
+}
+
+void KisToolTransformConfigWidget::slotRotateCCW()
+{
+    ToolTransformArgs *config = m_transaction->currentConfig();
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAZ(normalizeAngle(config->aZ() - M_PI_2));
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -1061,11 +1197,16 @@ void KisToolTransformConfigWidget::activateCustomWarpPoints(bool enabled)
     if (!enabled) {
         config->setEditingTransformPoints(false);
         setDefaultWarpPoints(densityBox->value());
+        config->setWarpCalculation(KisWarpTransformWorker::WarpCalculation::GRID);
     } else {
         config->setEditingTransformPoints(true);
+        config->setWarpCalculation(KisWarpTransformWorker::WarpCalculation::DRAW);
         setDefaultWarpPoints(0);
     }
 
+
+
+
     updateLockPointsButtonCaption();
 }
 
